//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using BSBMTextFactory = OQJ_BSBMTextFactory.BSBMTextFactory;


public static class BSAuxFuncs
{

	public static void InitCoordinateSystem3D(bool REC = false)
	{
		string name = (!REC) ? "CoordinateSystem3D" : "REC_CoordinateSystem3D";
		InitCoordinateSystem3D(name, REC);
	}

	static void InitCoordinateSystem3D(string crdsys_name, bool REC)
	{
		GameObject goCS = new GameObject(crdsys_name);
		goCS.AddComponent<MeshFilter>();
		goCS.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		float gridGray = 0.0f;
		float bkgrndGray = 0.0f;
		Color clrNegX = new Color();
		Color clrNegY = new Color();
		Color clrNegZ = new Color();
		Color clrPlsX = new Color();
		Color clrPlsY = new Color();
		Color clrPlsZ = new Color();


		if (!REC) 
		{
			gridGray = 0.56f;   
			bkgrndGray = BSData.BackgroundColor.r;

			float alpha = 0.25f;
			float semiVal = alpha * 0.73f + (1.0f - alpha) * bkgrndGray;
			float semiBgr = alpha * 0.0f + (1.0f - alpha) * bkgrndGray;
			clrNegX = new Color(semiVal, semiBgr, semiBgr);
			clrNegY = new Color(semiBgr, semiVal, semiBgr);
			clrNegZ = new Color(semiBgr, semiBgr, semiVal);
			clrPlsX = new Color(0.73f, 0.0f, 0.0f);
			clrPlsY = new Color(0.0f, 0.73f, 0.0f);
			clrPlsZ = new Color(0.0f, 0.0f, 0.73f);
		}


		Mesh wMesh = new Mesh();
		Color gridColor = new Color(gridGray, gridGray, gridGray);
		const int gridMax = 60;
		int[] idcs = new int[gridMax * 2 * 4 + 12];  
		int vi = 0;
		for (int i = -gridMax; i <= gridMax; i++)
		{
			if (i == 0) { continue; }

			verts.Add(new Vector3((float)-gridMax, -0.001f, (float)i));
			colors.Add(gridColor);
			verts.Add(new Vector3((float)gridMax, -0.001f, (float)i));
			colors.Add(gridColor);

			verts.Add(new Vector3((float)i, -0.001f, (float)-gridMax));
			colors.Add(gridColor);
			verts.Add(new Vector3((float)i, -0.001f, (float)gridMax));
			colors.Add(gridColor);

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;
			idcs[vi + 2] = vi + 2;
			idcs[vi + 3] = vi + 3;
			vi += 4;
		}

		verts.Add(new Vector3((float)-gridMax, 0.0f, 0.0f));
		colors.Add(clrNegX);	
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(clrNegX);	

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(clrPlsX);
		verts.Add(new Vector3((float)gridMax, 0.0f, 0.0f));
		colors.Add(clrPlsX);

		verts.Add(new Vector3(0.0f, (float)-gridMax, 0.0f));        
		colors.Add(clrNegY);
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(clrNegY);

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(clrPlsY);
		verts.Add(new Vector3(0.0f, (float)gridMax, 0.0f));         
		colors.Add(clrPlsY);

		verts.Add(new Vector3(0.0f, 0.0f, (float)-gridMax));
		colors.Add(clrNegZ);
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(clrNegZ);

		verts.Add(new Vector3((float) 0.0f, 0.0f, 0.0f));
		colors.Add(clrPlsZ);
		verts.Add(new Vector3(0.0f, 0.0f, (float) gridMax));
		colors.Add(clrPlsZ);

		idcs[vi] = vi;
		idcs[vi + 1] = vi + 1;	
		idcs[vi + 2] = vi + 2;	
		idcs[vi + 3] = vi + 3;
		idcs[vi + 4] = vi + 4;
		idcs[vi + 5] = vi + 5;
		idcs[vi + 6] = vi + 6;
		idcs[vi + 7] = vi + 7;
		idcs[vi + 8] = vi + 8;
		idcs[vi + 9] = vi + 9;
		idcs[vi + 10] = vi + 10;
		idcs[vi + 11] = vi + 11;

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = goCS.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		goCS.transform.localPosition = Vector3.zero;
		goCS.transform.localRotation = Quaternion.identity;
		goCS.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = goCS.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(goCS.name, goCS);
	}



	public static void InitCoordinateSystem2D(string name="CoordinateSystem2D", float gray=0.657f) 
	{
		GameObject goCS = new GameObject(name);
		goCS.AddComponent<MeshFilter>();
		goCS.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		const int gridMax = 30;
		Mesh wMesh = new Mesh();

		gray = 0.505f;
		Color gridColor = new Color(gray, gray, gray);
		int[] idcs = new int[gridMax * 2 * 4 + 4];	
		int vi = 0;	
		for (int i = -gridMax; i <= gridMax; i++)
		{
			if (i == 0) { continue; }

			verts.Add(new Vector3((float)-gridMax, (float)i, 0.001f));
			colors.Add(gridColor);	
			verts.Add(new Vector3((float) gridMax, (float)i, 0.001f));
			colors.Add(gridColor);	

			verts.Add(new Vector3((float)i, (float)-gridMax, 0.001f));
			colors.Add(gridColor);	
			verts.Add(new Vector3((float)i, (float) gridMax, 0.001f));
			colors.Add(gridColor);

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;	
			idcs[vi + 2] = vi + 2;	
			idcs[vi + 3] = vi + 3;
			vi += 4;	
		}

		verts.Add(new Vector3((float)-gridMax, 0.0f, 0.001f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));	
		verts.Add(new Vector3((float) gridMax, 0.0f, 0.001f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));	

		verts.Add(new Vector3(0.0f, (float)-gridMax, 0.001f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));
		verts.Add(new Vector3(0.0f, (float) gridMax, 0.001f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));

		idcs[vi] = vi;
		idcs[vi + 1] = vi + 1;	
		idcs[vi + 2] = vi + 2;	
		idcs[vi + 3] = vi + 3;

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = goCS.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		goCS.transform.localPosition = Vector3.zero;
		goCS.transform.localRotation = Quaternion.identity;
		goCS.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = goCS.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(goCS.name, goCS);

	} 



	public static void InitXYAxes()
	{
		GameObject XY_Axes = new GameObject("XY_Axes");
		XY_Axes.AddComponent<MeshFilter>();
		XY_Axes.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		const int gridMax = 30;
		int[] idcs = new int[8];


		verts.Add(new Vector3(-gridMax, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));
		verts.Add(new Vector3(gridMax, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));

		verts.Add(new Vector3(0.0f, -gridMax, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));	
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));	

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));	
		verts.Add(new Vector3(0.0f, gridMax, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));

		int vi = 0;
		idcs[vi] = vi;				
		idcs[vi + 1] = vi + 1;	
		idcs[vi + 2] = vi + 2;	
		idcs[vi + 3] = vi + 3;
		idcs[vi + 4] = vi + 4;		
		idcs[vi + 5] = vi + 5;
		idcs[vi + 6] = vi + 6;
		idcs[vi + 7] = vi + 7;

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = XY_Axes.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		XY_Axes.transform.localPosition = Vector3.zero;
		XY_Axes.transform.localRotation = Quaternion.identity;
		XY_Axes.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = XY_Axes.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(XY_Axes.name, XY_Axes);
	} 





	public static void CreateAuxLine()
	{
		CreateAuxLine(Color.black);
	}


	public static void CreateAuxLine(Color c, string name="AuxLine")
	{
		GameObject gObj = new GameObject(name);
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		int[] idcs = { 0, 1 };

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(c);	
		colors.Add(c);	

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);
		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);
	}


	
	public static void CreateAuxCircle()
	{
		GameObject gObj = new GameObject("AuxCircle");
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		const int numDiv = 48;
		const float aDeg = 360.0f / numDiv;
		Mesh wMesh = new Mesh();
		Color clr = new Color(0.34f, 0.34f, 0.34f);
		int[] idcs = new int[numDiv + 1];
		for (int i = 0; i < numDiv; i++)
		{
			float angle = Mathf.Deg2Rad * (aDeg * i);
			float cx = Mathf.Cos(angle);
			float cy = Mathf.Sin(angle);

			verts.Add(new Vector3(cx, cy, 0.001f));
			colors.Add(clr);

			idcs[i] = i;
		}
		idcs[numDiv] = 0;

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.LineStrip, 0);

		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;


		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);
	}



	public static void CreateAuxSquareEdge(string name="AuxSquareEdge")
	{
		GameObject gObj = new GameObject(name);
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		Color sqColor = new Color(0.34f, 0.34f, 0.34f);

		verts.Add(new Vector3(-0.5f, -0.5f, 0.0f));
		colors.Add(sqColor);	
		verts.Add(new Vector3( 0.5f, -0.5f, 0.0f));
		colors.Add(sqColor);	
		verts.Add(new Vector3( 0.5f,  0.5f, 0.0f));
		colors.Add(sqColor);	
		verts.Add(new Vector3(-0.5f,  0.5f, 0.0f));
		colors.Add(sqColor);

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		int[] idcs = { 0, 1, 2, 3, 0 };	
		wMesh.SetIndices(idcs, MeshTopology.LineStrip, 0);

		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);
	}



	public static void CreateAuxSphereGrid(bool isPlay=true)
	{
		GameObject gObj = new GameObject("AuxSphereGrid");
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		const int deg_itvl = 10;					
		const int numLong = 360 / deg_itvl;			
		const int numLati = 180 / deg_itvl - 1;		
		const float dfLati = 180.0f / (numLati + 1);		
		const float dfLong = 360.0f / numLong;	
		const float radius = 1.005f;
		
		Color gridColor = new Color(0.64f, 0.64f, 0.81f);       
		if (!isPlay)
		{
			gridColor = new Color(0.71f, 0.71f, 0.93f);       
		}

		int[] idcs = new int[((numLati + 1) * 2) * numLong + numLati * (numLong * 2)];
		int vi = 0;
		Vector3[,] lavs = new Vector3[numLati, numLong];	

		Vector3[] lgvs0 = new Vector3[numLati + 2];	
		for (int i = 0; i < lgvs0.Length; i++)
		{
			float agl = Mathf.Deg2Rad * (90.0f - dfLati * i);      
			float x = radius * Mathf.Cos(agl);	
			float y = radius * Mathf.Sin(agl);
			lgvs0[i] = new Vector3(x, y, 0.0f);	
		}

		for (int i = 0; i < numLong; i++)
		{
			float deg = i * dfLong;         
			Matrix4x4 rot = TH3DMath.GetRotation4x4(deg, Vector3.up);	

			Vector3 pv = lgvs0[0];	
			for (int j = 1; j < lgvs0.Length; j++)
			{
				Vector3 cv = rot * lgvs0[j];
				verts.Add(pv);	
				colors.Add(gridColor);	
				verts.Add(cv);
				colors.Add(gridColor);	
				pv = cv;	

				idcs[vi] = vi;
				idcs[vi + 1] = vi + 1;
				vi += 2;

				if(j < lgvs0.Length-1)
				{
					lavs[j - 1, i] = cv;	
				}		
			}
		}

		for (int i = 0; i < numLati; i++)
		{
			for (int j = 0; j < numLong-1; j++)
			{
				verts.Add(lavs[i, j]);	
				colors.Add(gridColor);
				verts.Add(lavs[i, j + 1]);
				colors.Add(gridColor);	

				idcs[vi] = vi;
				idcs[vi + 1] = vi + 1;
				vi += 2;
			}

			verts.Add(lavs[i, numLong-1]);	
			colors.Add(gridColor);
			verts.Add(lavs[i, 0]);
			colors.Add(gridColor);	

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;
			vi += 2;
		}	

		Mesh wMesh = new Mesh();
		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);


		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);

	} 




	public static float[] CalcImagePlaneData(float away_dist=1.0f)
	{
		float fov = BSData.CameraController3D.GetFieldOfView();
		float aspect = BSData.CameraController3D.GetAspectRatio();
		return CalcImagePlaneData(fov, aspect, Screen.height, away_dist);	
	}


	public static float[] CalcImagePlaneData(float fov, float aspect, int screen_h, float away_dist=1.0f)
	{
		float rad_hlf = fov / 2.0f * Mathf.Deg2Rad;

		float imageH = Mathf.Tan(rad_hlf) * away_dist * 2.0f;      
		float imageW = imageH * aspect;

		float WldPerPix = imageH / screen_h;

		return new float[] { imageW, imageH, WldPerPix, away_dist };
	}



	public static object[] CalcUIRectTransformN_LowerLeft(float[] bounds, float[] iplane_data, bool offset = false)
	{
		Vector3 camPos = BSData.CameraController3D.GetCameraPosition();
		Vector3 uI = BSData.CameraController3D.GetRightVector();
		Vector3 uJ = BSData.CameraController3D.GetUpVector();
		Vector3 uK = BSData.CameraController3D.GetForwardVector();

		Vector3[] camData = { camPos, uI, uJ, uK };
		return CalcUIRectTransformN_LowerLeft(bounds, camData, iplane_data, offset);
	}


	public static object[] CalcUIRectTransformN_LowerLeft(float[] bounds, Vector3[] cam_data, float[] iplane_data, 
																								bool offset = false)
	{
		float x = bounds[0];
		float y = bounds[1];
		float w = bounds[2];
		float h = bounds[3];

		Vector3 camPos = cam_data[0];
		Vector3 uI = cam_data[1];
		Vector3 uJ = cam_data[2];
		Vector3 uK = cam_data[3];

		float imgW = iplane_data[0];
		float imgH = iplane_data[1];
		float uperp = iplane_data[2];
		float awayDist = iplane_data[3];	

		Vector3 imgLowerLeft = camPos + awayDist * uK - (imgW * 0.5f) * uI - (imgH * 0.5f) * uJ;  
		Vector3 rectPos = imgLowerLeft + (x * imgW) * uI + (y * imgH) * uJ;      

		Vector3 rectScl = new Vector3(w * imgW, h * imgH, 1.0f);

		Quaternion rectRot = TH3DMath.CalcRotation_VW1toVW2(Vector3.right, -Vector3.forward, uI, -uK, Vector3.up);

		return new object[] { rectPos, rectRot, rectScl };
	}



	public static Vector3[] CalcBackPlaneData(Rect bounds, float d=1.0f) 
	{
		return CalcBackPlaneData(bounds.x, bounds.y, bounds.width, bounds.height, d);	
	}


	public static Vector3[] CalcBackPlaneData(float bx, float by, float bw, float bh, float d=1.0f) 
	{
		Rect rectBounds = new Rect(bx, by, bw, bh);

		float[] rbds2 = { rectBounds.x+rectBounds.width/2.0f,
					rectBounds.y+rectBounds.height/2.0f, rectBounds.width * 1.04f, rectBounds.height * 1.057f   };

		float[] iplaneData = CalcImagePlaneData(d); 

		Vector3[] initCamData = { Vector3.zero, Vector3.right, Vector3.up, Vector3.forward };
		object[] trs = CalcUIRectTransformN_LowerLeft(rbds2, initCamData, iplaneData);
		Vector3 difVtr = (Vector3)trs[0];  
		Vector3 scale = (Vector3)trs[2];

		return new Vector3[] { difVtr, scale };
	}





	public static Material GetSemiTransparentMaterial(float r, float g, float b, float a)
	{
		return GetSemiTransparentMaterial(new Color(r, g, b, a));
	}


	public static Material GetSemiTransparentMaterial(Color rgba)
	{
		Material mtrl = null;
		Shader trp_diffuse = Shader.Find("Legacy Shaders/Transparent/Diffuse");
		if (trp_diffuse != null)
		{
			mtrl = new Material(trp_diffuse);
		}
		else  
		{
			trp_diffuse = Shader.Find("Standard");
			mtrl = new Material(trp_diffuse);

			mtrl.SetOverrideTag("RenderType", "Transparent");
			mtrl.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
			mtrl.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
			mtrl.SetInt("_ZWrite", 0);
			mtrl.DisableKeyword("_ALPHATEST_ON");
			mtrl.DisableKeyword("_ALPHABLEND_ON");
			mtrl.EnableKeyword("_ALPHAPREMULTIPLY_ON");
			mtrl.renderQueue = 3000;  
		}

		mtrl.name = "SemiTransparent";
		mtrl.SetColor("_Color", rgba);

		return mtrl;
	}



	public static Material GetStandardMaterial(float r, float g, float b, float smoothness=0.2f)
	{
		return GetStandardMaterial(new Color(r, g, b), smoothness);
	}


	public static Material GetStandardMaterial(Color rgba, float smoothness=0.2f)
	{
		Shader standard = Shader.Find("Standard");
		Material mtrl = new Material(standard);
		mtrl.SetColor("_Color", rgba);
		mtrl.SetFloat("_Glossiness", smoothness);	

		return mtrl;
	}



	public static Material GetUnlitMaterial(float r, float g, float b)
	{
		return GetUnlitMaterial(new Color(r, g, b));
	}


	public static Material GetUnlitMaterial(Color c)
	{
		Shader unlit_color = Shader.Find("Unlit/Color");
		if (unlit_color == null) { return null; }

		Material mtrl = new Material(unlit_color);
		mtrl.SetColor("_Color", c);

		return mtrl;
	}



	public static void MeshRendererShadowOff(MeshRenderer mshRndr)
	{
		mshRndr.receiveShadows = false; 
		mshRndr.shadowCastingMode = UnityEngine.Rendering.ShadowCastingMode.Off;
	}


	public static void SetVisible(bool vflag, GameObject go) 
	{

		if (go == null)
		{
			Debug.Log("**** Err : GameObject is null  ..[BSAuxFuncs.SetVisible()] ****");
			return;
		}

		MeshRenderer mre = go.GetComponent<MeshRenderer>();
		if (mre != null && mre.enabled != vflag)
		{
			mre.enabled = vflag;
		}
	}


	public static void SetVisible(int vflag, GameObject go) 
	{
		SetVisible((vflag > 0) ? true : false, go);
	}



	public static Vector2 GetLetterSize(float cam_fov, float ltr_scale, float d=1.0f)
	{
		float c_letterRadio = BSData.c_LetterRatio;		

		float dians_hlf = cam_fov / 2.0f * Mathf.Deg2Rad;
		float hlf_imgH = Mathf.Tan(dians_hlf) * d;	
		float imgH = hlf_imgH * 2.0f;    
		float oneH = imgH * ltr_scale;
		float oneW = oneH * c_letterRadio;

		return new Vector2(oneW, oneH); 
	}



	public static GameObject GetMainCamera()
	{
		GameObject camera = GameObject.Find("Main Camera");
		if (camera == null)
		{
			camera = GameObject.Find("MainCamera");
		}

		return camera;
	}


	public static GameObject GetDirectionalLight()
	{
		GameObject dire_light = GameObject.Find("Directional Light");
		if(dire_light == null)
		{
			dire_light = GameObject.Find("DirectionalLight");
		}

		if (dire_light == null)
		{
			dire_light = new GameObject("Tmp_DLight");
			Light LC = dire_light.AddComponent<Light>();
			LC.color = new Color(1.0f, 0.9568f, 0.8392f);
			LC.type = LightType.Directional;
			LC.intensity = 0.3f;
			dire_light.transform.rotation = Quaternion.Euler(50, -30, 0);
		}

		return dire_light;
	}



	public static void InitText_EulerAngles(float d=1.0f, bool in_square_edge=false)  
	{
		if (GameObject.Find("Euler-x") != null)	{ return; }


		float fov = BSData.CameraController3D.GetFieldOfView();
		Vector2 ltr_size = GetLetterSize(fov, BSData.c_LetterScale_Regular, d);


		float rp_y = 0.73f;
		float rp_x_eulX = -0.43f;
		float rp_x_eulY = -0.2800989f;
		float rp_x_eulZ = -0.1301978f;
		if (in_square_edge)
		{
			rp_y = 0.48f;
			rp_x_eulX = -0.3150f;
			rp_x_eulY = -0.165099f;
			rp_x_eulZ = -0.01519778f;
		}


		string text = "x 0   ";      
		BSText btxX = BSBMTextFactory.MakeText(text, ltr_size, "Euler-x", d);
		btxX.SetRelativePosition3D(rp_x_eulX, rp_y);
		BSData.AuxTextList.Add(btxX);

		text = "y 0   ";            
		BSText btxY = BSBMTextFactory.MakeText(text, ltr_size, "Euler-y", d);
		btxY.SetRelativePosition3D(rp_x_eulY, rp_y);
		BSData.AuxTextList.Add(btxY);

		text = "z 0   ";            
		BSText btxZ = BSBMTextFactory.MakeText(text, ltr_size, "Euler-z", d);
		btxZ.SetRelativePosition3D(rp_x_eulZ, rp_y);
		BSData.AuxTextList.Add(btxZ);
	}




	public static void UpdateText_EulerAngles(int eulX, int eulY, int eulZ)
	{
		if (GameObject.Find("Euler-x") == null)	{ return; }

		BSText btxX = BSData.AuxTextList[0];
		BSText btxY = BSData.AuxTextList[1];
		BSText btxZ = BSData.AuxTextList[2];

		string ss = "" + eulX;
		string pad = GetPads(ss.Length, 4);     
		btxX.ChangeText(2, ss + pad);

		ss = "" + eulY;
		pad = GetPads(ss.Length, 4);            
		btxY.ChangeText(2, ss + pad);

		ss = "" + eulZ;                       
		pad = GetPads(ss.Length, 4);
		btxZ.ChangeText(2, ss + pad);
	}



	public static void SetVisibleText_EulerAngles(bool b)
	{

		if (GameObject.Find("Euler-x") == null)	{ return; }

		for (int i = 0; i < 3; i++)
		{
			BSText t = BSData.AuxTextList[i];
			MeshRenderer mre = t.gameObject.GetComponent<MeshRenderer>();
			if (mre != null && mre.enabled != b)
			{
				mre.enabled = b;
			}
		}
	}



	public static GameObject InitText_PixelCoord(float d=1.0f)  
	{
		GameObject go = GameObject.Find("PixelCoord");	
		if (go != null)	{ return go; }

		float fov = (BSData.CHAPTER_NR >= 3) ? BSData.CameraController3D.GetFieldOfView() : 60.0f;
		float tmpSize = BSData.c_LetterScale_Regular * 0.8f;
		Vector2 ltr_size = GetLetterSize(fov, tmpSize, d);


		float rp_x = 0.80f;
		float rp_y = -0.88f;


		string text = "(0, 0)      ";      
		BSText tx = BSBMTextFactory.MakeText(text, ltr_size, "PixelCoord", d);
		if (BSData.CHAPTER_NR >= 3)
		{
			tx.SetRelativePosition3D(rp_x, rp_y);
		}
		else
		{
			tx.SetRelativePosition2D(rp_x, rp_y);
		}

		BSData.AuxTextList.Add(tx);

		return tx.gameObject;
	}



	public static void Update_PixelCoord(int px, int py)  
	{
		BSText tx = BSData.AuxTextList[0];

		string sx = "" + px;
		string sy = "" + py;
		string padX = GetPads(sx.Length, 4);     
		string padY = GetPads(sy.Length, 4);     

		string text = "(" + sx + ", " + sy + ")" + padX + padY;
		tx.ChangeText(0, text);		
	}



	static string GetPads(int src_len, int max_len)
	{
		int pad_len = max_len - src_len;

		string pads = "";
		if (pad_len == 1)
		{
			pads = " ";
		}
		else if (pad_len == 2)
		{
			pads = "  ";
		}
		else if (pad_len == 3)
		{
			pads = "   ";
		}

		return pads;
	}



	public static void MoveCamera_PixelCoord()  
	{
		BSText tx = BSData.AuxTextList[0];
		tx.UpdatePosition();
	}




	public static void SetTransparentMaterial(THObject2D obj, float a)
	{
		SetTransparentMaterial(obj.gameObject, a);
	}

	public static void SetTransparentMaterial(THObject3D obj, float a)
	{
		SetTransparentMaterial(obj.gameObject, a);
	}


	public static void SetTransparentMaterial(GameObject go, float a)
	{
		Material mtrl = go.GetComponent<MeshRenderer>().material;
		Color dc = mtrl.GetColor("_Color");

		SetTransparentMaterial(go, dc.r, dc.g, dc.b, a);
	}


	public static void SetTransparentMaterial(THObject2D obj, float r, float g, float b, float a)
	{
		SetTransparentMaterial(obj.gameObject, r, g, b, a);
	}

	public static void SetTransparentMaterial(THObject3D obj, float r, float g, float b, float a)
	{
		SetTransparentMaterial(obj.gameObject, r, g, b, a);
	}


	public static void SetTransparentMaterial(GameObject go, float r, float g, float b, float a)
	{
		Material mtrl = GetSemiTransparentMaterial(r, g, b, a);
		go.GetComponent<MeshRenderer>().materials = new Material[] { mtrl };
	}



	public static void SetStandardMaterial(THObject2D obj, float r, float g, float b, float smoothness=0.2f)
	{
		SetStandardMaterial(obj.gameObject, r, g, b, smoothness);
	}

	public static void SetStandardMaterial(THObject3D obj, float r, float g, float b, float smoothness=0.2f)
	{
		SetStandardMaterial(obj.gameObject, r, g, b, smoothness);
	}


	public static void SetStandardMaterial(GameObject go, float r, float g, float b, float smoothness=0.2f)
	{
		Material mtrl = GetStandardMaterial(r, g, b, smoothness);
		go.GetComponent<MeshRenderer>().materials = new Material[] { mtrl };
	}



	public static void SetUnlitMaterial(THObject2D obj, float gray)
	{
		SetUnlitMaterial(obj.gameObject, gray, gray, gray);
	}

	public static void SetUnlitMaterial(THObject2D obj, float r, float g, float b)
	{
		SetUnlitMaterial(obj.gameObject, r, g, b);
	}


	public static void SetUnlitMaterial(THObject3D obj, float gray)
	{
		SetUnlitMaterial(obj.gameObject, gray, gray, gray);
	}

	public static void SetUnlitMaterial(THObject3D obj, float r, float g, float b)
	{
		SetUnlitMaterial(obj.gameObject, r, g, b);
	}


	public static void SetUnlitMaterial(GameObject go, float r, float g, float b)
	{
		Material mtrl = GetUnlitMaterial(r, g, b);
		go.GetComponent<MeshRenderer>().materials = new Material[] { mtrl };
	}




	public static void CreateAuxEditableCurve(Color c, int numDiv=256, string name="EditableCurve")  
	{
		GameObject gObj = new GameObject(name);
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		int[] idcs = new int[numDiv];
		
		Mesh wMesh = new Mesh();
		for (int i = 0; i < numDiv; i++)
		{
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(c);
			idcs[i] = i;
		}

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);
		wMesh.SetIndices(idcs, MeshTopology.LineStrip, 0);

		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);
	}



	public static void CreateLLBoundary()
	{
		GameObject gObj = new GameObject("LLBoundary");
		gObj.AddComponent<MeshFilter>();
		gObj.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		Color sqColor = new Color(0.41f, 0.41f, 0.41f);

		verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
		colors.Add(sqColor);
		verts.Add(new Vector3(0.0f, 1.0f, 0.0f));
		colors.Add(sqColor);
		verts.Add(new Vector3(1.0f, 1.0f, 0.0f));
		colors.Add(sqColor);
		verts.Add(new Vector3(1.0f, 0.0f, 0.0f));
		colors.Add(sqColor);

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		int[] idcs = { 0, 1, 2, 3, 0 };
		wMesh.SetIndices(idcs, MeshTopology.LineStrip, 0);

		MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		gObj.transform.localPosition = Vector3.zero;
		gObj.transform.localRotation = Quaternion.identity;
		gObj.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(gObj.name, gObj);
	}
	


	public static void InitQuaterGrid(float gray, int gridMax=6)
	{
		GameObject goQ = new GameObject("Quater_Grid");
		goQ.AddComponent<MeshFilter>();
		goQ.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		Color gridColor = new Color(gray, gray, gray);
		int[] idcs = new int[(gridMax +1) * 2 * 2];  
		int vi = 0;
		float yy = -0.0005f;	
		for (int i = 0; i <= gridMax; i++)
		{
			verts.Add(new Vector3( 0, yy, i));
			colors.Add(gridColor);
			verts.Add(new Vector3( gridMax, yy, i));
			colors.Add(gridColor);

			verts.Add(new Vector3(i, yy, 0));
			colors.Add(gridColor);
			verts.Add(new Vector3(i, yy,  gridMax));
			colors.Add(gridColor);

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;
			idcs[vi + 2] = vi + 2;
			idcs[vi + 3] = vi + 3;
			vi += 4;
		}

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);
		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = goQ.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		goQ.transform.localPosition = Vector3.zero;
		goQ.transform.localRotation = Quaternion.identity;
		goQ.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = goQ.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(goQ.name, goQ);
	}

} 
