//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------


using System.Collections;
using System.Collections.Generic;
using UnityEngine;



namespace OQJ_BSBMTextFactory
{

	public static class BSBMTextFactory
	{

		public static BSText MakeText(string text, Vector2 ltr_size, string name, int font)
		{
			return MakeText(text, ltr_size, name, 1.0f, font);
		}


		public static BSText MakeText(string text, Vector2 ltr_size, string name = "BMText", 
										float d=1.0f, int font=10)
		{
			GameObject gObj = new GameObject(name);
			gObj.AddComponent<MeshFilter>();
			gObj.AddComponent<MeshRenderer>();

			List<Vector3> verts = new List<Vector3>();
			List<Vector2> uvs = new List<Vector2>();
			int[] idcs = new int[text.Length * 4];

			QuadUV[] letter_uv = (font == BSText.c_ROBOT) ? c_uv_Roboto_Mono : c_uv_MPlus;

			float oneW = ltr_size.x;
			float oneH = ltr_size.y;
			for (int i = 0; i < text.Length; i++)
			{
				int id = (int)text[i];
				id -= 32;   

				float x0 = oneW * i;
				float y0 = 0.0f;

				verts.Add(new Vector3(x0, y0, 0.0f));                   
				verts.Add(new Vector3(x0, y0 + oneH, 0.0f));            
				verts.Add(new Vector3(x0 + oneW, y0 + oneH, 0.0f));     
				verts.Add(new Vector3(x0 + oneW, y0, 0.0f));            

				uvs.Add(new Vector2(letter_uv[id].u0, letter_uv[id].v0));
				uvs.Add(new Vector2(letter_uv[id].u0, letter_uv[id].v1));
				uvs.Add(new Vector2(letter_uv[id].u1, letter_uv[id].v1));
				uvs.Add(new Vector2(letter_uv[id].u1, letter_uv[id].v0));

				int iI = i * 4;
				idcs[iI] = iI;
				idcs[iI + 1] = iI + 1;
				idcs[iI + 2] = iI + 2;
				idcs[iI + 3] = iI + 3;
			}

			Mesh wMesh = new Mesh();
			wMesh.SetVertices(verts);
			wMesh.SetUVs(0, uvs);
			wMesh.SetIndices(idcs, MeshTopology.Quads, 0);

			MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
			mshFltr.mesh = wMesh;

			Shader trp_unlit = Shader.Find("Unlit/Transparent");
			Material mtrl_letter = new Material(trp_unlit);
			mtrl_letter.name = "AlphaTexture";

			string fontPath = (font == BSText.c_ROBOT) ? "images/sample_letters_2" : "images/f_1m_light";
			MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
			Texture2D fntPng = Resources.Load<Texture2D>(fontPath);

			mshRndr.material = mtrl_letter;
			mshRndr.material.SetTexture("_MainTex", fntPng);
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);

			gObj.transform.localPosition = Vector3.zero;
			gObj.transform.localRotation = Quaternion.identity;
			gObj.transform.localScale = Vector3.one;

			return new BSText(gObj, text, uvs.ToArray(), d, font);

		} 



		public static object[] ChangeText(BSText pTex, int from_idx, string ss)
		{
			if (from_idx + ss.Length != pTex.text.Length)
			{
				Debug.Log("**** Err : ChangeText.. [BSBMTextFactory] ****");
				return null;
			}

			List<Vector2> uvs = new List<Vector2>();
			for (int i = 0; i < from_idx; i++)
			{
				int t = i * 4;
				uvs.Add(pTex.UVs[t]);
				uvs.Add(pTex.UVs[t + 1]);
				uvs.Add(pTex.UVs[t + 2]);
				uvs.Add(pTex.UVs[t + 3]);
			}


			QuadUV[] letter_uv = (pTex.font == BSText.c_ROBOT) ? c_uv_Roboto_Mono : c_uv_MPlus;

			for (int i = 0; i < ss.Length; i++)
			{
				int id = (int)ss[i];
				id -= 32;   
				uvs.Add(new Vector2(letter_uv[id].u0, letter_uv[id].v0));
				uvs.Add(new Vector2(letter_uv[id].u0, letter_uv[id].v1));
				uvs.Add(new Vector2(letter_uv[id].u1, letter_uv[id].v1));
				uvs.Add(new Vector2(letter_uv[id].u1, letter_uv[id].v0));
			}


			var meshFltr = pTex.gameObject.GetComponent<MeshFilter>();
			meshFltr.mesh.SetUVs(0, uvs);

			return new object[] { pTex.text.Substring(0, from_idx) + ss, uvs.ToArray() };

		} 




		static readonly QuadUV[] c_uv_MPlus = 
		{
			new QuadUV(0f, 0.03125f, 0.9160156f, 1f),
			new QuadUV(0.03515625f, 0.06640625f, 0.9160156f, 1f),
			new QuadUV(0.0703125f, 0.1015625f, 0.9160156f, 1f),
			new QuadUV(0.1054688f, 0.1367188f, 0.9160156f, 1f),
			new QuadUV(0.140625f, 0.171875f, 0.9160156f, 1f),
			new QuadUV(0.1757813f, 0.2070313f, 0.9160156f, 1f),
			new QuadUV(0.2109375f, 0.2421875f, 0.9160156f, 1f),
			new QuadUV(0.2460938f, 0.2773438f, 0.9160156f, 1f),
			new QuadUV(0.28125f, 0.3125f, 0.9160156f, 1f),
			new QuadUV(0.3164063f, 0.3476563f, 0.9160156f, 1f),
			new QuadUV(0.3515625f, 0.3828125f, 0.9160156f, 1f),
			new QuadUV(0.3867188f, 0.4179688f, 0.9160156f, 1f),
			new QuadUV(0.421875f, 0.453125f, 0.9160156f, 1f),
			new QuadUV(0.4570313f, 0.4882813f, 0.9160156f, 1f),
			new QuadUV(0.4921875f, 0.5234375f, 0.9160156f, 1f),
			new QuadUV(0.5273438f, 0.5585938f, 0.9160156f, 1f),
			new QuadUV(0.5625f, 0.59375f, 0.9160156f, 1f),
			new QuadUV(0.5976563f, 0.6289063f, 0.9160156f, 1f),
			new QuadUV(0.6328125f, 0.6640625f, 0.9160156f, 1f),
			new QuadUV(0.6679688f, 0.6992188f, 0.9160156f, 1f),
			new QuadUV(0.703125f, 0.734375f, 0.9160156f, 1f),
			new QuadUV(0.7382813f, 0.7695313f, 0.9160156f, 1f),
			new QuadUV(0.7734375f, 0.8046875f, 0.9160156f, 1f),
			new QuadUV(0.8085938f, 0.8398438f, 0.9160156f, 1f),
			new QuadUV(0.84375f, 0.875f, 0.9160156f, 1f),
			new QuadUV(0.8789063f, 0.9101563f, 0.9160156f, 1f),
			new QuadUV(0.9140625f, 0.9453125f, 0.9160156f, 1f),
			new QuadUV(0.9492188f, 0.9804688f, 0.9160156f, 1f),
			new QuadUV(0f, 0.03125f, 0.828125f, 0.9121094f),
			new QuadUV(0.03515625f, 0.06640625f, 0.828125f, 0.9121094f),
			new QuadUV(0.0703125f, 0.1015625f, 0.828125f, 0.9121094f),
			new QuadUV(0.1054688f, 0.1367188f, 0.828125f, 0.9121094f),
			new QuadUV(0.140625f, 0.171875f, 0.828125f, 0.9121094f),
			new QuadUV(0.1757813f, 0.2070313f, 0.828125f, 0.9121094f),
			new QuadUV(0.2109375f, 0.2421875f, 0.828125f, 0.9121094f),
			new QuadUV(0.2460938f, 0.2773438f, 0.828125f, 0.9121094f),
			new QuadUV(0.28125f, 0.3125f, 0.828125f, 0.9121094f),
			new QuadUV(0.3164063f, 0.3476563f, 0.828125f, 0.9121094f),
			new QuadUV(0.3515625f, 0.3828125f, 0.828125f, 0.9121094f),
			new QuadUV(0.3867188f, 0.4179688f, 0.828125f, 0.9121094f),
			new QuadUV(0.421875f, 0.453125f, 0.828125f, 0.9121094f),
			new QuadUV(0.4570313f, 0.4882813f, 0.828125f, 0.9121094f),
			new QuadUV(0.4921875f, 0.5234375f, 0.828125f, 0.9121094f),
			new QuadUV(0.5273438f, 0.5585938f, 0.828125f, 0.9121094f),
			new QuadUV(0.5625f, 0.59375f, 0.828125f, 0.9121094f),
			new QuadUV(0.5976563f, 0.6289063f, 0.828125f, 0.9121094f),
			new QuadUV(0.6328125f, 0.6640625f, 0.828125f, 0.9121094f),
			new QuadUV(0.6679688f, 0.6992188f, 0.828125f, 0.9121094f),
			new QuadUV(0.703125f, 0.734375f, 0.828125f, 0.9121094f),
			new QuadUV(0.7382813f, 0.7695313f, 0.828125f, 0.9121094f),
			new QuadUV(0.7734375f, 0.8046875f, 0.828125f, 0.9121094f),
			new QuadUV(0.8085938f, 0.8398438f, 0.828125f, 0.9121094f),
			new QuadUV(0.84375f, 0.875f, 0.828125f, 0.9121094f),
			new QuadUV(0.8789063f, 0.9101563f, 0.828125f, 0.9121094f),
			new QuadUV(0.9140625f, 0.9453125f, 0.828125f, 0.9121094f),
			new QuadUV(0.9492188f, 0.9804688f, 0.828125f, 0.9121094f),
			new QuadUV(0f, 0.03125f, 0.7402344f, 0.8242188f),
			new QuadUV(0.03515625f, 0.06640625f, 0.7402344f, 0.8242188f),
			new QuadUV(0.0703125f, 0.1015625f, 0.7402344f, 0.8242188f),
			new QuadUV(0.1054688f, 0.1367188f, 0.7402344f, 0.8242188f),
			new QuadUV(0.140625f, 0.171875f, 0.7402344f, 0.8242188f),
			new QuadUV(0.1757813f, 0.2070313f, 0.7402344f, 0.8242188f),
			new QuadUV(0.2109375f, 0.2421875f, 0.7402344f, 0.8242188f),
			new QuadUV(0.2460938f, 0.2773438f, 0.7402344f, 0.8242188f),
			new QuadUV(0.28125f, 0.3125f, 0.7402344f, 0.8242188f),
			new QuadUV(0.3164063f, 0.3476563f, 0.7402344f, 0.8242188f),
			new QuadUV(0.3515625f, 0.3828125f, 0.7402344f, 0.8242188f),
			new QuadUV(0.3867188f, 0.4179688f, 0.7402344f, 0.8242188f),
			new QuadUV(0.421875f, 0.453125f, 0.7402344f, 0.8242188f),
			new QuadUV(0.4570313f, 0.4882813f, 0.7402344f, 0.8242188f),
			new QuadUV(0.4921875f, 0.5234375f, 0.7402344f, 0.8242188f),
			new QuadUV(0.5273438f, 0.5585938f, 0.7402344f, 0.8242188f),
			new QuadUV(0.5625f, 0.59375f, 0.7402344f, 0.8242188f),
			new QuadUV(0.5976563f, 0.6289063f, 0.7402344f, 0.8242188f),
			new QuadUV(0.6328125f, 0.6640625f, 0.7402344f, 0.8242188f),
			new QuadUV(0.6679688f, 0.6992188f, 0.7402344f, 0.8242188f),
			new QuadUV(0.703125f, 0.734375f, 0.7402344f, 0.8242188f),
			new QuadUV(0.7382813f, 0.7695313f, 0.7402344f, 0.8242188f),
			new QuadUV(0.7734375f, 0.8046875f, 0.7402344f, 0.8242188f),
			new QuadUV(0.8085938f, 0.8398438f, 0.7402344f, 0.8242188f),
			new QuadUV(0.84375f, 0.875f, 0.7402344f, 0.8242188f),
			new QuadUV(0.8789063f, 0.9101563f, 0.7402344f, 0.8242188f),
			new QuadUV(0.9140625f, 0.9453125f, 0.7402344f, 0.8242188f),
			new QuadUV(0.9492188f, 0.9804688f, 0.7402344f, 0.8242188f),
			new QuadUV(0f, 0.03125f, 0.6523438f, 0.7363281f),
			new QuadUV(0.03515625f, 0.06640625f, 0.6523438f, 0.7363281f),
			new QuadUV(0.0703125f, 0.1015625f, 0.6523438f, 0.7363281f),
			new QuadUV(0.1054688f, 0.1367188f, 0.6523438f, 0.7363281f),
			new QuadUV(0.140625f, 0.171875f, 0.6523438f, 0.7363281f),
			new QuadUV(0.1757813f, 0.2070313f, 0.6523438f, 0.7363281f),
			new QuadUV(0.2109375f, 0.2421875f, 0.6523438f, 0.7363281f),
			new QuadUV(0.2460938f, 0.2773438f, 0.6523438f, 0.7363281f),
			new QuadUV(0.28125f, 0.3125f, 0.6523438f, 0.7363281f),
			new QuadUV(0.3164063f, 0.3476563f, 0.6523438f, 0.7363281f),
			new QuadUV(0.3515625f, 0.3828125f, 0.6523438f, 0.7363281f),
			new QuadUV(0f, 0f, 0f, 0f)
		};



		static readonly QuadUV[] c_uv_Roboto_Mono = 
		{
			new QuadUV(0.04882813f, 0.09179688f, 0.9042969f, 1f),
			new QuadUV(0.09570313f, 0.1386719f, 0.9042969f, 1f),
			new QuadUV(0.1425781f, 0.1855469f, 0.9042969f, 1f),
			new QuadUV(0.1894531f, 0.2324219f, 0.9042969f, 1f),
			new QuadUV(0.2363281f, 0.2792969f, 0.9042969f, 1f),
			new QuadUV(0.2832031f, 0.3261719f, 0.9042969f, 1f),
			new QuadUV(0.3300781f, 0.3730469f, 0.9042969f, 1f),
			new QuadUV(0.3769531f, 0.4199219f, 0.9042969f, 1f),
			new QuadUV(0.4238281f, 0.4667969f, 0.9042969f, 1f),
			new QuadUV(0.4707031f, 0.5136719f, 0.9042969f, 1f),
			new QuadUV(0.5175781f, 0.5605469f, 0.9042969f, 1f),
			new QuadUV(0.5644531f, 0.6074219f, 0.9042969f, 1f),
			new QuadUV(0.6113281f, 0.6542969f, 0.9042969f, 1f),
			new QuadUV(0.6582031f, 0.7011719f, 0.9042969f, 1f),
			new QuadUV(0.7050781f, 0.7480469f, 0.9042969f, 1f),
			new QuadUV(0.7519531f, 0.7949219f, 0.9042969f, 1f),
			new QuadUV(0.7988281f, 0.8417969f, 0.9042969f, 1f),
			new QuadUV(0.8457031f, 0.8886719f, 0.9042969f, 1f),
			new QuadUV(0.8925781f, 0.9355469f, 0.9042969f, 1f),
			new QuadUV(0.9394531f, 0.9824219f, 0.9042969f, 1f),
			new QuadUV(0f, 0.04296875f, 0.8046875f, 0.9003906f),
			new QuadUV(0.046875f, 0.08984375f, 0.8046875f, 0.9003906f),
			new QuadUV(0.09375f, 0.1367188f, 0.8046875f, 0.9003906f),
			new QuadUV(0.140625f, 0.1835938f, 0.8046875f, 0.9003906f),
			new QuadUV(0.1875f, 0.2304688f, 0.8046875f, 0.9003906f),
			new QuadUV(0.234375f, 0.2773438f, 0.8046875f, 0.9003906f),
			new QuadUV(0.28125f, 0.3242188f, 0.8046875f, 0.9003906f),
			new QuadUV(0.328125f, 0.3710938f, 0.8046875f, 0.9003906f),
			new QuadUV(0.375f, 0.4179688f, 0.8046875f, 0.9003906f),
			new QuadUV(0.421875f, 0.4648438f, 0.8046875f, 0.9003906f),
			new QuadUV(0.46875f, 0.5117188f, 0.8046875f, 0.9003906f),
			new QuadUV(0.515625f, 0.5585938f, 0.8046875f, 0.9003906f),
			new QuadUV(0.5625f, 0.6054688f, 0.8046875f, 0.9003906f),
			new QuadUV(0.609375f, 0.6523438f, 0.8046875f, 0.9003906f),
			new QuadUV(0.65625f, 0.6992188f, 0.8046875f, 0.9003906f),
			new QuadUV(0.703125f, 0.7460938f, 0.8046875f, 0.9003906f),
			new QuadUV(0.75f, 0.7929688f, 0.8046875f, 0.9003906f),
			new QuadUV(0.796875f, 0.8398438f, 0.8046875f, 0.9003906f),
			new QuadUV(0.84375f, 0.8867188f, 0.8046875f, 0.9003906f),
			new QuadUV(0.890625f, 0.9335938f, 0.8046875f, 0.9003906f),
			new QuadUV(0.9375f, 0.9804688f, 0.8046875f, 0.9003906f),
			new QuadUV(0f, 0.04296875f, 0.7050781f, 0.8007813f),
			new QuadUV(0.046875f, 0.08984375f, 0.7050781f, 0.8007813f),
			new QuadUV(0.09375f, 0.1367188f, 0.7050781f, 0.8007813f),
			new QuadUV(0.140625f, 0.1835938f, 0.7050781f, 0.8007813f),
			new QuadUV(0.1875f, 0.2304688f, 0.7050781f, 0.8007813f),
			new QuadUV(0.234375f, 0.2773438f, 0.7050781f, 0.8007813f),
			new QuadUV(0.28125f, 0.3242188f, 0.7050781f, 0.8007813f),
			new QuadUV(0.328125f, 0.3710938f, 0.7050781f, 0.8007813f),
			new QuadUV(0.375f, 0.4179688f, 0.7050781f, 0.8007813f),
			new QuadUV(0.421875f, 0.4648438f, 0.7050781f, 0.8007813f),
			new QuadUV(0.46875f, 0.5117188f, 0.7050781f, 0.8007813f),
			new QuadUV(0.515625f, 0.5585938f, 0.7050781f, 0.8007813f),
			new QuadUV(0.5625f, 0.6054688f, 0.7050781f, 0.8007813f),
			new QuadUV(0.609375f, 0.6523438f, 0.7050781f, 0.8007813f),
			new QuadUV(0.65625f, 0.6992188f, 0.7050781f, 0.8007813f),
			new QuadUV(0.703125f, 0.7460938f, 0.7050781f, 0.8007813f),
			new QuadUV(0.75f, 0.7929688f, 0.7050781f, 0.8007813f),
			new QuadUV(0.796875f, 0.8398438f, 0.7050781f, 0.8007813f),
			new QuadUV(0.84375f, 0.8867188f, 0.7050781f, 0.8007813f),
			new QuadUV(0.890625f, 0.9335938f, 0.7050781f, 0.8007813f),
			new QuadUV(0.9375f, 0.9804688f, 0.7050781f, 0.8007813f),
			new QuadUV(0f, 0.04296875f, 0.6054688f, 0.7011719f),
			new QuadUV(0.046875f, 0.08984375f, 0.6054688f, 0.7011719f),
			new QuadUV(0.09375f, 0.1367188f, 0.6054688f, 0.7011719f),
			new QuadUV(0.140625f, 0.1835938f, 0.6054688f, 0.7011719f),
			new QuadUV(0.1875f, 0.2304688f, 0.6054688f, 0.7011719f),
			new QuadUV(0.234375f, 0.2773438f, 0.6054688f, 0.7011719f),
			new QuadUV(0.28125f, 0.3242188f, 0.6054688f, 0.7011719f),
			new QuadUV(0.328125f, 0.3710938f, 0.6054688f, 0.7011719f),
			new QuadUV(0.375f, 0.4179688f, 0.6054688f, 0.7011719f),
			new QuadUV(0.421875f, 0.4648438f, 0.6054688f, 0.7011719f),
			new QuadUV(0.46875f, 0.5117188f, 0.6054688f, 0.7011719f),
			new QuadUV(0.515625f, 0.5585938f, 0.6054688f, 0.7011719f),
			new QuadUV(0.5625f, 0.6054688f, 0.6054688f, 0.7011719f),
			new QuadUV(0.609375f, 0.6523438f, 0.6054688f, 0.7011719f),
			new QuadUV(0.65625f, 0.6992188f, 0.6054688f, 0.7011719f),
			new QuadUV(0.703125f, 0.7460938f, 0.6054688f, 0.7011719f),
			new QuadUV(0.75f, 0.7929688f, 0.6054688f, 0.7011719f),
			new QuadUV(0.796875f, 0.8398438f, 0.6054688f, 0.7011719f),
			new QuadUV(0.84375f, 0.8867188f, 0.6054688f, 0.7011719f),
			new QuadUV(0.890625f, 0.9335938f, 0.6054688f, 0.7011719f),
			new QuadUV(0.9375f, 0.9804688f, 0.6054688f, 0.7011719f),
			new QuadUV(0f, 0.04296875f, 0.5058594f, 0.6015625f),
			new QuadUV(0.046875f, 0.08984375f, 0.5058594f, 0.6015625f),
			new QuadUV(0.09375f, 0.1367188f, 0.5058594f, 0.6015625f),
			new QuadUV(0.140625f, 0.1835938f, 0.5058594f, 0.6015625f),
			new QuadUV(0.1875f, 0.2304688f, 0.5058594f, 0.6015625f),
			new QuadUV(0.234375f, 0.2773438f, 0.5058594f, 0.6015625f),
			new QuadUV(0.28125f, 0.3242188f, 0.5058594f, 0.6015625f),
			new QuadUV(0.328125f, 0.3710938f, 0.5058594f, 0.6015625f),
			new QuadUV(0.375f, 0.4179688f, 0.5058594f, 0.6015625f),
			new QuadUV(0.421875f, 0.4648438f, 0.5058594f, 0.6015625f),
			new QuadUV(0.46875f, 0.5117188f, 0.5058594f, 0.6015625f),
			new QuadUV(0.515625f, 0.5585938f, 0.5058594f, 0.6015625f),
			new QuadUV(0f, 0f, 0f, 0f),
		};

	} 




	public struct QuadUV
	{
		public float u0;
		public float u1;
		public float v0;
		public float v1;

		public QuadUV(float a, float b, float c, float d)
		{
			u0 = a;
			u1 = b;
			v0 = c;
			v1 = d;
		}
	}

} 


