//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;


public class BSCameraController2D : THObject2D
{
	Camera m_camera;

	Vector3 m_cameraPosition;
	
	Vector3 m_targetPosition;



	public BSCameraController2D(GameObject go) : base(go)
	{
		m_camera = go.GetComponent<Camera>();

		m_camera.clearFlags = CameraClearFlags.SolidColor;

		m_camera.backgroundColor = BSData.BackgroundColor;

		m_camera.orthographic = false;  

		m_camera.nearClipPlane = BSData.c_DefaultNearDist;

		m_camera.farClipPlane = BSData.c_DefaultFarDist;

		m_camera.fieldOfView = BSData.c_DefaultFOV;

	}


	public void SetView(Vector3 posi, Vector3 target, Vector3 up)  
	{
		Matrix4x4 viewMtx = TH3DMath.LookAtLH(posi, target, up);
		SetMatrix(viewMtx.inverse);

		m_cameraPosition = posi;
		m_targetPosition = target;
		float camDist = (m_targetPosition - m_cameraPosition).magnitude;


		UpdateAuxText();
	}



	void UpdateAuxText() 
	{
		BSData.AuxText_Sec.UpdatePosition();
		BSData.AuxText_Code.UpdatePosition();


		if (BSData.c_SHOW_COORDINATE_SCALE)
		{
			float camDist = (m_targetPosition - m_cameraPosition).magnitude;

			float scl = 1.0f;
			if (camDist < 10.0f)
			{
				scl = 0.5f;
			}

			float cH = 0.8f * scl;
			float cW = cH * BSData.c_LetterRatio;

			foreach (var btx in BSData.Coord2DScaleX)
			{
				BSAuxFuncs.SetVisible(1, btx.gameObject);

				string ss = btx.text;
				float wx = int.Parse(ss);
				btx.transform.localScale = new Vector3(scl, scl, 1.0f);
				btx.SetWorldPosition2D(wx - (cW * ss.Length) / 2.0f, -cH);
			}

			foreach (var btx in BSData.Coord2DScaleY)
			{
				BSAuxFuncs.SetVisible(1, btx.gameObject);

				string ss = btx.text;
				float wy = int.Parse(ss);
				btx.transform.localScale = new Vector3(scl, scl, 1.0f);
				btx.SetWorldPosition2D(-(cW * ss.Length) - 0.1f, wy - cH / 2.0f);
			}
		}
		else
		{
			foreach (var btx in BSData.Coord2DScaleX)
			{
				BSAuxFuncs.SetVisible(0, btx.gameObject);
			}

			foreach (var btx in BSData.Coord2DScaleY)
			{
				BSAuxFuncs.SetVisible(0, btx.gameObject);
			}
		}


		foreach(var t in BSData.AuxTextList)	
		{
			t.UpdatePosition();
		}
	}



	public void UpdateCameraData()
	{
		m_cameraPosition = transform.position;
		m_targetPosition = m_cameraPosition;
		m_targetPosition.z = 0;		

		UpdateAuxText();
	}	


		
	public Vector3 GetCameraPosition()
	{
		return m_cameraPosition;	
	}


	public Vector3 GetTargetPosition()
	{
		return m_targetPosition;	
	}


	public float GetAspectRatio()
	{
		return m_camera.aspect;
	}


	public float GetFieldOfView()
	{
		return m_camera.fieldOfView;
	}


	void SetMatrix(Matrix4x4 mtx)
	{
		Vector3 v = new Vector3(mtx.m03, mtx.m13, mtx.m23);
		Quaternion q = TH3DMath.GetRotationPart(mtx);
		this.transform.localRotation = q;	
		this.transform.localPosition = v;	

		float sx = Mathf.Sqrt(mtx.m00 * mtx.m00 + mtx.m10 * mtx.m10 + mtx.m20 * mtx.m20);	
		float sy = Mathf.Sqrt(mtx.m01 * mtx.m01 + mtx.m11 * mtx.m11 + mtx.m21 * mtx.m21);	
		float sz = Mathf.Sqrt(mtx.m02 * mtx.m02 + mtx.m12 * mtx.m12 + mtx.m22 * mtx.m22);
		this.transform.localScale = new Vector3(sx, sy, sz);
	}


}
