//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;





public class BSCameraController3D : THObject3D
{

	Camera m_camera;

	Vector3 m_cameraPosition;
	
	Vector3 m_targetPosition;

	float m_targetDistance;

	Vector3 m_uI;	
	Vector3 m_uJ;	
	Vector3 m_uK;	




	public BSCameraController3D(GameObject go) : base(go)
	{
		m_camera = go.GetComponent<Camera>();

		m_camera.clearFlags = CameraClearFlags.SolidColor;

		m_camera.backgroundColor = BSData.BackgroundColor;

		m_camera.orthographic = false;  

		m_camera.nearClipPlane = BSData.c_DefaultNearDist;

		m_camera.farClipPlane = BSData.c_DefaultFarDist;

		m_camera.fieldOfView = BSData.c_DefaultFOV;


		m_targetDistance = 50.0f;  

	}


	public void SetView(Vector3 posi, Vector3 target, Vector3 up)
	{
		Matrix4x4 viewMtx = TH3DMath.LookAtLH(posi, target, up);	
		SetMatrix(viewMtx.inverse);

		m_cameraPosition = posi;
		m_targetPosition = target;
		m_targetDistance = (target - posi).magnitude;
		m_uK = (target - posi).normalized;
		m_uI = Vector3.Cross(up, m_uK).normalized;		
		m_uJ = Vector3.Cross(m_uK, m_uI).normalized;	


		UpdateAuxText();
	}



	void UpdateAuxText()
	{
		BSData.AuxText_Sec.UpdatePosition();
		BSData.AuxText_Code.UpdatePosition();

		foreach(var t in BSData.AuxTextList)
		{
			t.UpdatePosition();
		}
	}

		
	public Vector3 GetCameraPosition()
	{
		return m_cameraPosition;	
	}


	public Quaternion GetCameraRotation()
	{
		return m_camera.transform.rotation;
	}


	public Vector3 GetTargetPosition()
	{
		return m_targetPosition;	
	}


	public Vector3 GetRightVector()
	{
		return m_uI;	
	}


	public Vector3 GetUpVector()
	{
		return m_uJ;	
	}


	public Vector3 GetForwardVector()
	{
		return m_uK;	
	}


	public Vector3 GetHorzDirection()
	{
		Vector3 dir = m_uK;
		dir.y = 0.0f;
		return dir.normalized;	
	}
	

	public float GetAspectRatio()	
	{
		return m_camera.aspect;
	}


	public float GetFieldOfView()
	{
		return m_camera.fieldOfView;
	}


	public float GetNearDist()
	{
		return m_camera.nearClipPlane;
	}


	public float GetFarDist()
	{
		return m_camera.farClipPlane;
	}



	public void UpdateCameraData()
	{
		Matrix4x4 curMtx = GetMatrix();
		m_uI = curMtx.GetColumn(0).normalized;		
		m_uJ = curMtx.GetColumn(1).normalized;		
		m_uK = curMtx.GetColumn(2).normalized;

		m_cameraPosition = curMtx.GetColumn(3);
		m_targetPosition = m_cameraPosition + m_targetDistance * m_uK;

		UpdateAuxText();
	}	



}
