//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using ClReel = OQJ_Main03.ClReel;
using ClEllipse = OQJ_MainPB.ClEllipse;
using OQJ_THShell2D;
using OQJ_THAlphaObject2D;
using OQJ_THBetaObject3D;
using OQJ_THAlphaObject3D;
using OQJ_THShell3D;


public abstract class BSSuperExe
{
	Vector3 pr_CameraPos = new Vector3(0.0f, 0.0f, -10.0f);
	Vector3 pr_TargetPos = new Vector3(0.0f, 0.0f, 0.0f);

	protected float i_worldW;
	protected float i_worldH;
	protected float i_pixW;
	protected float i_pixH;

	protected bool i_MOVE = false;
	protected int i_time = 0;
	protected int i_frameCount = 0;
	protected bool i_INITIALIZED = false;

	protected bool pr_LOAD_OBJECTS = false;
	protected bool pr_FIRST_RUN = true;
	protected bool pr_LOOP_CODE = true;

	readonly List<GameObject> pr_GObjList;
	readonly int pr_codeCount;


	private int m_FPS = 60;
	public int FPS
	{
		get { return m_FPS; }
	}

	protected void SetFPS(int fps)
	{
		m_FPS = fps;

		if (BSData.CHAPTER_NR <= 2)
		{
			Application.targetFrameRate = m_FPS;
		}
	}


	public float SecondPerFrame
	{
		get { return 1.0f / m_FPS; }
	}


	protected BSSuperExe(int num_code)
	{
		pr_codeCount = num_code;

		pr_GObjList = new List<GameObject>();

		pr_FIRST_RUN = true;
	}


	public void SuperInitialize()
	{
		ClearData();

		Initialize();
	}


	public abstract void Initialize();


	public abstract void Update();


	public void ChangeCode(int prnx)    
	{

		int code_nr = THSettings.CODE_NR;
		if (prnx < 0)
		{
			code_nr = (THSettings.CODE_NR - 1 >= 1) ? THSettings.CODE_NR - 1 : pr_codeCount;
		}
		else
		{
			code_nr = (THSettings.CODE_NR + 1 <= pr_codeCount) ? THSettings.CODE_NR + 1 : 1;
		}

		if (THSettings.CODE_NR == code_nr) { return; }

		THSettings.CODE_NR = code_nr;
		BSData.AuxText_Code.ChangeText(4, "" + THSettings.CODE_NR); 


		foreach (var go in pr_GObjList)
		{
			BSAuxFuncs.SetVisible(0, go);
		}

		SuperPreset();
	}


	public bool IsLoopCode()
	{
		return pr_LOOP_CODE;
	}


	public bool IsFirstRun()
	{
		return pr_FIRST_RUN;
	}


	public void False_FIRST_RUN()
	{
		pr_FIRST_RUN = false;
	}


	protected void SuperPreset()
	{
		ClearData();

		Preset();
	}


	protected abstract void Preset();


	protected void SetOutputImageSize(float world_len, int pix_len, bool update_view = false)
	{
		i_worldW = world_len;
		i_worldH = world_len;

		i_pixW = pix_len;
		i_pixH = pix_len;


		if (update_view)
		{
			pr_CameraPos.z = TH2DMath.CalcCameraPositionZ_2D(i_worldH, i_pixH);
			BSData.CameraController2D.SetView(pr_CameraPos, pr_TargetPos, Vector3.up);
		}
	}


	protected void SetCameraPosition_3D(Vector3 CamPos, Vector3 TrgPos, Vector3 uJ)
	{
		BSData.CameraController3D.SetView(CamPos, TrgPos, uJ);
	}


	protected void SetCameraPosition_2D(float x, float y, bool pix_fit = false)
	{
		float z = TH2DMath.CalcCameraPositionZ_2D(i_worldH, i_pixH, pix_fit);
		SetCameraPosition_2D(x, y, z);

	}


	protected void SetCameraPosition_2D(float x, float y, float z)
	{
		pr_TargetPos = new Vector3(x, y, 0);
		pr_CameraPos.x = x;
		pr_CameraPos.y = y;
		pr_CameraPos.z = z;
		BSData.CameraController2D.SetView(pr_CameraPos, pr_TargetPos, Vector3.up);
	}


	protected void SetCameraPosition_2D(Vector3 pos)
	{
		SetCameraPosition_2D(pos.x, pos.y, pos.z);
	}



	protected bool MatchCase(int c1, int c2 = -1, int c3 = -1, int c4 = -1, int c5 = -1, int c6 = -1)
	{
		if (THSettings.CODE_NR == c1 ||
			THSettings.CODE_NR == c2 ||
			THSettings.CODE_NR == c3 ||
			THSettings.CODE_NR == c4 ||
			THSettings.CODE_NR == c5 ||
			THSettings.CODE_NR == c6)
		{
			return true;
		}

		return false;
	}



	public int GetCodeCount()
	{
		return pr_codeCount;
	}


	void ClearData()
	{
		pr_FIRST_RUN = true;
		pr_LOOP_CODE = true;
		i_MOVE = false;
		i_frameCount = 0;
		i_INITIALIZED = false;
	}



	GameObject CreateObject_Common(string name)
	{
		if (!BSData.RunObjectDict.ContainsKey(name))    
		{
			Debug.Log(@"**** Err : 'name' Is Not Found in RunObjectDict at CreateObject_Common()**** : " + name);

			return null;
		}

		GameObject go = BSData.RunObjectDict[name];
		pr_GObjList.Add(go);

		return go;
	}



	GameObject CreateObject_Common_Ins(GameObject srcGo)
	{
		GameObject go = GameObject.Instantiate(srcGo);
		BSAuxFuncs.SetVisible(0, go);
		pr_GObjList.Add(go);

		return go;
	}




	protected THObject3D CreateObject_Obj3D(string name, bool shadow = false)
	{
		GameObject go = CreateObject_Common(name);
		if (go == null) { return null; }

		return new THObject3D(go, shadow);
	}



	protected THObject3D CreateObject_InsObj3D(GameObject srcGo, bool shadow=false)
	{
		return new THObject3D(CreateObject_Common_Ins(srcGo), shadow);
	}


	protected THObject3D CreateObject_InsOthers3D(GameObject srcGo, string name, bool shadow=false)
	{
		if (name.Equals("GrReel"))
		{
			return new ClReel(CreateObject_Common_Ins(srcGo), shadow);
		}

		return null;
	}


	protected THAlphaObject3D CreateObject_Alpha3D(string name, bool shadow=false)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THAlphaObject3D(go, shadow);
	}


	protected THAlphaObject3D CreateObject_InsAlpha3D(GameObject srcGo, bool shadow=false)
	{
		return new THAlphaObject3D(CreateObject_Common_Ins(srcGo), shadow);
	}


	protected THBetaObject3D CreateObject_Beta3D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THBetaObject3D(go);
	}


	protected THBetaObject3D CreateObject_InsBeta3D(GameObject srcGo)
	{
		return new THBetaObject3D(CreateObject_Common_Ins(srcGo));
	}


	protected THShell3D CreateObject_Shell3D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THShell3D(go);
	}


	protected THShell3D CreateObject_InsShell3D(GameObject srcGo)
	{
		return new THShell3D(CreateObject_Common_Ins(srcGo));
	}



	protected THObject3D CreateObject_Others(string name, bool shadow=false)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		if (name.Equals("GrReel"))
		{
			return new ClReel(go, shadow);
		}

		Debug.Log("**** Err : "+name+" is NOT FOUND in \'OTHERS\' .. CreateObject_Others ****");

		return null;
	}



	protected THObject2D CreateObject_Obj2D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THObject2D(go);
	}


	protected THShell2D CreateObject_Shell2D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THShell2D(go);
	}


	protected THAlphaObject2D CreateObject_Alpha2D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if(go == null) { return null; }

		return new THAlphaObject2D(go);
	}


	protected THObject2D CreateObject_Others2D(string name)
	{
		GameObject go = CreateObject_Common(name);
		if (go == null) { return null; }

		if (name.Equals("Combat2"))
		{
			return new OQJ_Sec230.Sec230.CustomJet(go);
		}
		else if (name.Equals("DbShell"))
		{
			return new OQJ_Sec230.Sec230.CustomShell(go);
		}
		else if (name.StartsWith("SHR_"))
		{
			return new ClEllipse(go);
		}

		return null;	
	}


	protected THObject2D CreateObject_FromScript2D(GameObject go)
	{
		BSAuxFuncs.SetVisible(0, go);
		pr_GObjList.Add(go);

		if (go.name.StartsWith("GUI_COMPONENT"))		
		{
			return new OQJ_Main02.THComponentObject(go);
		}

		return null;
	}



	protected THObject2D CreateObject_InsObj2D(GameObject srcGo)
	{
		return new THObject2D(CreateObject_Common_Ins(srcGo));
	}


	protected THAlphaObject2D CreateObject_InsAlpha2D(GameObject srcGo)
	{
		return new THAlphaObject2D(CreateObject_Common_Ins(srcGo));
	}


	protected THShell2D CreateObject_InsShell2D(GameObject srcGo)
	{
		return new THShell2D(CreateObject_Common_Ins(srcGo));
	}


	protected THObject2D CreateObject_InsOthers2D(GameObject srcGo)
	{
		if (srcGo.name.Equals("DbShell"))
		{
			return new OQJ_Sec230.Sec230.CustomShell(CreateObject_Common_Ins(srcGo));
		}

		return null;
	}

}



