//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using BSBMTextFactory = OQJ_BSBMTextFactory.BSBMTextFactory;



namespace OQJ_BSSuperMain
{
	public class BSSuperMain : MonoBehaviour
	{
		protected BSSuperExe[] m_ExeObjs;

		protected string[] m_RunObjRepres;

		protected int m_CoordinateSystemType;

		protected const int c_2D = 2;
		protected const int c_3D = 3;

		private float m_elapsedTime = -1;

		private float m_preTime = -1;  

		int CNT;	
		int i_frameCount;	


		void ControlFrameRate()
		{
			if (CNT <= 2)
			{ 
				CNT++;
				return; 
			}

			if (m_preTime < 0.0f)
			{
				m_preTime = Time.realtimeSinceStartup;
				m_elapsedTime = 0.0f;
				i_frameCount = 0;
			}

			m_elapsedTime += m_ExeObjs[THSettings.SEC_NR - 1].SecondPerFrame;
			float dt = Time.realtimeSinceStartup - m_preTime;
			float sleep = m_elapsedTime - dt;

			if (sleep > 0.0f)
			{
				System.Threading.Thread.Sleep(Mathf.RoundToInt(sleep * 1000));
			}

			while (dt < m_elapsedTime)
			{
				dt = Time.realtimeSinceStartup - m_preTime;
			}

			//i_frameCount++;

			if (dt >= 1.0f)
			{
				m_elapsedTime = 0.0f;
				m_preTime = Time.realtimeSinceStartup;

				//Debug.Log("FPS    :    " + i_frameCount);
				//i_frameCount = 0;
			}

		}



		private void Awake()
		{
			QualitySettings.vSyncCount = 0;
		}


		void Start()
		{
			m_elapsedTime = Time.realtimeSinceStartup;


			GameObject camera = GameObject.Find("Main Camera");
			if (camera == null)
			{
				camera = GameObject.Find("MainCamera");
			}

			if (m_CoordinateSystemType == c_2D)
			{
				BSData.CameraController2D = new BSCameraController2D(camera);
				Application.targetFrameRate = 60;
			}
			else if (m_CoordinateSystemType == c_3D)
			{
				BSData.CameraController3D = new BSCameraController3D(camera);
				Application.targetFrameRate = 9999;
			}


			BSData.ScreenW = Screen.width;
			BSData.ScreenH = Screen.height;


			Shader shader = Shader.Find("Custom/ColoredVertex");
			Material lineMaterial = new Material(shader);
			lineMaterial.hideFlags = HideFlags.HideAndDontSave;
			lineMaterial.shader.hideFlags = HideFlags.HideAndDontSave;
			BSData.LineMaterial = lineMaterial;



			BSData.RunObjectDict = new Dictionary<string, GameObject>();
			foreach (var repre in m_RunObjRepres)
			{
				GameObject go = GameObject.Find(repre);
				if(go == null)
				{
					Debug.LogFormat("**** NULL  :  {0} **** [BSSuperMain]", repre);
					continue;
				}

				BSData.RunObjectDict.Add(repre, go);
			}


			if (m_CoordinateSystemType == c_2D)
			{
				BSAuxFuncs.InitCoordinateSystem2D();

				GameObject dire_light = GameObject.Find("Directional Light");
				if (dire_light == null)
				{
					dire_light = GameObject.Find("DirectionalLight");
				}

				if (dire_light != null)
				{
					dire_light.transform.rotation = Quaternion.Euler(0, 0, 0);
				}

			}
			else if (m_CoordinateSystemType == c_3D)
			{
				BSAuxFuncs.InitCoordinateSystem3D();
				THUtil.InitXYZAxes();   

			}


			GameObject chBrd = GameObject.Find("ChapterBoard");
			if (chBrd != null)
			{
				BSAuxFuncs.SetVisible(0, chBrd.gameObject);
			}


			CheckInitSecAndCode();


			InitAuxObjects();


			InitAuxText();


			StandbyExe();

		}



		void CheckInitSecAndCode()
		{
			if (THSettings.SEC_NR <= 0 || m_ExeObjs.Length < THSettings.SEC_NR)
			{
				THSettings.SEC_NR = 1;
			}

			int numCode = m_ExeObjs[THSettings.SEC_NR - 1].GetCodeCount();
			if (THSettings.CODE_NR <= 0 || numCode < THSettings.CODE_NR)
			{
				THSettings.CODE_NR = 1;
			}
		}


		void InitAuxText()
		{
			float fov = BSData.c_DefaultFOV;
			if (m_CoordinateSystemType == c_3D)
			{
				fov = BSData.CameraController3D.GetFieldOfView();
			}
			else 
			{
				fov = BSData.CameraController2D.GetFieldOfView();
			}


			string sCh;
			if(BSData.CHAPTER_NR < 15)
			{
				sCh = (BSData.CHAPTER_NR < 10) ? " " + BSData.CHAPTER_NR : "" + BSData.CHAPTER_NR;
			}
			else
			{
				sCh = " " + (char)('A' + (BSData.CHAPTER_NR - 15));
			}

			string sSec = (THSettings.SEC_NR < 10) ? THSettings.SEC_NR + " " : "" + THSettings.SEC_NR;
			Vector2 ltr_size = BSAuxFuncs.GetLetterSize(fov, BSData.c_LetterScale_SecCode);
			string text = sCh + "-" + sSec;
			BSText ptx1 = BSBMTextFactory.MakeText(text, ltr_size, "AuxText_Ch_Sec");
			BSData.AuxText_Sec = ptx1;

			string sCode = "Code" + THSettings.CODE_NR;
			BSText ptx2 = BSBMTextFactory.MakeText(sCode, ltr_size, "AuxText_Code");
			BSData.AuxText_Code = ptx2;


			if (m_CoordinateSystemType == c_3D)
			{
				ptx1.SetRelativePosition3D(BSData.c_AuxTextRelaPos_Sec);

				Vector2 rp = (BSData.CHAPTER_NR == 3 && THSettings.SEC_NR == 20)
								? new Vector2(-0.59f, 0.80f) : BSData.c_AuxTextRelaPos_Code;
				ptx2.SetRelativePosition3D(rp);
			}
			else
			{   
				ptx1.SetRelativePosition2D(BSData.c_AuxTextRelaPos_Sec);
				ptx2.SetRelativePosition2D(BSData.c_AuxTextRelaPos_Code);
			}



			if (m_CoordinateSystemType == c_3D || !BSData.c_SHOW_COORDINATE_SCALE)
			{
				return;
			}

			float cH = 0.8f;
			float cW = cH * BSData.c_LetterRatio;
			ltr_size = new Vector2(cW, cH);

			const int max_tick = 30;

			for (int i = -max_tick; i <= max_tick; i += 5)
			{
				string ss = "" + i;
				string objName = "scale_x_" + ((i < 0) ? ("m" + (-i)) : "p" + i);
				BSText T_Co = BSBMTextFactory.MakeText(ss, ltr_size, objName);
				T_Co.SetWorldPosition2D(i - (cW * ss.Length) / 2.0f, -cH);
				BSData.Coord2DScaleX.Add(T_Co);
			}

			for (int i = -max_tick; i <= max_tick; i += 5)
			{
				if (i == 0) { continue; }

				string ss = "" + i;
				string objName = "scale_y_" + ((i < 0) ? ("m" + (-i)) : "p" + i);
				BSText T_Co = BSBMTextFactory.MakeText(ss, ltr_size, objName);
				T_Co.SetWorldPosition2D(-(cW * ss.Length) - 0.1f, i - cH / 2.0f);
				BSData.Coord2DScaleY.Add(T_Co);
			}

		}



		void StandbyExe()
		{
			foreach (var key in BSData.RunObjectDict.Keys)
			{
				if (BSData.RunObjectDict[key] == null)
				{
					Debug.LogFormat("**** NULL  :  {0}  ****", key);
				}
			}

			foreach (var obj in BSData.RunObjectDict.Values)    
			{
				BSAuxFuncs.SetVisible(0, obj);
			}

			m_ExeObjs[THSettings.SEC_NR - 1].SuperInitialize();
		}


		void Update()
		{
			if(BSData.CHAPTER_NR >= 3)
			{
				ControlFrameRate();
			}

			SuperPreProcess();

			if (m_ExeObjs[THSettings.SEC_NR - 1].IsLoopCode())
			{
				m_ExeObjs[THSettings.SEC_NR - 1].Update();
			}
			else
			{
				if (m_ExeObjs[THSettings.SEC_NR - 1].IsFirstRun())
				{
					m_ExeObjs[THSettings.SEC_NR - 1].Update();
				}
			}

			m_ExeObjs[THSettings.SEC_NR - 1].False_FIRST_RUN();  
		}



		protected virtual void InitAuxObjects()
		{
		}


		void SuperPreProcess()
		{


			if (THUtil.IsShiftDown() && THUtil.IsCtrlDown())
			{
				if (Input.GetKeyDown(KeyCode.Q))
				{
					m_ExeObjs[THSettings.SEC_NR - 1].ChangeCode(-1);
					return;
				}
				else if (Input.GetKeyDown(KeyCode.W))
				{
					m_ExeObjs[THSettings.SEC_NR - 1].ChangeCode(1);
					return;
				}
			}

		} 
	}

} 


