//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------


using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using BSBMTextFactory = OQJ_BSBMTextFactory.BSBMTextFactory;



public class BSText : THObject2D
{
	const int c_RELATIVE_2D = 1;

	const int c_RELATIVE_3D = 2;

	const int c_WORLD_2D = 3;


	public static readonly int c_MPLUS = 10;

	public static readonly int c_ROBOT = 20;



	int m_type;

	readonly float m_awayDist;		


	int m_font;
	public int font
	{
		get
		{
			return m_font;
		}
	}


	Vector2 m_rPos;
	public Vector2 relativePosition  
	{
		get
		{
			return m_rPos;
		}
	}


	Vector2 m_wPos2D;
	public Vector2 worldPos2D
	{
		get
		{
			return m_wPos2D;
		}
	}

	string m_text;
	public string text
	{
		get
		{
			return m_text;
		}
	}


	Vector2[] m_UVs;
	public Vector2[] UVs
	{
		get
		{
			return m_UVs;
		}
	}


	public BSText(GameObject go, string ss, Vector2[] uv_arr, float d=1.0f) : this(go, ss, uv_arr, d, c_MPLUS)
	{ }


	public BSText(GameObject go, string ss, Vector2[] uv_arr, float d, int font) : base(go) 
	{
		m_type = 0;

		m_text = ss;
		m_UVs = uv_arr;

		m_font = font;

		m_awayDist = d;
	}



	public void SetRelativePosition3D(Vector2 rel_pos)
	{
		m_type = c_RELATIVE_3D;
		m_rPos = rel_pos;

		Vector3 camPos = BSData.CameraController3D.GetCameraPosition();
		Vector3 uI = BSData.CameraController3D.GetRightVector();
		Vector3 uJ = BSData.CameraController3D.GetUpVector();
		Vector3 uK = BSData.CameraController3D.GetForwardVector();
		float fov = BSData.CameraController3D.GetFieldOfView();
		float aspect = BSData.CameraController3D.GetAspectRatio();
		float d = m_awayDist;



		float dians_hlf = fov / 2.0f * Mathf.Deg2Rad;
		float hlf_imgH_d_away = Mathf.Tan(dians_hlf) * d;      
		float hlf_imgW_d_away = hlf_imgH_d_away * aspect;
		Vector3 imgCtr = camPos + d * uK;  
		Vector3 texPos = imgCtr + rel_pos.x * hlf_imgW_d_away * uI + rel_pos.y * hlf_imgH_d_away * uJ;

		Quaternion texRot = BSData.CameraController3D.GetCameraRotation();	

		transform.localPosition = texPos;
		transform.localRotation = texRot;

		if(Mathf.Abs(fov - 60) > 0.1f)
		{
			float scl = Mathf.Tan(dians_hlf) / Mathf.Tan(30.0f * Mathf.Deg2Rad);
			transform.localScale = new Vector3(scl, scl, 1.0f);
		}
		else
		{
			transform.localScale = Vector3.one;
		}
	}


	public void SetRelativePosition3D(float x, float y)
	{
		SetRelativePosition3D(new Vector2(x, y));
	}



	public void SetRelativePosition2D(Vector2 rel_pos) 
	{
		m_type = c_RELATIVE_2D;
		m_rPos = rel_pos;

		Vector3 camPos = BSData.CameraController2D.GetCameraPosition();
		float fov = BSData.CameraController2D.GetFieldOfView();

		float rad_hlf = fov / 2.0f * Mathf.Deg2Rad;
		float hlf_imgH_1away = Mathf.Tan(rad_hlf);		
		float hlf_imgW_1away = hlf_imgH_1away * BSData.CameraController2D.GetAspectRatio();
		float tX = rel_pos.x * hlf_imgW_1away + camPos.x;
		float tY = rel_pos.y * hlf_imgH_1away + camPos.y;
		float tZ = camPos.z + 1.0f;  

		transform.localPosition = new Vector3(tX, tY, tZ);
	}


	public void SetRelativePosition2D(float rel_x, float rel_y) 
	{
		SetRelativePosition2D(new Vector2(rel_x, rel_y));
	}


	public void SetWorldPosition2D(Vector2 pos)
	{
		m_type = c_WORLD_2D;
		m_wPos2D = pos;

		transform.localPosition = new Vector3(pos.x, pos.y, 0.001f);
	}

	public void SetWorldPosition2D(float x, float y)
	{
		SetWorldPosition2D(new Vector2(x, y));
	}


	public void ChangeText(int idx, string ss)	
	{
		object[] val = BSBMTextFactory.ChangeText(this, idx, ss);
		if(val == null) { return; }

		m_text = (string)val[0];
		m_UVs = (Vector2[])val[1];
	}



	public void UpdatePosition()
	{
		switch (m_type)
		{
			case c_RELATIVE_2D : SetRelativePosition2D(m_rPos);	break;	

			case c_RELATIVE_3D : SetRelativePosition3D(m_rPos);	break;	

			case c_WORLD_2D : UpdateWorldPosition2D();	break;


			default : break;

		}
	}


	void UpdateWorldPosition2D()
	{
		SetWorldPosition2D(m_wPos2D);
	}

}

