//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;


public class THObject2D
{

	GameObject m_gameObject;


	public THObject2D(GameObject go)
	{
		m_gameObject = go;

		MeshRenderer mshRndr = go.GetComponent<MeshRenderer>();
		if (mshRndr != null)
		{
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);
		}
	}


	public GameObject gameObject
	{
		get { return m_gameObject; }
	}	


	public Transform transform
	{
		get { return m_gameObject.transform; }
	}	



	public void SetMatrix(THMatrix3x3 mtx)
	{
		transform.localRotation = TH2DMath.GetRotationPart(mtx);

		transform.localPosition = new Vector3(mtx[0, 2], mtx[1, 2], 0.0f);

		float sx = Mathf.Sqrt(mtx[0, 0] * mtx[0, 0] + mtx[1, 0] * mtx[1, 0]);	
		float sy = Mathf.Sqrt(mtx[0, 1] * mtx[0, 1] + mtx[1, 1] * mtx[1, 1]);	
		transform.localScale = new Vector3(sx, sy, 1.0f);
	}



	public void SetIdentity()
	{
		transform.localPosition = Vector3.zero;
		transform.localRotation = Quaternion.identity;
		transform.localScale = Vector3.one;
	}



	public void SetPosition(Vector2 pos)
	{
		transform.position = pos;
	}



	public void SetPosition(float x, float y)
	{
		SetPosition(new Vector2(x, y));
	}



	public Vector2 GetPosition()
	{
		return transform.position;
	}



	public void SetRotation(Quaternion rot)
	{
		transform.rotation = rot;
	}



	public void SetRotation(float degree)
	{
		transform.rotation = Quaternion.AngleAxis(degree, Vector3.forward);
	}



	public Quaternion GetRotation()
	{
		return transform.rotation;
	}



	public THMatrix3x3 GetMatrix()
	{
		Matrix4x4 mtx = transform.localToWorldMatrix;
		Vector3 c0 = new Vector3(mtx[0, 0], mtx[1, 0], 0.0f); 	
		Vector3 c1 = new Vector3(mtx[0, 1], mtx[1, 1], 0.0f); 	
		Vector3 c2 = new Vector3(mtx[0, 3], mtx[1, 3], 1.0f);

		return new THMatrix3x3(c0, c1, c2);	
	}



	public void SetScale(float s)
	{
		SetScale(s, s);
	}


	public void SetScale(float sx, float sy)
	{
		transform.localScale = new Vector3(sx, sy, 1.0f);
	}



}

