//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using UnityEngine;

public class THObject3D 
{

	GameObject m_gameObject;


	public THObject3D(GameObject go, bool shadow=false)
	{
		m_gameObject = go;

		MeshRenderer mshRndr = go.GetComponent<MeshRenderer>();
		if (mshRndr != null && !shadow)
		{
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);
		}
	}	


	public GameObject gameObject
	{
		get { return m_gameObject; }
	}	


	public Transform transform
	{
		get { return m_gameObject.transform; }
	}	


	public Matrix4x4 GetMatrix()
	{
		return transform.localToWorldMatrix;
	}


	public void SetMatrix(Matrix4x4 mtx)
	{
		Vector3 v = new Vector3(mtx.m03, mtx.m13, mtx.m23);
		Quaternion q = TH3DMath.GetRotationPart(mtx);
		transform.localRotation = q;	
		transform.localPosition = v;

		float sx = Mathf.Sqrt(mtx.m00 * mtx.m00 + mtx.m10 * mtx.m10 + mtx.m20 * mtx.m20);	
		float sy = Mathf.Sqrt(mtx.m01 * mtx.m01 + mtx.m11 * mtx.m11 + mtx.m21 * mtx.m21);	
		float sz = Mathf.Sqrt(mtx.m02 * mtx.m02 + mtx.m12 * mtx.m12 + mtx.m22 * mtx.m22);
		transform.localScale = new Vector3(sx, sy, sz);
	}




	public void SetIdentity()
	{
		transform.localPosition = Vector3.zero;
		transform.localRotation = Quaternion.identity;
		transform.localScale = Vector3.one;
	}



	public Vector3 GetWorldPosition()
	{
		return transform.position;
	}


	public Quaternion GetWorldRotation()
	{
		return transform.rotation;
	}


	public Vector3 GetLocalPosition()  
	{
		return transform.localPosition;
	}


	public Quaternion GetLocalRotation()
	{
		return transform.localRotation;
	}


	public Vector3 GetLocalScale()
	{
		return transform.localScale;
	}



	public void SetWorldPosition(Vector3 p)
	{
		transform.position = p;
	}


	public void SetWorldPosition(float x, float y, float z)
	{
		transform.position = new Vector3(x, y, z);
	}


	public void SetLocalPosition(Vector3 v)
	{
		transform.localPosition = v;
	}


	public void SetLocalPosition(float x, float y, float z)
	{
		SetLocalPosition(new Vector3(x, y, z));		
	}



	public void SetWorldRotation(Quaternion q)
	{
		transform.rotation = q;
	}


	public void SetWorldRotation(float deg, Vector3 axis)
	{
		transform.rotation = Quaternion.AngleAxis(deg, axis);
	}


	public void SetWorldRotation(float eul_x, float eul_y, float eul_z)
	{
		transform.rotation = Quaternion.Euler(eul_x, eul_y, eul_z);
	}


	public void SetLocalRotation(Quaternion q)
	{
		transform.localRotation = q;
	}

	public void SetLocalRotation(Matrix4x4 m4)
	{
		transform.localRotation = TH3DMath.GetRotationPart(m4);
	}

	public void SetLocalRotation(float deg, Vector3 axis)
	{
		transform.localRotation = Quaternion.AngleAxis(deg, axis);
	}

	public void SetLocalRotation(float eul_x, float eul_y, float eul_z)
	{
		transform.localRotation = Quaternion.Euler(eul_x, eul_y, eul_z);
	}

	public void SetLocalRotation(Vector3 v)
	{
		SetLocalRotation(v.x, v.y, v.z);
	}



	public void SetLocalScale(float scale)
	{
		transform.localScale = new Vector3(scale, scale, scale);
	}


	public void SetLocalScale(float sx, float sy, float sz)
	{
		transform.localScale = new Vector3(sx, sy, sz);
	}


	public void SetLocalScale(Vector3 s)
	{
		transform.localScale = s;
	}

} 

