//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;



public static class THUtil
{

	public static bool IsShiftDown()
	{
		if (Input.GetKey(KeyCode.LeftShift) || Input.GetKey(KeyCode.RightShift))
		{
			return true;
		}

		return false;
	}


	public static bool IsCtrlDown()
	{
		if (Input.GetKey(KeyCode.LeftControl) || Input.GetKey(KeyCode.RightControl))
		{
			return true;
		}

		return false;
	}


	public static bool IsZero(float v)
	{
		return (Mathf.Abs(v) < 0.00001f);
	}	


	public static bool IsEqual(float a, float b)
	{
		return (Mathf.Abs(a - b) < 0.00001f);
	}	


	public static bool IsEqual(Vector3 va, Vector3 vb)
	{
		return ((va - vb).sqrMagnitude < 0.00001f);
	}	

		
	public static void Swap(ref float a, ref float b)
	{
		float w = a;
		a = b;
		b = w;
	}


	public static bool IsZero_HighPrec(float v)
	{
		return (Mathf.Abs(v) < 0.0000001f);		
	}	



	public static void InitXYZAxes()
	{
		GameObject XYZ_Axes = new GameObject("XYZ_Axes_2");
		XYZ_Axes.AddComponent<MeshFilter>();
		XYZ_Axes.AddComponent<MeshRenderer>();

		List<Vector3> verts = new List<Vector3>();
		List<Color> colors = new List<Color>();

		Mesh wMesh = new Mesh();
		const int gridMax = 60;
		int[] idcs = new int[12];

		float bkgrndGray = BSData.BackgroundColor.r;
		float alpha = 0.25f; 	
		float semiVal = alpha * 0.73f + (1.0f - alpha) * bkgrndGray;
		float semiBgr = alpha * 0.0f + (1.0f - alpha) * bkgrndGray;

		// X axis	
		verts.Add(new Vector3((float)-gridMax, 0.0f, 0.0f));
		colors.Add(new Color(semiVal, semiBgr, semiBgr));
		verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
		colors.Add(new Color(semiVal, semiBgr, semiBgr));

		verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));
		verts.Add(new Vector3((float)gridMax, 0.0f, 0.0f));
		colors.Add(new Color(0.73f, 0.0f, 0.0f));

		// Y axis	
		verts.Add(new Vector3(0.0f, (float)-gridMax, 0.0f));
		colors.Add(new Color(semiBgr, semiVal, semiBgr));	
		verts.Add(new Vector3((float) 0.0f, 0.0f, 0.0f));
		colors.Add(new Color(semiBgr, semiVal, semiBgr));	

		verts.Add(new Vector3((float) 0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));	
		verts.Add(new Vector3(0.0f, (float) gridMax, 0.0f));
		colors.Add(new Color(0.0f, 0.73f, 0.0f));

		// Z axis	
		verts.Add(new Vector3(0.0f, 0.0f, (float)-gridMax));
		colors.Add(new Color(semiBgr, semiBgr, semiVal));
		verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
		colors.Add(new Color(semiBgr, semiBgr, semiVal));

		verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
		colors.Add(new Color(0.0f, 0.0f, 0.73f));
		verts.Add(new Vector3(0.0f, 0.0f, (float)gridMax));
		colors.Add(new Color(0.0f, 0.0f, 0.73f));

		int vi = 0;
		idcs[vi] = vi;				// X
		idcs[vi + 1] = vi + 1;	
		idcs[vi + 2] = vi + 2;	
		idcs[vi + 3] = vi + 3;
		idcs[vi + 4] = vi + 4;		// Y
		idcs[vi + 5] = vi + 5;
		idcs[vi + 6] = vi + 6;
		idcs[vi + 7] = vi + 7;
		idcs[vi + 8] = vi + 8;      // Z
		idcs[vi + 9] = vi + 9;
		idcs[vi + 10] = vi + 10;
		idcs[vi + 11] = vi + 11;

		wMesh.SetVertices(verts);
		wMesh.SetColors(colors);

		wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

		MeshFilter mshFltr = XYZ_Axes.GetComponent<MeshFilter>();
		mshFltr.mesh = wMesh;

		XYZ_Axes.transform.localPosition = Vector3.zero;
		XYZ_Axes.transform.localRotation = Quaternion.identity;
		XYZ_Axes.transform.localScale = Vector3.one;

		MeshRenderer mshRndr = XYZ_Axes.GetComponent<MeshRenderer>();
		mshRndr.material = BSData.LineMaterial;
		BSAuxFuncs.MeshRendererShadowOff(mshRndr);

		BSData.RunObjectDict.Add(XYZ_Axes.name, XYZ_Axes);

	}

} 

