//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec110
{
	public class Sec110 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		THObject2D Point;
		THObject2D Disk;
		THObject2D Circle;
		
		THObject2D PointB;
		THObject2D SmallPointR;


		// Instance Variables
		float i_shm = 0;
		float i_degree = 0;




		void Code1()
		{
			i_shm += 1;
			float x = 1.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);      
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(x, 0);
			Point.SetMatrix(T);
		}


		void Code2()
		{
			i_shm += 2;
			float x = 1.0f * Mathf.Cos(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(x, 0);
			Point.SetMatrix(T);
		}


		void Code3()
		{
			i_shm += 2;
			float A = 5.0f;
			float x = 0.5f * A * (-Mathf.Cos(i_shm * Mathf.Deg2Rad) + 1.0f);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(x, 0);
			Point.SetMatrix(T);
		}


		void Code4()
		{
			i_degree += 3;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degree);
			Disk.SetMatrix(R);
		}


		void Code5()
		{
			i_shm += 2;
			float min = 0.5f;
			float max = 2.5f;
			float A = max - min;
			float scale = 0.5f * A * (-Mathf.Cos(i_shm * Mathf.Deg2Rad) + 1.0f) + min;
			THMatrix3x3 S = TH2DMath.GetScale3x3(scale);
			Circle.SetMatrix(S);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec110 pr_instance = new Sec110(5);

		public static Sec110 PRGSI()
		{
			return pr_instance;
		}


		private Sec110(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				PointB = CreateObject_Obj2D("UnitPointB");
				SmallPointR = CreateObject_Obj2D("SmallPoint");
				Disk = CreateObject_Obj2D("DiskB");
				Circle = CreateObject_Obj2D("Circle");
				
				MeshRenderer mre = Circle.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.5f, 0.8f, 0.35f);

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			bool update_view = true;
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, PointB.gameObject);
				Point = PointB;
				
				Point.SetMatrix(THMatrix3x3.identity);
				
				i_shm = 0.0f;
				
				SetOutputImageSize(4.5f, 400, update_view);
			}
			else if (MatchCase(2, 3))
			{
				BSAuxFuncs.SetVisible(1, SmallPointR.gameObject);
				Point = SmallPointR;
				
				Point.SetMatrix(THMatrix3x3.identity);
				
				i_shm = 0.0f;
				
				SetOutputImageSize(10.2f, 400, update_view);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Disk.SetMatrix(THMatrix3x3.identity);
				
				i_degree = 0.0f;
				
				SetOutputImageSize(10.2f, 400, update_view);
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Circle.gameObject);
				Circle.SetMatrix(THMatrix3x3.identity);
				
				i_shm = 0.0f;
				
				SetOutputImageSize(10.2f, 400, update_view);
			}
		}

	}

}
