﻿//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_BSSuperMain;
using OQJ_Sec201;
using OQJ_Sec202;
using OQJ_Sec203;
using OQJ_Sec204;
using OQJ_Sec205;
using OQJ_Sec206;
using OQJ_Sec207;
using OQJ_Sec208;
using OQJ_Sec209;
using OQJ_Sec210;
using OQJ_Sec211;
using OQJ_Sec212;
using OQJ_Sec213;
using OQJ_Sec214;
using OQJ_Sec215;
using OQJ_Sec216;
using OQJ_Sec217;
using OQJ_Sec218;
using OQJ_Sec219;
using OQJ_Sec220;
using OQJ_Sec221;
using OQJ_Sec222;
using OQJ_Sec223;
using OQJ_Sec224;
using OQJ_Sec225;
using OQJ_Sec226;
using OQJ_Sec227;
using OQJ_Sec228;
using OQJ_Sec229;
using OQJ_Sec230;
using OQJ_Sec231;

using OQJ_RGAuxFuncs02;


namespace OQJ_Main02
{
	public class Main02 : BSSuperMain
	{
		public Main02()
		{
			BSSuperExe[] eos = 
			{
				Sec201.PRGSI(), Sec202.PRGSI(), Sec203.PRGSI(), Sec204.PRGSI(), Sec205.PRGSI(),
				Sec206.PRGSI(), Sec207.PRGSI(), Sec208.PRGSI(), Sec209.PRGSI(), Sec210.PRGSI(),

				Sec211.PRGSI(), Sec212.PRGSI(), Sec213.PRGSI(), Sec214.PRGSI(), Sec215.PRGSI(),
				Sec216.PRGSI(), Sec217.PRGSI(), Sec218.PRGSI(), Sec219.PRGSI(), Sec220.PRGSI(),

				Sec221.PRGSI(), Sec222.PRGSI(), Sec223.PRGSI(), Sec224.PRGSI(), Sec225.PRGSI(),					
				Sec226.PRGSI(), Sec227.PRGSI(), Sec228.PRGSI(), Sec229.PRGSI(), Sec230.PRGSI(),

				Sec231.PRGSI()
			};


			string[] repres =   
			{
				"DiskB", "DiskR", "SmallPointB", "Needle", "Ball",
				"SmallDiskB", "Foundation", "Tower", "Blades", "LPointB",
				"DirBar", "Rect", "TankBody", "GunBarrel", "HeliBody2D",
				"HeliRotor2D", "UShell2D", "CombatPlane", "CollDisk1", "CollDisk2",
				"CollDisk3", "CollRect2", "CollRect1", "CollRect3", "CollRect4",
				"Moon", "Ellipse", "Star", "Letter", "Cannon",
				"Battery", "EShell2D", "TipStar", "TipRed", "TipLetter",
				"Battleship_Body", "Battleship_HeadCannon", "Battleship_Cannon",
				"Battleship_Battery", "Battleship_Propeller",
				"RectB", "RectG", "UnitDiskB", "UnitDiskG",
				"Square2", "Cargo1", "Block2", "Floor0", "Mark0",

				"GrTria", "TanVtr", "Combat2", "DbShell", "WDR",
				"WDG", "WDB", "ObjA",

				"Vehicle", "Circuit_09", "ShiftDot", "Arrow2",
				"moR", "moG", "moB", "Circuit_24", "Square3",
				"DMaze", "BlackCover", "UnitTri_LL",
				"UnitRect_LL", "MiniMap_Cr", "MiniMap_Mz", "Dot_2",
				"xyC08", "xyC09", "xyC10", "xyC11", "Board_B",
				"Board_G", "Bubble"
			};


			BSData.CHAPTER_NR = 2;

			m_CoordinateSystemType = c_2D;
			m_ExeObjs = eos;
			m_RunObjRepres = repres;
		}


		protected override void InitAuxObjects()
		{
			BSAuxFuncs.CreateAuxCircle();

			BSAuxFuncs.CreateAuxLine();

			BSAuxFuncs.CreateAuxEditableCurve(Color.black, 120 + 1, "AuxArc");

			BSAuxFuncs.CreateAuxEditableCurve(new Color(0.7f, 0.7f, 0.7f), 3, "Convex");

			BSAuxFuncs.CreateAuxLine(new Color(1.0f, 1.0f, 0.0f), "AuxRay");

			BSAuxFuncs.CreateAuxLine(new Color(0.5f, 0.5f, 0.5f), "Line60");

			BSAuxFuncs.InitXYAxes();

			BSAuxFuncs.CreateLLBoundary();

			RGAuxFuncs02.CreateGridLines(16.0f, 50, "GridLines_XY");	

			BSAuxFuncs.CreateAuxSquareEdge();
		} 
	}



	public struct CellLoc
	{
		public int row;
		public int col;

		public CellLoc(int r, int c)
		{
			row = r;
			col = c;
		}
	}



	public class BasicCurve
	{
		Segment[] m_segments;

		float m_totalLength;


		public BasicCurve(List<Segment> list) : this(list.ToArray())
		{ }


		public BasicCurve(Segment[] segments)
		{
			m_segments = segments;

			m_totalLength = 0.0f;
			for (int i = 0; i < m_segments.Length; i++)
			{
				m_totalLength += m_segments[i].length;
			}
		}



		public Vector2 ComputePosition(float t)
		{
			float d = (t % 1.0f) * m_totalLength;

			float sum = 0.0f;
			for (int i = 0; i < m_segments.Length; i++)
			{
				var seg = m_segments[i];
				sum += seg.length;
				if (d < sum)
				{
					Vector2 pos;
					float l = d - (sum - seg.length);
					if (seg.type == Segment.LINE)
					{
						pos = seg.start + l * seg.direction;
					}
					else  // ARC
					{
						float wd = l / seg.length * seg.degree;
						Quaternion rot = Quaternion.AngleAxis(wd, Vector3.forward);
						pos = rot * (seg.start - seg.C);
						pos += seg.C;	
					}

					return pos;
				}
			}


			Debug.Log("Err : at ComputePositionAt().." + t + " , " + d);

			return Vector2.positiveInfinity;
		}



		public Vector2[] ComputePositionAndTangent(float t)
		{
			float d = (t % 1) * m_totalLength;

			float sum = 0.0f;
			for (int i = 0; i < m_segments.Length; i++)
			{
				var seg = m_segments[i];
				sum += seg.length;

				if (d < sum)
				{
					Vector2 pos;
					Vector2 dir;
					float l = d - (sum - seg.length);
					if (seg.type == Segment.LINE)
					{
						pos = seg.start + l * seg.direction;
						dir = seg.direction;
					}
					else  // ARC
					{
						float wd = l / seg.length * seg.degree;
						Quaternion rot = Quaternion.AngleAxis(wd, Vector3.forward);
						Vector2 w = rot * (seg.start - seg.C);
						pos = w + seg.C;

						float deg = (seg.degree > 0) ? 90 : -90;
						dir = (Quaternion.AngleAxis(deg, Vector3.forward) * w).normalized;
					}

					return new Vector2[] { pos, dir };
				}
			}


			Debug.Log("Err : at ComputePositionAndTangent().." + t + " , " + d);

			return null;
		}
	}



	public class Segment
	{
		public static readonly int LINE = 0;
		public static readonly int ARC = 1;

		private int m_type;
		
		private Vector2 m_start;
		private Vector2 m_end;
		private Vector2 m_direction;

		private Vector2 m_C;

		private float m_length;
		private float m_radius;
		private float m_degree;



		public int type
		{
			get { return m_type; }
		}	

		public Vector2 start
		{
			get { return m_start; }
		}	

		public Vector2 end
		{
			get { return m_end; }
		}	

		public Vector2 direction
		{
			get { return m_direction; }
		}	

		public Vector2 C
		{
			get { return m_C; }
		}	


		public float length
		{
			get { return m_length; }
		}	

		public float radius
		{
			get { return m_radius; }
		}	

		public float degree
		{
			get { return m_degree; }
		}	



		// // LINE
		public Segment(Vector2 s, Vector2 e)
		{
			m_type = LINE;
			m_start = s;
			m_end = e;

			Vector2 vSE = m_end - m_start;
			m_direction = vSE.normalized;
			m_length = vSE.magnitude;
		}


		// // ARC
		public Segment(Vector2 s, Vector2 e, Vector2 c)
		{
			m_type = ARC;
			m_start = s;
			m_end = e;
			m_C = c;

			m_radius = (m_start - m_C).magnitude;

			Vector2 vs = (m_start - m_C).normalized;
			Vector2 ve = (m_end - m_C).normalized;
			
			float dot = Vector2.Dot(vs, ve);

			float absDeg = (Mathf.Abs(dot) <= 0.999999f) ? (Mathf.Acos(dot) * Mathf.Rad2Deg) :
						 (dot > 0.0f ? 0.0f : 180.0f);     

			m_length = 2.0f * Mathf.PI * m_radius *  (absDeg / 360.0f);

			m_degree = (vs.x * ve.y - vs.y * ve.x) > 0 ? absDeg : -absDeg;
		}

	} // class Segment



	public class THComponentObject : THObject2D
	{
		Vector2[] m_uvs;


		public THComponentObject(GameObject go) : base(go)
		{
		}


		public void SetTexture(string tex_name)
		{
			MeshRenderer mshRndr = gameObject.GetComponent<MeshRenderer>();
			Texture2D texture = Resources.Load<Texture2D>(tex_name);
			mshRndr.material.SetTexture("_MainTex", texture);
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);
		}


		public void SetUVs(Vector2[] uv_arr)
		{
			List<Vector2> uvs = new List<Vector2>()
			{
				uv_arr[0], uv_arr[1], uv_arr[2], uv_arr[3]
			};

			MeshFilter mFilter = gameObject.GetComponent<MeshFilter>();
			mFilter.mesh.SetUVs(0, uvs);

			m_uvs = uv_arr;
		}


		public Vector2[] GetUVs()
		{
			return m_uvs;
		}

	} // class THComponentObject 


}


