﻿//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;


using Segment = OQJ_Main02.Segment;


namespace OQJ_RGAuxFuncs02
{
	public static class RGAuxFuncs02
	{

		public static bool CollisionTest_Point_Disk(Vector2 P, Vector2 C, float r)
		{
			float d = (P - C).magnitude;
			if (d <= r)
			{
				return true;
			}

			return false;
		}



		public static bool CollisionTest_Point_AABB(Vector2 P, Vector2[] minmax)
		{
			Vector2 min = minmax[0];
			Vector2 max = minmax[1];

			if (min.x <= P.x && P.x <= max.x &&
				min.y <= P.y && P.y <= max.y)
			{
				return true;
			}

			return false;
		}



		public static bool CollisionTest_Point_Rect(Vector2 P, Vector2 T, Vector2 lx, Vector2 ly, float wid, float hgt)
		{
			float w2 = wid * 0.5f;
			float h2 = hgt * 0.5f;

			Vector2 w = P - T;
			float a = Vector2.Dot(w, lx);
			float b = Vector2.Dot(w, ly);

			if (Mathf.Abs(a) <= w2 && Mathf.Abs(b) <= h2)
			{
				return true;
			}

			return false;
		}



		public static bool CollisionTest_Point_Triangle(Vector2 P, Vector2[] vrts)
		{
			if(vrts.Length != 3) { return false; }

			return CollisionTest_Point_Triangle(P, vrts[0], vrts[1], vrts[2]);	
		}



		public static bool CollisionTest_Point_Triangle(Vector2 P, Vector2 A, Vector2 B, Vector2 C)
		{
			// AP x AB
			Vector2 a = (P - A);
			Vector2 b = (B - A);
			if (a.x * b.y - a.y * b.x < 0.0f) { return false; }

			// BP x BC
			a = (P - B);
			b = (C - B);
			if (a.x * b.y - a.y * b.x < 0.0f) { return false; }

			// CP x CA
			a = (P - C);
			b = (A - C);
			if (a.x * b.y - a.y * b.x < 0.0f) { return false; }

			return true;
		}



		public static bool CollisionTest_Point_TriMesh(Vector2 P, Vector2[] crds)
		{
			for (int idx = 0; idx < crds.Length; idx += 3)
			{
				Vector2 A = crds[idx];
				Vector2 B = crds[idx + 1];
				Vector2 C = crds[idx + 2];

				// AP x AB
				Vector2 a = (P - A);
				Vector2 b = (B - A);
				if (a.x * b.y - a.y * b.x < 0.0f) { continue; }

				// BP x BC
				a = (P - B);
				b = (C - B);
				if (a.x * b.y - a.y * b.x < 0.0f) { continue; }

				// CP x CA
				a = (P - C);
				b = (A - C);
				if (a.x * b.y - a.y * b.x < 0.0f) { continue; }

				return true;
			}

			return false;
		}



		public static bool CollisionTest_Point_MultiAABB(Vector2 P, Vector2[] minmax)
		{
			for (int idx = 0; idx < minmax.Length; idx += 2)
			{
				Vector2 min = minmax[idx];
				Vector2 max = minmax[idx + 1];

				if (min.x <= P.x && P.x <= max.x &&
					min.y <= P.y && P.y <= max.y)
				{
					return true;
				}
			}

			return false;
		}



		public static Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			Vector3 R = TH2DMath.CalcIntersectionOf2Lines(P, va, Q, vb);
			float t1 = R.z;

			////if (t1 == float.PositiveInfinity) { }

			Vector2 R2 = R;	
			float t2 = Vector2.Dot(R2 - Q, vb);

			int b = (t1 > 0.0f && t2 < 0.0f) ? 1 : -1;
			return new Vector3(R2.x, R2.y, b);
		}



		public static void InterpolatePQ_Data(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3, List<Segment> segments)  
		{
			Vector2 R = R3;
			if (R3.z < 0.0f || IsLine(P, Q, R))
			{
				segments.Add(new Segment(P, Q));
				return;
			}

			float cosT = Vector2.Dot(-v, w);
			float sinT = Mathf.Sqrt(1.0f - cosT * cosT);
			float lenPR = (R - P).magnitude;
			float lenQR = (R - Q).magnitude;
			bool ccw = (-w.x * v.y + w.y * v.x > 0.0f) ? true : false;  
			Vector2 wr = ccw ? new Vector2(-w.y, w.x) : new Vector2(w.y, -w.x);  

			Vector2 S, C;
			if (lenPR >= lenQR)		
			{
				float a = lenQR;
				float r = a * (1 - cosT) / sinT;

				S = R - a * v;
				C = Q + r * wr;

				segments.Add(new Segment(P, S));
				segments.Add(new Segment(S, Q, C));
			}
			else
			{
				float a = lenPR;
				float r = a * (1 - cosT) / sinT;

				S = R + a * w;
				C = S + r * wr;

				segments.Add(new Segment(P, S, C));
				segments.Add(new Segment(S, Q));
			}
		}


		public static bool IsLine(Vector2 P, Vector2 Q, Vector2 R)
		{
			return THUtil.IsEqual((P + Q) * 0.5f, R);	// ((P + Q) * 0.5f - R).sqrMagnitude < 0.00001f;
		}



		public static GameObject MakeComponentRect(string name, bool center_is_orig = false, bool transparent = false)
		{
			return MakeComponentRect(name, Vector2.zero, Vector2.one, center_is_orig, transparent);
		}


		public static GameObject MakeComponentRect(string name, Vector2 min_uv, Vector2 max_uv, 
													bool center_is_orig = false, bool transparent = false)
		{
			GameObject gObj = new GameObject(name);
			gObj.AddComponent<MeshFilter>();
			gObj.AddComponent<MeshRenderer>();

			List<Vector3> verts = new List<Vector3>();
			List<Vector2> uvs = new List<Vector2>();

			float offs = (center_is_orig) ? 0.5f : 0.0f;
			verts.Add(new Vector3(0.0f - offs, 0.0f - offs, 0.0f));     // bottom-left
			verts.Add(new Vector3(0.0f - offs, 1.0f - offs, 0.0f));     // top-left
			verts.Add(new Vector3(1.0f - offs, 1.0f - offs, 0.0f));     // top-right
			verts.Add(new Vector3(1.0f - offs, 0.0f - offs, 0.0f));     // bottom-right

			float u0 = min_uv.x;
			float v0 = min_uv.y;
			float u1 = max_uv.x;
			float v1 = max_uv.y;
			uvs.Add(new Vector2(u0, v0));
			uvs.Add(new Vector2(u0, v1));
			uvs.Add(new Vector2(u1, v1));
			uvs.Add(new Vector2(u1, v0));


			Mesh wMesh = new Mesh();
			wMesh.SetVertices(verts);
			wMesh.SetUVs(0, uvs);
			wMesh.SetIndices(new int[] { 0, 1, 2, 3 }, MeshTopology.Quads, 0);

			MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
			mshFltr.mesh = wMesh;

			Shader shader_tex = (transparent) ? Shader.Find("Unlit/Transparent") : Shader.Find("Unlit/Texture");
			Material mtrl_tex = new Material(shader_tex);
			mtrl_tex.name = (transparent) ? "Unlit_Transparent_Texture" : "Unlit_Texture";

			MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
			mshRndr.material = mtrl_tex;

			gObj.transform.localPosition = Vector3.zero;
			gObj.transform.localRotation = Quaternion.identity;
			gObj.transform.localScale = Vector3.one;

			return gObj;
		}



		public static void CreateGridLines(float d=1.0f, float span=40, string name="GridLines")
		{
			float FOV = 60.0f;
			float aspect = Screen.width / (float)Screen.height;

			float h2 = Screen.height * 0.5f;
			float theta = FOV * 0.5f * Mathf.Deg2Rad;

			float H2 = Mathf.Tan(theta) * d;
			float W2 = H2 * aspect;

			float WldPerPix = H2 / h2;
			float len = span *  WldPerPix;        

			Vector3 PA0 = new Vector3(-W2,  H2, 0.0f);		// upper-left
			Vector3 PB0 = new Vector3(-W2, -H2, 0.0f);		// lower-left
			Vector3 PC0 = new Vector3(-W2, -H2, 0.0f);		// lower-left
			Vector3 PD0 = new Vector3( W2, -H2, 0.0f);     // lower-right

			List<Vector3> verts = new List<Vector3>();
			List<Color> colors = new List<Color>();

			float gr = 0.4f;
			Color gridColor = new Color(gr, gr, gr);

			int num = 120;
			int[] idcs = new int[num * 4];
			int vi = 0;
			for (int i = 0; i < num; i++)	
			{
				// vertical	lines ;; 左から右へ
				Vector3 pa = PA0 + i * len * Vector3.right;
				Vector3 pb = PB0 + i * len * Vector3.right;
				verts.Add(pa);
				verts.Add(pb);
				colors.Add(gridColor);
				colors.Add(gridColor);

				// horizontal lines		;; 下から上へ
				Vector3 pc = PC0 + i * len * Vector3.up;
				Vector3 pd = PD0 + i * len * Vector3.up;
				verts.Add(pc);
				verts.Add(pd);
				colors.Add(gridColor);
				colors.Add(gridColor);

				idcs[vi] = vi;
				idcs[vi + 1] = vi + 1;
				idcs[vi + 2] = vi + 2;
				idcs[vi + 3] = vi + 3;
				vi += 4;
			}

			GameObject go = new GameObject(name);
			go.AddComponent<MeshFilter>();
			go.AddComponent<MeshRenderer>();

			Mesh mesh = new Mesh();
			mesh.SetVertices(verts);
			mesh.SetColors(colors);

			mesh.SetIndices(idcs, MeshTopology.Lines, 0);

			MeshFilter mshFltr = go.GetComponent<MeshFilter>();
			mshFltr.mesh = mesh;

			go.transform.localPosition = Vector3.zero;
			go.transform.localRotation = Quaternion.identity;
			go.transform.localScale = Vector3.one;

			MeshRenderer mshRndr = go.GetComponent<MeshRenderer>();
			mshRndr.material = BSData.LineMaterial;
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);

			BSData.RunObjectDict.Add(go.name, go);
		}


		

		public static void MakeScreenshot_CTR(string filePath, int w, int h)
		{
			int ulX = Screen.width / 2 - w / 2;     
			int ulY = Screen.height / 2 - h / 2;
			OutputPNG(ulX, ulY, w, h, Screen.width, Screen.height, filePath);
		}


		static void OutputPNG(int ulX, int ulY, int imageW, int imageH, int screenW, int screenH, string filePath)
		{
			RenderTexture rt = new RenderTexture(screenW, screenH, 0, RenderTextureFormat.ARGB32);

			var CamComp = BSAuxFuncs.GetMainCamera().gameObject.GetComponent<Camera>();
			CamComp.targetTexture = rt;
			CamComp.Render();        

			RenderTexture saveRT = RenderTexture.active;
			RenderTexture.active = CamComp.targetTexture;

			Texture2D screenShot = new Texture2D(imageW, imageH, TextureFormat.ARGB32, false);
			screenShot.ReadPixels(new Rect(ulX, ulY, imageW, imageH), 0, 0, false);

			float igm = 1.0f / 1.4f;        
			Color[] pixArr = screenShot.GetPixels();
			for (int i = 0; i < pixArr.Length; i++)
			{
				//pixArr[i] = pixArr[i].gamma;		
				pixArr[i].r = Mathf.Pow(pixArr[i].r, igm);
				pixArr[i].g = Mathf.Pow(pixArr[i].g, igm);
				pixArr[i].b = Mathf.Pow(pixArr[i].b, igm);
			}

			screenShot.SetPixels(pixArr);

			screenShot.Apply(false);


			CamComp.targetTexture = null;
			RenderTexture.active = saveRT;

			byte[] imageData = screenShot.EncodeToPNG();
			System.IO.File.WriteAllBytes(filePath, imageData);
		}	

	}
} 

