//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec202
{
	public class Sec202 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Point;
		THObject2D Disk;


		// Instance Variables
		Vector2 i_position = new Vector2(-5.0f, 2.0f);
		float i_degree = 0.0f;




		void Code1()
		{
			i_position.x += 0.10f;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(i_position);
			Point.SetMatrix(T);
		}


		void Code2()
		{
			i_position.x += 0.10f;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(i_position);
			THMatrix3x3 Up = TH2DMath.GetTranslation3x3(0, 1);
			THMatrix3x3 Down = TH2DMath.GetTranslation3x3(0, -1);
			THMatrix3x3 M = T * Down * Up;
			Point.SetMatrix(M);
		}


		void Code3()
		{
			i_degree += 5;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degree);
			Disk.SetMatrix(R);
		}


		void Code4()
		{
			i_degree += 5;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degree);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(-5, 2);
			THMatrix3x3 M = T * R;
			Disk.SetMatrix(M);
		}


		void Code5()
		{
			i_degree += 5;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degree);
			i_position.x += 0.10f;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(i_position);
			THMatrix3x3 M = T * R;
			Disk.SetMatrix(M);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec202 pr_instance = new Sec202(5);

		public static Sec202 PRGSI()
		{
			return pr_instance;
		}


		private Sec202(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Point = CreateObject_Obj2D("SmallPointB");
				Disk = CreateObject_Obj2D("DiskB");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			
			if (MatchCase(1, 2))
			{
				i_position = new Vector2(-5.1f, 2.0f);
				BSAuxFuncs.SetVisible(1, Point.gameObject);
				
				Point.SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(3, 4, 5))
			{
				i_degree = 0.0f;
				if (MatchCase(5))
				{
					i_position = new Vector2(-5.1f, 2.0f);
				}
				
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				
				Disk.SetMatrix(THMatrix3x3.identity);
			}
		}

	}

}
