//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec203
{
	public class Sec203 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Needle;
		THObject2D Ball;
		THObject2D AuxCircle;


		// Instance Variables
		int i_revBall;
		int i_degNeedle;




		void Code1()
		{
			i_degNeedle -= 3;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degNeedle);
			Needle.SetMatrix(R);
		}


		void Code2()
		{
			i_revBall += 9;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_revBall);
			Vector2 pos = R * new Vector3(0, 1, 1);
			pos += new Vector2(0, 4);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
			Ball.SetMatrix(T);
			
			Needle.SetMatrix(THMatrix3x3.identity);
		}


		void Code3()
		{
			THMatrix3x3 localBall = TH2DMath.GetTranslation3x3(0, 5);
			
			i_degNeedle -= 3;
			THMatrix3x3 localNeedle = TH2DMath.GetRotation3x3(i_degNeedle);
			
			THMatrix3x3 worldBall = localNeedle * localBall;
			THMatrix3x3 worldNeedle = localNeedle;
			
			Ball.SetMatrix(worldBall);
			Needle.SetMatrix(worldNeedle);
		}


		void Code4()
		{
			i_revBall += 9;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_revBall);
			Vector2 pos = R * new Vector3(0, 1, 1);
			pos += new Vector2(0, 4);
			THMatrix3x3 localBall = TH2DMath.GetTranslation3x3(pos);
			
			i_degNeedle -= 3;
			THMatrix3x3 localNeedle = TH2DMath.GetRotation3x3(i_degNeedle);
			
			THMatrix3x3 worldBall = localNeedle * localBall;
			THMatrix3x3 worldNeedle = localNeedle;
			
			Ball.SetMatrix(worldBall);
			Needle.SetMatrix(worldNeedle);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(2))
			{
				AuxCircle.SetPosition(0, 4);
			}
			else if (MatchCase(4))
			{
				THMatrix3x3 M = Needle.GetMatrix() * TH2DMath.GetTranslation3x3(0, 4);
				AuxCircle.SetMatrix(M);
			}
		}


		static Sec203 pr_instance = new Sec203(4);

		public static Sec203 PRGSI()
		{
			return pr_instance;
		}


		private Sec203(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Needle = CreateObject_Obj2D("Needle");
				Ball = CreateObject_Obj2D("Ball");
				AuxCircle = CreateObject_Obj2D("AuxCircle");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_revBall = 0;
			i_degNeedle = 0;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Needle.gameObject);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Needle.gameObject);
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Needle.SetMatrix(THMatrix3x3.identity);
				Ball.SetMatrix(THMatrix3x3.identity);
				AuxCircle.SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Needle.gameObject);
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				
				Needle.SetMatrix(THMatrix3x3.identity);
				Ball.SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Needle.gameObject);
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Needle.SetMatrix(THMatrix3x3.identity);
				Ball.SetMatrix(THMatrix3x3.identity);
				AuxCircle.SetMatrix(THMatrix3x3.identity);
			}
		}

	}

}
