//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec204
{
	public class Sec204 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Disk;
		THObject2D AuxCircle;


		// Instance Variables
		int i_degRev;
		int i_degRot;
		
		readonly Vector3 c_initPos = new Vector3(3, 0, 1);




		void Code1()
		{
			i_degRev += 3;
			THMatrix3x3 Rev = TH2DMath.GetRotation3x3(i_degRev);
			Vector2 pos = Rev * c_initPos;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
			Disk.SetMatrix(T);
		}


		void Code2()
		{
			i_degRot += 9;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degRot);
			
			i_degRev += 3;
			THMatrix3x3 Rev = TH2DMath.GetRotation3x3(i_degRev);
			Vector2 pos = Rev * c_initPos;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
			
			THMatrix3x3 M = T * R;
			Disk.SetMatrix(M);
		}


		void Code3()
		{
			i_degRot += 6;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degRot);
			
			i_degRev += 3;
			THMatrix3x3 Rev = TH2DMath.GetRotation3x3(i_degRev);
			Vector2 pos = Rev * c_initPos;
			pos += new Vector2(1, 2);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
			
			THMatrix3x3 M = T * R;
			Disk.SetMatrix(M);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec204 pr_instance = new Sec204(3);

		public static Sec204 PRGSI()
		{
			return pr_instance;
		}


		private Sec204(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Disk = CreateObject_Obj2D("SmallDiskB");
				AuxCircle = CreateObject_Obj2D("AuxCircle");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(7.5f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_degRev = 0;
			i_degRot = 0;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Disk.SetMatrix(THMatrix3x3.identity);
				
				AuxCircle.SetMatrix(TH2DMath.GetScale3x3(3));
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Disk.SetMatrix(THMatrix3x3.identity);
				
				AuxCircle.SetMatrix(TH2DMath.GetTranslation3x3(1, 2) *
				TH2DMath.GetScale3x3(3));
			}
		}

	}

}
