//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec207
{
	public class Sec207 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Foundation;
		THObject2D Tower;
		THObject2D Blades;


		// Instance Variables
		float i_shmFound = 0.0f;
		float i_shmTower = 0.0f;
		float i_degBlades = 0.0f;
		
		readonly Vector2 c_attachPos_Tower = new Vector2(0.0f, 5.0f);
		readonly Vector2 c_attachPos_Blades = new Vector2(0.0f, 2.8f);




		void Code1()
		{
			i_degBlades += 8;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degBlades);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(c_attachPos_Blades);
			THMatrix3x3 localBlades = T * R;
			
			i_shmTower += 2;
			float mx = 3.4f * Mathf.Sin(i_shmTower * Mathf.Deg2Rad);
			THMatrix3x3 localTower = TH2DMath.GetTranslation3x3(mx, 5.0f);
			
			THMatrix3x3 localFound = THMatrix3x3.identity;
			
			THMatrix3x3 worldBlades = localFound * localTower * localBlades;
			THMatrix3x3 worldTower = localFound * localTower;
			THMatrix3x3 worldFound = localFound;
			
			Blades.SetMatrix(worldBlades);
			Tower.SetMatrix(worldTower);
			Foundation.SetMatrix(worldFound);
		}


		void Code2()
		{
			i_degBlades += 8;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degBlades);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(c_attachPos_Blades);
			THMatrix3x3 localBlades = T * R;
			
			i_shmTower += 2;
			float mx = 3.4f * Mathf.Sin(i_shmTower * Mathf.Deg2Rad);
			THMatrix3x3 localTower = TH2DMath.GetTranslation3x3(mx, 5.0f);
			
			i_shmFound += 2;
			float my = 3.0f * Mathf.Sin(i_shmFound * Mathf.Deg2Rad);
			THMatrix3x3 localFound = TH2DMath.GetTranslation3x3(0.0f, my);
			
			THMatrix3x3 worldBlades = localFound * localTower * localBlades;
			THMatrix3x3 worldTower = localFound * localTower;
			THMatrix3x3 worldFound = localFound;
			
			Blades.SetMatrix(worldBlades);
			Tower.SetMatrix(worldTower);
			Foundation.SetMatrix(worldFound);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec207 pr_instance = new Sec207(2);

		public static Sec207 PRGSI()
		{
			return pr_instance;
		}


		private Sec207(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Foundation = CreateObject_Obj2D("Foundation");
				Tower = CreateObject_Obj2D("Tower");
				Blades = CreateObject_Obj2D("Blades");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 5);
			
			SetFPS(30);
			
			

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			
			i_shmFound = 0.0f;
			i_shmTower = 0.0f;
			i_degBlades = 0.0f;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Blades.gameObject);
				BSAuxFuncs.SetVisible(1, Tower.gameObject);
				BSAuxFuncs.SetVisible(1, Foundation.gameObject);
				
				Blades.SetMatrix(THMatrix3x3.identity);
				Tower.SetMatrix(THMatrix3x3.identity);
				Foundation.SetMatrix(THMatrix3x3.identity);
			}
		}

	}

}
