//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec209
{
	public class Sec209 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Rect;
		THObject2D DirBar;
		
		THObject2D Body;
		THObject2D Barrel;


		// Instance Variables
		int i_forwardDegree = 0;
		int i_degBody = 0;
		int i_degBarrel = 0;




		void Code1()
		{
			Vector2 curPos = Rect.GetPosition();
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			float d = 0.05f;
			Vector2 newPos = (Input.GetKey(KeyCode.S)) ? curPos + d * forwardDir : curPos;
			
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newPos);
			Rect.SetMatrix(T);
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Rect.SetMatrix(R);
		}


		void Code3()
		{
			Vector2 curPos = Rect.GetPosition();
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			float d = 0.05f;
			Vector2 newPos = (Input.GetKey(KeyCode.S)) ? curPos + d * forwardDir : curPos;
			
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 M = T * R;
			Rect.SetMatrix(M);
		}


		void Code4()
		{
			THMatrix3x3 localBarrel = THMatrix3x3.identity;
			THMatrix3x3 localBody = THMatrix3x3.identity;
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
		}


		void Code5()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curPos = Body.GetPosition();
			Vector2 newPos = (i_MOVE) ? curPos + 0.05f * direBody : curPos;
			
			THMatrix3x3 localBarrel = THMatrix3x3.identity;
			
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 localBody = traBody * rotBody;
			
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
		}


		void Code6()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 2;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degBarrel += 4;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degBarrel -= 4;
			}
			
			THMatrix3x3 rotBarrel = TH2DMath.GetRotation3x3(i_degBarrel);
			THMatrix3x3 localBarrel = rotBarrel;
			
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			THMatrix3x3 localBody = rotBody;
			
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
		}


		void Code7()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 2;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degBarrel += 4;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degBarrel -= 4;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			// localBarrel
			THMatrix3x3 rotBarrel = TH2DMath.GetRotation3x3(i_degBarrel);
			THMatrix3x3 localBarrel = rotBarrel;
			
			// localBody
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curPos = Body.GetPosition();
			Vector2 newPos = (i_MOVE) ? curPos + 0.05f * direBody : curPos;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (THSettings.CODE_NR == 1 || THSettings.CODE_NR == 2)
			{
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(Rect.GetPosition());
				DirBar.SetMatrix(T * R);
			}
		}


		static Sec209 pr_instance = new Sec209(7);

		public static Sec209 PRGSI()
		{
			return pr_instance;
		}


		private Sec209(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Rect = CreateObject_Obj2D("Rect");
				DirBar = CreateObject_Obj2D("DirBar");
				Body = CreateObject_Obj2D("TankBody");
				Barrel = CreateObject_Obj2D("GunBarrel");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(7.5f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_forwardDegree = 0;
			i_degBody = 0;
			i_degBarrel = 0;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				BSAuxFuncs.SetVisible(1, DirBar.gameObject);
				Rect.SetMatrix(THMatrix3x3.identity);
				DirBar.SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				Rect.SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(4, 5, 6, 7))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Barrel.SetMatrix(THMatrix3x3.identity);
			}
		}

	}

}
