//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec210
{
	public class Sec210 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Body;
		THObject2D Rotor;


		// Instance Variables
		int i_degBody = 0;
		int i_degRotor = 0;
		
		readonly Vector2 c_attachPos_Rotor = new Vector2(0.0f, 0.340f);      




		void Code1()
		{
			THMatrix3x3 localRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localBody = THMatrix3x3.identity;
			
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
		}


		void Code2()
		{
			i_degRotor += 12;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			THMatrix3x3 localBody = THMatrix3x3.identity;
			
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			i_degRotor += 12;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			THMatrix3x3 localBody = TH2DMath.GetRotation3x3(i_degBody);
			
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
		}


		void Code4()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			// localRotor
			i_degRotor += 12;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			// localBody	
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curP = Body.GetPosition();
			Vector2 newP = (i_MOVE) ? (curP + 0.075f * direBody) : curP;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec210 pr_instance = new Sec210(4);

		public static Sec210 PRGSI()
		{
			return pr_instance;
		}


		private Sec210(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Body = CreateObject_Obj2D("HeliBody2D");
				Rotor = CreateObject_Obj2D("HeliRotor2D");

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_degBody = 0;
			i_degRotor = 0;
			
			if (MatchCase(1, 2, 3, 4))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Rotor.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Rotor.SetMatrix(THMatrix3x3.identity);
			}
		}

	}

}
