//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject2D;
using OQJ_THShell2D;



namespace OQJ_Sec211
{
	public class Sec211 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Body;
		THObject2D Rotor;
		
		THShell2D OneShell;
		THShell2D Shell_src;
		THShell2D[] Shell = new THShell2D[c_numShell];


		// Instance Variables
		int i_degBody = 0;
		int i_degRotor = 0;
		
		const int c_numShell = 50;
		readonly Vector2 c_attachPos_Rotor = new Vector2(0.0f, 0.340f);      




		void Code1()
		{
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(45);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			THMatrix3x3 localBody = THMatrix3x3.identity;
			
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
			
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = new Vector2(0.0f, 1.25f);
				OneShell.SetPosition(startPos);
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.25f * Vector2.up;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(45);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			THMatrix3x3 localBody = rotBody;
			
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
			
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = 1.25f * direBody;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = direBody;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.25f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			// localRotor
			i_degRotor += 12;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			// localBody	
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curP = Body.GetPosition();
			Vector2 newP = (i_MOVE) ? (curP + 0.075f * direBody) : curP;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
			
			//// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = newP + 1.25f * direBody;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = direBody;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.25f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code4()
		{
			i_frameCount++;
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Body.lastShootFrame + 3)
				{
					bShoot = true;
					Body.lastShootFrame = i_frameCount;
				}
			}
			
			// localRotor
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(45);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			// localBody
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			Vector2 newP = Vector2.zero;
			THMatrix3x3 localBody = rotBody;
			
			// world matrix
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
			
			//// Shell
			for (int i = 0; i < Shell.Length; i++)
			{
				if (Shell[i].active)
				{
					Vector2 newShellPos = Shell[i].GetPosition() + 0.80f * Shell[i].direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						Shell[i].active = false;
					}
					
					Shell[i].SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						Shell[i].active = true;
						Shell[i].direction = direBody;
						
						Vector2 startPos = newP + 1.25f * direBody;
						Shell[i].SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
		}


		void Code5()
		{
			i_frameCount++;
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Body.lastShootFrame + 3)
				{
					bShoot = true;
					Body.lastShootFrame = i_frameCount;
				}
			}
			
			// localRotor
			i_degRotor += 12;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			// localBody
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curP = Body.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.075f * direBody : curP;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
			
			// Shell
			for (int i = 0; i < Shell.Length; i++)
			{
				if (Shell[i].active)
				{
					Vector2 newShellPos = Shell[i].GetPosition() + 0.80f * Shell[i].direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						Shell[i].active = false;
					}
					
					Shell[i].SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						Shell[i].active = true;
						Shell[i].direction = direBody;
						
						Vector2 startPos = newP + 1.25f * direBody;
						Shell[i].SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
			
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec211 pr_instance = new Sec211(5);

		public static Sec211 PRGSI()
		{
			return pr_instance;
		}


		private Sec211(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Body = CreateObject_Alpha2D("HeliBody2D");
				Rotor = CreateObject_Obj2D("HeliRotor2D");
				Shell_src = CreateObject_Shell2D("UShell2D");
				
				// Instantiate 
				Shell[0] = Shell_src;
				for (int i = 1; i < Shell.Length; i++)
				{
					Shell[i] = CreateObject_InsShell2D(Shell_src.gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_degBody = 0;
			i_degRotor = 0;
			
			if (MatchCase(1, 2, 3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Rotor.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Rotor.SetMatrix(THMatrix3x3.identity);
				
				BSAuxFuncs.SetVisible(1, Shell_src.gameObject);
				OneShell = Shell_src;
				OneShell.SetMatrix(THMatrix3x3.identity);
				OneShell.active = false;
				OneShell.SetPosition(-1000, 0);
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Rotor.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Rotor.SetMatrix(THMatrix3x3.identity);
				
				Body.lastShootFrame = 0;
				
				for (int i = 0; i < Shell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Shell[i].gameObject);
					Shell[i].SetMatrix(THMatrix3x3.identity);
					Shell[i].active = false;
					Shell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
			
		}

	}

}
