//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject2D;
using OQJ_THShell2D;



namespace OQJ_Sec212
{
	public class Sec212 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Body;
		THAlphaObject2D Barrel;
		
		THShell2D OneShell;
		THShell2D Shell_src;
		THShell2D[] Shell = new THShell2D[c_numShell];


		// Instance Variables
		int i_degBody = 0;
		int i_degBarrel = 0;
		
		const int c_numShell = 50;




		void Code1()
		{
			if (Input.GetKey(KeyCode.D))
			{
				i_degBarrel += 4;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degBarrel -= 4;
			}
			
			THMatrix3x3 rotBarrel = TH2DMath.GetRotation3x3(i_degBarrel);
			THMatrix3x3 localBarrel = rotBarrel;
			
			THMatrix3x3 localBody = THMatrix3x3.identity;
			
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
			
			// Shell
			Vector2 direBarrel = rotBarrel * new Vector3(0, 1, 1);
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = 1.52f * direBarrel;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = direBarrel;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.25f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 2;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degBarrel += 4;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degBarrel -= 4;
			}
			
			THMatrix3x3 rotBarrel = TH2DMath.GetRotation3x3(i_degBarrel);
			THMatrix3x3 localBarrel = rotBarrel;
			
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			THMatrix3x3 localBody = rotBody;
			
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
			
			// Shell
			THMatrix3x3 R2 = TH2DMath.GetRotation3x3(i_degBarrel + i_degBody);
			Vector2 direBarrel = R2 * new Vector3(0, 1, 1);
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = 1.52f * direBarrel;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = direBarrel;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.25f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code3()
		{
			i_frameCount++;
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 2;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degBarrel += 4;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degBarrel -= 4;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Barrel.lastShootFrame + 3)
				{
					bShoot = true;
					Barrel.lastShootFrame = i_frameCount;
				}
			}
			
			// localBarrel
			THMatrix3x3 rotBarrel = TH2DMath.GetRotation3x3(i_degBarrel);
			THMatrix3x3 localBarrel = rotBarrel;
			
			// localBody
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curP = Body.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.05f * direBody : curP;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldBarrel = localBody * localBarrel;
			THMatrix3x3 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Body.SetMatrix(worldBody);
			
			// Shell
			THMatrix3x3 R2 = TH2DMath.GetRotation3x3(i_degBarrel + i_degBody);
			Vector2 direBarrel = R2 * new Vector3(0, 1, 1);
			for (int i = 0; i < Shell.Length; i++)
			{
				if (Shell[i].active)
				{
					Vector2 newShellPos = Shell[i].GetPosition() + 0.80f * Shell[i].direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						Shell[i].active = false;
					}
					
					Shell[i].SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						Shell[i].active = true;
						Shell[i].direction = direBarrel;
						
						Vector2 startPos = newP + 1.52f * direBarrel;
						Shell[i].SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec212 pr_instance = new Sec212(3);

		public static Sec212 PRGSI()
		{
			return pr_instance;
		}


		private Sec212(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Body = CreateObject_Alpha2D("TankBody");
				Barrel = CreateObject_Alpha2D("GunBarrel");
				Shell_src = CreateObject_Shell2D("UShell2D");
				
				// Instantiate 
				Shell[0] = Shell_src;
				for (int i = 1; i < Shell.Length; i++)
				{
					Shell[i] = CreateObject_InsShell2D(Shell_src.gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(7.5f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_degBody = 0;
			i_degBarrel = 0;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Barrel.SetMatrix(THMatrix3x3.identity);
				
				BSAuxFuncs.SetVisible(1, Shell_src.gameObject);
				OneShell = Shell_src;
				OneShell.SetMatrix(THMatrix3x3.identity);
				OneShell.active = false;
				OneShell.SetPosition(-1000, 0);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(THMatrix3x3.identity);
				Barrel.SetMatrix(THMatrix3x3.identity);
				
				Barrel.lastShootFrame = 0;
				
				for (int i = 0; i < Shell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Shell[i].gameObject);
					Shell[i].SetMatrix(THMatrix3x3.identity);
					Shell[i].active = false;
					Shell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				OneShell = Shell[0];
			}
		}

	}

}
