//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject2D;
using OQJ_THShell2D;
using static OQJ_RGAuxFuncs02.RGAuxFuncs02;



namespace OQJ_Sec214
{
	public class Sec214 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Plane;
		THShell2D OneShell;
		
		THAlphaObject2D Disk0;
		THAlphaObject2D Disk1;
		THAlphaObject2D Disk2;
		
		THAlphaObject2D Rect0;
		THAlphaObject2D Rect1;
		THAlphaObject2D Rect2;
		THAlphaObject2D Rect3;


		// Instance Variables
		float i_forwardDegree = 0.0f;
		
		THAlphaObject2D[] i_collisionObjects = new THAlphaObject2D[0];
		THAlphaObject2D[] pr_collisionObjects = new THAlphaObject2D[0];
		
		float i_shm;
		float i_deg;
		
		readonly Vector2[] c_initRectBounds0 = { new Vector2(-1.8f, -0.8f), new Vector2(1.8f, 0.8f) };
		readonly Vector2[] c_initRectBounds1 = { new Vector2(-3.0f, -0.6f), new Vector2(3.0f, 0.6f) };
		readonly Vector2[] c_initRectBounds2 = { new Vector2(-4.4f, -0.4f), new Vector2(4.4f, 0.4f) };
		readonly Vector2[] c_initRectBounds3 = { new Vector2(-1.2f, -1.2f), new Vector2(1.2f, 1.2f) };
		
		readonly Color c_HitRed0 = new Color(0.5f, 0.0f, 0.0f);
		readonly Color c_HitRed1 = new Color(0.6f, 0.2f, 0.2f);
		readonly Color c_DiskBlue0 = new Color(0.559f, 0.636f, 0.855f);
		readonly Color c_DiskBlue1 = new Color(0.678f, 0.566f, 0.755f);




		void Code1()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.12f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = newP + 2.1f * forwardDir;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = forwardDir;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.12f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
		}


		void Code3()
		{
			// <-- Code2と同じ記述
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.12f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
			// Code2と同じ記述 -->
			
			
			Disk0.SetPosition(6.0f, 8.0f);
			Disk1.SetPosition(-12.0f, 0.0f);
			Disk2.SetPosition(12.0f, -6.0f);
			
			// Collision Test
			Vector2 shell_pos = OneShell.GetPosition();
			
			foreach (var obj in i_collisionObjects)
			{
				Vector2 C = obj.GetPosition();
				float r = obj.initRadius;
				bool bColl = CollisionTest_Point_Disk(shell_pos, C, r);
				if (bColl)
				{
					Hit(obj);
					break;
				}
			}
		}


		void Code4()
		{
			// <-- Code2と同じ記述
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.12f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
			// Code2と同じ記述 -->
			
			
			T = TH2DMath.GetTranslation3x3(11.2f, -6.2f);   
			Rect0.SetMatrix(T);
			
			T = TH2DMath.GetTranslation3x3(10f, 6.6f);      
			Rect1.SetMatrix(T);
			
			R = TH2DMath.GetRotation3x3(-30f);
			T = TH2DMath.GetTranslation3x3(-11f, -6f);      
			M = T * R;
			Rect2.SetMatrix(M);
			
			R = TH2DMath.GetRotation3x3(120f);
			T = TH2DMath.GetTranslation3x3(-12f, 6.6f);     
			M = T * R;
			Rect3.SetMatrix(M);
			
			
			// Collision Test 
			Vector2 shell_pos = OneShell.GetPosition();
			
			foreach (var obj in i_collisionObjects)
			{
				Vector2[] bounds = obj.initRectBounds;
				THMatrix3x3 mtx = obj.GetMatrix();
				Vector2 P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bool bColl = CollisionTest_Point_AABB(P, bounds);
				if (bColl)
				{
					Hit(obj);
					break;
				}
			}
		}


		void Code5()
		{
			// <-- Code2と同じ記述
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.12f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
			// Code2と同じ記述 -->
			
			
			i_shm += 2;
			float scale = 1.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad) + 2.0f;
			THMatrix3x3 S = TH2DMath.GetScale3x3(scale);
			T = TH2DMath.GetTranslation3x3(11f, 6f);
			M = T * S;
			Disk0.SetMatrix(M);
			
			float my = 2.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			T = TH2DMath.GetTranslation3x3(11.0f, -5.0f + my);
			Rect1.SetMatrix(T);
			
			i_deg += 2;
			R = TH2DMath.GetRotation3x3(i_deg);
			T = TH2DMath.GetTranslation3x3(-11f, 5.4f);
			M = T * R;
			Rect2.SetMatrix(M);
			
			S = TH2DMath.GetScale3x3(1.0f, scale);
			R = TH2DMath.GetRotation3x3(60f);
			T = TH2DMath.GetTranslation3x3(-11f, -5.6f);
			M = T * R * S;
			Rect3.SetMatrix(M);
			
			
			// Collision Test 
			Vector2 shell_pos = OneShell.GetPosition();
			
			foreach (var obj in i_collisionObjects)
			{
				bool bColl = CollisionTest_AllTypes(obj, shell_pos);
				if (bColl)
				{
					Hit(obj);
					break;
				}
			}
		}




		bool CollisionTest_AllTypes(THAlphaObject2D obj, Vector2 shell_pos)
		{
			bool bColl = false;
			switch (obj.GetType())
			{
				case THAlphaObject2D.DISK:
				Vector2 C = obj.GetPosition();
				float r = obj.GetDiskScale() * obj.initRadius;
				bColl = CollisionTest_Point_Disk(shell_pos, C, r);
				break;
				
				case THAlphaObject2D.RECT:
				Vector2[] bounds = obj.initRectBounds;
				THMatrix3x3 mtx = obj.GetMatrix();
				Vector2 P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bColl = CollisionTest_Point_AABB(P, bounds);
				break;
				
				default: break;
			}
			
			return bColl;
		}
		
		
		
		void Hit(THAlphaObject2D obj)
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = 6;
				
				MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
				mre.materials = obj.hitMaterials;
			}
			
			OneShell.SetPosition(-1000, 0);
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(3, 4, 5))
			{
				foreach (var o in pr_collisionObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
							mre.materials = o.originalMaterials;
						}
					}
				}
			}
		}


		static Sec214 pr_instance = new Sec214(5);

		public static Sec214 PRGSI()
		{
			return pr_instance;
		}


		private Sec214(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				
				Plane = CreateObject_Alpha2D("CombatPlane");
				Disk0 = CreateObject_Alpha2D("CollDisk1");
				Disk1 = CreateObject_Alpha2D("CollDisk2");
				Disk2 = CreateObject_Alpha2D("CollDisk3");
				
				Rect0 = CreateObject_Alpha2D("CollRect2");
				
				Rect1 = CreateObject_Alpha2D("CollRect1");
				Rect2 = CreateObject_Alpha2D("CollRect3");
				Rect3 = CreateObject_Alpha2D("CollRect4");
				OneShell = CreateObject_Shell2D("UShell2D");
				
				
				Disk0.SetData(THAlphaObject2D.DISK, 1.0f);
				Disk1.SetData(THAlphaObject2D.DISK, 1.66f);
				Disk2.SetData(THAlphaObject2D.DISK, 2.4f);
				
				Rect0.SetData(THAlphaObject2D.RECT, c_initRectBounds0);
				Rect1.SetData(THAlphaObject2D.RECT, c_initRectBounds1);
				Rect2.SetData(THAlphaObject2D.RECT, c_initRectBounds2);
				Rect3.SetData(THAlphaObject2D.RECT, c_initRectBounds3);
				
				THAlphaObject2D[] wObjs = { Disk0, Disk1, Disk2, Rect0, Rect1, Rect2, Rect3 };
				Shader standard = Shader.Find("Standard");
				foreach (var obj in wObjs)
				{
					var mre = obj.gameObject.GetComponent<MeshRenderer>();
					
					if (obj.GetType() == THAlphaObject2D.DISK)
					{
						mre.materials[0].SetColor("_Color", c_DiskBlue0);
						mre.materials[1].SetColor("_Color", c_DiskBlue1);
					}
					
					obj.originalMaterials = mre.materials;
					
					Material[] hitMtrls = new Material[2];
					hitMtrls[0] = new Material(standard);
					hitMtrls[0].name = "CollRed0";
					hitMtrls[0].SetColor("_Color", c_HitRed0);
					hitMtrls[1] = new Material(standard);
					hitMtrls[1].name = "CollRed1";
					hitMtrls[1].SetColor("_Color", c_HitRed1);
					
					obj.hitMaterials = hitMtrls;
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_forwardDegree = 0.0f;
			i_shm = 0.0f;
			i_deg = 0.0f;
			
			
			foreach (var o in pr_collisionObjects)
			{
				if (o.collCount > 0)
				{
					o.collCount = 0;
					MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
					mre.materials = o.originalMaterials;
				}
			}
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, OneShell.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				OneShell.SetMatrix(THMatrix3x3.identity);
				
				OneShell.SetPosition(-1000, 0);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Disk0.gameObject);
				BSAuxFuncs.SetVisible(1, Disk1.gameObject);
				BSAuxFuncs.SetVisible(1, Disk2.gameObject);
				BSAuxFuncs.SetVisible(1, OneShell.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				OneShell.SetMatrix(THMatrix3x3.identity);
				
				OneShell.SetPosition(-1000, 0);
				
				Disk0.SetMatrix(THMatrix3x3.identity);
				Disk1.SetMatrix(THMatrix3x3.identity);
				Disk2.SetMatrix(THMatrix3x3.identity);
				
				i_collisionObjects = new THAlphaObject2D[] { Disk0, Disk1, Disk2 };
				pr_collisionObjects = i_collisionObjects;
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Rect0.gameObject);
				BSAuxFuncs.SetVisible(1, Rect1.gameObject);
				BSAuxFuncs.SetVisible(1, Rect2.gameObject);
				BSAuxFuncs.SetVisible(1, Rect3.gameObject);
				BSAuxFuncs.SetVisible(1, OneShell.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				OneShell.SetMatrix(THMatrix3x3.identity);
				
				OneShell.SetPosition(-1000, 0);
				
				Rect0.SetMatrix(THMatrix3x3.identity);
				Rect1.SetMatrix(THMatrix3x3.identity);
				Rect2.SetMatrix(THMatrix3x3.identity);
				Rect3.SetMatrix(THMatrix3x3.identity);
				
				i_collisionObjects = new THAlphaObject2D[] { Rect0, Rect1, Rect2, Rect3 };
				pr_collisionObjects = i_collisionObjects;
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Disk0.gameObject);
				BSAuxFuncs.SetVisible(1, Rect1.gameObject);
				BSAuxFuncs.SetVisible(1, Rect2.gameObject);
				BSAuxFuncs.SetVisible(1, Rect3.gameObject);
				BSAuxFuncs.SetVisible(1, OneShell.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				OneShell.SetMatrix(THMatrix3x3.identity);
				
				OneShell.SetPosition(-1000, 0);
				
				Disk0.SetMatrix(THMatrix3x3.identity);
				Rect1.SetMatrix(THMatrix3x3.identity);
				Rect2.SetMatrix(THMatrix3x3.identity);
				Rect3.SetMatrix(THMatrix3x3.identity);
				
				i_collisionObjects = new THAlphaObject2D[] { Disk0, Rect1, Rect2, Rect3 };
				pr_collisionObjects = i_collisionObjects;
			}
		}

	}

}
