//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGMeshData02;
using OQJ_THAlphaObject2D;
using OQJ_THShell2D;
using static OQJ_RGAuxFuncs02.RGAuxFuncs02;



namespace OQJ_Sec215
{
	public class Sec215 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Plane;
		THShell2D OneShell;
		
		THAlphaObject2D Moon;
		THAlphaObject2D Star;
		THAlphaObject2D Ellipse;
		THAlphaObject2D Letter;


		// Instance Variables
		float i_forwardDegree = 0.0f;
		float i_deg = 0.0f;
		
		THAlphaObject2D[] i_collisionObjects = new THAlphaObject2D[0];
		THAlphaObject2D[] pr_collisionObjects = new THAlphaObject2D[0];
		
		
		readonly Vector2 c_posMoon = new Vector2(9, 5);
		readonly Vector2 c_posEllipse = new Vector2(-12, 6);
		readonly Vector2 c_posStar = new Vector2(-12, -5);
		readonly Vector2 c_posLetter = new Vector2(10, -5.5f);
		
		readonly Color c_HitRed0 = new Color(0.5f, 0.0f, 0.0f);
		readonly Color c_HitRed1 = new Color(0.6f, 0.2f, 0.2f);




		void Code1()
		{
			// <-- 2-14節 Code2と同じ記述
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curPos = Plane.GetPosition();
			Vector2 newPos = (i_MOVE) ? curPos + 0.12f * forwardDir : curPos;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
			// 2-14節 Code2と同じ記述 -->
			
			
			// Object Motion
			T = TH2DMath.GetTranslation3x3(c_posMoon);
			Moon.SetMatrix(T);
			
			T = TH2DMath.GetTranslation3x3(c_posEllipse);
			Ellipse.SetMatrix(T);
			
			T = TH2DMath.GetTranslation3x3(c_posStar);
			Star.SetMatrix(T);
			
			T = TH2DMath.GetTranslation3x3(c_posLetter);
			Letter.SetMatrix(T);
			
			
			// Collision Test		
			Vector2 shell_pos = OneShell.GetPosition();
			
			foreach (var obj in i_collisionObjects)
			{
				THMatrix3x3 mtx = obj.GetMatrix();
				Vector2 P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bool bColl = CollisionTest_Point_TriMesh(P, obj.initTriangleCoords);
				if (bColl)
				{
					Hit(obj);
					break;
				}
			}
		}


		void Code2()
		{
			// <-- 2-14節 Code2と同じ記述
			if (!i_INITIALIZED)
			{
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.initShootDirection = new Vector2(0.0f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curPos = Plane.GetPosition();
			Vector2 newPos = (i_MOVE) ? curPos + 0.12f * forwardDir : curPos;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
			
			// Shell
			if (Input.GetKeyDown(KeyCode.A))
			{
				Vector2 startPos = M * Plane.initShootPosition;
				OneShell.SetPosition(startPos);
				
				OneShell.direction = (M * Plane.initShootDirection).normalized;
			}
			else
			{
				Vector2 newShellPos = OneShell.GetPosition() + 0.50f * OneShell.direction;
				OneShell.SetPosition(newShellPos);
			}
			// 2-14節 Code2と同じ記述 -->
			
			
			// Object Motion
			i_deg += 1.0f;
			R = TH2DMath.GetRotation3x3(i_deg);
			
			T = TH2DMath.GetTranslation3x3(c_posMoon);
			M = T * R;
			Moon.SetMatrix(M);
			
			T = TH2DMath.GetTranslation3x3(c_posEllipse);
			M = T * R;
			Ellipse.SetMatrix(M);
			
			T = TH2DMath.GetTranslation3x3(c_posStar);
			M = T * R;
			Star.SetMatrix(M);
			
			T = TH2DMath.GetTranslation3x3(c_posLetter);
			M = T * R;
			Letter.SetMatrix(M);
			
			
			// Collision Test		
			Vector2 shell_pos = OneShell.GetPosition();
			
			foreach (var obj in i_collisionObjects)
			{
				THMatrix3x3 mtx = obj.GetMatrix();
				Vector2 P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bool bColl = CollisionTest_Point_TriMesh(P, obj.initTriangleCoords);
				if (bColl)
				{
					Hit(obj);
					break;
				}
			}
		}




		
		void Hit(THAlphaObject2D obj)
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = 6;
				
				MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
				mre.materials = obj.hitMaterials;
			}
			
			OneShell.SetPosition(-1000, 0);
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				foreach (var o in pr_collisionObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
							mre.materials = o.originalMaterials;
						}
					}
				}
			}
		}


		static Sec215 pr_instance = new Sec215(2);

		public static Sec215 PRGSI()
		{
			return pr_instance;
		}


		private Sec215(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Plane = CreateObject_Alpha2D("CombatPlane");
				Moon = CreateObject_Alpha2D("Moon");
				Ellipse = CreateObject_Alpha2D("Ellipse");
				Star = CreateObject_Alpha2D("Star");
				Letter = CreateObject_Alpha2D("Letter");
				OneShell = CreateObject_Shell2D("UShell2D");
				
				Vector2[] verts = RGMeshData02.GetVertsData(RGMeshData02.c_ELLIPSE);
				Ellipse.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_LETTER);
				Letter.SetData(THAlphaObject2D.TRIMESH, verts);
				
				
				THAlphaObject2D[] wObjs = { Moon, Ellipse, Star, Letter };
				Shader standard = Shader.Find("Standard");
				foreach (var obj in wObjs)
				{
					var mre = obj.gameObject.GetComponent<MeshRenderer>();
					
					obj.originalMaterials = mre.materials;
					
					Material[] hitMtrls = new Material[1];
					hitMtrls[0] = new Material(standard);
					hitMtrls[0].name = "CollRed1";
					hitMtrls[0].SetColor("_Color", c_HitRed1);
					
					obj.hitMaterials = hitMtrls;
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_forwardDegree = 0.0f;
			i_deg = 0.0f;
			
			
			foreach (var o in pr_collisionObjects)
			{
				if (o.collCount > 0)
				{
					o.collCount = 0;
					MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
					mre.materials = o.originalMaterials;
				}
			}
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Moon.gameObject);
				BSAuxFuncs.SetVisible(1, Star.gameObject);
				BSAuxFuncs.SetVisible(1, Ellipse.gameObject);
				BSAuxFuncs.SetVisible(1, Letter.gameObject);
				BSAuxFuncs.SetVisible(1, OneShell.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				OneShell.SetMatrix(THMatrix3x3.identity);
				
				OneShell.SetPosition(-1000, 0);
				
				Moon.SetMatrix(THMatrix3x3.identity);
				Star.SetMatrix(THMatrix3x3.identity);
				Ellipse.SetMatrix(THMatrix3x3.identity);
				Letter.SetMatrix(THMatrix3x3.identity);
				
				i_collisionObjects = new THAlphaObject2D[] { Moon, Ellipse, Star, Letter };
				pr_collisionObjects = i_collisionObjects;
				
				if (MatchCase(1))
				{
					Vector2[] verts = RGMeshData02.GetVertsData(RGMeshData02.c_MOON);
					Moon.SetData(THAlphaObject2D.TRIMESH, verts);
					
					verts = RGMeshData02.GetVertsData(RGMeshData02.c_STAR);
					Star.SetData(THAlphaObject2D.TRIMESH, verts);
				}
				else
				{
					Vector2[] verts = RGMeshData02.GetVertsData(RGMeshData02.c_LOOSE_MOON);
					Moon.SetData(THAlphaObject2D.TRIMESH, verts);
					
					verts = RGMeshData02.GetVertsData(RGMeshData02.c_LOOSE_STAR);
					Star.SetData(THAlphaObject2D.TRIMESH, verts);
				}
			}
		}

	}

}
