//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGMeshData02;
using OQJ_THAlphaObject2D;
using OQJ_THShell2D;



namespace OQJ_Sec216
{
	public class Sec216 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Battery_src;
		THAlphaObject2D[] Battery;
		THAlphaObject2D[] storedBattery = new THAlphaObject2D[2];
		
		THAlphaObject2D Cannon_src;
		THAlphaObject2D[] Cannon;
		THAlphaObject2D[] storedCannon = new THAlphaObject2D[4];
		
		THShell2D EShell_src;
		THShell2D[] EShell = new THShell2D[c_numEShell];


		// Instance Variables
		float i_shm = 0;
		
		THAlphaObject2D[] i_shootObjects = new THAlphaObject2D[0];
		
		const int c_numEShell = 30;




		void Code1()
		{
			i_shm += 2;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localCannon = TH2DMath.GetTranslation3x3(x, 0);
			
			THMatrix3x3 rotBattery = TH2DMath.GetRotation3x3(-120);
			THMatrix3x3 traBattery = TH2DMath.GetTranslation3x3(-10, 4);
			THMatrix3x3 localBattery = traBattery * rotBattery;
			
			THMatrix3x3 worldCannon = localBattery * localCannon;
			THMatrix3x3 worldBattery = localBattery;
			
			Cannon[0].SetMatrix(worldCannon);
			Battery[0].SetMatrix(worldBattery);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Cannon[0].lastShootFrame = 8;
				Cannon[0].shootInterval = 12;
				Cannon[0].initShootDirection = new Vector2(0, 1);
				Cannon[0].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[0].shellSpeed = 0.40f;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			// Cannon
			i_shm += 2;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(x, 0);
			Cannon[0].SetMatrix(T);
			
			// Shell
			bool bShoot = false;
			if (i_frameCount >= Cannon[0].lastShootFrame + Cannon[0].shootInterval)
			{
				Cannon[0].lastShootFrame = i_frameCount;
				bShoot = true;
			}
			
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						THMatrix3x3 M = Cannon[0].GetMatrix();
						Vector2 startPos = M * Cannon[0].initShootPosition;
						Vector2 shootDir = (M * Cannon[0].initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Cannon[0].shellSpeed;
						
						shell.SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Cannon[0].lastShootFrame = 8;
				Cannon[0].shootInterval = 12;
				Cannon[0].initShootDirection = new Vector2(0, 1);
				Cannon[0].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[0].shellSpeed = 0.40f;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			// <-- Code1と同じ記述
			// Cannon
			i_shm += 2;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localCannon = TH2DMath.GetTranslation3x3(x, 0);
			
			THMatrix3x3 rotBattery = TH2DMath.GetRotation3x3(-120);
			THMatrix3x3 traBattery = TH2DMath.GetTranslation3x3(-10, 4);
			THMatrix3x3 localBattery = traBattery * rotBattery;
			
			THMatrix3x3 worldCannon = localBattery * localCannon;
			THMatrix3x3 worldBattery = localBattery;
			
			Cannon[0].SetMatrix(worldCannon);
			Battery[0].SetMatrix(worldBattery);
			// Code1と同じ記述 -->
			
			
			// Shell
			bool bShoot = false;
			if (i_frameCount >= Cannon[0].lastShootFrame + Cannon[0].shootInterval)
			{
				Cannon[0].lastShootFrame = i_frameCount;
				bShoot = true;
			}
			
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						THMatrix3x3 M = Cannon[0].GetMatrix();
						Vector2 startPos = M * Cannon[0].initShootPosition;
						Vector2 shootDir = (M * Cannon[0].initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Cannon[0].shellSpeed;
						
						shell.SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				Cannon[0].lastShootFrame = 8;
				Cannon[0].shootInterval = 12;  
				Cannon[0].initShootDirection = new Vector2(0, 1);
				Cannon[0].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[0].shellSpeed = 0.40f;
				
				Cannon[1].lastShootFrame = 8;
				Cannon[1].shootInterval = 12;
				Cannon[1].initShootDirection = new Vector2(0, 1);
				Cannon[1].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[1].shellSpeed = 0.40f;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			// Cannon
			THMatrix3x3 T0 = TH2DMath.GetTranslation3x3(-2.0f, 0.0f);
			Cannon[0].SetMatrix(T0);
			THMatrix3x3 T1 = TH2DMath.GetTranslation3x3(2.0f, 0.0f);
			Cannon[1].SetMatrix(T1);
			
			
			// Shell
			var curShooters = new List<THAlphaObject2D>();
			
			foreach (var obj in Cannon)
			{
				if (i_frameCount >= obj.lastShootFrame + obj.shootInterval)
				{
					obj.lastShootFrame = i_frameCount;
					curShooters.Add(obj);
				}
			}
			
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (curShooters.Count > 0)
					{
						THAlphaObject2D obj = curShooters[0];
						THMatrix3x3 M = obj.GetMatrix();
						Vector2 startPos = M * obj.initShootPosition;
						Vector2 shootDir = (M * obj.initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = obj.shellSpeed;
						
						shell.SetPosition(startPos);
						
						curShooters.RemoveAt(0);
					}
				}
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				Cannon[0].lastShootFrame = 8;
				Cannon[0].shootInterval = 15;
				Cannon[0].initShootDirection = new Vector2(0, 1);
				Cannon[0].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[0].shellSpeed = 0.40f;
				
				Cannon[1].lastShootFrame = 12;
				Cannon[1].shootInterval = 18;
				Cannon[1].initShootDirection = new Vector2(0, 1);
				Cannon[1].initShootPosition = new Vector2(0.0f, 1.75f);
				Cannon[1].shellSpeed = 0.40f;
				
				i_shootObjects = Cannon;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			EnemyMotion5();
			EShellMotion();
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				for (int i = 0; i < Cannon.Length; i++)
				{
					Cannon[i].lastShootFrame = 8 + i * 4;
					Cannon[i].shootInterval = 15 + i * 3;
					Cannon[i].initShootDirection = new Vector2(0, 1);
					Cannon[i].initShootPosition = new Vector2(0.0f, 1.75f);
					Cannon[i].shellSpeed = 0.40f;
				}
				
				i_shootObjects = Cannon;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			EnemyMotion6();
			EShellMotion();
		}




		void EnemyMotion5()
		{
			THMatrix3x3 rotBattery = TH2DMath.GetRotation3x3(-120);     
			THMatrix3x3 traBattery = TH2DMath.GetTranslation3x3(-10, 4);    
			THMatrix3x3 localBattery = traBattery * rotBattery;
			
			i_shm += 2.0f;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localCannon0 = TH2DMath.GetTranslation3x3(x - 2, 0);
			THMatrix3x3 localCannon1 = TH2DMath.GetTranslation3x3(x + 2, 0);
			
			THMatrix3x3 worldCannon0 = localBattery * localCannon0;
			THMatrix3x3 worldCannon1 = localBattery * localCannon1;
			THMatrix3x3 worldBattery = localBattery;
			
			Cannon[0].SetMatrix(worldCannon0);
			Cannon[1].SetMatrix(worldCannon1);
			Battery[0].SetMatrix(worldBattery);
		} 
		
		
		void EnemyMotion6()
		{
			// Battery[0] & Cannon[0],[1]
			THMatrix3x3 rotBattery0 = TH2DMath.GetRotation3x3(-120);        
			THMatrix3x3 traBattery0 = TH2DMath.GetTranslation3x3(-10, 4);   
			THMatrix3x3 localBattery0 = traBattery0 * rotBattery0;
			
			i_shm += 2.0f;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localCannon0 = TH2DMath.GetTranslation3x3(x - 2, 0);
			THMatrix3x3 localCannon1 = TH2DMath.GetTranslation3x3(x + 2, 0);
			
			THMatrix3x3 worldCannon0 = localBattery0 * localCannon0;
			THMatrix3x3 worldCannon1 = localBattery0 * localCannon1;
			THMatrix3x3 worldBattery0 = localBattery0;
			
			Cannon[0].SetMatrix(worldCannon0);
			Cannon[1].SetMatrix(worldCannon1);
			Battery[0].SetMatrix(worldBattery0);
			
			// Battery[1] & Cannon[2],[3]
			THMatrix3x3 rotBattery1 = TH2DMath.GetRotation3x3(120);     
			THMatrix3x3 traBattery1 = TH2DMath.GetTranslation3x3(10, 4);    
			THMatrix3x3 localBattery1 = traBattery1 * rotBattery1;
			
			THMatrix3x3 localCannon2 = TH2DMath.GetTranslation3x3(x - 2, 0);
			THMatrix3x3 localCannon3 = TH2DMath.GetTranslation3x3(x + 2, 0);
			
			THMatrix3x3 worldCannon2 = localBattery1 * localCannon2;
			THMatrix3x3 worldCannon3 = localBattery1 * localCannon3;
			THMatrix3x3 worldBattery1 = localBattery1;
			
			Cannon[2].SetMatrix(worldCannon2);
			Cannon[3].SetMatrix(worldCannon3);
			Battery[1].SetMatrix(worldBattery1);
			
		} 
		
		
		void EShellMotion()  
		{
			var curShooters = new List<THAlphaObject2D>();
			
			foreach (var obj in i_shootObjects)
			{
				if (i_frameCount >= obj.lastShootFrame + obj.shootInterval)
				{
					obj.lastShootFrame = i_frameCount;
					curShooters.Add(obj);
				}
			}
			
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 20 || Mathf.Abs(newShellPos.y) > 12)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (curShooters.Count > 0)
					{
						THAlphaObject2D obj = curShooters[0];
						THMatrix3x3 M = obj.GetMatrix();
						
						Vector2 startPos = M * obj.initShootPosition;
						Vector2 shootDir = (M * obj.initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = obj.shellSpeed;
						
						shell.SetPosition(startPos);
						
						curShooters.RemoveAt(0);
					}
				}
			}
		} 


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec216 pr_instance = new Sec216(6);

		public static Sec216 PRGSI()
		{
			return pr_instance;
		}


		private Sec216(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Cannon_src = CreateObject_Alpha2D("Cannon");
				Battery_src = CreateObject_Alpha2D("Battery");
				EShell_src = CreateObject_Shell2D("EShell2D");
				
				// Instantiate 
				EShell[0] = EShell_src;
				for (int i = 1; i < EShell.Length; i++)
				{
					EShell[i] = CreateObject_InsShell2D(EShell_src.gameObject);
				}
				
				storedCannon[0] = Cannon_src;
				for (int i = 1; i < storedCannon.Length; i++)
				{
					storedCannon[i] = CreateObject_InsAlpha2D(Cannon_src.gameObject);
				}
				
				storedBattery[0] = Battery_src;
				for (int i = 1; i < storedBattery.Length; i++)
				{
					storedBattery[i] = CreateObject_InsAlpha2D(Battery_src.gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_shm = 0.0f;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Cannon_src.gameObject);
				BSAuxFuncs.SetVisible(1, Battery_src.gameObject);
				
				Battery = new THAlphaObject2D[] { Battery_src };
				Cannon = new THAlphaObject2D[] { Cannon_src };
				
				Battery[0].SetMatrix(THMatrix3x3.identity);
				Cannon[0].SetMatrix(THMatrix3x3.identity);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Cannon_src.gameObject);
				Cannon = new THAlphaObject2D[] { Cannon_src };
				Cannon[0].SetMatrix(THMatrix3x3.identity);
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Cannon_src.gameObject);
				BSAuxFuncs.SetVisible(1, Battery_src.gameObject);
				
				Battery = new THAlphaObject2D[] { Battery_src };
				Cannon = new THAlphaObject2D[] { Cannon_src };
				
				Battery[0].SetMatrix(THMatrix3x3.identity);
				Cannon[0].SetMatrix(THMatrix3x3.identity);
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
			else if (MatchCase(4, 5))
			{
				if (MatchCase(5))
				{
					BSAuxFuncs.SetVisible(1, Battery_src.gameObject);
					Battery = new THAlphaObject2D[] { Battery_src };
					Battery[0].SetMatrix(THMatrix3x3.identity);
				}
				
				Cannon = new THAlphaObject2D[2];
				for (int i = 0; i < Cannon.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedCannon[i].gameObject);
					Cannon[i] = storedCannon[i];
					Cannon[i].SetMatrix(THMatrix3x3.identity);
				}
				
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
			}
			else if (MatchCase(6))
			{
				Battery = new THAlphaObject2D[2];
				for (int i = 0; i < Battery.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedBattery[i].gameObject);
					Battery[i] = storedBattery[i];
					Battery[i].SetMatrix(THMatrix3x3.identity);
				}
				
				Cannon = new THAlphaObject2D[Battery.Length * 2];
				for (int i = 0; i < Cannon.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedCannon[i].gameObject);
					Cannon[i] = storedCannon[i];
					Cannon[i].SetMatrix(THMatrix3x3.identity);
				}
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
		}

	}

}
