//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGMeshData02;
using OQJ_THAlphaObject2D;
using OQJ_THShell2D;
using static OQJ_RGAuxFuncs02.RGAuxFuncs02;



namespace OQJ_Sec217
{
	public class Sec217 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THAlphaObject2D Plane;
		
		THAlphaObject2D Star;
		THAlphaObject2D TipStar;
		THAlphaObject2D TipRed;
		THAlphaObject2D Letter;
		THAlphaObject2D TipLetter;
		
		THAlphaObject2D Battery_src;
		THAlphaObject2D[] Battery;
		THAlphaObject2D[] storedBattery = new THAlphaObject2D[2];
		
		THAlphaObject2D Cannon_src;
		THAlphaObject2D[] Cannon;
		THAlphaObject2D[] storedCannon = new THAlphaObject2D[4];
		
		THAlphaObject2D Body;
		THAlphaObject2D HeadCannon;
		
		THAlphaObject2D BSBattery_src;
		THAlphaObject2D[] SideBattery;
		THAlphaObject2D[] storedSideBattery = new THAlphaObject2D[4];
		
		THAlphaObject2D BSCannon_src;
		THAlphaObject2D[] SideCannon;
		THAlphaObject2D[] storedSideCannon = new THAlphaObject2D[8];
		
		THAlphaObject2D Propeller_src;
		THAlphaObject2D[] Propeller = new THAlphaObject2D[3];
		
		THShell2D UShell_src;
		THShell2D[] UShell = new THShell2D[c_numUShell];
		
		THShell2D EShell_src;
		THShell2D[] EShell = new THShell2D[c_numEShell];


		// Instance Variables
		float i_forwardDegree = 0.0f;
		
		THAlphaObject2D[] i_shootObjects = new THAlphaObject2D[0];
		THAlphaObject2D[] i_collisionObjects = new THAlphaObject2D[0];
		
		THAlphaObject2D[] pr_collisionObjects = new THAlphaObject2D[0];  
		
		float[] i_degPropeller = { 0f, 0f, 0f };
		
		System.Random i_random;
		int i_curBSRegion = 0;
		Vector2 i_nextBSPosition;
		
		bool i_HEAD_SHOOT = false;
		float i_degBattleship = 0.0f;
		float i_shm = 0;
		float i_degHeadCannon = 0.0f;
		float i_preHeadCannon = 0.0f;
		float i_toDegHeadCannon = 0.0f;
		float i_fromDegHeadCannon = 0.0f;
		int i_toDegCount = -1;
		
		int i_sailCount = 0;    
		
		
		readonly Vector2 c_attachPos_HeadCannon = new Vector2(6.112f, 0.0f);
		readonly Vector2 c_attachPos_SideCannonL = new Vector2(-0.5f, 0.0f);
		readonly Vector2 c_attachPos_SideCannonR = new Vector2(0.5f, 0.0f);
		
		readonly float[] c_deg_SideBattery = { 170.635f, 188.982f, 351.018f, 369.365f };
		
		readonly Vector2[] c_attachPos_SideBattery =
		{
		new Vector2(3.37771f,  0.84528f),
		new Vector2(-3.42748f,  0.87038f),
		new Vector2(-3.42748f, -0.87038f),
		new Vector2(3.37771f, -0.84528f)
		};
		
		readonly Vector2 c_attachPos_Propeller0 = new Vector2(-5.29429f, 0.0f);
		readonly Vector2 c_attachPos_Propeller1 = new Vector2(0.0f, 0.0f);
		readonly Vector2 c_attachPos_Propeller2 = new Vector2(5.20455f, 0.0f);
		
		const int c_maxSailCount = 1500;
		
		readonly Vector2[] c_BSRegionPoints =
		{
		new Vector2(14.2f, 0.0f),		
		
		new Vector2(14.2f, 5.5f),		
		new Vector2(0.0f, 5.5f),		
		new Vector2(-14.2f, 5.5f),		
		
		new Vector2(-14.2f, 0.0f),		
		
		new Vector2(-14.2f, -5.5f),		
		new Vector2(0.0f, -5.5f),		
		new Vector2(14.2f, -5.5f)		
		};
		
		const int c_numUShell = 30;
		const int c_numEShell = 80;
		
		readonly Color[] c_HitRed = {
		new Color(0.5f, 0.0f, 0.0f),
		new Color(0.6f, 0.2f, 0.2f),
		new Color(0.4f, 0.1f, 0.1f),
		new Color(0.6f, 0.0f, 0.0f),
		new Color(0.45f, 0.12f, 0.23f),
		new Color(0.33f, 0.12f, 0.05f)
		};
		




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Plane.lastShootFrame = 0;
				Plane.shootInterval = 3;
				Plane.initShootDirection = new Vector2(0, 1);
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.shellSpeed = 0.80f;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Plane.lastShootFrame = 0;
				Plane.shootInterval = 3;
				Plane.initShootDirection = new Vector2(0, 1);
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.shellSpeed = 0.80f;
				
				for (int i = 0; i < Cannon.Length; i++)
				{
					Cannon[i].lastShootFrame = 8 + i * 4;
					Cannon[i].shootInterval = 15 + i * 3;
					Cannon[i].initShootDirection = new Vector2(0, 1);
					Cannon[i].initShootPosition = new Vector2(0.0f, 1.75f);
					Cannon[i].shellSpeed = 0.40f;
				}
				
				i_shootObjects = Cannon;
				i_collisionObjects = Cannon;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
			
			EnemyMotion2();
			EShellMotion();
			
			// UShell vs Enemy
			CollisionTest(UShell, i_collisionObjects);
			
			// EShell vs User
			CollisionTest(EShell, new THAlphaObject2D[] { Plane });
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Plane.lastShootFrame = 0;
				Plane.shootInterval = 3;
				Plane.initShootDirection = new Vector2(0, 1);
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.shellSpeed = 0.80f;
				
				Star.lastShootFrame = 5;
				Star.shootInterval = 20;
				Star.initShootPosition = new Vector2(3.04f, 0.89f);
				Star.initShootDirection = Star.initShootPosition.normalized;
				Star.shellSpeed = 0.25f;
				
				Letter.lastShootFrame = 10;
				Letter.shootInterval = 18;
				Letter.initShootPosition = new Vector2(-2.14f, -2.61f);
				Letter.initShootDirection = new Vector2(0.0f, -1.0f);
				Letter.shellSpeed = 0.30f;
				
				i_shootObjects = new THAlphaObject2D[] { Star, Letter };
				i_collisionObjects = i_shootObjects;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
			
			EnemyMotion3();
			EShellMotion();
			
			// UShell vs Enemy
			CollisionTest(UShell, i_collisionObjects);
			// EShell vs User
			CollisionTest(EShell, new THAlphaObject2D[] { Plane });
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				// HeadCannon
				HeadCannon.lastShootFrame = 40;
				HeadCannon.shootInterval = 5;
				HeadCannon.initShootDirection = new Vector2(1, 0);
				HeadCannon.initShootPosition = new Vector2(1.95f, 0.0f);
				HeadCannon.shellSpeed = 0.54f;
				
				// SideCannon
				SideCannon[0].lastShootFrame = 6;
				SideCannon[0].shootInterval = 16;
				SideCannon[0].initShootDirection = new Vector2(0, -1);
				SideCannon[0].initShootPosition = new Vector2(0.0f, -2.45f);
				SideCannon[0].shellSpeed = 0.40f;
				
				SideCannon[1].lastShootFrame = 12;
				SideCannon[1].shootInterval = 16;
				SideCannon[1].initShootDirection = new Vector2(0, -1);
				SideCannon[1].initShootPosition = new Vector2(0.0f, -2.45f);
				SideCannon[1].shellSpeed = 0.40f;
				
				i_shootObjects = new THAlphaObject2D[] { SideCannon[0], SideCannon[1], HeadCannon };
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			EnemyMotion4();
			EShellMotion();
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				// HeadCannon
				HeadCannon.lastShootFrame = 40;
				HeadCannon.shootInterval = 5;
				HeadCannon.initShootDirection = new Vector2(1, 0);
				HeadCannon.initShootPosition = new Vector2(1.95f, 0.0f);
				HeadCannon.shellSpeed = 0.54f;
				
				// SideCannon
				for (int i = 0; i < SideCannon.Length; i++)
				{
					SideCannon[i].lastShootFrame = 6 * i;
					SideCannon[i].shootInterval = 16;
					SideCannon[i].initShootDirection = new Vector2(0, -1);
					SideCannon[i].initShootPosition = new Vector2(0.0f, -2.45f);
					SideCannon[i].shellSpeed = 0.40f;
				}
				
				
				i_shootObjects = new THAlphaObject2D[]
				{
					SideCannon[0], SideCannon[1], SideCannon[2], SideCannon[3],
					SideCannon[4], SideCannon[5], SideCannon[6], SideCannon[7],
					HeadCannon
				};
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			EnemyMotion5();
			EShellMotion();
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				// Plane
				Plane.lastShootFrame = 0;
				Plane.shootInterval = 3;
				Plane.initShootDirection = new Vector2(0, 1);
				Plane.initShootPosition = new Vector2(0.0f, 2.1f);
				Plane.shellSpeed = 0.80f;
				
				// HeadCannon
				HeadCannon.lastShootFrame = 40;
				HeadCannon.shootInterval = 5;
				HeadCannon.initShootDirection = new Vector2(1, 0);
				HeadCannon.initShootPosition = new Vector2(1.95f, 0.0f);
				HeadCannon.shellSpeed = 0.54f;
				
				// SideCannon
				for (int i = 0; i < SideCannon.Length; i++)
				{
					SideCannon[i].lastShootFrame = 6 * i;
					SideCannon[i].shootInterval = 16;
					SideCannon[i].initShootDirection = new Vector2(0, -1);
					SideCannon[i].initShootPosition = new Vector2(0.0f, -2.45f);
					SideCannon[i].shellSpeed = 0.40f;
				}
				
				i_shootObjects = new THAlphaObject2D[]
				{
					SideCannon[0], SideCannon[1], SideCannon[2], SideCannon[3],
					SideCannon[4], SideCannon[5], SideCannon[6], SideCannon[7],
					HeadCannon
				};
				
				i_collisionObjects = new THAlphaObject2D[1] { Body };
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
			
			EnemyMotion6();
			EShellMotion();
			
			// UShell vs Enemy
			CollisionTest(UShell, i_collisionObjects);
			// EShell vs User
			CollisionTest(EShell, new THAlphaObject2D[] { Plane });
		}




		void UserMotion()       
		{                       
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_forwardDegree);
			Vector2 forwardDir = R * new Vector3(0, 1, 1);
			
			Vector2 curP = Plane.GetPosition();
			Vector2 newP = (i_MOVE) ? curP + 0.20f * forwardDir : curP;
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 M = T * R;
			Plane.SetMatrix(M);
		}
		
		
		
		
		
		void EnemyMotion2()
		{
			// Battery[0] & Cannon[0],[1]
			THMatrix3x3 rotBattery0 = TH2DMath.GetRotation3x3(-120);
			THMatrix3x3 traBattery0 = TH2DMath.GetTranslation3x3(-10, 4);
			THMatrix3x3 localBattery0 = traBattery0 * rotBattery0;
			
			i_shm += 2.0f;
			float x = 3.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localCannon0 = TH2DMath.GetTranslation3x3(x - 2, 0);
			THMatrix3x3 localCannon1 = TH2DMath.GetTranslation3x3(x + 2, 0);
			
			THMatrix3x3 worldCannon0 = localBattery0 * localCannon0;
			THMatrix3x3 worldCannon1 = localBattery0 * localCannon1;
			THMatrix3x3 worldBattery0 = localBattery0;
			
			Cannon[0].SetMatrix(worldCannon0);
			Cannon[1].SetMatrix(worldCannon1);
			Battery[0].SetMatrix(worldBattery0);
			
			// Battery[1] & Cannon[2],[3]
			THMatrix3x3 rotBattery1 = TH2DMath.GetRotation3x3(120);
			THMatrix3x3 traBattery1 = TH2DMath.GetTranslation3x3(10, 4);
			THMatrix3x3 localBattery1 = traBattery1 * rotBattery1;
			
			THMatrix3x3 localCannon2 = TH2DMath.GetTranslation3x3(x - 2, 0);
			THMatrix3x3 localCannon3 = TH2DMath.GetTranslation3x3(x + 2, 0);
			
			THMatrix3x3 worldCannon2 = localBattery1 * localCannon2;
			THMatrix3x3 worldCannon3 = localBattery1 * localCannon3;
			THMatrix3x3 worldBattery1 = localBattery1;
			
			Cannon[2].SetMatrix(worldCannon2);
			Cannon[3].SetMatrix(worldCannon3);
			Battery[1].SetMatrix(worldBattery1);
			
		} 
		
		
		void EnemyMotion3()
		{
			i_shm += 2.0f;
			float deg = 15.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad) - 20.0f; // [-35, 5]
			THMatrix3x3 R = TH2DMath.GetRotation3x3(deg);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(-12.5f, -5.5f);
			THMatrix3x3 M = T * R;
			Star.SetMatrix(M);
			
			deg = 20.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad) - 20.0f;  // [-40, 0]
			R = TH2DMath.GetRotation3x3(deg);
			T = TH2DMath.GetTranslation3x3(11, 6);
			M = T * R;
			Letter.SetMatrix(M);
		}
		
		
		void EnemyMotion4()
		{
			i_shm += 2.0f;
			float rdg0 = 30.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 leftTra = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonL);
			THMatrix3x3 rightTra = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonR);
			THMatrix3x3 Rot = TH2DMath.GetRotation3x3(rdg0);
			THMatrix3x3 localSideCannon0 = leftTra * Rot;
			THMatrix3x3 localSideCannon1 = rightTra * Rot;
			
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[0]);
			THMatrix3x3 R = TH2DMath.GetRotation3x3(c_deg_SideBattery[0]);
			THMatrix3x3 localSideBattery0 = T * R;
			
			THMatrix3x3 localBody = THMatrix3x3.identity;
			
			THMatrix3x3 worldSideCannon0 = localBody * localSideBattery0 * localSideCannon0;
			THMatrix3x3 worldSideCannon1 = localBody * localSideBattery0 * localSideCannon1;
			THMatrix3x3 worldSideBattery0 = localBody * localSideBattery0;
			THMatrix3x3 worldBody = localBody;
			
			SideCannon[0].SetMatrix(worldSideCannon0);
			SideCannon[1].SetMatrix(worldSideCannon1);
			SideBattery[0].SetMatrix(worldSideBattery0);
			Body.SetMatrix(worldBody);
			
			
			T = TH2DMath.GetTranslation3x3(c_attachPos_HeadCannon);
			R = Rot;
			THMatrix3x3 localHeadCannon = T * R;
			THMatrix3x3 worldHeadCannon = localBody * localHeadCannon;
			HeadCannon.SetMatrix(worldHeadCannon);
		} 
		
		
		void EnemyMotion5()
		{
			// local matrix : SideCannon
			i_shm += 2.0f;
			float deg0 = 30.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			float deg1 = 30.0f * Mathf.Sin((i_shm + 120) * Mathf.Deg2Rad);
			float deg2 = 30.0f * Mathf.Sin((i_shm + 180) * Mathf.Deg2Rad);
			float deg3 = 30.0f * Mathf.Sin((i_shm + 300) * Mathf.Deg2Rad);
			THMatrix3x3[] Rot =
			{
				TH2DMath.GetRotation3x3(deg0), TH2DMath.GetRotation3x3(deg1),
				TH2DMath.GetRotation3x3(deg2), TH2DMath.GetRotation3x3(deg3)
			};
			THMatrix3x3 leftTra = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonL);
			THMatrix3x3 rightTra = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonR);
			THMatrix3x3[] localSideCannon = new THMatrix3x3[SideCannon.Length];
			for (int idx = 0; idx < SideCannon.Length; idx += 2)
			{
				localSideCannon[idx] = leftTra * Rot[idx / 2];
				localSideCannon[idx + 1] = rightTra * Rot[idx / 2];
			}
			
			// local matrix : SideBattery
			THMatrix3x3 T, R;
			THMatrix3x3[] localSideBattery = new THMatrix3x3[SideBattery.Length];
			for (int i = 0; i < SideBattery.Length; i++)
			{
				R = TH2DMath.GetRotation3x3(c_deg_SideBattery[i]);
				T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[i]);
				localSideBattery[i] = T * R;
			}
			
			// local matrix : HeadCannon
			T = TH2DMath.GetTranslation3x3(c_attachPos_HeadCannon);
			R = Rot[0];
			THMatrix3x3 localHeadCannon = T * R;
			
			// local matrix : Body
			float degBody = 10.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			THMatrix3x3 localBody = TH2DMath.GetRotation3x3(degBody);
			
			
			// world matrix : SideCannon
			for (int i = 0; i < SideCannon.Length; i++)
			{
				THMatrix3x3 worldSideCannon = localBody * localSideBattery[i / 2] * localSideCannon[i];
				SideCannon[i].SetMatrix(worldSideCannon);
			}
			
			// world matrix : SideBattery
			for (int i = 0; i < SideBattery.Length; i++)
			{
				THMatrix3x3 worldSideBattery = localBody * localSideBattery[i];
				SideBattery[i].SetMatrix(worldSideBattery);
			}
			
			// world matrix : HeadCannon
			THMatrix3x3 worldHeadCannon = localBody * localHeadCannon;
			HeadCannon.SetMatrix(worldHeadCannon);
			
			// world matrix : Body
			THMatrix3x3 worldBody = localBody;
			Body.SetMatrix(worldBody);
			
		} 
		
		
		void EnemyMotion6()
		{
			if (i_frameCount % 300 == 0)
			{
				i_HEAD_SHOOT = !i_HEAD_SHOOT;
				if (i_HEAD_SHOOT)
				{
					HeadCannon.shootInterval = 7;
				}
				else
				{
					HeadCannon.shootInterval = 300;
				}
			}
			
			if (i_toDegCount < 0)
			{
				Vector2 wv = Plane.GetPosition() - HeadCannon.GetPosition();
				Vector2 toPln = (wv.magnitude >= 0.01f) ? wv.normalized : Vector2.right;
				Vector2 wDir = Body.GetMatrix() * new Vector3(1, 0, 0);
				float wDot = Vector2.Dot(toPln, wDir);
				float wCrs = wDir.x * toPln.y - wDir.y * toPln.x;  
				
				if (-1.0f < wDot && wDot < 1.0f)
				{
					i_degHeadCannon = Mathf.Acos(wDot) * Mathf.Rad2Deg;
				}
				else 
				{
					i_degHeadCannon = 90.0f - wDot * 90.0f;  
				}
				
				if (wCrs < -0.00001f) { i_degHeadCannon = -i_degHeadCannon; }
				
				if (i_degHeadCannon > 160.0f) { i_degHeadCannon = 160f; }
				else if (i_degHeadCannon < -160.0f) { i_degHeadCannon = -160f; }
				
				if (Mathf.Abs(i_degHeadCannon - i_preHeadCannon) > 180.0f)
				{
					i_toDegHeadCannon = i_degHeadCannon;
					i_fromDegHeadCannon = i_preHeadCannon;
					i_toDegCount = 0;
				}
			}
			
			if (i_toDegCount >= 0)
			{
				const int wT = 40;
				i_toDegCount++;
				i_degHeadCannon = i_fromDegHeadCannon + (i_toDegHeadCannon - i_fromDegHeadCannon) * i_toDegCount / (float)wT;
				
				if (i_toDegCount == wT) { i_toDegCount = -1; }
			}
			
			i_preHeadCannon = i_degHeadCannon;
			
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degHeadCannon);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(c_attachPos_HeadCannon);
			THMatrix3x3 localHeadCannon = T * R;
			
			
			i_shm += 1.20f;
			float rdg0 = 30.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			float rdg1 = 30.0f * Mathf.Sin((i_shm + 120) * Mathf.Deg2Rad);
			float rdg2 = 30.0f * Mathf.Sin((i_shm + 180) * Mathf.Deg2Rad);
			float rdg3 = 30.0f * Mathf.Sin((i_shm + 300) * Mathf.Deg2Rad);
			THMatrix3x3[] wR =
			{
				TH2DMath.GetRotation3x3(rdg0), TH2DMath.GetRotation3x3(rdg1),
				TH2DMath.GetRotation3x3(rdg2), TH2DMath.GetRotation3x3(rdg3)
			};
			THMatrix3x3 TL = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonL);
			THMatrix3x3 TR = TH2DMath.GetTranslation3x3(c_attachPos_SideCannonR);
			THMatrix3x3[] localCannon = new THMatrix3x3[SideCannon.Length];
			for (int idx = 0; idx < SideCannon.Length; idx += 2)
			{
				localCannon[idx] = TL * wR[idx / 2];
				localCannon[idx + 1] = TR * wR[idx / 2];
			}
			
			
			THMatrix3x3[] localBattery = new THMatrix3x3[SideBattery.Length];
			R = TH2DMath.GetRotation3x3((360f - 189.365f));
			T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[0]);
			localBattery[0] = T * R;
			
			R = TH2DMath.GetRotation3x3((360f - 171.018f));
			T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[1]);
			localBattery[1] = T * R;
			
			R = TH2DMath.GetRotation3x3((360f - 8.982f));
			T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[2]);
			localBattery[2] = T * R;
			
			R = TH2DMath.GetRotation3x3((360f + 9.365f));
			T = TH2DMath.GetTranslation3x3(c_attachPos_SideBattery[3]);
			localBattery[3] = T * R;
			
			
			i_degPropeller[0] += 4.0f;
			i_degPropeller[1] += 2.4f;
			i_degPropeller[2] -= 5.3f;
			THMatrix3x3[] localPropeller =
			{
				TH2DMath.GetTranslation3x3(c_attachPos_Propeller0) *
				TH2DMath.GetRotation3x3(i_degPropeller[0]),
				TH2DMath.GetTranslation3x3(c_attachPos_Propeller1) *
				TH2DMath.GetRotation3x3(i_degPropeller[1]),
				TH2DMath.GetTranslation3x3(c_attachPos_Propeller2) *
				TH2DMath.GetRotation3x3(i_degPropeller[2]),
			};
			
			
			Vector2 curPos = Body.GetPosition();
			R = TH2DMath.GetRotation3x3(i_degBattleship);
			Vector2 ua = new Vector2(R.m00, R.m10);                 
			Vector2 ub = (i_nextBSPosition - curPos).normalized;    
			float dot = Vector2.Dot(ua, ub);
			float crs = ua.x * ub.y - ua.y * ub.x;
			if (Mathf.Abs(1.0f - dot) >= 0.01f)  
			{
				i_degBattleship += ((crs > 0.0f) ? 0.3f : -0.3f);
			}
			
			
			
			R = TH2DMath.GetRotation3x3(i_degBattleship);
			Vector2 forwardDir = new Vector2(R.m00, R.m10);
			Vector2 newPos = curPos + 0.04f * forwardDir;
			T = TH2DMath.GetTranslation3x3(newPos);
			THMatrix3x3 localBody = T * R;
			
			i_sailCount++;
			if ((newPos - i_nextBSPosition).magnitude < 2.5f || i_sailCount >= c_maxSailCount)
			{
				int nextBSRegion = (i_curBSRegion + 3 + i_random.Next(0, 3)) % 8;  
				i_sailCount = 0;
				
				int iS = nextBSRegion;
				int iE = (nextBSRegion + 1) % 8;
				float t = (float)i_random.NextDouble();     
				i_nextBSPosition = (1.0f - t) * c_BSRegionPoints[iS] + t * c_BSRegionPoints[iE];
				
				i_curBSRegion = nextBSRegion;
			}
			
			
			THMatrix3x3 worldHeadCannon = localBody * localHeadCannon;
			HeadCannon.SetMatrix(worldHeadCannon);
			
			for (int idx = 0; idx < SideCannon.Length; idx += 2)
			{
				THMatrix3x3 worldCannon = localBody * localBattery[idx / 2] * localCannon[idx];
				SideCannon[idx].SetMatrix(worldCannon);
				worldCannon = localBody * localBattery[idx / 2] * localCannon[idx + 1];
				SideCannon[idx + 1].SetMatrix(worldCannon);
			}
			
			for (int i = 0; i < SideBattery.Length; i++)
			{
				THMatrix3x3 worldBattery = localBody * localBattery[i];
				SideBattery[i].SetMatrix(worldBattery);
			}
			
			for (int i = 0; i < Propeller.Length; i++)
			{
				THMatrix3x3 worldPropeller = localBody * localPropeller[i];
				Propeller[i].SetMatrix(worldPropeller);
			}
			
			THMatrix3x3 worldBody = localBody;
			Body.SetMatrix(worldBody);
			
		} 
		
		
		
		void UShellMotion()   
		{
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Plane.lastShootFrame + Plane.shootInterval)
				{
					Plane.lastShootFrame = i_frameCount;
					bShoot = true;
				}
			}
			
			foreach (var shell in UShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 30 || Mathf.Abs(newShellPos.y) > 18)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (bShoot)
					{
						THMatrix3x3 M = Plane.GetMatrix();
						
						Vector2 startPos = M * Plane.initShootPosition;
						Vector2 shootDir = (M * Plane.initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Plane.shellSpeed;
						
						shell.SetPosition(startPos);
						
						bShoot = false;
					}
				}
			}
		} 
		
		
		void EShellMotion()  
		{
			var curShooters = new List<THAlphaObject2D>();
			
			foreach (var obj in i_shootObjects)
			{
				if (i_frameCount >= obj.lastShootFrame + obj.shootInterval)
				{
					obj.lastShootFrame = i_frameCount;
					curShooters.Add(obj);
				}
			}
			
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					Vector2 newShellPos = shell.GetPosition() + shell.speed * shell.direction;
					if (Mathf.Abs(newShellPos.x) > 30 || Mathf.Abs(newShellPos.y) > 18)
					{
						shell.active = false;
					}
					
					shell.SetPosition(newShellPos);
				}
				else
				{
					if (curShooters.Count > 0)
					{
						THAlphaObject2D obj = curShooters[0];
						THMatrix3x3 M = obj.GetMatrix();
						
						Vector2 startPos = M * obj.initShootPosition;
						Vector2 shootDir = (M * obj.initShootDirection).normalized;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = obj.shellSpeed;
						
						shell.SetPosition(startPos);
						
						curShooters.RemoveAt(0);
					}
				}
			}
		} 
		
		
		
		void CollisionTest(THShell2D[] shell_array, THAlphaObject2D[] target_array)
		{
			foreach (var shell in shell_array)
			{
				if (!shell.active) { continue; }
				
				Vector2 shell_pos = shell.GetPosition();
				
				foreach (var target in target_array)
				{
					bool bColl = CollisionTest_AllTypes(target, shell_pos);
					if (bColl)
					{
						Hit(target);
						
						shell.SetPosition(-1000, 0);
						shell.active = false;
					}
				} 
			} 
		}
		
		
		bool CollisionTest_AllTypes(THAlphaObject2D obj, Vector2 shell_pos)
		{
			THMatrix3x3 mtx;
			Vector2 P;
			
			bool bColl = false;
			switch (obj.GetType())
			{
				case THAlphaObject2D.DISK:
				Vector2 C = obj.GetPosition();
				float r = obj.GetDiskScale() * obj.initRadius;
				bColl = CollisionTest_Point_Disk(shell_pos, C, r);
				break;
				
				case THAlphaObject2D.RECT:
				mtx = obj.GetMatrix();
				P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bColl = CollisionTest_Point_AABB(P, obj.initRectBounds);
				break;
				
				case THAlphaObject2D.MULTI_RECT:
				mtx = obj.GetMatrix();
				P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bColl = CollisionTest_Point_MultiAABB(P, obj.initRectBounds);
				break;
				
				case THAlphaObject2D.TRIMESH:
				mtx = obj.GetMatrix();
				P = mtx.inverse * TH2DMath.ToVector3(shell_pos);
				bColl = CollisionTest_Point_TriMesh(P, obj.initTriangleCoords);
				break;
			}
			
			return bColl;
		}
		
		
		
		
		void Hit(THAlphaObject2D obj)
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = 6;
				
				MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
				mre.materials = obj.hitMaterials;
			}
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(2, 3, 6))
			{
				foreach (var o in pr_collisionObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
							mre.materials = o.originalMaterials;
						}
					}
				}
				
				if (MatchCase(3))
				{
					TipStar.SetMatrix(Star.GetMatrix());
					TipLetter.SetMatrix(Letter.GetMatrix());
					
					TipRed.SetMatrix(Star.GetMatrix());
				}
			}
		}


		static Sec217 pr_instance = new Sec217(6);

		public static Sec217 PRGSI()
		{
			return pr_instance;
		}


		private Sec217(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Plane = CreateObject_Alpha2D("CombatPlane");
				Cannon_src = CreateObject_Alpha2D("Cannon");
				Battery_src = CreateObject_Alpha2D("Battery");
				Star = CreateObject_Alpha2D("Star");
				TipStar = CreateObject_Alpha2D("TipStar");
				TipRed = CreateObject_Alpha2D("TipRed");
				Letter = CreateObject_Alpha2D("Letter");
				TipLetter = CreateObject_Alpha2D("TipLetter");
				Body = CreateObject_Alpha2D("Battleship_Body");
				HeadCannon = CreateObject_Alpha2D("Battleship_HeadCannon");
				BSCannon_src = CreateObject_Alpha2D("Battleship_Cannon");
				BSBattery_src = CreateObject_Alpha2D("Battleship_Battery");
				Propeller_src = CreateObject_Alpha2D("Battleship_Propeller");
				UShell_src = CreateObject_Shell2D("UShell2D");
				EShell_src = CreateObject_Shell2D("EShell2D");
				
				//// Instantiate 
				EShell[0] = EShell_src;
				for (int i = 1; i < EShell.Length; i++)
				{
					EShell[i] = CreateObject_InsShell2D(EShell_src.gameObject);
				}
				
				UShell[0] = UShell_src;
				for (int i = 1; i < UShell.Length; i++)
				{
					UShell[i] = CreateObject_InsShell2D(UShell_src.gameObject);
				}
				
				storedCannon[0] = Cannon_src;
				for (int i = 1; i < storedCannon.Length; i++)
				{
					storedCannon[i] = CreateObject_InsAlpha2D(Cannon_src.gameObject);
				}
				
				storedBattery[0] = Battery_src;
				for (int i = 1; i < storedBattery.Length; i++)
				{
					storedBattery[i] = CreateObject_InsAlpha2D(Battery_src.gameObject);
				}
				
				storedSideCannon[0] = BSCannon_src;
				for (int i = 1; i < storedSideCannon.Length; i++)
				{
					storedSideCannon[i] = CreateObject_InsAlpha2D(BSCannon_src.gameObject);
				}
				
				storedSideBattery[0] = BSBattery_src;
				for (int i = 1; i < storedSideBattery.Length; i++)
				{
					storedSideBattery[i] = CreateObject_InsAlpha2D(BSBattery_src.gameObject);
				}
				
				Propeller[0] = Propeller_src;
				for (int i = 0; i < Propeller.Length; i++)
				{
					Propeller[i] = CreateObject_InsAlpha2D(Propeller_src.gameObject);
				}
				
				
				//// Set verts data 
				Vector2[] verts = RGMeshData02.GetVertsData(RGMeshData02.c_PLAIN);
				Plane.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_CANNON);
				for (int i = 0; i < storedCannon.Length; i++)
				{
					storedCannon[i].SetData(THAlphaObject2D.MULTI_RECT, verts);
				}
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_LOOSE_STAR);
				Star.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_LOOSE_LETTER);
				Letter.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_BATTLESHIP_BODY);
				Body.SetData(THAlphaObject2D.TRIMESH, verts);
				
				
				//// Materials
				Shader standard = Shader.Find("Standard");
				
				// Plane
				MeshRenderer meshRndr = Plane.gameObject.GetComponent<MeshRenderer>();
				Plane.originalMaterials = meshRndr.materials;
				
				Material[] wMtrls = new Material[meshRndr.materials.Length];
				for (int i = 0; i < meshRndr.materials.Length; i++)
				{
					wMtrls[i] = new Material(standard);
					wMtrls[i].name = "CollRed" + i;
					wMtrls[i].SetColor("_Color", c_HitRed[i]);
				}
				Plane.hitMaterials = wMtrls;
				
				// Cannon (stored)
				meshRndr = storedCannon[0].gameObject.GetComponent<MeshRenderer>();
				Material[] hitMtrls = new Material[meshRndr.materials.Length];
				int[] widx = new int[hitMtrls.Length];
				widx[0] = 1; widx[1] = 0; widx[2] = 2;
				for (int i = 0; i < hitMtrls.Length; i++)
				{
					hitMtrls[i] = new Material(standard);
					hitMtrls[i].name = "CollRed" + i;
					hitMtrls[i].SetColor("_Color", c_HitRed[widx[i]]);
				}
				
				for (int i = 0; i < storedCannon.Length; i++)
				{
					meshRndr = storedCannon[i].gameObject.GetComponent<MeshRenderer>();
					storedCannon[i].originalMaterials = meshRndr.materials;
					storedCannon[i].hitMaterials = hitMtrls;
				}
				
				// Star & Letter
				hitMtrls = new Material[1];
				hitMtrls[0] = new Material(standard);
				hitMtrls[0].name = "CollRed" + 1;
				hitMtrls[0].SetColor("_Color", c_HitRed[1]);
				
				THAlphaObject2D[] wObjs = { Star, Letter };
				for (int i = 0; i < wObjs.Length; i++)
				{
					meshRndr = wObjs[i].gameObject.GetComponent<MeshRenderer>();
					wObjs[i].originalMaterials = meshRndr.materials;
					wObjs[i].hitMaterials = hitMtrls;
				}
				
				// Battleship Body
				meshRndr = Body.gameObject.GetComponent<MeshRenderer>();
				Body.originalMaterials = meshRndr.materials;
				
				hitMtrls = new Material[meshRndr.materials.Length];
				for (int i = 0; i < hitMtrls.Length; i++)
				{
					hitMtrls[i] = new Material(standard);
					hitMtrls[i].name = "CollRed" + i;
					hitMtrls[i].SetColor("_Color", c_HitRed[5 - i]);
				}
				Body.hitMaterials = hitMtrls;

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(12.6f, 400);
			
			SetCameraPosition_2D(0, 0);
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			i_forwardDegree = 0.0f;
			i_shm = 0.0f;
			
			i_HEAD_SHOOT = false;
			i_degBattleship = 0.0f;
			i_degHeadCannon = 0.0f;
			i_preHeadCannon = 0.0f;
			i_toDegHeadCannon = 0.0f;
			i_fromDegHeadCannon = 0.0f;
			i_degPropeller = new float[] { 0f, 0f, 0f };
			i_toDegCount = -1;
			i_sailCount = 0;
			
			
			foreach (var o in pr_collisionObjects)
			{
				if (o.collCount > 0)
				{
					o.collCount = 0;
					MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
					mre.materials = o.originalMaterials;
				}
			}
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				Plane.SetMatrix(THMatrix3x3.identity);
				
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetMatrix(THMatrix3x3.identity);
					UShell[i].active = false;
					UShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				Plane.SetMatrix(THMatrix3x3.identity);
				
				Battery = new THAlphaObject2D[2];
				for (int i = 0; i < Battery.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedBattery[i].gameObject);
					Battery[i] = storedBattery[i];
					Battery[i].SetMatrix(THMatrix3x3.identity);
				}
				
				Cannon = new THAlphaObject2D[Battery.Length * 2];
				for (int i = 0; i < Cannon.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedCannon[i].gameObject);
					Cannon[i] = storedCannon[i];
					Cannon[i].SetMatrix(THMatrix3x3.identity);
				}
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].active = false;
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetMatrix(THMatrix3x3.identity);
					UShell[i].active = false;
					UShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				pr_collisionObjects = new THAlphaObject2D[]
				{ Cannon[0], Cannon[1], Cannon[2], Cannon[3], Plane };
				
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Star.gameObject);
				BSAuxFuncs.SetVisible(1, TipStar.gameObject);
				BSAuxFuncs.SetVisible(1, Letter.gameObject);
				BSAuxFuncs.SetVisible(1, TipLetter.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				Star.SetMatrix(THMatrix3x3.identity);
				TipStar.SetMatrix(THMatrix3x3.identity);
				Letter.SetMatrix(THMatrix3x3.identity);
				TipLetter.SetMatrix(THMatrix3x3.identity);
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].active = false;
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetMatrix(THMatrix3x3.identity);
					UShell[i].active = false;
					UShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				pr_collisionObjects = new THAlphaObject2D[] { Star, Letter, Plane };
				
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, HeadCannon.gameObject);
				
				Body.SetMatrix(THMatrix3x3.identity);
				HeadCannon.SetMatrix(THMatrix3x3.identity);
				
				int numSB = (THSettings.CODE_NR == 4) ? 1 : 4;
				
				
				SideBattery = new THAlphaObject2D[numSB];
				for (int i = 0; i < SideBattery.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedSideBattery[i].gameObject);
					SideBattery[i] = storedSideBattery[i];
					SideBattery[i].SetMatrix(THMatrix3x3.identity);
				}
				
				
				SideCannon = new THAlphaObject2D[SideBattery.Length * 2];
				for (int i = 0; i < SideCannon.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedSideCannon[i].gameObject);
					SideCannon[i] = storedSideCannon[i];
					SideCannon[i].SetMatrix(THMatrix3x3.identity);
				}
				
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].active = false;
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, HeadCannon.gameObject);
				
				Plane.SetMatrix(THMatrix3x3.identity);
				Body.SetMatrix(THMatrix3x3.identity);
				HeadCannon.SetMatrix(THMatrix3x3.identity);
				
				
				SideBattery = new THAlphaObject2D[4];
				for (int i = 0; i < SideBattery.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedSideBattery[i].gameObject);
					SideBattery[i] = storedSideBattery[i];
					SideBattery[i].SetMatrix(THMatrix3x3.identity);
				}
				
				
				SideCannon = new THAlphaObject2D[SideBattery.Length * 2];
				for (int i = 0; i < SideCannon.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, storedSideCannon[i].gameObject);
					SideCannon[i] = storedSideCannon[i];
					SideCannon[i].SetMatrix(THMatrix3x3.identity);
				}
				
				
				foreach (var prop in Propeller)
				{
					BSAuxFuncs.SetVisible(1, prop.gameObject);
					prop.SetMatrix(THMatrix3x3.identity);
				}
				
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetMatrix(THMatrix3x3.identity);
					EShell[i].active = false;
					EShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetMatrix(THMatrix3x3.identity);
					UShell[i].active = false;
					UShell[i].transform.localPosition = new Vector3(-1000, 0, 0);
				}
				
				pr_collisionObjects = new THAlphaObject2D[2] { Body, Plane };
				
				
				
				i_random = new System.Random();
				
				i_degBattleship = 180;
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degBattleship);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(20.0f, 0.0f);
				Body.SetMatrix(T * R);
				
				i_curBSRegion = 0;
				int nextBSRegion = (i_curBSRegion + 3 + i_random.Next(0, 3)) % 8;  
				
				int iS = nextBSRegion;
				int iE = (nextBSRegion + 1) % 8;
				float t = (float)i_random.NextDouble();     
				i_nextBSPosition = t * c_BSRegionPoints[iS] + (1.0f - t) * c_BSRegionPoints[iE];
				
				i_curBSRegion = nextBSRegion;
			}
		}

	}

}
