//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec220
{
	public class Sec220 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Green;
		THObject2D Blue;


		// Instance Variables
		float i_rectW = 6.0f;
		float i_rectH = 2.0f;
		float i_diskRad = 1.0f;
		
		int i_degGreen;
		bool pr_isHitColor;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				i_degGreen = 0;
				i_rectW = 6.0f;
				i_rectH = 2.0f;
				i_diskRad = 1.0f;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.R))
			{
				i_degGreen = (THUtil.IsShiftDown()) ? i_degGreen - 1 : i_degGreen + 1;
				THMatrix3x3 M = TH2DMath.GetTranslation3x3(0, 5) * TH2DMath.GetRotation3x3(i_degGreen);
				Green.SetMatrix(M);
			}
			
			Vector2 posB = Blue.GetPosition();
			
			float speed = 0.05f;
			if (Input.GetKey(KeyCode.H))
			{
				posB.x -= speed;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				posB.x += speed;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				posB.y -= speed;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				posB.y += speed;
			}
			
			Blue.SetPosition(posB);
			
			Vector2 posG = Green.GetPosition();
			if (i_degGreen % 90 == 0)
			{
				bool bColl;
				if (i_degGreen % 180 == 0)
				{
					bColl = CollisionTest_Disk_AABB(posB, posG, i_diskRad, i_rectW, i_rectH);
				}
				else
				{
					bColl = CollisionTest_Disk_AABB(posB, posG, i_diskRad, i_rectH, i_rectW);
				}
				
				Hit(bColl);
			}
			else
			{
				THMatrix3x3 M = Green.GetMatrix();
				Vector2 ui = M.GetColumn(0).normalized;
				Vector2 uj = M.GetColumn(1).normalized;
				bool bColl = CollisionTest_Disk_OBB(posB, posG, ui, uj, i_diskRad, i_rectW, i_rectH);
				Hit(bColl);
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				i_rectW = 6.0f;
				i_rectH = 2.0f;
				i_diskRad = 1.0f;
				i_MOVE = true;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			if (i_MOVE)
			{
				GreenMotion(1);
			}
			
			Vector2 posB = Blue.GetPosition();
			
			float speed = 0.10f;
			if (Input.GetKey(KeyCode.H))
			{
				posB.x -= speed;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				posB.x += speed;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				posB.y -= speed;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				posB.y += speed;
			}
			
			Blue.SetPosition(posB);
			
			Vector2 posG = Green.GetPosition();
			THMatrix3x3 M = Green.GetMatrix();
			Vector2 ui = M.GetColumn(0).normalized;  // 横方向の単位ベクトル
			Vector2 uj = M.GetColumn(1).normalized;  // 縦方向の単位ベクトル
			bool bColl = CollisionTest_Disk_OBB(posB, posG, ui, uj, i_diskRad, i_rectW, i_rectH);
			Hit(bColl);
		}




		// 長方形が2D空間内の任意の位置にある場合
		bool CollisionTest_Disk_AABB(Vector2 pos_disk, Vector2 pos_rect, float r, float w, float h)
		{
			return CollisionTest_Disk_AABB(pos_disk - pos_rect, r, w, h);
		}
		
		
		// 長方形の中心が原点にある場合
		bool CollisionTest_Disk_AABB(Vector2 C, float r, float w, float h)
		{
			// 第1象限へ対称移動
			C.x = Mathf.Abs(C.x);
			C.y = Mathf.Abs(C.y);
			
			float halfW = w * 0.5f;
			float halfH = h * 0.5f;
			if ((C.x >= halfW + r) || (C.y >= halfH + r))
			{
				return false;
			}
			
			float ax = C.x - halfW;
			float ay = C.y - halfH;
			if (ax >= 0.0f && ay >= 0.0f)
			{
				if (ax * ax + ay * ay >= r * r)
				{
					return false;
				}
			}
			
			return true;
		}
		
		
		
		bool CollisionTest_Disk_OBB(Vector2 pos_disk, Vector2 pos_rect, Vector2 ui, Vector2 uj,
		float r, float w, float h)
		{
			Vector2 v = pos_disk - pos_rect;
			float s = Vector2.Dot(v, ui);
			float t = Vector2.Dot(v, uj);
			
			return CollisionTest_Disk_AABB(new Vector2(s, t), r, w, h);
		}
		
		
		
		void GreenMotion(int id)
		{
			if (i_degGreen == 0 && id == 1)
			{
				Blue.SetPosition(0, 8);
			}
			
			if (id == 0)
			{
				i_degGreen++;
				float x = 6.0f * Mathf.Sin(i_degGreen * Mathf.Deg2Rad);
				THMatrix3x3 M = TH2DMath.GetTranslation3x3(x, 5) * TH2DMath.GetRotation3x3(30);
				Green.SetMatrix(M);
			}
			else if (id == 1)
			{
				i_degGreen++;
				THMatrix3x3 M = TH2DMath.GetRotation3x3(i_degGreen) * TH2DMath.GetTranslation3x3(3, 0);
				Green.SetMatrix(M);
			}
		}
		
		
		
		void Hit(bool coll)
		{
			if (coll)
			{
				if (!pr_isHitColor)
				{
					var mre = Green.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.8f, 0.2f, 0.1f, 1.0f));
					
					mre = Blue.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.7f, 0.1f, 0.1f, 0.6f));
				}
				
				pr_isHitColor = true;
			}
			else
			{
				if (pr_isHitColor)
				{
					var mre = Green.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.2703f, 0.7075f, 0.2703f, 1.0f));
					
					mre = Blue.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.2078f, 0.4705f, 0.6980f, 0.6f));
				}
				
				pr_isHitColor = false;
			}
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				Vector3 pos = Blue.GetPosition();
				pos.z = -0.001f;
				Blue.transform.localPosition = pos;
			}
		}


		static Sec220 pr_instance = new Sec220(2);

		public static Sec220 PRGSI()
		{
			return pr_instance;
		}


		private Sec220(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Green = CreateObject_Obj2D("RectG");
				Blue = CreateObject_Obj2D("UnitDiskB");
				
				var trpMtrl = BSAuxFuncs.GetSemiTransparentMaterial(0.2078f, 0.4705f, 0.6980f, 0.6f);
				var mre = Blue.gameObject.GetComponent<MeshRenderer>();
				mre.material = trpMtrl;

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				BSAuxFuncs.SetVisible(1, Green.gameObject);
				
				Blue.SetMatrix(THMatrix3x3.identity);
				Green.SetMatrix(THMatrix3x3.identity);
				
				Green.SetPosition(0, 5);
				
				i_degGreen = 0;
				Hit(false);
			}
		}

	}

}
