//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using CellLoc = OQJ_Main02.CellLoc;



namespace OQJ_Sec222
{
	public class Sec222 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Square;
		THObject2D Cargo_src;
		THObject2D[] CargoArray;
		THObject2D[] Cargo;
		THObject2D Block_src;
		THObject2D[] Block;
		THObject2D Floor_src;
		THObject2D[] Floor;
		THObject2D Mark_src;
		THObject2D[] Mark;


		// Instance Variables
		int[,] i_cellInfo;
		
		int i_moveCount;
		Vector2 i_moveVtr;
		float i_moveSpeed;
		int i_pushCargoIndex;
		int i_degSquare;
		
		bool[] i_CargoState;    // BLUE or GREEN
		
		readonly bool c_BLUE = true;
		readonly bool c_GREEN = false;
		readonly float c_cellLen = 2.0f;
		readonly int c_maxMoveCount = 20;
		
		int pr_halfStageSize;
		Material[] pr_mtrlGreen;
		Material[] pr_mtrlBlue;




		void Code1()
		{
			if (i_MOVE) // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(newSqrPos);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKeyDown(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKeyDown(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKeyDown(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKeyDown(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
		}


		void Code2()
		{
			if (i_MOVE) // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(newSqrPos);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKeyDown(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKeyDown(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKeyDown(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKeyDown(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			
			Vector2 posSqr = Square.GetPosition();
			bool CANCEL_MOVE = CollisionTest_vs_Block(posSqr, i_moveVtr);
			if (CANCEL_MOVE)
			{
				i_MOVE = false;
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(posSqr);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
			}
		}


		void Code3()
		{
			if (i_MOVE) // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(newSqrPos);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
				
				if (i_pushCargoIndex >= 0)
				{
					Vector2 newCrgPos = Cargo[i_pushCargoIndex].GetPosition() + i_moveVtr;
					Cargo[i_pushCargoIndex].SetPosition(newCrgPos);
				}
				
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKeyDown(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKeyDown(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKeyDown(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKeyDown(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			// Square vs Cargo
			Vector2 posSqr = Square.GetPosition();
			i_pushCargoIndex = CollisionTest_vs_Cargo(posSqr, i_moveVtr);
		}


		void Code4()
		{
			if (i_MOVE)  // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(newSqrPos);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
				
				if (i_pushCargoIndex >= 0)
				{
					Vector2 newCrgPos = Cargo[i_pushCargoIndex].GetPosition() + i_moveVtr;
					Cargo[i_pushCargoIndex].SetPosition(newCrgPos);
				}
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKeyDown(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKeyDown(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKeyDown(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKeyDown(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			// Square vs Cargo
			Vector2 posSqr = Square.GetPosition();
			i_pushCargoIndex = CollisionTest_vs_Cargo(posSqr, i_moveVtr);
			
			bool CANCEL_MOVE = false;
			if (i_pushCargoIndex < 0)  // Squareが単独で移動
			{
				// Square vs Block
				CANCEL_MOVE = CollisionTest_vs_Block(posSqr, i_moveVtr);
			}
			else  // SquareがCargoを押しながら移動
			{
				Vector2 posCrg = Cargo[i_pushCargoIndex].GetPosition();
				
				// Cargo vs Block
				CANCEL_MOVE = CollisionTest_vs_Block(posCrg, i_moveVtr);
				
				if (!CANCEL_MOVE)
				{
					// Cargo vs Cargo
					int collIdx = CollisionTest_vs_Cargo(posCrg, i_moveVtr, i_pushCargoIndex);
					if (collIdx >= 0)
					{
						CANCEL_MOVE = true;
					}
				}
			}
			
			
			if (CANCEL_MOVE)
			{
				i_MOVE = false;
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(posSqr);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				SetupStage(1);
				
				i_INITIALIZED = true;
			}
			
			if (i_MOVE)  // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(newSqrPos);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
				
				if (i_pushCargoIndex >= 0)
				{
					Vector2 newCrgPos = Cargo[i_pushCargoIndex].GetPosition() + i_moveVtr;
					Cargo[i_pushCargoIndex].SetPosition(newCrgPos);
				}
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
					
					// Cargoの色を変更する処理
					if (i_pushCargoIndex >= 0)
					{
						CellLoc loc = GetCellLocation(Cargo[i_pushCargoIndex].GetPosition());
						
						if (i_cellInfo[loc.row, loc.col] == 2) // CargoがMarkに移動した場合
						{
							if (i_CargoState[i_pushCargoIndex] == c_GREEN)
							{
								i_CargoState[i_pushCargoIndex] = c_BLUE;
								SetCargoColor(i_pushCargoIndex, c_BLUE);
							}
						}
						else // CargoがFloorに移動した場合
						{
							if (i_CargoState[i_pushCargoIndex] == c_BLUE)
							{
								i_CargoState[i_pushCargoIndex] = c_GREEN;
								SetCargoColor(i_pushCargoIndex, c_GREEN);
							}
						}
					}
				}
				
				return;
			}
			
			
			if (Input.GetKeyDown(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKeyDown(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKeyDown(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKeyDown(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			// Square vs Cargo
			Vector2 posSqr = Square.GetPosition();
			i_pushCargoIndex = CollisionTest_vs_Cargo(posSqr, i_moveVtr);
			
			bool CANCEL_MOVE = false;
			if (i_pushCargoIndex < 0)  // Squareが単独で移動
			{
				// Square vs Block
				CANCEL_MOVE = CollisionTest_vs_Block(posSqr, i_moveVtr);
			}
			else  // SquareがCargoを押しながら移動
			{
				Vector2 posCrg = Cargo[i_pushCargoIndex].GetPosition();
				
				// Cargo vs Block
				CANCEL_MOVE = CollisionTest_vs_Block(posCrg, i_moveVtr);
				
				if (!CANCEL_MOVE)
				{
					// Cargo vs Cargo
					int collIdx = CollisionTest_vs_Cargo(posCrg, i_moveVtr, i_pushCargoIndex);
					if (collIdx >= 0)
					{
						CANCEL_MOVE = true;
					}
				}
			}
			
			
			if (CANCEL_MOVE)
			{
				i_MOVE = false;
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(posSqr);
				THMatrix3x3 M = T * R;
				Square.SetMatrix(M);
			}
		}




		bool CollisionTest_vs_Block(Vector2 pos, Vector2 move_vtr)
		{
			CellLoc nextLoc = GetCellLocation(pos);
			
			float eps = 0.001f;
			if (move_vtr.y > eps)
			{
				nextLoc.row--;
			}
			else if (move_vtr.y < -eps)
			{
				nextLoc.row++;
			}
			else if (move_vtr.x < -eps)
			{
				nextLoc.col--;
			}
			else if (move_vtr.x > eps)
			{
				nextLoc.col++;
			}
			
			
			// 進もうとしているセルがBlockかどうか
			if (i_cellInfo[nextLoc.row, nextLoc.col] == 0)
			{
				return true;
			}
			
			return false;
		}
		
		
		int CollisionTest_vs_Cargo(Vector2 pos, Vector2 move_vtr, int crg_idx = -1)
		{
			CellLoc nextLoc = GetCellLocation(pos);
			
			float eps = 0.001f;
			if (move_vtr.y > eps)
			{
				nextLoc.row--;
			}
			else if (move_vtr.y < -eps)
			{
				nextLoc.row++;
			}
			else if (move_vtr.x < -eps)
			{
				nextLoc.col--;
			}
			else if (move_vtr.x > eps)
			{
				nextLoc.col++;
			}
			
			
			for (int i = 0; i < Cargo.Length; i++)
			{
				if (i == crg_idx) { continue; }
				
				CellLoc crgLoc = GetCellLocation(Cargo[i].GetPosition());
				if (nextLoc.row == crgLoc.row && nextLoc.col == crgLoc.col)
				{
					return i;
				}
			}
			
			return -1;
		}
		
		
		CellLoc GetCellLocation(Vector2 pos)
		{
			
			
			int row = pr_halfStageSize - Mathf.RoundToInt(pos.y / c_cellLen);
			int col = pr_halfStageSize + Mathf.RoundToInt(pos.x / c_cellLen);
			
			return new CellLoc(row, col);
		}
		
		
		
		void SetStageData(int[,] cell_info)
		{
			i_cellInfo = new int[cell_info.GetLength(0), cell_info.GetLength(1)];
			
			pr_halfStageSize = cell_info.GetLength(0) / 2;     
			
			int idxB = 0;   
			int idxF = 0;   
			int idxM = 0;   
			for (int i = 0; i < cell_info.GetLength(0); i++)
			{
				for (int j = 0; j < cell_info.GetLength(1); j++)
				{
					int val = cell_info[i, j];
					
					Vector2 pos = GetCellCenterPosition(i, j);  
					if (val == 0)      
					{
						BSAuxFuncs.SetVisible(1, Block[idxB].gameObject);
						Block[idxB].SetPosition(pos);
						idxB++;
					}
					else if (val == 1) 
					{
						BSAuxFuncs.SetVisible(1, Floor[idxF].gameObject);
						Floor[idxF].SetPosition(pos);
						idxF++;
					}
					else if (val == 2) 
					{
						BSAuxFuncs.SetVisible(1, Mark[idxM].gameObject);
						Mark[idxM].SetPosition(pos);
						idxM++;
					}
					
					i_cellInfo[i, j] = val;
				} 
				
			} 
			
		} 
		
		
		void SetSquareStartLocation(CellLoc loc, int dir = 0) 
		{
			i_degSquare = 360 - 90 * dir;
			THMatrix3x3 R = TH2DMath.GetRotation3x3(i_degSquare);
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(GetCellCenterPosition(loc));
			Square.SetMatrix(T * R);
		}
		
		
		void AddCargo(CellLoc[] locs)
		{
			Cargo = new THObject2D[locs.Length];
			
			for (int i = 0; i < Cargo.Length; i++)
			{
				Vector2 pos = GetCellCenterPosition(locs[i]);
				BSAuxFuncs.SetVisible(1, CargoArray[i].gameObject);
				CargoArray[i].SetPosition(pos);
				Cargo[i] = CargoArray[i];
				
				var mre = Cargo[i].gameObject.GetComponent<MeshRenderer>();
				mre.materials = pr_mtrlGreen;
			}
			
			i_pushCargoIndex = -1;
			i_CargoState = new bool[Cargo.Length];
		}
		
		
		// Loc to Pos	~Pos : center of Cell
		Vector2 GetCellCenterPosition(CellLoc loc)
		{
			float x = (-pr_halfStageSize + loc.col) * c_cellLen;
			float y = (pr_halfStageSize - loc.row) * c_cellLen;
			
			return new Vector2(x, y);
		}
		
		Vector2 GetCellCenterPosition(int row, int col)
		{
			return GetCellCenterPosition(new CellLoc(row, col));
		}
		
		
		void SetCargoColor(int idx, bool isBlue)
		{
			var mre = Cargo[idx].gameObject.GetComponent<MeshRenderer>();
			
			if (isBlue)
			{
				mre.materials = pr_mtrlBlue;
			}
			else
			{
				mre.materials = pr_mtrlGreen;
			}
		}
		
		
		void SetupStage(int nr)
		{
			if (nr == 1)
			{
				var cellInfo = new int[,]       
				{
					{0, 0, 0, 0,  0,  0, 0, 0, 0},
					{0, 1, 1, 2,  2,  1, 1, 2, 0},
					{0, 0, 1, 0,  0,  1, 0, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 0, 0, 0,  0,  0, 0, 0, 0}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(7, 1), 1); 
				
				CellLoc[] locs = { new CellLoc(6, 3), new CellLoc(6, 4), new CellLoc(6, 5) };
				AddCargo(locs);
			}
			else if (nr == 2)
			{
				var cellInfo = new int[,]       
				{
					{0, 0, 0, 0,  0,  0, 0, 0, 0},
					{0, 1, 1, 2,  0,  0, 0, 0, 0},
					{0, 1, 1, 1,  2,  1, 1, 1, 0},
					{0, 0, 0, 1,  0,  1, 0, 1, 0},
					{0, 0, 0, 1,  0,  1, 0, 1, 0},
					{0, 0, 0, 1,  0,  1, 0, 1, 0},
					{0, 0, 1, 1,  1,  1, 0, 1, 0},
					{0, 0, 1, 1,  1,  1, 1, 2, 0},
					{0, 0, 0, 0,  0,  0, 0, 0, 0}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(7, 2), 1); 
				
				CellLoc[] locs = { new CellLoc(5, 3), new CellLoc(6, 4), new CellLoc(2, 3) };
				AddCargo(locs);
			}
			else if (nr == 3)
			{
				var cellInfo = new int[,]
				{
					{0, 0, 0, 0,  0,  0, 0, 0, 0},
					{0, 0, 2, 1,  1,  1, 2, 2, 0},
					{0, 1, 1, 0,  0,  1, 0, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 0, 1, 0,  1,  1, 0, 0, 0},
					{0, 0, 1, 0,  0,  1, 0, 0, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 0,  1,  1, 1, 1, 0},
					{0, 0, 0, 0,  0,  0, 0, 0, 0}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(7, 1), 0); 
				
				CellLoc[] locs = { new CellLoc(2, 2), new CellLoc(2, 5), new CellLoc(6, 3) };
				AddCargo(locs);
				
			}
			else  
			{
				var cellInfo = new int[,]       
				{
					{0, 0, 0, 0,  0,  0, 0, 0, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 0,  1,  0, 1, 1, 0},
					{0, 1, 1, 2,  2,  2, 1, 1, 0},
					{0, 1, 1, 0,  1,  0, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 1, 1, 1,  1,  1, 1, 1, 0},
					{0, 0, 0, 0,  0,  0, 0, 0, 0}
				};
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(7, 1), 1); 
				
				CellLoc[] locs = { new CellLoc(2, 2), new CellLoc(2, 4), new CellLoc(2, 6) };
				AddCargo(locs);
			}
			
		} 


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			Vector3 pos = Square.GetPosition();
			pos.z = -0.002f;
			Square.transform.localPosition = pos;
			
			foreach (var obj in Cargo)
			{
				pos = obj.GetPosition();
				pos.z = -0.002f;
				obj.transform.localPosition = pos;
			}
		}


		static Sec222 pr_instance = new Sec222(5);

		public static Sec222 PRGSI()
		{
			return pr_instance;
		}


		private Sec222(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				
				Square = CreateObject_Obj2D("Square2");
				Cargo_src = CreateObject_Obj2D("Cargo1");
				Block_src = CreateObject_Obj2D("Block2");
				Floor_src = CreateObject_Obj2D("Floor0");
				Mark_src = CreateObject_Obj2D("Mark0");
				
				
				// Instantiate Block & Floor --
				const int numElem = 64;
				Block = new THObject2D[numElem];
				Block[0] = Block_src;
				for (int i = 1; i < Block.Length; i++)
				{
					Block[i] = CreateObject_InsObj2D(Block_src.gameObject);
				}
				
				var mre = Cargo_src.gameObject.GetComponent<MeshRenderer>();
				pr_mtrlGreen = mre.materials;
				
				Shader standard = Shader.Find("Standard");
				pr_mtrlBlue = new Material[2];
				pr_mtrlBlue[0] = new Material(standard);
				pr_mtrlBlue[0].name = "CrgBlue0";
				pr_mtrlBlue[0].SetColor("_Color", new Color(0.1f, 0.1f, 0.8f, 1.0f));
				pr_mtrlBlue[1] = new Material(standard);
				pr_mtrlBlue[1].name = "CrgBlue1";
				pr_mtrlBlue[1].SetColor("_Color", new Color(0.25f, 0.4f, 0.55f, 1.0f));
				
				
				Floor = new THObject2D[81];
				Floor[0] = Floor_src;
				for (int i = 1; i < Floor.Length; i++)
				{
					Floor[i] = CreateObject_InsObj2D(Floor_src.gameObject);
				}
				
				Mark = new THObject2D[6];
				Mark[0] = Mark_src;
				for (int i = 1; i < Mark.Length; i++)
				{
					Mark[i] = CreateObject_InsObj2D(Mark_src.gameObject);
				}
				
				CargoArray = new THObject2D[6];
				CargoArray[0] = Cargo_src;
				for (int i = 1; i < CargoArray.Length; i++)
				{
					CargoArray[i] = CreateObject_InsObj2D(Cargo_src.gameObject);
				}
				
				Cargo = new THObject2D[0];

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(12.6f, 400);
			
			SetCameraPosition_2D(0, 0);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(THMatrix3x3.identity);
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetMatrix(THMatrix3x3.identity);
				
				var cellInfo = new int[,]   
				{
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(4, 4), 0); 
				
				i_MOVE = false;
				i_moveSpeed = c_cellLen / c_maxMoveCount;
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetMatrix(THMatrix3x3.identity);
				
				var cellInfo = new int[,]       
				{
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 0, 0,  1,  0, 0, 1, 1},
					{1, 1, 0, 1,  1,  1, 0, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 0, 1,  1,  1, 0, 1, 1},
					{1, 1, 0, 0,  1,  0, 0, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(4, 4), 0); 
				
				i_MOVE = false;
				i_moveSpeed = c_cellLen / c_maxMoveCount;
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetMatrix(THMatrix3x3.identity);
				
				var cellInfo = new int[,]       
				{
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(4, 4), 0); 
				
				CellLoc[] locs = { new CellLoc(2, 4) };
				AddCargo(locs);
				
				i_MOVE = false;
				i_moveSpeed = c_cellLen / c_maxMoveCount;
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetMatrix(THMatrix3x3.identity);
				
				var cellInfo = new int[,]       
				{
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  0,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 0, 1, 1,  1,  1, 1, 0, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1},
					{1, 1, 1, 1,  0,  1, 1, 1, 1},
					{1, 1, 1, 1,  1,  1, 1, 1, 1}
				};
				
				SetStageData(cellInfo);
				
				SetSquareStartLocation(new CellLoc(4, 4), 0); 
				
				CellLoc[] locs = { new CellLoc(2, 2), new CellLoc(2, 6) };
				AddCargo(locs);
				
				i_MOVE = false;
				i_moveSpeed = c_cellLen / c_maxMoveCount;
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetMatrix(THMatrix3x3.identity);
				
				i_MOVE = false;
				i_moveSpeed = c_cellLen / c_maxMoveCount;
			}
		}

	}

}
