//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;



namespace OQJ_Sec223
{
	public class Sec223 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Disk;
		THObject2D Triangle;


		// Instance Variables
		float i_rotDeg;
		
		float i_scaleX;
		float i_scaleY;

		int[,] i_vi = new int[8, 3];
		
		bool pr_isHitColor;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Disk.SetPosition(-4, 0);
				
				i_scaleX = 2.0f;
				i_scaleY = 2.0f;
				Triangle.SetMatrix(TH2DMath.GetScale3x3(i_scaleX, i_scaleY));
				
				i_INITIALIZED = true;
			}
			
			// Disk
			Vector2 T = Disk.GetPosition();
			
			float speed = 0.10f;
			if (Input.GetKey(KeyCode.H))
			{
				T.x -= speed;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				T.x += speed;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				T.y -= speed;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				T.y += speed;
			}
			
			Disk.SetPosition(T);
			
			
			// Triangle
			if (Input.GetKey(KeyCode.R) || Input.GetKey(KeyCode.S) || Input.GetKey(KeyCode.T))
			{
				if (Input.GetKey(KeyCode.R))
				{
					i_rotDeg += THUtil.IsShiftDown() ? -0.5f : 0.5f;
				}
				else if (Input.GetKey(KeyCode.S))
				{
					i_scaleX += THUtil.IsShiftDown() ? -0.02f : 0.02f;
				}
				else if (Input.GetKey(KeyCode.T))
				{
					i_scaleY += THUtil.IsShiftDown() ? -0.02f : 0.02f;
				}
				
				THMatrix3x3 R = TH2DMath.GetRotation3x3(i_rotDeg);
				THMatrix3x3 S = TH2DMath.GetScale3x3(i_scaleX, i_scaleY);
				Triangle.SetMatrix(R * S);
			}
			
			
			// Collision Test
			THMatrix3x3 M = Triangle.GetMatrix();
			Vector2 A = Vector2.zero;						
			Vector2 B = M * new Vector3(0.8f, 1.8f, 1.0f);
			Vector2 C = M * new Vector3(2.4f, 0.0f, 1.0f);
			float r = 1.0f;
			
			bool bColl = CollisionTest_Disk_Triangle(T, r, A, B, C);
			Hit(bColl);
		}


		bool CollisionTest_Disk_Triangle(Vector2 T, float r, Vector2 A, Vector2 B, Vector2 C)
		{
			return CollisionTest_Disk_Triangle(T, r, new Vector2[] { A, B, C });
		}


		bool CollisionTest_Disk_Triangle(Vector2 T, float r,  Vector2[] vrts)
		{
			float rr = r * r;
			int k = 0;

			// AT x AB
			Vector2 a = (T - vrts[0]);
			Vector2 b = (vrts[1] - vrts[0]);
			if (a.x * b.y - a.y * b.x >= 0.0f) { k += 4; }

			// BT x BC
			a = (T - vrts[1]);
			b = (vrts[2] - vrts[1]);
			if (a.x * b.y - a.y * b.x >= 0.0f) { k += 2; }

			// CT x CA
			a = (T - vrts[2]);
			b = (vrts[0] - vrts[2]);
			if (a.x * b.y - a.y * b.x >= 0.0f) { k += 1; }

			if (k == 7) { return true; }

			int i0 = i_vi[k, 0];
			int i1 = i_vi[k, 1];
			int i2 = i_vi[k, 2];
			return (ComputeSqrDistance_Point_Segment(T, vrts[i0], vrts[i1]) <= rr) ||
					(i2 != -1 && ComputeSqrDistance_Point_Segment(T, vrts[i1], vrts[i2]) <= rr);
		}


		float ComputeSqrDistance_Point_Segment(Vector2 P, Vector2 A, Vector2 B)
		{
			Vector2 AB = B - A;
			Vector2 AP = P - A;

			float dot1 = Vector2.Dot(AP, AB);
			if(dot1 < 0.0f)
			{
				return AP.sqrMagnitude;
			}

			float dot2 = AB.sqrMagnitude;
			if(dot1 > dot2)
			{
				return (P - B).sqrMagnitude;	
			}

			return AP.sqrMagnitude - (dot1 * dot1) / dot2;
		}


		void Hit(bool coll)
		{
			if (coll)
			{
				if (!pr_isHitColor)
				{
					var mre = Triangle.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.8f, 0.2f, 0.1f, 1.0f));
					
					mre = Disk.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.7f, 0.1f, 0.1f, 0.6f));
				}
				
				pr_isHitColor = true;
			}
			else
			{
				if (pr_isHitColor)
				{
					var mre = Triangle.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.2703f, 0.7075f, 0.2703f, 1.0f));
					
					mre = Disk.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.2078f, 0.4705f, 0.6980f, 0.6f));
				}
				
				pr_isHitColor = false;
			}
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1))
			{
				var pos = Disk.transform.localPosition;
				pos.z = -0.025f;
				Disk.transform.localPosition = pos;
			}
		}


		static Sec223 pr_instance = new Sec223(1);

		public static Sec223 PRGSI()
		{
			return pr_instance;
		}


		private Sec223(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Disk = CreateObject_Obj2D("UnitDiskB");
				Triangle = CreateObject_Obj2D("GrTria");
				
				var trpMtrl = BSAuxFuncs.GetSemiTransparentMaterial(0.2078f, 0.4705f, 0.6980f, 0.88f);
				var mre = Disk.gameObject.GetComponent<MeshRenderer>();
				mre.material = trpMtrl;
				
				mre = Triangle.gameObject.GetComponent<MeshRenderer>();
				mre.material.SetColor("_Color", new Color(0.2703f, 0.7075f, 0.2703f, 1.0f));

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(8.0f, 400);     
			SetCameraPosition_2D(2.3f, 2.2f);       
			
			SetFPS(30);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			i_rotDeg = 0.0f;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, Triangle.gameObject);
				Disk.SetIdentity();
				Triangle.SetIdentity();

				const int A = 0, B = 1, C = 2;
				i_vi[1, 0] = A;
				i_vi[1, 1] = B;
				i_vi[1, 2] = C;

				i_vi[2, 0] = C;
				i_vi[2, 1] = A;	
				i_vi[2, 2] = B;	

				i_vi[3, 0] =  A;
				i_vi[3, 1] =  B;
				i_vi[3, 2] = -1;

				i_vi[4, 0] = B;
				i_vi[4, 1] = C;	
				i_vi[4, 2] = A;

				i_vi[5, 0] =  B;
				i_vi[5, 1] =  C;
				i_vi[5, 2] = -1;

				i_vi[6, 0] =  C;
				i_vi[6, 1] =  A;
				i_vi[6, 2] = -1;

				i_vi[7, 0] = -1;
				i_vi[7, 1] = -1;
				i_vi[7, 2] = -1;

				Triangle.SetScale(2);	
			}
		}

	}

}
