//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec224
{
	public class Sec224 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Rect;
		THObject2D Disk;
		THObject2D Ray;
		THObject2D LP;
		THObject2D[] CrsDot = new THObject2D[2];
		
		THObject2D Dot;
		
		THObject2D Sta1;
		THObject2D Sta2;
		THObject2D Dir1;
		THObject2D Dir2;
		THObject2D Line1;
		THObject2D Line2;


		// Instance Variables
		Vector2 R0;
		Vector2 P1, P2;
		
		readonly float c_RectW = 6.0f;
		readonly float c_RectH = 2.0f;
		
		readonly Vector2[] c_initMinMax = { new Vector2(-3, -1), new Vector2(3, 1) };
		
		float pr_degRay;
		float pr_degRect;
		
		float pr_deg1;
		float pr_deg2;




		void Code1()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L)) 
			{
				MoveRect();
			}
			
			if (Input.GetKey(KeyCode.E))
			{
				ChangeRayDirection();
			}
			
			Vector2 v = GetRayDirection(); 
			
			THMatrix3x3 M = Rect.GetMatrix();	
			Vector2 min = M * TH2DMath.ToVector3(c_initMinMax[0]);
			Vector2 max = M * TH2DMath.ToVector3(c_initMinMax[1]);
			
			Vector2[] res = CollisionTest_Ray_AABB(R0, v, new Vector2[] { min, max });
			Hit(res);
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L) ||
			Input.GetKey(KeyCode.R)) 
			{
				MoveRect();
			}
			
			if (Input.GetKey(KeyCode.E))
			{
				ChangeRayDirection();
			}
			
			Vector2 v = GetRayDirection();
			
			THMatrix3x3 M = Rect.GetMatrix();
			Vector2 ui = M.GetColumn(0);
			Vector2 uj = M.GetColumn(1);
			Vector2 T = Rect.GetPosition();
			
			Vector2[] res = CollisionTest_Ray_Rect(R0, v, T, ui, uj, c_RectW, c_RectH);
			Hit(res);
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L) )
			
			{
				MoveDisk();
			}
			
			if (Input.GetKey(KeyCode.E))
			{
				ChangeRayDirection();
			}
			
			Vector2 v = GetRayDirection();
			
			Vector2 C = Disk.GetPosition();
			float r = Disk.transform.localScale.x;
			
			Vector2[] res = CollisionTest_Ray_Disk(R0, v, C, r);
			Hit(res);
		}


		void Code4()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L))
			{
				RotateLine();
			}
			
			Vector2 v1 = GetDirection_v1();
			Vector2 v2 = GetDirection_v2();
			
			Vector3 T = CalcIntersectionOf2Lines(P1, v1, P2, v2);
			
			if (T.z != float.PositiveInfinity) 
			{ 
				Dot.SetPosition(T);
			}
		}




		
		
		
		Vector2[] CollisionTest_Ray_AABB(Vector2 R0, Vector2 v, Vector2[] min_max)
		{
			const float eps = 0.000001f;
			
			float xmin = min_max[0].x;
			float ymin = min_max[0].y;
			float xmax = min_max[1].x;
			float ymax = min_max[1].y;
			
			float inX, outX;
			if (THUtil.IsZero(v.x))
			{
				if (R0.x < xmin || xmax < R0.x) { return null; }
				
				inX  = float.NegativeInfinity;
				outX = float.PositiveInfinity;
			}
			else
			{
				inX  = (xmin - R0.x) / v.x;
				outX = (xmax - R0.x) / v.x;
				if (v.x < -eps) { THUtil.Swap(ref inX, ref outX); }
			}
			
			float inY, outY;
			if (THUtil.IsZero(v.y))
			{
				if (R0.y < ymin || ymax < R0.y) { return null; }
				
				inY  = float.NegativeInfinity;
				outY = float.PositiveInfinity;
			}
			else
			{
				inY  = (ymin - R0.y) / v.y;
				outY = (ymax - R0.y) / v.y;
				if (v.y < -eps) { THUtil.Swap(ref inY, ref outY); }
			}
			
			
			if (!(inX < outY && inY < outX)) { return null; }
			
			if (inX < 0.0f && inY < 0.0f) { return null; }
			
			float tmin = Mathf.Max(inX, inY);
			float tmax = Mathf.Min(outX, outY);
			
			return new Vector2[] { R0 + tmin * v, R0 + tmax * v };
		}
		

		float[] ComputeT_Ray_AABB(Vector2 R0, Vector2 v, Vector2[] min_max)
		{
			const float eps = 0.000001f;
			
			float xmin = min_max[0].x;
			float ymin = min_max[0].y;
			float xmax = min_max[1].x;
			float ymax = min_max[1].y;
			
			float inX, outX;
			if (THUtil.IsZero(v.x))
			{
				if (R0.x < xmin || xmax < R0.x) { return null; }
				
				inX  = float.NegativeInfinity;
				outX = float.PositiveInfinity;
			}
			else
			{
				inX  = (xmin - R0.x) / v.x;
				outX = (xmax - R0.x) / v.x;
				if (v.x < -eps) { THUtil.Swap(ref inX, ref outX); }
			}
			
			float inY, outY;
			if (THUtil.IsZero(v.y))
			{
				if (R0.y < ymin || ymax < R0.y) { return null; }
				
				inY  = float.NegativeInfinity;
				outY = float.PositiveInfinity;
			}
			else
			{
				inY  = (ymin - R0.y) / v.y;
				outY = (ymax - R0.y) / v.y;
				if (v.y < -eps) { THUtil.Swap(ref inY, ref outY); }
			}
			
			
			if (!(inX < outY && inY < outX)) { return null; }
			
			if (inX < 0.0f && inY < 0.0f) { return null; }

			float tmin = Mathf.Max(inX, inY);
			float tmax = Mathf.Min(outX, outY);

			return new float[] { tmin, tmax };
		}

		
		Vector2[] CollisionTest_Ray_Rect(Vector2 R0, Vector2 v, Vector2 T, Vector2 ui, Vector2 uj, float w, float h)
		{
			Vector2 m = R0 - T;
			float s = Vector2.Dot(m, ui);	
			float t = Vector2.Dot(m, uj);
			Vector2 locR0 = new Vector2(s, t);
			
			s = Vector2.Dot(v, ui);		
			t = Vector2.Dot(v, uj);
			Vector2 locV = new Vector2(s, t).normalized;
			
			Vector2 max = new Vector2(w / 2, h / 2);
			Vector2 min = -max;

			float[] ti = ComputeT_Ray_AABB(locR0, locV, new Vector2[] { min, max });
			if (ti == null) { return null; }

			return new Vector2[] { R0 + ti[0] * v, R0 + ti[1] * v };
		}
		
		
		
		public static Vector2[] CollisionTest_Ray_Disk(Vector2 R0, Vector2 v, Vector2 C, float r)
		{
			Vector2 locR0 = R0 - C;
			float b = Vector2.Dot(locR0, v);
			float c = locR0.sqrMagnitude - r * r;
			
			float D = b * b - c;
			if(D < 0.0f) { return null; }
			
			float tmin = -b - Mathf.Sqrt(D);
			float tmax = -b + Mathf.Sqrt(D);
			
			if (tmin < 0.0f) { return null; }
			
			Vector2 Pa = R0 + tmin * v;
			Vector2 Pb = R0 + tmax * v;
			
			return new Vector2[] { Pa, Pb };
		}
		
		
		
		Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 a, Vector2 Q, Vector2 b)
		{
			return TH2DMath.CalcIntersectionOf2Lines(P, a, Q, b);
		}
		
		
		
		void MoveRect()
		{
			Vector2 pos = Rect.GetPosition();	
			
			if (Input.GetKey(KeyCode.H))
			{
				pos.x -= 0.025f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				pos.x += 0.025f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				pos.y -= 0.025f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				pos.y += 0.025f;
			}
			
			
			if (MatchCase(1))
			{
				Rect.SetPosition(pos);
			}
			else
			{
				if (Input.GetKey(KeyCode.R))
				{
					pr_degRect += THUtil.IsShiftDown() ? -0.5f : 0.5f;
				}
				
				THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
				THMatrix3x3 R = TH2DMath.GetRotation3x3(pr_degRect);
				Rect.SetMatrix(T * R);
			}
		}
		
		
		
		void MoveDisk()
		{
			Vector2 pos = Disk.GetPosition();	
			
			if (Input.GetKey(KeyCode.H))
			{
				pos.x -= 0.025f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				pos.x += 0.025f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				pos.y -= 0.025f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				pos.y += 0.025f;
			}
			
			Disk.SetPosition(pos);
		}
		
		
		
		void ChangeRayDirection()
		{
			pr_degRay += THUtil.IsShiftDown() ? -0.2f : 0.2f;
			
			Ray.SetRotation(pr_degRay);
		}
		
		
		Vector2 GetRayDirection()
		{
			return TH2DMath.GetRotation3x3(pr_degRay) * Vector2.up;
		}
		
		
		
		
		void RotateLine()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J))
			{
				pr_deg1 += Input.GetKey(KeyCode.H) ? 0.15f : -0.15f;
				Vector2 pos = Line1.GetPosition();
				THMatrix3x3 M = TH2DMath.GetTranslation3x3(pos) * TH2DMath.GetRotation3x3(pr_deg1) *
				TH2DMath.GetScale3x3(50.0f);
				Line1.SetMatrix(M);
				Dir1.SetRotation(pr_deg1);
			}
			
			if (Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L))
			{
				pr_deg2 += Input.GetKey(KeyCode.K) ? 0.15f : -0.15f;
				Vector2 pos = Line2.GetPosition();
				THMatrix3x3 M = TH2DMath.GetTranslation3x3(pos) * TH2DMath.GetRotation3x3(pr_deg2) *
				TH2DMath.GetScale3x3(50.0f);
				Line2.SetMatrix(M);
				Dir2.SetRotation(pr_deg2);
			}
		}
		
		
		
		Vector2 GetDirection_v1()
		{
			return Dir1.GetRotation() * Vector2.right;
		}
		
		
		Vector2 GetDirection_v2()
		{
			return Dir2.GetRotation() * Vector2.right;
		}
		
		
		
		void Hit(Vector2[] res)
		{
			if (res != null)
			{
				CrsDot[0].transform.position = new Vector3(res[0].x, res[0].y, -0.006f);
				CrsDot[1].transform.position = new Vector3(res[1].x, res[1].y, -0.006f);
			}
			else
			{
				CrsDot[0].SetPosition(-1000, 0);
				CrsDot[1].SetPosition(-1000, 0);
			}
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec224 pr_instance = new Sec224(4);

		public static Sec224 PRGSI()
		{
			return pr_instance;
		}


		private Sec224(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Rect = CreateObject_Obj2D("RectB");
				Ray = CreateObject_Obj2D("AuxRay");
				LP = CreateObject_Obj2D("LPointB");
				Disk = CreateObject_Obj2D("UnitDiskB");
				
				Line1 = CreateObject_InsObj2D(Ray.gameObject);
				Line2 = CreateObject_InsObj2D(Ray.gameObject);
				
				var src_Dir = CreateObject_Obj2D("TanVtr");  
				Dir1 = CreateObject_InsObj2D(src_Dir.gameObject);
				Dir2 = CreateObject_InsObj2D(src_Dir.gameObject);
				
				CrsDot[0] = CreateObject_InsObj2D(Disk.gameObject);
				CrsDot[1] = CreateObject_InsObj2D(Disk.gameObject);
				
				Sta1 = CreateObject_InsObj2D(LP.gameObject);
				Sta2 = CreateObject_InsObj2D(LP.gameObject);
				Sta1.gameObject.GetComponent<MeshRenderer>().material = BSAuxFuncs.GetUnlitMaterial(0.18f, 0.52f, 0.89f);
				Sta2.gameObject.GetComponent<MeshRenderer>().material = BSAuxFuncs.GetUnlitMaterial(0.18f, 0.52f, 0.89f);
				
				Dot = CreateObject_Obj2D("UnitDiskG");
				Dot.gameObject.GetComponent<MeshRenderer>().material = BSAuxFuncs.GetUnlitMaterial(0.86f, 0.86f, 0.86f);
				
				Rect.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetSemiTransparentMaterial(0.393f, 0.70f, 0.90f, 0.365f);
				
				Disk.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetSemiTransparentMaterial(0.64f, 0.84f, 0.75f, 0.4f);
				
				LP.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetUnlitMaterial(0.40f, 0.565f, 0.91f);
				
				Material lmtr= BSAuxFuncs.GetUnlitMaterial(0.08f, 0.08f, 0.080f);
				Line1.gameObject.GetComponent<MeshRenderer>().material = lmtr;
				Line2.gameObject.GetComponent<MeshRenderer>().material = lmtr;
				
				Mesh mesh = Ray.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { Vector2.zero, Vector2.up });
				
				mesh = Line1.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { -Vector2.right, Vector2.right });
				
				mesh = Line2.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { -Vector2.right, Vector2.right });

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(10.2f, 400);
			
			SetCameraPosition_2D(0, 0);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				BSAuxFuncs.SetVisible(1, Ray.gameObject);
				BSAuxFuncs.SetVisible(1, LP.gameObject);
				Rect.SetIdentity();
				Ray.SetIdentity();
				LP.SetIdentity();
				
				for (int i = 0; i < 2; i++)
				{
					BSAuxFuncs.SetVisible(1, CrsDot[i].gameObject);
					CrsDot[i].SetIdentity();
					CrsDot[i].SetPosition(-1000, 0);
					CrsDot[i].SetScale(0.08f);
				}
				
				pr_degRay = -45;
				pr_degRect = 0;
				
				Rect.SetPosition(4, 4);
				
				LP.transform.position = new Vector3(-4.0f, -4.0f, -0.006f);
				LP.SetScale(0.8f);
				R0 = LP.GetPosition();
				
				Ray.SetPosition(LP.GetPosition());
				Ray.SetRotation(pr_degRay);
				Ray.SetScale(1, 50);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, Ray.gameObject);
				BSAuxFuncs.SetVisible(1, LP.gameObject);
				Disk.SetIdentity();
				Ray.SetIdentity();
				LP.SetIdentity();
				
				for (int i = 0; i < 2; i++)
				{
					BSAuxFuncs.SetVisible(1, CrsDot[i].gameObject);
					CrsDot[i].SetIdentity();
					CrsDot[i].SetPosition(-1000, 0);
					CrsDot[i].SetScale(0.08f);
				}
				
				Disk.SetPosition(4, 4);
				Disk.SetScale(1.8f);
				
				pr_degRay = -45;
				
				LP.transform.position = new Vector3(-4.0f, -4.0f, -0.006f);
				LP.SetScale(0.8f);
				R0 = LP.GetPosition();
				
				Ray.SetPosition(LP.GetPosition());
				Ray.SetRotation(pr_degRay);
				Ray.SetScale(1, 50);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Line1.gameObject);
				BSAuxFuncs.SetVisible(1, Line2.gameObject);
				BSAuxFuncs.SetVisible(1, Sta1.gameObject);
				BSAuxFuncs.SetVisible(1, Sta2.gameObject);
				BSAuxFuncs.SetVisible(1, Dir1.gameObject);
				BSAuxFuncs.SetVisible(1, Dir2.gameObject);
				Line1.SetIdentity();
				Line2.SetIdentity();
				Sta1.SetIdentity();
				Sta2.SetIdentity();
				Dir1.SetIdentity();
				Dir2.SetIdentity();
				
				Vector3 pos1 = new Vector3(-10, -6, 0);
				Vector3 pos2 = new Vector3(10, 6, 0);
				Line1.SetScale(50.0f);
				Line2.SetScale(50.0f);
				Line1.SetPosition(pos1);
				Line2.SetPosition(pos2);
				
				pos1.z = -0.01f;
				pos2.z = -0.01f;
				Dir1.transform.position = pos1;
				Dir2.transform.position = pos2;
				Dir1.SetScale(0.92f);
				Dir2.SetScale(0.92f);
				
				pos1.z = -0.02f;
				pos2.z = -0.02f;
				Sta1.transform.position = pos1;
				Sta2.transform.position = pos2;
				Sta1.SetScale(0.5f);
				Sta2.SetScale(0.5f);
				
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				Dot.SetPosition(-1000, 0);
				Dot.SetScale(0.085f);
				
				pr_deg1 = 60.0f;
				pr_deg2 = 185.0f;
				Line1.SetRotation(pr_deg1);
				Line2.SetRotation(pr_deg2);
				Dir1.SetRotation(pr_deg1);
				Dir2.SetRotation(pr_deg2);
				
				P1 = pos1;
				P2 = pos2;
			}
		}

	}

}

