//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGAuxFuncs02;
using static OQJ_BSMouseListener.BSMouseListener;



namespace OQJ_Sec225
{
	public class Sec225 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		THObject2D[] VisCtrl;
		THObject2D[] Line;
		THObject2D[] Arc;
		
		THObject2D[] Convex;
		
		THObject2D[] TanArr;
		THObject2D[] PntArr;


		// Instance Variables
		int pr_numControl;
		
		int pr_DrawIndex;
		
		int pr_HF;
		int pr_hoverIndex;
		int pr_selectedIndex;
		bool pr_CONTROL_CHANGED;
		
		Vector2 pr_preDir;
		
		float pr_w2;
		float pr_h2;
		float pr_WldPerPix;
		
		readonly float c_radPnt = 0.27f * 0.6f;
		readonly float c_widTan = 1.8f;
		readonly float c_hgtTan = 0.2315f * 2 * 0.82f;
		
		Material pr_PointMtrl;
		Material pr_TangentMtrl;
		Material pr_hoverMtrl;
		
		readonly float c_fov = 60.0f;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				AddControl(2, 3, 50);
				AddControl(26, 8, -28);
				
				i_INITIALIZED = true;
			}
			
			
			if (AnyControlChanged())
			{
				Vector2 P = GetControlPosition(0);
				Vector2 v = GetControlTangent(0);
				Vector2 Q = GetControlPosition(1);
				Vector2 w = GetControlTangent(1);
				
				Vector3 R = CalcIntersectionOf2Lines(P, v, Q, w);
				InterpolatePQ(P, v, Q, w, R);
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				AddControl(0, 4, 60);
				AddControl(7, 7, -65);
				AddControl(14, 4, 76);
				AddControl(19, 12, -70);
				AddControl(23, 6, 52);
				AddControl(27, 9, -40);
				
				i_INITIALIZED = true;
			}
			
			
			if (AnyControlChanged())
			{
				int numControl = 6;
				for (int i = 0; i < numControl - 1; i++)
				{
					Vector2 P = GetControlPosition(i);
					Vector2 v = GetControlTangent(i);
					Vector2 Q = GetControlPosition(i + 1);
					Vector2 w = GetControlTangent(i + 1);
					
					Vector3 R = CalcIntersectionOf2Lines(P, v, Q, w);
					InterpolatePQ(P, v, Q, w, R);
				}
				
			}
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				AddControl(2, 3, 50);
				AddControl(26, 8, -28);
				
				i_INITIALIZED = true;
			}
			
			
			if (AnyControlChanged())
			{
				Vector2 P = GetControlPosition(0);
				Vector2 v = GetControlTangent(0);
				Vector2 Q = GetControlPosition(1);
				Vector2 w = GetControlTangent(1);
				
				Vector4 R = CalcIntersectionOf2Lines_ver2(P, v, Q, w);
				InterpolatePQ_ver2(P, v, Q, w, R);
			}
		}




		void AddControl(Vector3 v)
		{
			if(pr_numControl >= Line.Length) { Debug.Log("Err 1");   return; }
			
			VisCtrl[pr_numControl].SetPosition(v);
			VisCtrl[pr_HF + pr_numControl].SetPosition(v);
			VisCtrl[pr_HF + pr_numControl].SetRotation(v.z);
			
			pr_numControl++;
		}
		
		
		void AddControl(float x, float y, float deg)
		{
			AddControl(new Vector3(x, y, deg));
		}
		
		
		
		Vector2 GetControlPosition(int idx)
		{
			return VisCtrl[idx].GetPosition();
		}
		
		
		
		Vector2 GetControlTangent(int idx)
		{
			return VisCtrl[idx + pr_HF].GetRotation() * Vector2.right;
		}
		
		
		
		bool AnyControlChanged()
		{
			if (pr_CONTROL_CHANGED)
			{
				ClearCurve();
			}
			
			return pr_CONTROL_CHANGED;
		}
		
		
		
		void InterpolatePQ(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3)  
		{
			if (R3.z < 0.0f)
			{
				DrawLine(P, Q);
				pr_DrawIndex++;
				return;
			}
			
			Vector2 R = R3;
			
			float cosT = Vector2.Dot(-v, w);
			float sinT = Mathf.Sqrt(1.0f - cosT * cosT);
			float lenPR = (R - P).magnitude;
			float lenQR = (R - Q).magnitude;
			bool ccw = (-w.x * v.y + w.y * v.x > 0.0f) ? true : false;  
			Vector2 wr = ccw ? new Vector2(-w.y, w.x) : new Vector2(w.y, -w.x);  
			
			Vector2 S, C;
			if (lenPR >= lenQR)		
			{
				float a = lenQR;
				float r = a * (1 - cosT) / sinT;
				
				S = R - a * v;
				C = Q + r * wr;
				
				DrawLine(P, S);		
				DrawArc(S, Q, C);
			}
			else
			{
				float a = lenPR;
				float r = a * (1 - cosT) / sinT;
				
				S = R + a * w;
				C = S + r * wr;
				
				DrawLine(S, Q);
				DrawArc(P, S, C);
			}
			
			DrawAuxLine(P, Q, R);
			pr_DrawIndex++;
		}
		
		
		
		
		
		void InterpolatePQ_ver2(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector4 R4)  
		{
			Vector2 R = R4;
			float t1 = R4.z;
			float t2 = R4.w;
			
			float cos2T = Vector2.Dot(-v, w);
			float sin2T = Mathf.Sqrt(1.0f - cos2T * cos2T);
			float lenPR = (R - P).magnitude;
			float lenQR = (R - Q).magnitude;
			
			if (t1 > 0.0f && t2 < 0.0f)
			{
				bool ccw = (-w.x * v.y + w.y * v.x > 0.0f) ? true : false;  
				Vector2 wr = ccw ? new Vector2(-w.y, w.x) : new Vector2(w.y, -w.x);  
				
				Vector2 S, C;
				if (lenPR >= lenQR)
				{
					float a = lenQR;
					float r = a * (1 - cos2T) / sin2T;
					
					S = R - a * v;
					C = Q + r * wr;
					
					DrawLine(P, S);
					DrawArc(S, Q, C);
				}
				else
				{
					float a = lenPR;
					float r = a * (1 - cos2T) / sin2T;
					
					S = R + a * w;
					C = S + r * wr;
					
					DrawLine(S, Q);
					DrawArc(P, S, C);
				}
				
				DrawAuxLine(P, Q, R);
			}
			else if (t1 < 0.0f && t2 > 0.0f)
			{
				int sign = (v.x * (-w.y) - v.y * (-w.x) > 0.0f) ? 1 : -1;  
				Vector2 vr = (sign > 0) ? new Vector2(-v.y, v.x) : new Vector2(v.y, -v.x);  
				
				Vector2 S, C;
				if (lenPR >= lenQR)
				{
					float a = lenPR;
					float r = a * (1 - cos2T) / sin2T;
					
					S = R - a * w;
					C = P + r * vr;
					
					DrawLine(Q, S);
					DrawArc(P, S, C, true);
				}
				else
				{
					float a = lenQR;
					float r = a * (1 - cos2T) / sin2T;
					
					S = R + a * v;
					C = S + r * vr;
					
					DrawLine(P, S);
					DrawArc(S, Q, C, true);
				}
			}
			else if (t1 > 0.0f && t2 > 0.0f)
			{
				float a = lenQR;
				float r = a * (1 - cos2T) / sin2T;
				Vector2 S = R + a * v;
				int sign = (v.x * (-w.y) - v.y * (-w.x) > 0.0f) ? 1 : -1;  
				Vector2 vr = (sign > 0) ? new Vector2(-v.y, v.x) : new Vector2(v.y, -v.x);  
				Vector2 C = S + r * vr;
				
				DrawLine(P, S);
				DrawArc(S, Q, C, true);
			}
			else  
			{
				
				
				float a = lenPR;
				float r = a * (1 - cos2T) / sin2T;
				Vector2 S = R - a * w;
				int sign = (w.x * v.y - w.y * v.x > 0.0f) ? 1 : -1;  
				Vector2 wr = (sign > 0) ? new Vector2(-w.y, w.x) : new Vector2(w.y, -w.x);	
				Vector2 C = S + r * wr;
				
				DrawLine(S, Q);
				DrawArc(P, S, C, true);
			}
			
			pr_DrawIndex++;
		}
		
		
		
		Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			Vector3 R = TH2DMath.CalcIntersectionOf2Lines(P, va, Q, vb);
			float t1 = R.z;
			
			
			
			Vector2 R2 = R;	
			float t2 = Vector2.Dot(R2 - Q, vb);
			
			int b = (t1 > 0.0f && t2 < 0.0f) ? 1 : -1;
			return new Vector3(R2.x, R2.y, b);
		}
		
		
		Vector4 CalcIntersectionOf2Lines_ver2(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			Vector3 R = TH2DMath.CalcIntersectionOf2Lines(P, va, Q, vb);
			float t1 = R.z;
			
			
			
			Vector2 R2 = R;	
			float t2 = Vector2.Dot(R2 - Q, vb);
			
			return new Vector4(R2.x, R2.y, t1, t2);
		}
		
		
		
		void DrawLine(Vector2 P, Vector2 Q)
		{
			if(pr_DrawIndex >= Line.Length) { return; }
			
			List<Vector3> list = new List<Vector3>() { P, Q };
			
			Mesh mesh = Line[pr_DrawIndex].gameObject.GetComponent<MeshFilter>().mesh;
			mesh.SetVertices(list);
		}
		
		
		void DrawArc(Vector2 P, Vector2 Q, Vector2 C, bool over180=false)
		{
			if(pr_DrawIndex >= Arc.Length) { return; }
			
			float deg = TH2DMath.CalcDegree_V1toV2(P - C, Q - C);   
			if (over180)
			{
				if(deg > 0)	{ deg = -(360.0f - deg); }
				else		{ deg = 360.0f + deg; }
			}
			
			const int numDiv = 120;
			
			Vector2 w = P - C;
			
			List<Vector3> list = new List<Vector3>();
			for (int i = 0; i <= numDiv; i++)
			{
				float rd = ((float)i / numDiv) * deg;
				Quaternion rot = Quaternion.AngleAxis(rd, Vector3.forward);
				Vector2 wp = rot * w;
				
				list.Add(wp + C);
			}
			
			Mesh mesh = Arc[pr_DrawIndex].gameObject.GetComponent<MeshFilter>().mesh;
			mesh.SetVertices(list);
		}
		
		
		void DrawAuxLine(Vector2 P, Vector2 Q, Vector2 R)
		{
			if(pr_DrawIndex >= Convex.Length) { return; }
			
			List<Vector3> list = new List<Vector3>() { P, R, Q };
			Mesh mesh = Convex[pr_DrawIndex].gameObject.GetComponent<MeshFilter>().mesh;
			mesh.SetVertices(list);
		}
		
		
		void ClearCurve()
		{
			List<Vector3> list = new List<Vector3>() { Vector3.zero, Vector3.zero };
			for (int i = 0; i < Line.Length; i++)
			{
				Mesh mesh = Line[i].gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(list);
			}
			
			
			list = new List<Vector3>() { Vector3.zero, Vector3.zero, Vector3.zero };
			for (int i = 0; i < Convex.Length; i++)
			{
				Mesh mesh = Convex[i].gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(list);
			}
			
			
			list.Clear();
			for (int i = 0; i <= 120; i++) {  list.Add(Vector3.zero);  }
			
			for (int i = 0; i < Arc.Length; i++)
			{
				Mesh mesh = Arc[i].gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(list);
			}
			
			pr_DrawIndex = 0;
		}
		
		
		void SetHoverColor(int nI)
		{
			if(pr_hoverIndex != -1)
			{
				ClearHoverColor();
			}
			
			MeshRenderer mr = VisCtrl[nI].gameObject.GetComponent<MeshRenderer>();
			mr.material = pr_hoverMtrl;
			
			pr_hoverIndex = nI;
		}
		
		
		void ClearHoverColor()
		{ 
			MeshRenderer mr = VisCtrl[pr_hoverIndex].gameObject.GetComponent<MeshRenderer>();
			if (VisCtrl[pr_hoverIndex].gameObject.name.StartsWith("Small"))
			{
				mr.material = pr_PointMtrl;
			}
			else
			{
				mr.material = pr_TangentMtrl;
			}
			
			pr_hoverIndex = -1;
		}
		
		
		void InitialSetup(float d=1.0f)
		{
			pr_w2 = Screen.width / 2;
			pr_h2 = Screen.height / 2;
			
			float theta = c_fov * 0.5f * Mathf.Deg2Rad;
			float tanT = Mathf.Tan(theta);
			float H2 = d * tanT;
			pr_WldPerPix = H2 / pr_h2;
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			MouseDragProcess();
			
			if (MatchCase(1, 2, 3))
			{
				int tmpIdx = -1;
				float mx = Input.mousePosition.x;
				float my = Input.mousePosition.y;
				float cx = pr_w2;
				float cy = pr_h2;
				
				Vector2 TrgPos = BSData.CameraController2D.GetCameraPosition();
				float x = (mx - cx) * pr_WldPerPix;
				float y = (my - cy) * pr_WldPerPix;
				Vector2 P = new Vector2(x, y) + TrgPos;
				
				if (pr_selectedIndex == -1)
				{
					for (int i = 0; i < pr_numControl; i++)
					{
						Vector2 C = VisCtrl[i].GetPosition();
						
						bool b = RGAuxFuncs02.CollisionTest_Point_Disk(P, C, c_radPnt * 1.8f);
						if (b)
						{
							tmpIdx = i;
							break;
						}
					}
					
					if (tmpIdx == -1)
					{
						for (int i = pr_HF; i < pr_HF + pr_numControl; i++)
						{
							THMatrix3x3 M = VisCtrl[i].GetMatrix();
							Vector2 objOrig = M.GetColumn(2);       
							Vector2 lx = M.GetColumn(0).normalized;
							Vector2 ly = M.GetColumn(1).normalized;
							
							Vector2 T = objOrig + (0.5f * c_widTan) * lx;
							bool b = RGAuxFuncs02.CollisionTest_Point_Rect(P, T, lx, ly, c_widTan, c_hgtTan);
							if (b)
							{
								tmpIdx = i;
								break;
							}
						}
					}
					
					
					if (tmpIdx != -1)
					{
						SetHoverColor(tmpIdx);
					}
					else if (pr_hoverIndex != -1)
					{
						ClearHoverColor();
					}
				}
				
				
				if (IsMousePressed())
				{
					pr_selectedIndex = -1;
					
					if (tmpIdx != -1)
					{
						pr_selectedIndex = tmpIdx;
						
						if (tmpIdx >= pr_HF)	
						{
							pr_preDir = (P - VisCtrl[tmpIdx - pr_HF].GetPosition()).normalized;
						}
					}
				}
				
				
				pr_CONTROL_CHANGED = false;
				
				if (pr_selectedIndex >= 0)
				{
					if (IsMouseReleased())
					{
						pr_selectedIndex = -1;
						return;
					}
					
					if (IsMouseDragged())
					{
						int pdx = (pr_selectedIndex < pr_HF) ? pr_selectedIndex : pr_selectedIndex - pr_HF;
						
						THObject2D Pnt = VisCtrl[pdx];
						THObject2D Tan = VisCtrl[pdx + pr_HF];
						
						if (pr_selectedIndex < pr_HF)  
						{
							Vector2 df = GetMouseDifference();
							Vector2 mv = new Vector2(df.x * pr_WldPerPix, df.y * pr_WldPerPix);
							
							Pnt.SetPosition(Pnt.GetPosition() + mv);
							Tan.SetPosition(Tan.GetPosition() + mv);
						}
						else  
						{
							P = new Vector2(x, y) + TrgPos;
							Vector2 curDir = (P - Pnt.GetPosition()).normalized;
							
							float deg = TH2DMath.CalcDegree_V1toV2(pr_preDir, curDir);
							Quaternion rot = Quaternion.AngleAxis(deg, Vector3.forward) * Tan.GetRotation();
							Tan.SetRotation(rot);
							
							pr_preDir = curDir;
						}
						
						pr_CONTROL_CHANGED = true;
					}
				}
				
				
				if (!i_INITIALIZED) { pr_CONTROL_CHANGED = true; }
			}
		}


		void PostProcess()
		{
			for (int i = 0; i < pr_numControl; i++)
			{
				var wp = VisCtrl[i].transform.position;
				wp.z = -0.01f;
				VisCtrl[i].transform.position = wp;
			}
			
			for (int i = 0; i < pr_DrawIndex; i++)
			{
				var wp = Convex[i].transform.position;
				wp.z = 0.001f;
				Convex[i].transform.position = wp;
			}
		}


		static Sec225 pr_instance = new Sec225(3);

		public static Sec225 PRGSI()
		{
			return pr_instance;
		}


		private Sec225(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				var src_Point = CreateObject_Obj2D("SmallPointB");
				var src_Tangent = CreateObject_Obj2D("TanVtr");  
				var src_Line = CreateObject_Obj2D("AuxLine");
				var src_Arc = CreateObject_Obj2D("AuxArc");
				var src_Convex = CreateObject_Obj2D("Convex");
				
				// Lines & Arcs, Tangents
				const int max = 10;
				pr_HF = max;
				PntArr = new THObject2D[max];
				TanArr = new THObject2D[max];
				Line = new THObject2D[max];
				Arc = new THObject2D[max];
				Convex = new THObject2D[max];
				
				pr_hoverMtrl = BSAuxFuncs.GetUnlitMaterial(0.9f, 0.45f, 0.45f);
				pr_PointMtrl = BSAuxFuncs.GetUnlitMaterial(0.23f, 0.41f, 0.90f);
				pr_TangentMtrl = BSAuxFuncs.GetUnlitMaterial(0.45f, 0.88f, 0.45f);
				src_Point.gameObject.GetComponent<MeshRenderer>().material = pr_PointMtrl;
				src_Tangent.gameObject.GetComponent<MeshRenderer>().material = pr_TangentMtrl;
				
				for (int i = 0; i < max; i++)
				{
					PntArr[i] = CreateObject_InsObj2D(src_Point.gameObject);
					TanArr[i] = CreateObject_InsObj2D(src_Tangent.gameObject);
					
					Line[i] = CreateObject_InsObj2D(src_Line.gameObject);
					Arc[i] = CreateObject_InsObj2D(src_Arc.gameObject);
					Convex[i] = CreateObject_InsObj2D(src_Convex.gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(8.8f, 400); 
			
			SetCameraPosition_2D(14, 7);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (MatchCase(1, 2, 3))
			{
				CrdSys.transform.position = new Vector3(0.0f, 0.0f, 0.001f);	
				
				VisCtrl = new THObject2D[pr_HF * 2];   
				for (int i = 0; i < pr_HF; i++)
				{
					BSAuxFuncs.SetVisible(1, Line[i].gameObject);
					BSAuxFuncs.SetVisible(1, Arc[i].gameObject);
					BSAuxFuncs.SetVisible(1, Convex[i].gameObject);
					Line[i].SetIdentity();
					Arc[i].SetIdentity();
					Convex[i].SetIdentity();
					
					BSAuxFuncs.SetVisible(1, PntArr[i].gameObject);
					BSAuxFuncs.SetVisible(1, TanArr[i].gameObject);
					PntArr[i].SetIdentity();
					TanArr[i].SetIdentity();
					PntArr[i].SetScale(0.6f);
					TanArr[i].SetScale(1.0f, 0.75f);
					PntArr[i].SetPosition(-1000, 0);
					TanArr[i].SetPosition(-1000, 0);
					
					VisCtrl[i] = PntArr[i];             
					VisCtrl[pr_HF + i] = TanArr[i];
					
					VisCtrl[i].gameObject.GetComponent<MeshRenderer>().material = pr_PointMtrl;
					VisCtrl[pr_HF + i].gameObject.GetComponent<MeshRenderer>().material = pr_TangentMtrl;
				}
				
				pr_numControl = 0;
				pr_hoverIndex = -1;
				pr_selectedIndex = -1;
				
				float camZ = BSData.CameraController2D.GetCameraPosition().z;
				InitialSetup(Mathf.Abs(camZ));
			}
		}

	}

}
