//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using Segment = OQJ_Main02.Segment;
using BasicCurve = OQJ_Main02.BasicCurve;
using OQJ_RGAuxFuncs02;



namespace OQJ_Sec226
{
	public class Sec226 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Vehicle;
		THObject2D Circuit;
		
		THObject2D[] VisCtrl;
		THObject2D[] Line;
		THObject2D[] Arc;
		
		THObject2D[] TanArr;
		THObject2D[] PntArr;
		
		THObject2D Dot;


		// Instance Variables
		BasicCurve Curve;
		float t;
		
		int pr_numControl;
		
		int pr_DrawIndex;
		int pr_HF;
		
		Material pr_PointMtrl;
		Material pr_TangentMtrl;
		




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Vector2[] arrP = GetPositionArray(1);
				Vector2[] arrT = GetTangentArray(1);
				
				List<Segment> segments = new List<Segment>();
				int numControl = arrP.Length;
				for (int i = 0; i < numControl; i++)
				{
					int i1 = (i + 1) % numControl;
					Vector2 P = arrP[i];
					Vector2 v = arrT[i];
					Vector2 Q = arrP[i1];
					Vector2 w = arrT[i1];
					
					Vector3 iR = CalcIntersectionOf2Lines(P, v, Q, w);
					InterpolatePQ_Data(P, v, Q, w, iR, segments);
				}
				
				Curve = new BasicCurve(segments);
				t = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.K))
			{
				t += 0.002f;
			}
			else if (Input.GetKey(KeyCode.J))
			{
				t -= 0.002f;
			}
			
			Vector2 pos = Curve.ComputePosition(t);
			Dot.SetPosition(pos);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Vector2[] arrP = GetPositionArray(2);
				Vector2[] arrT = GetTangentArray(2);
				
				List<Segment> segments = new List<Segment>();
				int numControl = arrP.Length;
				for (int i = 0; i < numControl; i++)
				{
					int i1 = (i + 1) % numControl;
					Vector2 P = arrP[i];
					Vector2 v = arrT[i];
					Vector2 Q = arrP[i1];
					Vector2 w = arrT[i1];
					
					Vector3 iR = CalcIntersectionOf2Lines(P, v, Q, w);
					InterpolatePQ_Data(P, v, Q, w, iR, segments);
				}
				
				Curve = new BasicCurve(segments);
				t = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.K))
			{
				t += 0.0025f;
			}
			else if (Input.GetKey(KeyCode.J))
			{
				t -= 0.0025f;
			}
			
			Vector2[] vrr = Curve.ComputePositionAndTangent(t);
			Vector2 pos = vrr[0];
			Vector2 dir = vrr[1];
			
			THMatrix3x3 T = TH2DMath.GetTranslation3x3(pos);
			THMatrix3x3 R = TH2DMath.CalcRotation3x3_V1toV2(Vector2.up, dir);
			Vehicle.SetMatrix(T * R);
		}




		
		
		void InterpolatePQ_Data(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3, List<Segment> segments)  
		{
			RGAuxFuncs02.InterpolatePQ_Data(P, v, Q, w, R3, segments);
		}
		
		
		void AddControl(Vector3 v)
		{
			if(pr_numControl >= Line.Length) { Debug.Log("Err 1");   return; }
			
			VisCtrl[pr_numControl].SetPosition(v);
			VisCtrl[pr_HF + pr_numControl].SetPosition(v);
			VisCtrl[pr_HF + pr_numControl].SetRotation(v.z);
			
			pr_numControl++;
		}
		
		
		void AddControl(float x, float y, float deg)
		{
			AddControl(new Vector3(x, y, deg));
		}
		
		
		
		void InterpolatePQ(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3)  
		{
			Vector2 R = R3;
			if (R3.z < 0.0f || RGAuxFuncs02.IsLine(P, Q, R))
			{
				DrawLine(P, Q);
				pr_DrawIndex++;
				return;
			}
			
			
			float cosT = Vector2.Dot(-v, w);
			float sinT = Mathf.Sqrt(1.0f - cosT * cosT);
			float lenPR = (R - P).magnitude;
			float lenQR = (R - Q).magnitude;
			bool ccw = (-w.x * v.y + w.y * v.x > 0.0f) ? true : false;  
			Vector2 wr = ccw ? new Vector2(-w.y, w.x) : new Vector2(w.y, -w.x);  
			
			Vector2 S, C;
			if (lenPR >= lenQR)		
			{
				float a = lenQR;
				float r = a * (1 - cosT) / sinT;
				
				S = R - a * v;
				C = Q + r * wr;
				
				DrawLine(P, S);		
				DrawArc(S, Q, C);
			}
			else
			{
				float a = lenPR;
				float r = a * (1 - cosT) / sinT;
				
				S = R + a * w;
				C = S + r * wr;
				
				DrawArc(P, S, C);
				DrawLine(S, Q);
			}
			
			
			pr_DrawIndex++;
		}
		
		
		
		Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			return RGAuxFuncs02.CalcIntersectionOf2Lines(P, va, Q, vb);
		}
		
		
		
		void DrawLine(Vector2 P, Vector2 Q)
		{
			if(pr_DrawIndex >= Line.Length) { return; }
			
			List<Vector3> list = new List<Vector3>() { P, Q };
			
			Mesh mesh = Line[pr_DrawIndex].gameObject.GetComponent<MeshFilter>().mesh;
			mesh.SetVertices(list);
		}
		
		
		void DrawArc(Vector2 P, Vector2 Q, Vector2 C, bool over180=false)
		{
			if(pr_DrawIndex >= Arc.Length) { return; }
			
			float deg = TH2DMath.CalcDegree_V1toV2(P - C, Q - C);   
			if (over180)
			{
				if(deg > 0)	{ deg = -(360.0f - deg); }
				else		{ deg = 360.0f + deg; }
			}
			
			const int numDiv = 120;
			
			Vector2 w = P - C;
			
			List<Vector3> list = new List<Vector3>();
			for (int i = 0; i <= numDiv; i++)
			{
				float rd = ((float)i / numDiv) * deg;
				Quaternion rot = Quaternion.AngleAxis(rd, Vector3.forward);
				Vector2 wp = rot * w;
				
				list.Add(wp + C);
			}
			
			Mesh mesh = Arc[pr_DrawIndex].gameObject.GetComponent<MeshFilter>().mesh;
			mesh.SetVertices(list);
		}
		
		
		
		Vector2[] GetPositionArray(int id)
		{
			Vector2[] posArray;
			
			if (id == 1)
			{
				Vector2[] arr =
				{
					new Vector2(11, 2),
					new Vector2(7, 6),
					new Vector2(-7, 6),
					new Vector2(-11, 2),
					
					new Vector2(-11, -2),
					new Vector2(-7, -6),
					new Vector2(7, -6),
					new Vector2(11, -2)
				};
				
				posArray = arr;
			}
			else  
			{
				Vector2[] arr =
				{
					new Vector2(0.6864f,  -2.8424f),
					new Vector2(6.8235f,  -4.7510f),
					new Vector2(11.2785f, -4.2672f),
					new Vector2(9.2406f,  -0.8154f),
					new Vector2(4.0288f,   2.4506f),
					
					new Vector2(-0.6864f, 2.8424f),
					new Vector2(-6.8235f, 4.7510f),
					new Vector2(-11.2785f, 4.2672f),
					new Vector2(-9.2406f, 0.8154f),
					new Vector2(-4.0288f, -2.4506f)
				};
				
				posArray = arr;
			}
			
			return posArray;
		}
		
		
		Vector2[] GetTangentArray(int id)
		{
			Vector2[] tanArray;
			
			if (id == 1)
			{
				Vector2[] arr =
				{
					Vector2.up,
					Vector2.left,
					Vector2.left,
					Vector2.down,
					
					Vector2.down,
					Vector2.right,
					Vector2.right,
					Vector2.up
				};
				
				tanArray = arr;
			}
			else  
			{
				Vector2[] arr =
				{
					new Vector2(0.9966f, -0.0828f),
					new Vector2(0.8317f, -0.5552f),
					new Vector2(-0.3935f, 0.9193f),
					new Vector2(-0.6323f, 0.7747f),
					new Vector2(-0.9966f, 0.0828f),
					
					new Vector2(-0.9966f, 0.0828f),
					new Vector2(-0.8317f, 0.5552f),
					new Vector2(0.3935f, -0.9193f),
					new Vector2(0.6323f, -0.7747f),
					new Vector2(0.9966f, -0.0828f)
				};
				
				tanArray = arr;
			}
			
			return tanArray;
		}
		


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1))
			{
				var wp = Dot.transform.position;
				wp.z = -0.015f;
				Dot.transform.position = wp;
			}
			else if (MatchCase(2))
			{
				var wp = Vehicle.transform.position;
				wp.z = -0.02f;
				Vehicle.transform.position = wp;
			}
			
			BSData.CameraController2D.UpdateCameraData();
		}


		static Sec226 pr_instance = new Sec226(2);

		public static Sec226 PRGSI()
		{
			return pr_instance;
		}


		private Sec226(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Vehicle = CreateObject_Obj2D("Vehicle");
				Circuit = CreateObject_Obj2D("Circuit_09");
				Dot = CreateObject_Obj2D("ShiftDot");
				
				BSAuxFuncs.SetUnlitMaterial(Dot, 0.084f, 0.849f, 0.7308f);
				
				Material[] mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.792f, 0.792f, 0.792f),		
					BSAuxFuncs.GetUnlitMaterial(0.5566f, 0.5566f, 0.5566f),		
					BSAuxFuncs.GetUnlitMaterial(0.868f, 0.868f, 0.868f),		
					BSAuxFuncs.GetUnlitMaterial(0.74f, 0.74f, 0.74f),		
				};
				Circuit.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				
				var src_Point = CreateObject_Obj2D("SmallPointB");
				var src_Tangent = CreateObject_Obj2D("TanVtr");  
				var src_Line = CreateObject_Obj2D("AuxLine");
				var src_Arc = CreateObject_Obj2D("AuxArc");
				
				// Lines & Arcs, Tangents
				const int max = 8;
				pr_HF = max;
				PntArr = new THObject2D[max];
				TanArr = new THObject2D[max];
				Line = new THObject2D[max];
				Arc = new THObject2D[max];
				
				pr_PointMtrl = BSAuxFuncs.GetUnlitMaterial(0.23f, 0.41f, 0.90f);
				pr_TangentMtrl = BSAuxFuncs.GetUnlitMaterial(0.45f, 0.88f, 0.45f);
				src_Point.gameObject.GetComponent<MeshRenderer>().material = pr_PointMtrl;
				src_Tangent.gameObject.GetComponent<MeshRenderer>().material = pr_TangentMtrl;
				
				for (int i = 0; i < max; i++)
				{
					PntArr[i] = CreateObject_InsObj2D(src_Point.gameObject);
					TanArr[i] = CreateObject_InsObj2D(src_Tangent.gameObject);
				}
				
				
				for (int i = 0; i < max; i++)
				{
					Line[i] = CreateObject_InsObj2D(src_Line.gameObject);
				}
				
				for (int i = 0; i < max; i++)
				{
					Arc[i] = CreateObject_InsObj2D(src_Arc.gameObject);
				}
				
				
				VisCtrl = new THObject2D[pr_HF * 2];   
				for (int i = 0; i < pr_HF; i++)
				{
					Line[i].SetIdentity();
					Arc[i].SetIdentity();
					
					float s = 1.3f;
					PntArr[i].SetScale(0.35f  * s);
					TanArr[i].SetScale(0.67f * s, 0.60f * s);
					
					VisCtrl[i] = PntArr[i];             
					VisCtrl[pr_HF + i] = TanArr[i];
				}
				
				
				{	
					Vector2[] arrP = GetPositionArray(1);
					Vector2[] arrT = GetTangentArray(1);
					
					int numControl = arrP.Length;
					for (int i = 0; i < numControl; i++)
					{
						int i1 = (i + 1) % numControl;
						Vector2 P = arrP[i];
						Vector2 v = arrT[i];
						Vector2 Q = arrP[i1];
						Vector2 w = arrT[i1];
						
						Vector3 iR = CalcIntersectionOf2Lines(P, v, Q, w);
						InterpolatePQ(P, v, Q, w, iR);
					}
					
					for (int i = 0; i < arrP.Length; i++)
					{
						float deg = TH2DMath.CalcDegree_V1toV2(Vector2.right, arrT[i]);
						AddControl(arrP[i].x, arrP[i].y, deg);
					}
				}
				

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			BSData.c_SHOW_COORDINATE_SCALE = false;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				Dot.SetScale(0.2f);
				
				for (int i = 0; i < pr_HF; i++)
				{
					BSAuxFuncs.SetVisible(1, Line[i].gameObject);
					BSAuxFuncs.SetVisible(1, Arc[i].gameObject);
					Line[i].SetIdentity();
					Arc[i].SetIdentity();
					
					BSAuxFuncs.SetVisible(1, PntArr[i].gameObject);
					BSAuxFuncs.SetVisible(1, TanArr[i].gameObject);
				}
				
				SetCameraPosition_2D(0.0f, 0.0f, -13);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Vehicle.gameObject);
				Vehicle.SetIdentity();
				BSAuxFuncs.SetVisible(1, Circuit.gameObject);
				Circuit.SetIdentity();
				
				SetCameraPosition_2D(0, 0, -13);
			}
		}

	}

}
