//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGMeshData02;
using OQJ_RGAuxFuncs02;
using CellLoc = OQJ_Main02.CellLoc;
using Segment = OQJ_Main02.Segment;
using BasicCurve = OQJ_Main02.BasicCurve;



namespace OQJ_Sec227
{
	public class Sec227 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		
		THObject2D Disk;
		THObject2D Plane;
		
		THObject2D Arrow;
		
		THObject2D Needle;
		THObject2D Ball;
		
		THObject2D Body;
		THObject2D Rotor;
		
		THObject2D Red;
		THObject2D Green;
		THObject2D Blue;
		
		THObject2D Vehicle;
		THObject2D Circuit;
		
		THObject2D Square;
		THObject2D Maze;
		THObject2D Cover;
		
		THObject2D AuxCircle;
		THObject2D AuxLine;
		
		GameObject MainCamera;
		
		THObject2D Dot;
		


		// Instance Variables
		int i_degRev;
		int i_degRot;
		
		int i_forwardDegree;
		Vector2 i_forwardDir;
		
		int i_shmBall;
		int i_revBall;
		int i_degNeedle;
		
		int i_degBody;
		int i_degRotor;
		
		int i_shmB;
		int i_shmG;
		int i_shmR;
		
		readonly Vector2 c_attachPos_Rotor = new Vector2(0.0f, 0.340f);      
		readonly Vector2 c_attachPos_Green = new Vector2(0.0f, 3.45f);
		
		int[,] i_cellInfo;
		int i_moveCount;
		Vector2 i_moveVtr;
		float i_moveSpeed;
		float i_degSquare;
		
		BasicCurve Curve = null;
		float t;
		
		bool i_flag;	
		int pr_halfStageSize = 17;
		
		readonly Vector2 c_StartPosition = new Vector2(26, -30);
		readonly float c_cellLen = 2.0f;
		readonly int c_maxMoveCount = 20;




		void Code1()
		{
			i_degRot += 6;
			Quaternion rot = Quaternion.AngleAxis(i_degRot, Vector3.forward);
			
			i_degRev += 3;
			Quaternion rev = Quaternion.AngleAxis(i_degRev, Vector3.forward);
			Vector2 pos = rev * new Vector2(3, 0);
			pos += new Vector2(1, 2);
			
			Disk.transform.position = pos;
			Disk.transform.rotation = rot;
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_forwardDegree += 2;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_forwardDegree -= 2;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			Quaternion rot = Quaternion.AngleAxis(i_forwardDegree, Vector3.forward);
			Vector2 forwardDir = rot * new Vector2(0, 1);
			
			Vector2 curP = Plane.transform.position;
			Vector2 newP = (i_MOVE) ? curP + 0.20f * forwardDir : curP;
			
			Plane.transform.position = newP;
			Plane.transform.rotation = rot;
			
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Quaternion rot = Quaternion.AngleAxis(-60, Vector3.forward);
				Arrow.transform.rotation = rot;
				i_forwardDir = rot * new Vector2(0, 1);
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.C))
			{
				Arrow.transform.position = Vector2.zero;
				i_MOVE = false;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			Vector2 curP = Arrow.transform.position;
			Vector2 newP = (i_MOVE) ? curP + 0.1f * i_forwardDir : curP;
			Arrow.transform.position = newP;
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				Ball.transform.SetParent(Needle.transform);
				
				Needle.transform.localPosition = new Vector2(-4, 1);
				
				i_INITIALIZED = true;
			}
			
			i_shmBall += 9;
			float s = CalcShmValue(i_shmBall, 1.0f, 1.5f);
			Ball.transform.localScale = new Vector3(s, s, 1);
			
			i_revBall += 9;
			Quaternion rev = Quaternion.AngleAxis(i_revBall, Vector3.forward);
			Vector2 pos = rev * new Vector3(0, 1);
			pos += new Vector2(0, 4);
			Ball.transform.localPosition = pos;
			
			i_degNeedle -= 3;
			Needle.transform.localRotation = Quaternion.AngleAxis(i_degNeedle, Vector3.forward);
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				Rotor.transform.SetParent(Body.transform);
				
				Rotor.transform.localPosition = c_attachPos_Rotor;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			// Rotor
			i_degRotor += 12;
			Quaternion rotRotor = Quaternion.AngleAxis(i_degRotor, Vector3.forward);
			Rotor.transform.localRotation = rotRotor;
			
			// Body	
			Quaternion rotBody = Quaternion.AngleAxis(i_degBody, Vector3.forward);
			Vector2 direBody = rotBody * new Vector2(0, 1);
			
			Vector2 curP = Body.transform.localPosition;
			Vector2 newP = (i_MOVE) ? (curP + 0.075f * direBody) : curP;
			
			Body.transform.localPosition = newP;
			Body.transform.localRotation = rotBody;
			
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				Blue.transform.SetParent(Green.transform);
				Green.transform.SetParent(Red.transform);
				
				Green.transform.localPosition = c_attachPos_Green;
				Red.transform.localRotation = TH2DMath.GetRotation(20);
				
				i_INITIALIZED = true;
			}
			
			// Blue
			i_shmB += 4;
			float x = CalcShmValue(i_shmB, -4.2f, 4.2f);
			Blue.transform.localPosition = new Vector2(x, 0.2f);
			
			float s = CalcShmValue(i_shmB + 90, 1.0f, 2.0f);
			Blue.transform.localScale = new Vector3(1, s, 1);
			
			// Green
			i_shmG += 2;
			float deg = CalcShmValue(i_shmG, -30.0f, 30.0f);
			Green.transform.localRotation = TH2DMath.GetRotation(deg);
			
			// Red
			i_shmR += 1;
			x = CalcShmValue(i_shmR, 6.0f, 12.0f);
			Red.transform.localPosition = new Vector2(x, 3);
			
		}


		void Code7()
		{
			Vector3 E = MainCamera.transform.position;
			
			if (Input.GetKey(KeyCode.H))   
			{
				E.x -= 0.1f;	
			}
			else if (Input.GetKey(KeyCode.L))    
			{
				E.x += 0.1f;	
			}
			else if (Input.GetKey(KeyCode.J))    
			{
				E.y -= 0.1f;	
			}
			else if (Input.GetKey(KeyCode.K))   
			{
				E.y += 0.1f;	
			}
			else
			{
				return;
			}
			
			MainCamera.transform.position = E;
			
			Vector3 pos = new Vector3(E.x, E.y, 0);
			Dot.transform.position = pos;	
			
		}


		void Code8()
		{
			if (!i_INITIALIZED)
			{
				Curve = ConstructCurve();
				
				t = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.K))
			{
				t += 0.001f;
				if(t > 1.0f) { t -= 1.0f; }
			}
			else if (Input.GetKey(KeyCode.J))
			{
				t -= 0.001f;
				if(t < 0.0f) { t += 1.0f; }
			}
			
			Vector2[] vrr = Curve.ComputePositionAndTangent(t);
			Vector2 pos = vrr[0];
			Vector2 dir = vrr[1];
			
			Quaternion rot = TH2DMath.CalcRotation_V1toV2(Vector2.up, dir);
			Vehicle.transform.position = pos;
			Vehicle.transform.rotation = rot;
			
			Vector3 E = new Vector3(pos.x, pos.y, -7.2f);
			MainCamera.transform.position = E;
		}


		void Code9()
		{
			if (!i_INITIALIZED)
			{
				Square.transform.position = c_StartPosition;
				
				i_INITIALIZED = true;
			}
			
			MoveSquare();
			
			Vector2 pos = Square.transform.position;
			
			Vector3 E = new Vector3(pos.x, pos.y, -20.0f);
			MainCamera.transform.position = E;
			
			if (Input.GetKeyDown(KeyCode.C))
			{
				i_flag = !i_flag;	
				ShowBlackCover(i_flag);
			}
			
		}




		
		float CalcShmValue(float deg, float min, float max)
		{
			float A = max - min;
			float val = 0.5f * A * (-Mathf.Cos(deg * Mathf.Deg2Rad) + 1.0f) + min;
			return val;
		}
		
		
		Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			return RGAuxFuncs02.CalcIntersectionOf2Lines(P, va, Q, vb);
		}
		
		
		
		void InterpolatePQ_Data(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3, List<Segment> segments)  
		{
			RGAuxFuncs02.InterpolatePQ_Data(P, v, Q, w, R3, segments);
		}
		
		
		BasicCurve ConstructCurve()
		{
			if(Curve != null) { return Curve; }
			
			Vector2[] ps = RGMeshData02.GetVertsData(RGMeshData02.c_CIRCUIT_SPINE);
			
			Vector2[] dirs = new Vector2[(ps.Length - 1) / 2 + 1];
			for (int i = 0; i < dirs.Length - 1; i++)
			{
				int n = i * 2;
				dirs[i] = (ps[n + 1] - ps[n]).normalized;
			}
			
			dirs[dirs.Length - 1] = dirs[0];
			
			List<Segment> segments = new List<Segment>();
			for (int i = 0; i < ps.Length; i++)
			{
				int i1 = (i + 1) % ps.Length;
				int n = i / 2;
				int n1 = i1 / 2;
				Vector2 P = ps[i];
				Vector2 v = dirs[n];
				Vector2 Q = ps[i1];
				Vector2 w = dirs[n1];
				
				Vector3 iR = CalcIntersectionOf2Lines(P, v, Q, w);      
				InterpolatePQ_Data(P, v, Q, w, iR, segments);
			}
			
			Curve = new BasicCurve(segments);
			return Curve;
		}
		
		
		void MoveSquare()
		{
			if (i_MOVE)  // セル移動処理
			{
				i_moveCount++;
				Vector2 wp = Square.transform.position;
				Vector2 newSqrPos = wp + i_moveVtr;
				
				Quaternion rotSqr = Quaternion.AngleAxis(i_degSquare, Vector3.forward);
				Square.transform.position = newSqrPos;
				Square.transform.rotation = rotSqr;
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKey(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKey(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKey(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKey(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			// Square vs Block
			Vector2 posSqr = Square.transform.position;
			bool CANCEL_MOVE = CollisionTest_vs_Block(posSqr, i_moveVtr);
			
			if (CANCEL_MOVE)
			{
				i_MOVE = false;
				Quaternion rotSqr = Quaternion.AngleAxis(i_degSquare, Vector3.forward);
				Square.transform.position = posSqr;
				Square.transform.rotation = rotSqr;
			}
		}	
		
		
		void ShowBlackCover(bool b)
		{
			int vi = b ? 1 : 0;
			BSAuxFuncs.SetVisible(vi, Cover.gameObject);
		}
		
		
		bool CollisionTest_vs_Block(Vector2 pos, Vector2 move_vtr)
		{
			CellLoc nextLoc = GetCellLocation(pos);
			
			float eps = 0.001f;
			if (move_vtr.y > eps)
			{
				nextLoc.row--;
			}
			else if (move_vtr.y < -eps)
			{
				nextLoc.row++;
			}
			else if (move_vtr.x < -eps)
			{
				nextLoc.col--;
			}
			else if (move_vtr.x > eps)
			{
				nextLoc.col++;
			}
			
			
			// 進もうとしているセルがBlockかどうか
			if (i_cellInfo[nextLoc.row, nextLoc.col] == 0)
			{
				return true;
			}
			
			return false;
		}
		
		
		CellLoc GetCellLocation(Vector2 pos)
		{
			int row = pr_halfStageSize - Mathf.RoundToInt(pos.y / c_cellLen);
			int col = pr_halfStageSize + Mathf.RoundToInt(pos.x / c_cellLen);
			
			return new CellLoc(row, col);
		}
		


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

				case 8 :	Code8();  break;

				case 9 :	Code9();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			
			if (MatchCase(4))
			{
				THMatrix3x3 M = Needle.GetMatrix() * TH2DMath.GetTranslation3x3(0, 4);
				AuxCircle.SetMatrix(M);
			}
			else if (MatchCase(8))
			{
				var wp = Vehicle.transform.position;
				wp.z = -0.006f;
				Vehicle.transform.position = wp;
			}
			else if (MatchCase(9))
			{
				Vector3 pos = Square.GetPosition();
				pos.z = -0.01f;
				Square.transform.position = pos;
				Cover.transform.position = pos;
			}
			
			BSData.CameraController2D.UpdateCameraData();
			
		}


		static Sec227 pr_instance = new Sec227(9);

		public static Sec227 PRGSI()
		{
			return pr_instance;
		}


		private Sec227(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Disk = CreateObject_Obj2D("SmallDiskB");
				Plane = CreateObject_Obj2D("CombatPlane");
				Arrow = CreateObject_Obj2D("Arrow2");
				Needle = CreateObject_Obj2D("Needle");
				Ball = CreateObject_Obj2D("Ball");
				Body = CreateObject_Alpha2D("HeliBody2D");
				Rotor = CreateObject_Obj2D("HeliRotor2D");
				Red = CreateObject_Obj2D("moR");
				Green = CreateObject_Obj2D("moG");
				Blue = CreateObject_Obj2D("moB");
				Dot = CreateObject_Obj2D("UnitDiskB");
				Vehicle = CreateObject_Obj2D("Vehicle");
				Circuit = CreateObject_Obj2D("Circuit_24");
				Square = CreateObject_Obj2D("Square3");
				Maze = CreateObject_Obj2D("DMaze");
				Cover = CreateObject_Obj2D("BlackCover");

				BSAuxFuncs.SetUnlitMaterial(Cover, 0.15f);
				
				Material[] mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.705f, 0.705f, 0.705f),
					BSAuxFuncs.GetUnlitMaterial(0.7452f, 0.7452f, 0.7452f),
					BSAuxFuncs.GetUnlitMaterial(0.5566f, 0.5566f, 0.5566f),
					BSAuxFuncs.GetUnlitMaterial(0.8396f, 0.8396f, 0.8396f),
				};
				Circuit.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				
				mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.0754f, 0.0754f, 0.0754f),
					BSAuxFuncs.GetUnlitMaterial(0.3207f, 0.1744f, 0.1679f),
					BSAuxFuncs.GetUnlitMaterial(0.3604f, 0.566f, 0.4276f),
					BSAuxFuncs.GetUnlitMaterial(0.4464f, 0.7452f, 0.4944f),
					BSAuxFuncs.GetUnlitMaterial(0.11f, 0.39f, 0.80f),
					BSAuxFuncs.GetUnlitMaterial(0.54f, 0.687f, 0.887f),
					BSAuxFuncs.GetUnlitMaterial(0.2364f, 0.2447f, 0.3113f)
				};
				Maze.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				
				AuxCircle = CreateObject_Obj2D("AuxCircle");
				
				AuxLine = CreateObject_Obj2D("Line60");
				Mesh mesh = AuxLine.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { Vector2.zero, Vector2.up });
				
				BSAuxFuncs.SetUnlitMaterial(Arrow, 0.28f, 0.915f, 0.63f);
				
				// Main Camera
				MainCamera = BSAuxFuncs.GetMainCamera();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			BSData.c_SHOW_COORDINATE_SCALE = true;
			i_frameCount = 0;
			
			SetFPS(60);
			
			if (MatchCase(1))
			{
				SetFPS(30);
				
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Disk.SetIdentity();
				
				AuxCircle.SetMatrix(TH2DMath.GetTranslation3x3(1, 2) *
				TH2DMath.GetScale3x3(3));
				
				i_degRev = 0;
				i_degRot = 0;
				
				SetOutputImageSize(7.5f, 400);
				SetCameraPosition_2D(0, 0);
			}
			else if (MatchCase(2))
			{
				SetFPS(30);
				
				BSAuxFuncs.SetVisible(1, Plane.gameObject);
				Plane.SetIdentity();
				
				i_forwardDegree = 0;
				
				SetOutputImageSize(12.6f, 400);
				SetCameraPosition_2D(0, 0);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Arrow.gameObject);
				Arrow.SetIdentity();
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				AuxLine.SetIdentity();
				
				AuxLine.transform.rotation = TH2DMath.GetRotation(-60);
				AuxLine.transform.localScale = new Vector3(1, 40, 1);
				
				SetOutputImageSize(10.2f, 400);     
				SetCameraPosition_2D(10, 6);
			}
			else if (MatchCase(4))
			{
				SetFPS(30);
				CrdSys.transform.position = new Vector3(0, 0, 0.002f);
				
				BSAuxFuncs.SetVisible(1, Needle.gameObject);
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Needle.SetIdentity();
				Ball.SetIdentity();
				AuxCircle.SetIdentity();
				
				i_shmBall = 0;
				i_revBall = 0;
				i_degNeedle = 0;
				
				SetOutputImageSize(10.2f, 400);
				SetCameraPosition_2D(-4, 1);
			}
			else if (MatchCase(5))
			{
				SetFPS(30);
				
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Rotor.gameObject);
				
				Body.SetIdentity();
				Rotor.SetIdentity();
				
				i_degBody = 0;
				i_degRotor = 0;
				i_MOVE = false;
				
				SetOutputImageSize(10.2f, 400);
				SetCameraPosition_2D(0, 0);
			}
			else if (MatchCase(6))
			{
				SetFPS(30);
				
				BSAuxFuncs.SetVisible(1, Red.gameObject);
				BSAuxFuncs.SetVisible(1, Green.gameObject);
				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				
				Red.SetIdentity();
				Green.SetIdentity();
				Blue.SetIdentity();
				
				SetOutputImageSize(10.2f, 400);			
				SetCameraPosition_2D(10, 6);
			}
			else if (MatchCase(7))
			{
				SetFPS(30);
				
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				Dot.transform.localScale = new Vector3(0.3f, 0.3f, 1.0f);
				
				SetCameraPosition_2D(0, 0, -18);
			}
			else if (MatchCase(8))
			{
				BSData.c_SHOW_COORDINATE_SCALE = false;
				CrdSys.SetScale(1.5f);
				
				BSAuxFuncs.SetVisible(1, Vehicle.gameObject);
				Vehicle.SetIdentity();
				BSAuxFuncs.SetVisible(1, Circuit.gameObject);
				Circuit.SetIdentity();
				
				Vehicle.transform.localScale = new Vector3(0.8f, 0.8f, 1.0f);
				
				SetCameraPosition_2D(0, 0, -18);
			}
			else if (MatchCase(9))
			{
				SetFPS(60);
				
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetIdentity();
				BSAuxFuncs.SetVisible(1, Maze.gameObject);
				Maze.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cover.gameObject);
				Cover.SetIdentity();
				Cover.SetScale(6, 6);
				i_flag = true;
				
				i_moveSpeed = c_cellLen / c_maxMoveCount;
				i_cellInfo = RGMeshData02.GetStageInfo(0);
				
				SetCameraPosition_2D(c_StartPosition.x, c_StartPosition.y, -20);    
			}
		}

	}

}
