//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGMeshData02;
using OQJ_RGAuxFuncs02;
using CellLoc = OQJ_Main02.CellLoc;
using Segment = OQJ_Main02.Segment;
using BasicCurve = OQJ_Main02.BasicCurve;



namespace OQJ_Sec228
{
	public class Sec228 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		THObject2D AxisXY;
		
		THObject2D Triangle;
		THObject2D Rect;
		
		THObject2D Vehicle;
		THObject2D Circuit;
		
		THObject2D Square;
		THObject2D Maze;
		THObject2D Cover;
		
		THObject2D MiniMap;
		THObject2D MiniMap_Cr;
		THObject2D MiniMap_Mz;
		THObject2D Dot;
		
		THObject2D GridLines_XY;
		
		GameObject MainCamera;
		GameObject Text_Pixel;


		// Instance Variables
		float i_WldPerPix;
		float i_cx;
		float i_cy;
		
		int i_locX;
		int i_locY;
		int i_width;
		int i_height;
		
		BasicCurve Curve;
		float t;
		
		Vector3 i_vEF;
		
		int[,] i_cellInfo;
		int i_moveCount;
		Vector2 i_moveVtr;
		float i_moveSpeed;
		float i_degSquare;
		
		readonly Vector2 c_StartPosition = new Vector2(26, -30);
		readonly float c_cellLen = 2.0f;
		readonly int c_maxMoveCount = 20;
		
		int pr_halfStageSize = 17;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				float H2 = 9.0f;
				
				float FOV = 60.0f;
				float tanT = Mathf.Tan(FOV * 0.5f * Mathf.Deg2Rad);
				float d = H2 / tanT;
				
				MainCamera.transform.position = new Vector3(0, 0, -d);
				
				i_INITIALIZED = true;
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				float aspect = Screen.width / (float)Screen.height;
				float W2 = 12.0f;
				float H2 = W2 / aspect;
				
				float FOV = 60.0f;
				float tanT = Mathf.Tan(FOV * 0.5f * Mathf.Deg2Rad);
				float d = H2 / tanT;
				
				MainCamera.transform.position = new Vector3(0, 0, -d);
				
				i_INITIALIZED = true;
			}
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Vector3 pos = MainCamera.transform.position;
				float d = -pos.z;
				
				float aspect = Screen.width / (float)Screen.height;
				float FOV = 60.0f;
				float tanT = Mathf.Tan(FOV * 0.5f * Mathf.Deg2Rad);
				float H2 = d * tanT;
				float W2 = H2 * aspect;
				
				Triangle.transform.localScale = new Vector3(W2, H2, 1);
				Triangle.transform.position = new Vector2(0, 0);
				
				i_INITIALIZED = true;
			}
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				Vector3 E = MainCamera.transform.position;
				float d = -E.z;
				
				float FOV = 60.0f;
				float theta = FOV * 0.5f * Mathf.Deg2Rad;
				float w2 = Screen.width * 0.5f;
				float h2 = Screen.height * 0.5f;
				float cx = w2;
				float cy = h2;
				
				float H2 = d * Mathf.Tan(theta);
				float WldPerPix = H2 / h2;
				
				float mx = 200;
				float my = 100;
				float rw = 250;
				float rh = 150;
				float dx = mx - cx;
				float dy = my - cy;
				
				float RW = rw * WldPerPix;
				float RH = rh * WldPerPix;
				float s  = dx * WldPerPix;
				float t  = dy * WldPerPix;
				
				Vector2 P = E;
				Vector2 M = P + new Vector2(s, t);
				
				Rect.transform.position = M;
				Rect.transform.localScale = new Vector3(RW, RH, 1);
				
				i_INITIALIZED = true;
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				Vector3 E = MainCamera.transform.position;
				float d = -E.z;
				
				float FOV = 60.0f;
				float theta = FOV * 0.5f * Mathf.Deg2Rad;
				float w2 = Screen.width * 0.5f;
				float h2 = Screen.height * 0.5f;
				i_cx = w2;
				i_cy = h2;
				
				float H2 = d * Mathf.Tan(theta);
				i_WldPerPix = H2 / h2;
				
				i_locX   = 1;
				i_locY   = 1;
				i_width  = 2;
				i_height = 2;
				
				i_INITIALIZED = true;
			}
			
			
			bool shift = THUtil.IsShiftDown();
			
			if (Input.GetKeyDown(KeyCode.H))
			{
				if (shift) { i_width--; }
				else       { i_locX--; }
			}
			else if (Input.GetKeyDown(KeyCode.L))
			{
				if (shift) { i_width++; }
				else       { i_locX++; }
			}
			else if (Input.GetKeyDown(KeyCode.J))
			{
				if (shift) { i_height--; }
				else       { i_locY--; }
			}
			else if (Input.GetKeyDown(KeyCode.K))
			{
				if (shift) { i_height++; }
				else       { i_locY++; }
			}
			
			i_locX   = Mathf.Clamp(i_locX, 0, 20);
			i_locY   = Mathf.Clamp(i_locY, 0, 20);
			i_width  = Mathf.Clamp(i_width, 1, 20);
			i_height = Mathf.Clamp(i_height, 1, 20);
			
			float mx = i_locX * 50;
			float my = i_locY * 50;
			float rw = i_width * 50;
			float rh = i_height * 50;
			float dx = mx - i_cx;
			float dy = my - i_cy;
			
			float RW = rw * i_WldPerPix;
			float RH = rh * i_WldPerPix;
			float s  = dx * i_WldPerPix;
			float t  = dy * i_WldPerPix;
			
			Vector2 P = MainCamera.transform.position;
			Vector2 M = P + new Vector2(s, t);
			
			Rect.transform.position = M;
			Rect.transform.localScale = new Vector3(RW, RH, 1);
			
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				Curve = ConstructCurve();
				
				t = 0.0f;
				i_MOVE = true;
				
				Dot.transform.SetParent(MiniMap.transform);
				
				MiniMap.transform.localPosition = new Vector3(0, 0, -0.02f);
				MiniMap.transform.localScale = new Vector3(0.5f, 0.5f, 1);
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			if (!i_MOVE) { return; }
			
			t += 0.001f;
			if (t > 1.0f) { t -= 1.0f; }
			
			Vector2[] vrr = Curve.ComputePositionAndTangent(t);
			Vector2 pos = vrr[0];
			Vector2 dir = vrr[1];
			
			Quaternion rot = TH2DMath.CalcRotation_V1toV2(Vector2.up, dir);
			Vehicle.transform.position = new Vector3(pos.x, pos.y, -0.01f);   
			Vehicle.transform.rotation = rot;
			
			Dot.transform.localPosition = new Vector3(pos.x, pos.y, -0.01f);
			
			Vector3 mp = MiniMap.transform.localPosition;
			if (Input.GetKey(KeyCode.H))
			{
				mp.x -= 0.05f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				mp.x += 0.05f;
			}
			if (Input.GetKey(KeyCode.J))
			{
				mp.y -= 0.05f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				mp.y += 0.05f;
			}
			
			MiniMap.transform.localPosition = mp;
			
		}


		void Code7()
		{
			if (!i_INITIALIZED)
			{
				Curve = ConstructCurve();
				
				t = 0.0f;
				
				Vector3 E0 = new Vector3(0, 0, -7.2f);
				Vector3 F0 = new Vector3(-5.0f, 1.6f, -0.02f);
				i_vEF = F0 - E0;
				
				Dot.transform.SetParent(MiniMap.transform);
				MiniMap.transform.localScale = new Vector3(0.055f, 0.055f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.K))
			{
				t += 0.001f;
				if (t > 1.0f) { t -= 1.0f; }
			}
			else if (Input.GetKey(KeyCode.J))
			{
				t -= 0.001f;
				if (t < 0.0f) { t += 1.0f; }
			}
			
			Vector2[] vrr = Curve.ComputePositionAndTangent(t);
			Vector2 pos = vrr[0];
			Vector2 dir = vrr[1];
			
			Quaternion rot = TH3DMath.CalcRotation_V1toV2(Vector2.up, dir);
			Vehicle.transform.position = new Vector3(pos.x, pos.y, -0.01f);
			Vehicle.transform.rotation = rot;
			
			Vector3 E = pos;
			E.z = -7.2f;
			MainCamera.transform.position = E;
			
			Dot.transform.localPosition = new Vector3(pos.x, pos.y, -0.01f);
			
			Vector3 F = E + i_vEF;
			MiniMap.transform.localPosition = F;
			
		}


		void Code8()
		{
			if (!i_INITIALIZED)
			{
				MiniMap.SetMatrix(THMatrix3x3.identity);
				
				i_INITIALIZED = true;
			}
		}


		void Code9()
		{
			if (!i_INITIALIZED)
			{
				Square.SetPosition(c_StartPosition);
				
				Vector3 E0 = new Vector3(0, 0, -20);
				Vector3 F0 = new Vector3(-14.4f, 3.0f, -0.02f);
				i_vEF = F0 - E0;
				
				Dot.transform.SetParent(MiniMap_Mz.transform);
				MiniMap.transform.localScale = new Vector3(0.11f, 0.11f, 1.0f);
				
				i_INITIALIZED = true;
			}
			
			MoveSquare();
			
			Vector2 pos = Square.transform.position;
			
			Vector3 E = new Vector3(pos.x, pos.y, -20.0f);
			MainCamera.transform.position = E;
			
			Dot.transform.localPosition = new Vector3(pos.x, pos.y, -0.01f);
			
			Vector3 F = E + i_vEF;
			MiniMap.transform.localPosition = F;
			
		}




		Vector3 CalcIntersectionOf2Lines(Vector2 P, Vector2 va, Vector2 Q, Vector2 vb)
		{
			return RGAuxFuncs02.CalcIntersectionOf2Lines(P, va, Q, vb);
		}
		
		
		
		void InterpolatePQ_Data(Vector2 P, Vector2 v, Vector2 Q, Vector2 w, Vector3 R3, List<Segment> segments)  
		{
			RGAuxFuncs02.InterpolatePQ_Data(P, v, Q, w, R3, segments);
		}
		
		
		BasicCurve ConstructCurve()
		{
			if(Curve != null) { return Curve; }
			
			Vector2[] ps = RGMeshData02.GetVertsData(RGMeshData02.c_CIRCUIT_SPINE);
			
			Vector2[] dirs = new Vector2[(ps.Length - 1) / 2 + 1];
			for (int i = 0; i < dirs.Length - 1; i++)
			{
				int n = i * 2;
				dirs[i] = (ps[n + 1] - ps[n]).normalized;
			}
			
			dirs[dirs.Length - 1] = dirs[0];
			
			List<Segment> segments = new List<Segment>();
			for (int i = 0; i < ps.Length; i++)
			{
				int i1 = (i + 1) % ps.Length;
				int n = i / 2;
				int n1 = i1 / 2;
				Vector2 P = ps[i];
				Vector2 v = dirs[n];
				Vector2 Q = ps[i1];
				Vector2 w = dirs[n1];
				
				Vector3 iR = CalcIntersectionOf2Lines(P, v, Q, w);      
				InterpolatePQ_Data(P, v, Q, w, iR, segments);
			}
			
			Curve = new BasicCurve(segments);
			return Curve;
		}
		
		
		void MoveSquare()
		{
			if (i_MOVE)  // セル移動処理
			{
				i_moveCount++;
				Vector2 newSqrPos = Square.GetPosition() + i_moveVtr;
				
				Quaternion rotSqr = Quaternion.AngleAxis(i_degSquare, Vector3.forward);
				Square.transform.position = newSqrPos;
				Square.transform.rotation = rotSqr;
				
				if (i_moveCount == c_maxMoveCount)
				{   // あるセルに到達
					i_MOVE = false;
				}
				
				return;
			}
			
			
			if (Input.GetKey(KeyCode.H))    // 左のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(-i_moveSpeed, 0.0f);
				i_degSquare = 90;
			}
			else if (Input.GetKey(KeyCode.L))    // 右のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(i_moveSpeed, 0.0f);
				i_degSquare = 270;
			}
			else if (Input.GetKey(KeyCode.J))    // 下のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, -i_moveSpeed);
				i_degSquare = 180;
			}
			else if (Input.GetKey(KeyCode.K))    // 上のセルへ進む
			{
				i_MOVE = true;
				i_moveCount = 0;
				i_moveVtr = new Vector2(0.0f, i_moveSpeed);
				i_degSquare = 0;
			}
			else
			{
				// キーが押されていないときは、以降の処理は行わない
				return;
			}
			
			
			// Square vs Block
			Vector2 posSqr = Square.GetPosition();
			bool CANCEL_MOVE = CollisionTest_vs_Block(posSqr, i_moveVtr);
			
			if (CANCEL_MOVE)
			{
				i_MOVE = false;
				Quaternion rotSqr = Quaternion.AngleAxis(i_degSquare, Vector3.forward);
				Square.transform.position = posSqr;
				Square.transform.rotation = rotSqr;
			}
		}	
		
		
		bool CollisionTest_vs_Block(Vector2 pos, Vector2 move_vtr)
		{
			CellLoc nextLoc = GetCellLocation(pos);
			
			float eps = 0.001f;
			if (move_vtr.y > eps)
			{
				nextLoc.row--;
			}
			else if (move_vtr.y < -eps)
			{
				nextLoc.row++;
			}
			else if (move_vtr.x < -eps)
			{
				nextLoc.col--;
			}
			else if (move_vtr.x > eps)
			{
				nextLoc.col++;
			}
			
			
			// 進もうとしているセルがBlockかどうか
			if (i_cellInfo[nextLoc.row, nextLoc.col] == 0)
			{
				return true;
			}
			
			return false;
		}
		
		
		CellLoc GetCellLocation(Vector2 pos)
		{
			int row = pr_halfStageSize - Mathf.RoundToInt(pos.y / c_cellLen);
			int col = pr_halfStageSize + Mathf.RoundToInt(pos.x / c_cellLen);
			
			return new CellLoc(row, col);
		}
		


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

				case 8 :	Code8();  break;

				case 9 :	Code9();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(4, 5))
			{
				if (GameObject.Find("PixelCoord") != null)
				{
					int px = Mathf.RoundToInt(Input.mousePosition.x);
					int py = Mathf.RoundToInt(Input.mousePosition.y);
					if (!(px > Screen.width || px < 0 || py > Screen.height || py < 0))
					{
						BSAuxFuncs.Update_PixelCoord(px, py);
					}
				}
			}
			else if (MatchCase(9))
			{
				Vector3 pos = Square.GetPosition();
				pos.z = -0.01f;
				Square.transform.position = pos;
				Cover.transform.position = pos;
			}
			
			BSData.CameraController2D.UpdateCameraData();
		}


		static Sec228 pr_instance = new Sec228(9);

		public static Sec228 PRGSI()
		{
			return pr_instance;
		}


		private Sec228(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				AxisXY = CreateObject_Obj2D("XY_Axes");
				Triangle = CreateObject_Obj2D("UnitTri_LL");
				Rect = CreateObject_Obj2D("UnitRect_LL");
				Vehicle = CreateObject_Obj2D("Vehicle");
				Circuit = CreateObject_Obj2D("Circuit_24");
				MiniMap_Cr = CreateObject_Obj2D("MiniMap_Cr");
				Square = CreateObject_Obj2D("Square3");
				Maze = CreateObject_Obj2D("DMaze");
				Cover = CreateObject_Obj2D("BlackCover");
				MiniMap_Mz = CreateObject_Obj2D("MiniMap_Mz");
				Dot = CreateObject_Obj2D("ShiftDot");
				
				BSAuxFuncs.SetUnlitMaterial(Cover, 0.15f);
				
				Material[] mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.705f, 0.705f, 0.705f),
					BSAuxFuncs.GetUnlitMaterial(0.7452f, 0.7452f, 0.7452f),
					BSAuxFuncs.GetUnlitMaterial(0.5566f, 0.5566f, 0.5566f),
					BSAuxFuncs.GetUnlitMaterial(0.8396f, 0.8396f, 0.8396f),
				};
				Circuit.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.0754f, 0.0754f, 0.0754f),
					BSAuxFuncs.GetUnlitMaterial(0.3207f, 0.1744f, 0.1679f),
					BSAuxFuncs.GetUnlitMaterial(0.3604f, 0.566f, 0.4276f),
					BSAuxFuncs.GetUnlitMaterial(0.4464f, 0.7452f, 0.4944f),
					BSAuxFuncs.GetUnlitMaterial(0.11f, 0.39f, 0.80f),
					BSAuxFuncs.GetUnlitMaterial(0.54f, 0.687f, 0.887f),
					BSAuxFuncs.GetUnlitMaterial(0.2364f, 0.2447f, 0.3113f)
				};
				Maze.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.69f, 0.69f, 0.712f),
					BSAuxFuncs.GetUnlitMaterial(0.52f, 0.57f, 0.75f),
				};
				MiniMap_Cr.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				mtrls = new Material[]
				{
					BSAuxFuncs.GetUnlitMaterial(0.54f, 0.687f, 0.887f),
					BSAuxFuncs.GetUnlitMaterial(0.4464f, 0.7452f, 0.4944f),
					BSAuxFuncs.GetUnlitMaterial(0.30f, 0.30f, 0.30f),
				};
				MiniMap_Mz.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				GridLines_XY = CreateObject_Obj2D("GridLines_XY");
				GridLines_XY.gameObject.GetComponent<MeshRenderer>().material.renderQueue = 1900;
				
				BSAuxFuncs.SetUnlitMaterial(Triangle, 0.375f, 0.981f, 0.46f);
				BSAuxFuncs.SetUnlitMaterial(Rect, 0.6155f, 0.9816f, 0.7044f);
				
				Text_Pixel = BSAuxFuncs.InitText_PixelCoord(1.0f);
				
				MainCamera = BSAuxFuncs.GetMainCamera();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			BSAuxFuncs.SetVisible(0, Text_Pixel.gameObject);	
			BSData.c_SHOW_COORDINATE_SCALE = true;
			
			i_frameCount = 0;
			SetFPS(60);
			
			Dot.transform.SetParent(null);
			
			if (MatchCase(1, 2))
			{
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Triangle.gameObject);
				Triangle.SetIdentity();
				
				SetOutputImageSize(9.4f, 400);
				SetCameraPosition_2D(0, 0);         
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(0, CrdSys.gameObject);
				BSData.c_SHOW_COORDINATE_SCALE = false;
				
				BSAuxFuncs.SetVisible(1, AxisXY.gameObject);
				AxisXY.SetIdentity();
				AxisXY.transform.position = new Vector3(0, 0, 0.001f);
				
				Vector2 E = MatchCase(4) ? new Vector2(-2, -3) : new Vector2(5, 2);
				
				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				Rect.SetIdentity();
				BSAuxFuncs.SetVisible(1, GridLines_XY.gameObject);
				GridLines_XY.SetIdentity();
				GridLines_XY.SetPosition(E);
				
				BSAuxFuncs.SetVisible(1, Text_Pixel.gameObject);
				
				SetCameraPosition_2D(E.x, E.y, -16);
				BSAuxFuncs.MoveCamera_PixelCoord();  
			}
			
			else if (MatchCase(6))
			{
				BSData.c_SHOW_COORDINATE_SCALE = false;
				CrdSys.SetScale(1.5f);
				
				BSAuxFuncs.SetVisible(1, Vehicle.gameObject);
				Vehicle.SetIdentity();
				BSAuxFuncs.SetVisible(1, Circuit.gameObject);
				Circuit.SetIdentity();
				
				
				BSAuxFuncs.SetVisible(1, MiniMap_Cr.gameObject);
				MiniMap_Cr.SetIdentity();
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				
				MiniMap = MiniMap_Cr;
				
				SetCameraPosition_2D(0, 2.8f, -34);			
			}
			else if (MatchCase(7))
			{
				BSData.c_SHOW_COORDINATE_SCALE = false;
				CrdSys.SetScale(1.5f);
				
				BSAuxFuncs.SetVisible(1, Vehicle.gameObject);
				Vehicle.SetIdentity();
				BSAuxFuncs.SetVisible(1, Circuit.gameObject);
				Circuit.SetIdentity();
				
				Vehicle.transform.localScale = new Vector3(0.8f, 0.8f, 1.0f);
				
				
				BSAuxFuncs.SetVisible(1, MiniMap_Cr.gameObject);
				MiniMap_Cr.SetIdentity();
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				
				MiniMap = MiniMap_Cr;
			}
			else if (MatchCase(8))
			{
				BSAuxFuncs.SetVisible(1, MiniMap_Mz.gameObject);
				MiniMap_Mz.SetIdentity();
				
				MiniMap = MiniMap_Mz;
				
				SetCameraPosition_2D(0, 0, -20);	
			}
			else if (MatchCase(9))
			{
				SetFPS(60);
				
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetIdentity();
				BSAuxFuncs.SetVisible(1, Maze.gameObject);
				Maze.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cover.gameObject);
				Cover.SetIdentity();
				Cover.SetScale(6.0f);
				
				Square.SetPosition(0, 0);
				
				BSAuxFuncs.SetVisible(1, MiniMap_Mz.gameObject);
				MiniMap_Mz.SetIdentity();
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				
				MiniMap = MiniMap_Mz;
				
				i_moveSpeed = c_cellLen / c_maxMoveCount;
				i_cellInfo = RGMeshData02.GetStageInfo(0);
				
				SetCameraPosition_2D(c_StartPosition.x, c_StartPosition.y, -20);	
			}
		}

	}

}
