//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_Main02;
using OQJ_THAlphaObject2D;
using OQJ_RGMeshData02;
using static OQJ_RGAuxFuncs02.RGAuxFuncs02;
using static OQJ_BSMouseListener.BSMouseListener;



namespace OQJ_Sec229
{
	public class Sec229 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;
		THObject2D CS_AF;
		
		THObject2D Dot;
		THObject2D[] Card = new THObject2D[4];
		
		THObject2D Board;
		THObject2D Board_B;
		THObject2D Board_G;
		
		THAlphaObject2D Moon;
		THAlphaObject2D Star;
		THAlphaObject2D Ellipse;
		THAlphaObject2D Letter;
		THAlphaObject2D Block;
		THAlphaObject2D Disk;
		
		THObject2D Body;
		THObject2D Rotor;
		THObject2D Bubble;
		
		THObject2D AuxLine;
		THObject2D TmpSqEd;
		
		GameObject MainCamera;
		GameObject Text_Pixel;
		
		THComponentObject[] CharRect = new THComponentObject[16];  
		THObject3D[] LLBoundary = new THObject3D[16];


		// Instance Variables
		int i_degBody = 0;
		int i_degRotor = 0;
		readonly Vector2 c_attachPos_Rotor = new Vector2(0.0f, 0.340f);      
		
		bool i_SET_VIEW;
		
		int i_turnCardIndex = -1;
		
		THAlphaObject2D i_selectObject;
		Vector2 i_M1;
		
		readonly Vector2[] c_CardMinMax = 
		{ 
		new Vector2(-1.5f, -2.5f),	new Vector2(1.5f,  2.5f)
		};
		
		readonly float c_BoardW = 8; 
		readonly float c_BoardH = 6;
		readonly Vector2 c_BoardCenter = Vector2.zero;
		
		Vector2 A, B, C, D;
		
		readonly Vector2 pr_posQ1 = new Vector2(9, 5);
		readonly Vector2 pr_posQ2 = new Vector2(-12, 6);
		readonly Vector2 pr_posQ3 = new Vector2(-12, -5);
		readonly Vector2 pr_posQ4 = new Vector2(10, -5.5f);
		
		readonly Vector2[] c_initRectBounds1 = { new Vector2(-3.0f, -0.6f), new Vector2(3.0f, 0.6f) };
		
		THAlphaObject2D[] i_targets = new THAlphaObject2D[0];
		
		int pr_turnCounter;
		
		float pr_shm;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				float RW = 0.4f;
				float RH = 1.2f;
				Vector3 V0 = new Vector3(12, 8, -0.02f);
				
				for (int i = 0; i < 8; i++)
				{
					Vector3 LL = V0 + new Vector3(i * RW, 0, 0);
					
					CharRect[i].transform.localPosition = LL;
					CharRect[i].transform.localScale = new Vector3(RW, RH, 1);
					CharRect[i].SetTexture("images/sample_letters_2");
				}
				
				i_INITIALIZED = true;
			}
			
			System.DateTime time = System.DateTime.Now;
			int hour = time.Hour;  
			int min = time.Minute;
			int sec = time.Second;
			string sH = (hour < 10) ? "0" + hour : "" + hour;
			string sM = (min < 10) ? "0" + min : "" + min;
			string sS = (sec < 10) ? "0" + sec : "" + sec;
			
			string str = sH + ":" + sM + ":" + sS;
			for (int i = 0; i < str.Length; i++)
			{
				CharRect[i].SetUVs(GetUVs(str[i]));
			}
			
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				float RW = 0.32f;
				float RH = 0.94f; 
				Vector3 V0 = new Vector3(1.4f, 1.5f, -0.01f);
				
				for (int i = 0; i < 16; i++)
				{
					Vector3 LL = V0 + new Vector3(i * RW, 0, 0);
					
					CharRect[i].transform.SetParent(Bubble.transform);
					CharRect[i].transform.localPosition = LL;
					CharRect[i].transform.localScale = new Vector3(RW, RH, 1);
					
					CharRect[i].SetTexture("images/sample_letters_2");
				}
				
				i_INITIALIZED = true;
			}
			
			HelicopterMotion();
			
			Vector2 pos = Body.GetPosition();
			
			Vector3 posBubble = new Vector3(pos.x + 0.4f, pos.y + 0.4f, -0.03f);
			Bubble.transform.localPosition = posBubble;
			
			for (int i = 0; i < CharRect.Length; i++)  
			{
				CharRect[i].SetUVs(GetUVs(' '));
			}
			
			string str = string.Format("x {0:f1}  y {1:f1}", pos.x, pos.y);
			for (int i = 0; i < str.Length; i++)
			{
				CharRect[i].SetUVs(GetUVs(str[i]));
			}
			
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				
				Vector3 E = MainCamera.transform.position;
				float d = -E.z;
				
				float fov = 60.0f;
				float theta = fov * 0.5f * Mathf.Deg2Rad;
				float w2 = Screen.width * 0.5f;
				float h2 = Screen.height * 0.5f;
				float cx = w2;
				float cy = h2;
				
				float H2 = d * Mathf.Tan(theta);
				float WldPerPix = H2 / h2;
				
				float mx = 100;
				float my = 100;
				float rw = 16;
				float rh = 44;
				float RW = rw * WldPerPix;
				float RH = rh * WldPerPix;
				
				Vector2 P = E;		
				string str = "Hello World!";
				for (int i = 0; i < str.Length; i++)
				{
					float dx = mx - cx;
					float dy = my - cy;
					float s  = dx * WldPerPix;
					float t  = dy * WldPerPix;
					Vector2 M = P + new Vector2(s, t);
					
					CharRect[i].transform.localPosition = M;
					CharRect[i].transform.localScale = new Vector3(RW, RH, 1);
					CharRect[i].SetTexture("images/sample_letters_2");
					CharRect[i].SetUVs(GetUVs(str[i]));
					
					mx += rw;
				}
				
				i_INITIALIZED = true;
			}
		}


		void Code4()
		{
			if (IsMousePressed())
			{
				float mx = Input.mousePosition.x;
				float my = Input.mousePosition.y;
				Vector2 pos = ConvertScreenToWorld(mx, my);
				Dot.transform.position = pos;
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				Card[0].transform.position = new Vector2(8, 5);
				Card[0].transform.rotation = TH2DMath.GetRotation(-50);
				Card[1].transform.position = new Vector2(-8, 5);
				Card[1].transform.rotation = TH2DMath.GetRotation(30);
				Card[2].transform.position = new Vector2(-8, -5);
				Card[2].transform.rotation = TH2DMath.GetRotation(-80);
				Card[3].transform.position = new Vector2(8, -5);
				Card[3].transform.rotation = TH2DMath.GetRotation(-120);
				
				i_turnCardIndex = -1;
				
				i_INITIALIZED = true;
			}
			
			if (IsMousePressed() && i_turnCardIndex == -1)
			{
				Vector2 Md = Input.mousePosition;	
				Vector2 M  = ConvertScreenToWorld(Md);
				
				for (int i = 0; i < Card.Length; i++)
				{
					THMatrix3x3 mtx = Card[i].GetMatrix();
					Vector2  P = mtx.inverse * TH2DMath.ToVector3(M);
					bool bColl = CollisionTest_Point_AABB(P, c_CardMinMax);
					
					if (bColl)
					{
						i_turnCardIndex = i;
						break;
					}	
				}
			}
			
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				i_targets = new THAlphaObject2D[]
				{
					Moon, Ellipse, Star, Letter
				};
				
				i_INITIALIZED = true;
			}
			
			if (IsMousePressed())
			{
				Vector2 Md = Input.mousePosition;	
				Vector2 M  = ConvertScreenToWorld(Md);
				
				foreach (var obj in i_targets)
				{
					THMatrix3x3 mtx = obj.GetMatrix();
					Vector2  P = mtx.inverse * TH2DMath.ToVector3(M);
					bool bColl = CollisionTest_Point_TriMesh(P, obj.initTriangleCoords);
					if (bColl)
					{
						SelectObject(obj);
						break;
					}
				}
			}
			
		}


		void Code7()
		{
			if (!i_INITIALIZED)
			{
				i_targets = new THAlphaObject2D[]
				{
					Block, Moon, Disk, Star
				};
				
				i_selectObject = null;
				
				i_INITIALIZED = true;
			}
			
			if (IsMousePressed())  
			{
				Vector2 Md = Input.mousePosition;
				Vector2 M1  = ConvertScreenToWorld(Md);
				
				foreach (var obj in i_targets)
				{
					bool bColl = CollisionTest_Point_AllTypes(M1, obj);
					if (bColl)
					{
						SelectObject(obj);
						
						i_selectObject = obj;
						i_M1 = M1;
						break;
					}
				}
			}
			
			
			if (i_selectObject != null)
			{
				if (IsMouseDragged())
				{
					Vector2 Md = Input.mousePosition;
					Vector2 M2  = ConvertScreenToWorld(Md);
					
					Vector3 dif = M2 - i_M1;
					i_selectObject.transform.position += dif;
					i_M1 = M2;
				}
				else if(IsMouseReleased())
				{
					
					i_selectObject = null;
				}
			}
			
		}


		void Code8()
		{
			if (Input.GetKeyDown(KeyCode.V))
			{
				i_SET_VIEW = !i_SET_VIEW;
			}
			
			if (i_SET_VIEW)
			{
				Vector2 P = c_BoardCenter;
				float w2 = Screen.width * 0.5f;
				float h2 = Screen.height * 0.5f;
				
				float RW = c_BoardW;
				float RH = c_BoardH;
				float rw = 400;
				float rh = 300;
				float WldPerPix = RH / rh;
				
				float FOV = 60.0f;
				float theta = FOV * 0.5f * Mathf.Deg2Rad;
				float H2 = h2 * WldPerPix;
				float d = H2 / Mathf.Tan(theta);
				
				MainCamera.transform.position = new Vector3(P.x, P.y, -d);
			}
			else
			{
				MainCamera.transform.position = new Vector3(0, 0, -20);
			}
			
			
			if (Input.GetKeyDown(KeyCode.T))
			{
				string filePath = "screenshot1.png";
				MakeScreenshot(filePath, 400, 300);
				
				Debug.Log("= Make Screenshot 1 =");
			}
		}


		void Code9()
		{
			if (Input.GetKeyDown(KeyCode.V))
			{
				i_SET_VIEW = !i_SET_VIEW;
			}
			
			if (i_SET_VIEW)
			{
				Vector2 P = (A + C) * 0.5f;
				float w2 = Screen.width * 0.5f;
				float h2 = Screen.height * 0.5f;
				
				float RW = (D - A).magnitude;
				float RH = (A - B).magnitude;
				float rw = 270;
				float rh = 480;
				float WldPerPix = RH / rh;
				
				float FOV = 60.0f;
				float theta = FOV * 0.5f * Mathf.Deg2Rad;
				float H2 = h2 * WldPerPix;
				float d = H2 / Mathf.Tan(theta);
				
				MainCamera.transform.position = new Vector3(P.x, P.y, -d);
			}
			else
			{
				MainCamera.transform.position = new Vector3(0, 0, -20);
			}
			
			
			if (Input.GetKeyDown(KeyCode.T))
			{
				string filePath = "screenshot2.png";
				MakeScreenshot(filePath, 270, 480);
				
				Debug.Log("= Make Screenshot 2 =");
			}
		}




		
		
		Vector2 ConvertScreenToWorld(Vector2 Md)
		{
			return ConvertScreenToWorld(Md.x, Md.y); 
		}
		
		
		Vector2 ConvertScreenToWorld(float mx, float my)
		{
			Vector3 E = MainCamera.transform.position;
			
			float d = -E.z;
			float FOV = 60.0f;
			float theta = FOV * 0.5f * Mathf.Deg2Rad;
			float w2 = Screen.width * 0.5f;
			float h2 = Screen.height * 0.5f;
			float cx = w2;
			float cy = h2;
			
			float H2 = d * Mathf.Tan(theta);
			float WldPerPix = H2 / h2;
			
			float s = (mx - cx) * WldPerPix;
			float t = (my - cy) * WldPerPix;
			
			Vector2 P = E;
			Vector2 pos = P + new Vector2(s, t);	
			
			return pos;
		}
		
		
		void MakeScreenshot(string filePath, int w, int h)
		{
			BSAuxFuncs.SetVisible(0, TmpSqEd.gameObject);
			MakeScreenshot_CTR(filePath, w, h);
			BSAuxFuncs.SetVisible(1, TmpSqEd.gameObject);
		}
		
		
		void SelectObject(THAlphaObject2D obj)
		{
			if (MatchCase(6))
			{
				if (obj.collCount <= 0)
				{
					obj.collCount = 8;
					
					MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
					mre.materials = obj.hitMaterials;
				}
			}
			else if (MatchCase(7))
			{
				MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
				mre.materials = obj.hitMaterials;
			}
		}
		
		
		bool CollisionTest_Point_AllTypes(Vector2 P, THAlphaObject2D obj)
		{
			THMatrix3x3 mtx;
			
			bool bColl = false;
			switch (obj.GetType())
			{
				case THAlphaObject2D.DISK:
				Vector2 C = obj.GetPosition();
				float r = obj.GetDiskScale() * obj.initRadius;
				bColl = CollisionTest_Point_Disk(P, C, r);
				break;
				
				case THAlphaObject2D.RECT:
				mtx = obj.GetMatrix();
				P = mtx.inverse * TH2DMath.ToVector3(P);
				bColl = CollisionTest_Point_AABB(P, obj.initRectBounds);
				break;
				
				case THAlphaObject2D.TRIMESH:
				mtx = obj.GetMatrix();
				P = mtx.inverse * TH2DMath.ToVector3(P);
				bColl = CollisionTest_Point_TriMesh(P, obj.initTriangleCoords);
				break;
			}
			
			return bColl;
		}
		
		
		void HelicopterMotion()   
		{                  
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody += 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody -= 1;
			}
			
			if (Input.GetKeyDown(KeyCode.S))   
			{
				i_MOVE = !i_MOVE;
			}
			
			// localRotor
			i_degRotor += 8;
			THMatrix3x3 rotRotor = TH2DMath.GetRotation3x3(i_degRotor);
			THMatrix3x3 traRotor = TH2DMath.GetTranslation3x3(c_attachPos_Rotor);
			THMatrix3x3 localRotor = traRotor * rotRotor;
			
			// localBody	
			THMatrix3x3 rotBody = TH2DMath.GetRotation3x3(i_degBody);
			Vector2 direBody = rotBody * new Vector3(0, 1, 1);
			
			Vector2 curP = Body.GetPosition();
			Vector2 newP = (i_MOVE) ? (curP + 0.05f * direBody) : curP;
			THMatrix3x3 traBody = TH2DMath.GetTranslation3x3(newP);
			THMatrix3x3 localBody = traBody * rotBody;
			
			// world matrix
			THMatrix3x3 worldRotor = localBody * localRotor;
			THMatrix3x3 worldBody = localBody;
			
			Rotor.SetMatrix(worldRotor);
			Body.SetMatrix(worldBody);
		}
		
		
		
		Vector2[] GetUVs(char ch)
		{
			return RGMeshData02.GetUVs_SF(ch);
		}
		


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

				case 8 :	Code8();  break;

				case 9 :	Code9();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			MouseDragProcess();
			
			if (MatchCase(1))
			{
				pr_shm += 0.5f;
				float mx = 3 * Mathf.Sin(pr_shm * Mathf.Deg2Rad);
				Block.SetPosition(12 + mx, 7);
			}
			else if (MatchCase(7))
			{
				if (i_selectObject == null)
				{
					foreach (var o in i_targets)
					{
						MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
						mre.materials = o.originalMaterials;
					}
				}
			}
		}


		void PostProcess()
		{
			if (MatchCase(3, 8, 9))
			{
				if (GameObject.Find("PixelCoord") != null)
				{
					int px = Mathf.RoundToInt(Input.mousePosition.x);
					int py = Mathf.RoundToInt(Input.mousePosition.y);
					if (!(px > Screen.width || px < 0 || py > Screen.height || py < 0))
					{
						BSAuxFuncs.Update_PixelCoord(px, py);
					}
				}
				
				if (MatchCase(3))
				{
					for (int i = 0; i < CharRect.Length; i++)
					{
						Vector3 wp = CharRect[i].transform.position;
						if (wp.x < -100) { continue; }
						
						LLBoundary[i].SetWorldPosition(wp);
						LLBoundary[i].SetLocalScale(CharRect[i].transform.localScale);
					}
				}
				
				if (MatchCase(8, 9))
				{
					Vector3 E = MainCamera.transform.position;
					
					float d = 1;
					float fov = 60.0f;
					float theta = fov * 0.5f * Mathf.Deg2Rad;
					float w2 = Screen.width * 0.5f;
					float h2 = Screen.height * 0.5f;
					
					float H2 = d * Mathf.Tan(theta);
					float WldPerPix = H2 / h2;
					float rw = 400;
					float rh = 300;
					if (MatchCase(9))
					{
						rw = 270;
						rh = 480;
					}
					
					float RW = rw * WldPerPix;
					float RH = rh * WldPerPix;
					
					TmpSqEd.transform.localScale = new Vector3(RW, RH, 1);		
					TmpSqEd.transform.position = E + Vector3.forward;
				}
			}
			else if (MatchCase(5))
			{
				if (i_turnCardIndex >= 0)
				{
					pr_turnCounter++;
					Quaternion q = Quaternion.AngleAxis(9.0f, Vector3.up);
					THObject2D obj = Card[i_turnCardIndex];
					obj.transform.rotation = obj.transform.rotation * q;
					
					if (pr_turnCounter == 20)
					{
						i_turnCardIndex = -1;
						pr_turnCounter = 0;
					}
				}
			}
			else if (MatchCase(6))
			{
				foreach (var o in i_targets)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
							mre.materials = o.originalMaterials;
						}
					}
				}
			}
			else if (MatchCase(7))
			{
				if (i_selectObject == null)
				{
					Vector2 Md = Input.mousePosition;	
					Vector2 M  = ConvertScreenToWorld(Md);
					
					foreach (var obj in i_targets)
					{
						bool bColl = CollisionTest_Point_AllTypes(M, obj);
						if (bColl)
						{
							MeshRenderer mre = obj.gameObject.GetComponent<MeshRenderer>();
							mre.materials = obj.hitMaterials;
							break;
						}
					}
				}
			}
			
			BSData.CameraController2D.UpdateCameraData();
		}


		static Sec229 pr_instance = new Sec229(9);

		public static Sec229 PRGSI()
		{
			return pr_instance;
		}


		private Sec229(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				Body = CreateObject_Obj2D("HeliBody2D");
				Rotor = CreateObject_Obj2D("HeliRotor2D");
				Dot = CreateObject_Obj2D("Dot_2");
				Card[0] = CreateObject_Obj2D("xyC08");
				Card[1] = CreateObject_Obj2D("xyC09");
				Card[2] = CreateObject_Obj2D("xyC10");
				Card[3] = CreateObject_Obj2D("xyC11");
				Block = CreateObject_Alpha2D("CollRect1");
				Disk = CreateObject_Alpha2D("CollDisk2");
				Board_B = CreateObject_Obj2D("Board_B");
				Board_G = CreateObject_Obj2D("Board_G");
				Moon = CreateObject_Alpha2D("Moon");
				Ellipse = CreateObject_Alpha2D("Ellipse");
				Star = CreateObject_Alpha2D("Star");
				Letter = CreateObject_Alpha2D("Letter");
				
				AuxLine = CreateObject_Obj2D("AuxLine");
				Mesh mesh = AuxLine.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { Vector2.zero, Vector2.up });
				
				TmpSqEd = CreateObject_InsObj2D(BSData.RunObjectDict["AuxSquareEdge"]);
				BSAuxFuncs.SetUnlitMaterial(TmpSqEd, 0.8f, 0.0f, 0.0f);
				
				for (int i = 0; i < CharRect.Length; i++)		
				{
					CharRect[i] = (THComponentObject)CreateObject_FromScript2D(
					MakeComponentRect("GUI_COMPONENT_" + i, false, true));
					
					CharRect[i].gameObject.name = "CharRect_" + i;
				}
				
				var src_LLBoundary = CreateObject_Obj3D("LLBoundary");
				for (int i = 0; i < LLBoundary.Length; i++)
				{
					LLBoundary[i] = CreateObject_InsObj3D(src_LLBoundary.gameObject);
				}
				
				
				BSAuxFuncs.SetUnlitMaterial(Dot, 0.3f, 0.53f, 0.87f);
				
				Text_Pixel = BSAuxFuncs.InitText_PixelCoord(1.0f);
				
				
				MainCamera = BSAuxFuncs.GetMainCamera();
				
				
				Vector2[] verts = RGMeshData02.GetVertsData(RGMeshData02.c_ELLIPSE);
				Ellipse.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_LETTER);
				Letter.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_MOON);
				Moon.SetData(THAlphaObject2D.TRIMESH, verts);
				
				verts = RGMeshData02.GetVertsData(RGMeshData02.c_STAR);
				Star.SetData(THAlphaObject2D.TRIMESH, verts);
				
				
				Disk.SetData(THAlphaObject2D.DISK, 1.66f);
				Disk.originalMaterials = new Material[]
				{ BSAuxFuncs.GetUnlitMaterial(0.8764f, 0.8764f, 0.9528f), BSAuxFuncs.GetUnlitMaterial(0.528f, 0.528f, 0.868f) };
				Disk.hitMaterials = new Material[]
				{ BSAuxFuncs.GetUnlitMaterial(0.89f, 0.8f, 0.77f), BSAuxFuncs.GetUnlitMaterial(0.89f, 0.67f, 0.63f) };
				Disk.gameObject.GetComponent<MeshRenderer>().materials = Disk.originalMaterials;
				
				Block.SetData(THAlphaObject2D.RECT, c_initRectBounds1);
				Block.originalMaterials = new Material[]
				{ BSAuxFuncs.GetUnlitMaterial(0.63f, 0.934f, 0.6674f), BSAuxFuncs.GetUnlitMaterial(0.4475f, 0.84f, 0.4817f) };
				Block.hitMaterials = new Material[]
				{ BSAuxFuncs.GetUnlitMaterial(0.89f, 0.8f, 0.77f), BSAuxFuncs.GetUnlitMaterial(0.89f, 0.67f, 0.63f) };
				Block.gameObject.GetComponent<MeshRenderer>().materials = Block.originalMaterials;
				
				
				THAlphaObject2D[] wObjs = { Moon, Ellipse, Star, Letter };
				Color selectColor = new Color(0.934f, 0.695f, 0.595f);
				foreach (var obj in wObjs)
				{
					var mre = obj.gameObject.GetComponent<MeshRenderer>();
					
					obj.originalMaterials = mre.materials;
					
					Color clr = (obj != Ellipse) ? selectColor : new Color(0.934f, 0.543f, 0.543f);
					Material[] hitMtrls = { BSAuxFuncs.GetUnlitMaterial(clr) };
					hitMtrls[0].SetColor("_Color", clr);
					
					obj.hitMaterials = hitMtrls;
				}
				
				Bubble = CreateObject_Obj2D("Bubble");
				Material[] mtrls = 
				{ 
					BSAuxFuncs.GetUnlitMaterial(0.935f, 0.935f, 0.935f),
					BSAuxFuncs.GetUnlitMaterial(0.6067f, 0.7179f, 0.95f)
				};
				Bubble.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				
				CS_AF = CreateObject_InsObj2D(CrdSys.gameObject);
				Shader shader = Shader.Find("Custom/CLV_AF");
				Material lineAF = new Material(shader);
				lineAF.hideFlags = HideFlags.HideAndDontSave;
				lineAF.shader.hideFlags = HideFlags.HideAndDontSave;
				CS_AF.gameObject.GetComponent<MeshRenderer>().material = lineAF;

				pr_LOAD_OBJECTS = true;
			}

			

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			BSAuxFuncs.SetVisible(0, Text_Pixel.gameObject);	
			BSData.c_SHOW_COORDINATE_SCALE = true;
			
			i_frameCount = 0;
			SetFPS(60);
			
			for (int i = 0; i < CharRect.Length; i++)
			{
				CharRect[i].transform.SetParent(null);
			}
			
			if (MatchCase(1, 2, 3))
			{
				if (MatchCase(1))
				{
					BSAuxFuncs.SetVisible(1, Block.gameObject);
					Block.SetIdentity();
					
					Block.SetRotation(50);
					pr_shm = 0;
					
					Material[] mtrls =
					{
						BSAuxFuncs.GetUnlitMaterial(0.728f,0.830f,0.724f),
						BSAuxFuncs.GetUnlitMaterial(0.551f,0.764f,0.515f)
					};
					Block.gameObject.GetComponent<MeshRenderer>().materials = mtrls;
				}
				else if (MatchCase(2))
				{
					BSAuxFuncs.SetVisible(1, Body.gameObject);
					BSAuxFuncs.SetVisible(1, Rotor.gameObject);
					BSAuxFuncs.SetVisible(1, Bubble.gameObject);
					Body.SetIdentity();
					Rotor.SetIdentity();
					Bubble.SetIdentity();
					
					i_degBody = 0;
					i_degRotor = 0;
					i_MOVE = false;
				}
				else if (MatchCase(3))
				{
					for (int i = 0; i < LLBoundary.Length; i++)
					{
						BSAuxFuncs.SetVisible(1, LLBoundary[i].gameObject);
						LLBoundary[i].SetIdentity();
						LLBoundary[i].SetWorldPosition(-1000, 0, 0);
					}
					
					BSAuxFuncs.SetVisible(1, Text_Pixel.gameObject);
					BSAuxFuncs.MoveCamera_PixelCoord();  
				}
				
				for (int i = 0; i < CharRect.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, CharRect[i].gameObject);
					CharRect[i].SetIdentity();
					CharRect[i].SetPosition(-1000, 0);
				}
				
				SetCameraPosition_2D(0, 0, -17.7f);
			}
			
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Dot.gameObject);
				Dot.SetIdentity();
				
				SetCameraPosition_2D(0, 0, -15);
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(0, CrdSys.gameObject);
				
				BSAuxFuncs.SetVisible(1, CS_AF.gameObject);
				CS_AF.SetIdentity();			
				
				for (int i = 0; i < Card.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Card[i].gameObject);
					Card[i].SetIdentity();
				}
				
				pr_turnCounter = 0;
				
				SetCameraPosition_2D(0, 0, -16.2f);
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Moon.gameObject);
				BSAuxFuncs.SetVisible(1, Ellipse.gameObject);
				BSAuxFuncs.SetVisible(1, Star.gameObject);
				BSAuxFuncs.SetVisible(1, Letter.gameObject);
				Moon.SetIdentity();
				Ellipse.SetIdentity();
				Star.SetIdentity();
				Letter.SetIdentity();
				
				THAlphaObject2D[] objs = { Moon, Ellipse, Star, Letter };
				foreach (var o in objs)
				{
					MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
					mre.materials = o.originalMaterials;
				}
				
				Moon.SetPosition(pr_posQ1);
				Ellipse.SetPosition(pr_posQ2);
				Star.SetPosition(pr_posQ3);
				Letter.SetPosition(pr_posQ4);
				
				SetOutputImageSize(10.2f, 400);
				SetCameraPosition_2D(0, 0);
			}
			
			else if (MatchCase(7))
			{
				BSAuxFuncs.SetVisible(1, Block.gameObject);
				BSAuxFuncs.SetVisible(1, Moon.gameObject);
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				BSAuxFuncs.SetVisible(1, Star.gameObject);
				Block.SetIdentity();
				Moon.SetIdentity();
				Disk.SetIdentity();
				Star.SetIdentity();
				
				THAlphaObject2D[] objs = { Block, Moon, Disk, Star };
				foreach (var o in objs)
				{
					MeshRenderer mre = o.gameObject.GetComponent<MeshRenderer>();
					mre.materials = o.originalMaterials;
				}
				
				Block.SetPosition(pr_posQ1);
				Moon.SetPosition(pr_posQ2);
				Disk.SetPosition(pr_posQ3);
				Star.SetPosition(pr_posQ4);
				
				Block.SetRotation(30);
				Moon.SetRotation(30);
				Star.SetRotation(20);
				
				SetOutputImageSize(10.2f, 400);
				SetCameraPosition_2D(0, 0);
			}
			else if (MatchCase(8))
			{
				BSAuxFuncs.SetVisible(1, Board_B.gameObject);
				Board_B.SetIdentity();
				BSAuxFuncs.SetVisible(1, TmpSqEd.gameObject);
				TmpSqEd.SetIdentity();
				
				Board = Board_B;
				i_SET_VIEW = false;
				
				BSAuxFuncs.SetVisible(1, Text_Pixel.gameObject);
				BSAuxFuncs.MoveCamera_PixelCoord();  
			}
			else if (MatchCase(9))
			{
				BSAuxFuncs.SetVisible(1, Board_G.gameObject);
				Board_G.SetIdentity();
				BSAuxFuncs.SetVisible(1, TmpSqEd.gameObject);
				TmpSqEd.SetIdentity();
				
				Board = Board_G;
				float brdW = 4.5f;        
				float brdH = 8;
				i_SET_VIEW = false;
				
				Vector2 brdCtr = new Vector2(-5, 2);
				Board.SetPosition(brdCtr);
				
				float hfW = brdW * 0.5f;
				float hfH = brdH * 0.5f;
				A = brdCtr + new Vector2(hfW, hfH);
				B = A - brdH * Vector2.up;
				C = B - brdW * Vector2.right;
				D = A - brdW * Vector2.right;
				
				BSAuxFuncs.SetVisible(1, Text_Pixel.gameObject);
				BSAuxFuncs.MoveCamera_PixelCoord();  
			}
		}

	}

}
