﻿//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;


namespace OQJ_THAlphaObject2D
{
	public class THAlphaObject2D : THObject2D
	{
		public const int NONE = 0;
		public const int DISK = 1;
		public const int RECT = 2;
		public const int MULTI_RECT = 3;
		public const int TRIMESH = 4;

		int collisionType = NONE;


		public THAlphaObject2D(GameObject go) : base(go) { }


		float m_initRad;
		public float initRadius
		{
			get
			{
				if (collisionType == DISK)
				{
					return m_initRad;
				}
				else
				{
					return 0.0f;
				}
			}
		}


		Vector2[] m_initRectBds;
		public Vector2[] initRectBounds  // for MULTI_RECT {lw0, up0, lw1, up1, lw2, up2, .. }
		{
			get
			{
				if (collisionType == RECT || collisionType == MULTI_RECT)
				{
					return m_initRectBds;
				}
				else
				{
					return new Vector2[0];
				}
			}
		}



		Vector2[] m_initTriCrds;
		public Vector2[] initTriangleCoords
		{
			get
			{
				if (collisionType == TRIMESH)
				{
					return m_initTriCrds;
				}
				else
				{
					return new Vector2[0];
				}
			}
		}


		public int collCount;

		public Material[] originalMaterials;
		public Material[] hitMaterials;


		//// Shoot Data 
		public int lastShootFrame;

		public int shootInterval;

		Vector2 m_initShootDir;
		public Vector3 initShootDirection
		{
			get
			{
				return new Vector3(m_initShootDir.x, m_initShootDir.y, 0.0f);
			}

			set
			{
				m_initShootDir = value;
			}
		}


		Vector2 m_initShootPos;
		public Vector3 initShootPosition
		{
			get
			{
				return new Vector3(m_initShootPos.x, m_initShootPos.y, 1.0f);
			}

			set
			{
				m_initShootPos = value;
			}
		}


		public float shellSpeed;


		public int GetType()
		{
			return collisionType;
		}


		public float GetDiskScale()
		{
			if (collisionType != DISK) { return 1.0f; }

			THMatrix3x3 M = GetMatrix();
			float s = Mathf.Sqrt(M.m00 * M.m00 + M.m10 * M.m10);
			return s;
		}


		public void SetData(int t, float rad)   // for Disk
		{
			SetData(t, null, rad);
		}


		public void SetData(int t, Vector2[] crd_arr, float rad = 0.0f)
		{
			collisionType = t;
			m_initRad = rad;

			m_initTriCrds = new Vector2[0];
			m_initRectBds = new Vector2[0];

			if (crd_arr == null)
			{
				// .. .. 
			}
			else
			{
				if (collisionType == TRIMESH)
				{
					m_initTriCrds = crd_arr;
				}
				else if (collisionType == RECT || collisionType == MULTI_RECT)
				{
					m_initRectBds = crd_arr;
				}
			}
		}
	}
}

