﻿//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections.Generic;
using UnityEngine;


namespace OQJ_RGAuxFuncs03
{
	public static class RGAuxFuncs03
	{

		public static void CreateRightZupCoordinateSystem()  // Blender Coordinate System
		{
			GameObject goCS = new GameObject("RZ_CS");
			goCS.AddComponent<MeshFilter>();
			goCS.AddComponent<MeshRenderer>();

			List<Vector3> verts = new List<Vector3>();
			List<Color> colors = new List<Color>();

			float gridGray = 0.56f;   // backgroundColor 0.608
			float bkgrndGray = BSData.BackgroundColor.r;
			float alpha = 0.25f;
			float semiVal = alpha * 0.73f + (1.0f - alpha) * bkgrndGray;
			float semiBgr = alpha * 0.0f + (1.0f - alpha) * bkgrndGray;

			Color clrNegX = new Color(semiVal, semiBgr, semiBgr);
			Color clrNegY = new Color(semiBgr, semiVal, semiBgr);
			Color clrNegZ = new Color(semiBgr, semiBgr, semiVal);
			Color clrPlsX = new Color(0.73f, 0.0f, 0.0f);
			Color clrPlsY = new Color(0.0f, 0.73f, 0.0f);
			Color clrPlsZ = new Color(0.0f, 0.0f, 0.73f);


			Mesh wMesh = new Mesh();
			Color gridColor = new Color(gridGray, gridGray, gridGray);
			const int gridMax = 60;
			int[] idcs = new int[gridMax * 2 * 4 + 12];  // 12 : X, Y, Z axes
			int vi = 0;
			for (int i = -gridMax; i <= gridMax; i++)
			{
				if (i == 0) { continue; }

				// parallel X
				verts.Add(new Vector3((float)-gridMax, -0.001f, (float)i));
				colors.Add(gridColor);
				verts.Add(new Vector3((float)gridMax, -0.001f, (float)i));
				colors.Add(gridColor);

				// parallel Z
				verts.Add(new Vector3((float)i, -0.001f, (float)-gridMax));
				colors.Add(gridColor);
				verts.Add(new Vector3((float)i, -0.001f, (float)gridMax));
				colors.Add(gridColor);

				idcs[vi] = vi;
				idcs[vi + 1] = vi + 1;
				idcs[vi + 2] = vi + 2;
				idcs[vi + 3] = vi + 3;
				vi += 4;
			}

			//////// X axis	
			// Minus X
			verts.Add(new Vector3(-gridMax, 0.0f, 0.0f));
			colors.Add(clrNegX);
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrNegX);

			// Plus X
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsX);
			verts.Add(new Vector3(gridMax, 0.0f, 0.0f));
			colors.Add(clrPlsX);

			//////// Y axis		**** FORWARD ****
			// Minus Y
			verts.Add(new Vector3(0.0f, 0.0f, -gridMax));
			colors.Add(clrNegY);
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrNegY);

			// Plus Y
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsY);
			verts.Add(new Vector3(0.0f, 0.0f, gridMax));
			colors.Add(clrPlsY);

			//////// Z axis		**** UP ****
			// Minus Z
			verts.Add(new Vector3(0.0f, -gridMax, 0.0f));
			colors.Add(clrNegZ);
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrNegZ);

			// Plus Z
			verts.Add(new Vector3(0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsZ);
			verts.Add(new Vector3(0.0f, gridMax, 0.0f));
			colors.Add(clrPlsZ);

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;
			idcs[vi + 2] = vi + 2;
			idcs[vi + 3] = vi + 3;
			idcs[vi + 4] = vi + 4;
			idcs[vi + 5] = vi + 5;
			idcs[vi + 6] = vi + 6;
			idcs[vi + 7] = vi + 7;
			idcs[vi + 8] = vi + 8;
			idcs[vi + 9] = vi + 9;
			idcs[vi + 10] = vi + 10;
			idcs[vi + 11] = vi + 11;

			wMesh.SetVertices(verts);
			wMesh.SetColors(colors);

			wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

			MeshFilter mshFltr = goCS.GetComponent<MeshFilter>();
			mshFltr.mesh = wMesh;

			goCS.transform.localPosition = Vector3.zero;
			goCS.transform.localRotation = Quaternion.identity;
			goCS.transform.localScale = Vector3.one;

			MeshRenderer mshRndr = goCS.GetComponent<MeshRenderer>();
			mshRndr.material = BSData.LineMaterial;
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);

			BSData.RunObjectDict.Add(goCS.name, goCS);
		}
		




		public static void CreateArc90()
		{
			GameObject gObj = new GameObject("Arc90");
			gObj.AddComponent<MeshFilter>();
			gObj.AddComponent<MeshRenderer>();

			List<Vector3> verts = new List<Vector3>();
			List<Color> colors = new List<Color>();

			Color clr = new Color(0.33f, 0.33f, 0.33f);
			float staDeg = 45.0f;
			float endDeg = 135.0f;

			int numDiv = 24;
			float aDeg = (endDeg - staDeg) / numDiv;
			float rad = 2.4f;
			Mesh wMesh = new Mesh();
			int[] idcs = new int[numDiv + 1 + 2];
			for (int i = 0; i < idcs.Length; i++)
			{
				idcs[i] = i;
			}


			verts.Add(Vector3.zero);
			colors.Add(clr);

			for (int i = 0; i <= numDiv; i++)
			{
				float angle = Mathf.Deg2Rad * (staDeg + aDeg * i);
				float vx = rad * Mathf.Cos(angle);
				float vy = rad * Mathf.Sin(angle);

				verts.Add(new Vector3(vx, vy, 0.0f));
				colors.Add(clr);
			}

			verts.Add(Vector3.zero);
			colors.Add(clr);


			wMesh.SetVertices(verts);
			wMesh.SetColors(colors);
			wMesh.SetIndices(idcs, MeshTopology.LineStrip, 0);

			MeshFilter mshFltr = gObj.GetComponent<MeshFilter>();
			mshFltr.mesh = wMesh;

			gObj.transform.localPosition = Vector3.zero;
			gObj.transform.localRotation = Quaternion.identity;
			gObj.transform.localScale = Vector3.one;

			MeshRenderer mshRndr = gObj.GetComponent<MeshRenderer>();
			mshRndr.material = BSData.LineMaterial;
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);

			BSData.RunObjectDict.Add(gObj.name, gObj);
		}
	} 

}

