//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec306
{
	public class Sec306 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		THObject3D CS_2D;

		THObject2D Green;
		THObject2D Rect;

		GameObject Blue;
		GameObject Triangle;
		MeshFilter pr_msFltr;


		// Instance Variables
		float pr_degBl;
		float pr_degGr;
		
		Vector2 pr_E;
		Vector2 pr_F;

		bool pr_isHitColor;

		readonly Vector3[] pr_initRectVerts = new Vector3[]
		{
			new Vector3(-0.5f, -0.5f, 1), new Vector3(-0.5f, 0.5f, 1),
			new Vector3( 0.5f, 0.5f, 1), new Vector3(0.5f, -0.5f, 1),
		};




		void Code1()
		{
			Vector3 a = new Vector3(0, 0, 2);
			Vector3 b = new Vector3(3, 0, 1);
			Vector3 c = Vector3.Cross(a, b);  
			Debug.Log(c);
			
			c = Vector3.Cross(b, a);          
			Debug.Log(c);
		}



		void Code2()
		{
			Vector3 A = new Vector3(3.92f, 4.35f, -4.31f);
			Vector3 B = new Vector3(0.52f, 4.0f, -7.95f);
			Vector3 C = new Vector3(0.0f, 4.0f, -5.0f);
			
			Vector3 a = A - C;
			Vector3 b = B - C;
			Vector3 n = Vector3.Cross(a, b).normalized;
			
			Debug.Log("(" + n.x + ", " + n.y + ", " + n.z + ")");
		}



		void Code3()
		{
			ObjectMotion();

			Vector2[] vtsABC = GetGreenVerts();
			Vector2[] vtsDEF = GetBlueVerts();

			bool bColl = CollisionTest_Triangle_Triangle(vtsABC, vtsDEF);
			Hit(bColl);
		}



		void Code4()
		{
			ObjectMotion();

			Vector2[] vtsTri  = GetTriangleVerts();
			Vector2[] vtsRect = GetRectVerts();

			bool bColl = CollisionTest_Triangle_Rect(vtsTri, vtsRect);
			Hit(bColl);
		}




		bool CollisionTest_Triangle_Triangle(Vector2[] vtsABC, Vector2[] vtsDEF)
		{
			// vtsABC[0] : A
			// vtsABC[1] : B
			// vtsABC[2] : C
			// vtsDEF[0] : D
			// vtsDEF[1] : E
			// vtsDEF[2] : F

			return false;
		}



		bool CollisionTest_Triangle_Rect(Vector2[] vtsTri, Vector2[] vtsRect)
		{
			// vtsRect[0] : A
			// vtsRect[1] : B
			// vtsRect[2] : C
			// vtsRect[3] : D

			return false;
		}





///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		Vector2[] GetBlueVerts()
		{
			Quaternion rot = Blue.transform.rotation;

			Vector3 D = Blue.transform.position;
			Vector3 E = D + rot * pr_E;
			Vector3 F = D + rot * pr_F;

			return new Vector2[] { D, E, F };
		}


		Vector2[] GetGreenVerts()
		{
			THMatrix3x3 M = Green.GetMatrix();
			Vector2 A = M * new Vector3(0, 0, 1);
			Vector2 B = M * new Vector3(0.8f, 1.8f, 1.0f);
			Vector2 C = M * new Vector3(2.4f, 0.0f, 1.0f);

			return new Vector2[] { A, B, C };
		}


		Vector2[] GetTriangleVerts()
		{
			Quaternion rot = Triangle.transform.rotation;

			Vector3 P = Triangle.transform.position;
			Vector3 Q = P + rot * pr_E;
			Vector3 R = P + rot * pr_F;

			return new Vector2[] { P, Q, R };
		}


		Vector2[] GetRectVerts()
		{
			THMatrix3x3 M = Rect.GetMatrix();

			Vector2[] vrts = new Vector2[]
			{
				M * pr_initRectVerts[0],	// pr_initRectVerts : Hom crd
				M * pr_initRectVerts[1],
				M * pr_initRectVerts[2],
				M * pr_initRectVerts[3]
			};

			return vrts;
		}



		GameObject MakeTriangle()
		{
			List<Vector3> verts = new List<Vector3>();
			List<Vector3> norms = new List<Vector3>();
			int[] idcs = { 0, 1, 2 };

			pr_E = new Vector2(-0.6f, 8.1f);
			pr_F = new Vector2(3.4f, 6.2f);

			verts.Add(Vector2.zero);
			verts.Add(pr_E);
			verts.Add(pr_F);

			norms.Add(-Vector3.forward);
			norms.Add(-Vector3.forward);
			norms.Add(-Vector3.forward);

			Mesh objMesh = new Mesh();
			objMesh.SetVertices(verts);
			objMesh.SetNormals(norms);
			objMesh.SetIndices(idcs, MeshTopology.Triangles, 0);

			GameObject go = new GameObject("Tria2");
			MeshFilter mshFltr = go.AddComponent<MeshFilter>();
			mshFltr.mesh = objMesh;
			pr_msFltr = mshFltr;

			MeshRenderer mshRndr = go.AddComponent<MeshRenderer>();
			mshRndr.material = BSAuxFuncs.GetSemiTransparentMaterial(0.09f, 0.265f, 0.78f, 0.6f);

			return go;
		}



		void ObjectMotion()
		{
			if (MatchCase(3) && 
				(Input.GetKey(KeyCode.Y) || Input.GetKey(KeyCode.U) || 
				Input.GetKey(KeyCode.I) || Input.GetKey(KeyCode.O))	)
			{
				float aa = 0.05f;
				bool isF = THUtil.IsShiftDown();	
				if (Input.GetKey(KeyCode.Y))
				{
					if(!isF)	pr_E.x -= aa;
					else		pr_F.x -= aa;
				}
				else if (Input.GetKey(KeyCode.O))
				{
					if(!isF)	pr_E.x += aa;
					else		pr_F.x += aa;
				}
				else if (Input.GetKey(KeyCode.U))
				{
					if(!isF)	pr_E.y -= aa;
					else		pr_F.y -= aa;
				}
				else if (Input.GetKey(KeyCode.I))
				{
					if(!isF)	pr_E.y += aa;
					else		pr_F.y += aa;
				}

				List<Vector3> vrts = new List<Vector3>{ Vector2.zero, pr_E, pr_F };
				pr_msFltr.mesh.SetVertices(vrts);
			}
			else if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
				Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L))
			{
				Vector2 pos = Blue.transform.position;

				float speed = 0.10f;
				if (Input.GetKey(KeyCode.H))
				{
					pos.x -= speed;
				}
				else if (Input.GetKey(KeyCode.J))
				{
					pos.y -= speed;
				}
				else if (Input.GetKey(KeyCode.K))
				{
					pos.y += speed;
				}
				else if (Input.GetKey(KeyCode.L))
				{
					pos.x += speed;
				}

				Blue.transform.position = pos;
			}
			else if (Input.GetKey(KeyCode.R))
			{
				pr_degBl += THUtil.IsShiftDown() ? -1.0f : 1.0f;
				Blue.transform.rotation = Quaternion.AngleAxis(pr_degBl, Vector3.forward);
			}

			else if (Input.GetKey(KeyCode.Q))
			{
				pr_degGr += THUtil.IsShiftDown() ? -1.0f : 1.0f;

				if (MatchCase(3))
				{ Green.SetRotation(pr_degGr); }
				else
				{ Rect.SetRotation(pr_degGr); }
			}
		}


		void Hit(bool coll)
		{
			if (coll)
			{
				if (!pr_isHitColor)
				{
					var mre = Blue.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.8f, 0.2f, 0.1f, 0.6f));
				}
				
				pr_isHitColor = true;
			}
			else
			{
				if (pr_isHitColor)
				{
					var mre = Blue.gameObject.GetComponent<MeshRenderer>();
					mre.material.SetColor("_Color", new Color(0.09f, 0.265f, 0.78f, 0.6f));
				}
				
				pr_isHitColor = false;
			}
		}



		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec306 pr_instance = new Sec306(4);

		public static Sec306 PRGSI()
		{
			return pr_instance;
		}


		private Sec306(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				CS_2D = CreateObject_Obj3D("TmpCS_2D");
				Green = CreateObject_Obj2D("GrTria");
				Rect = CreateObject_Obj2D("UnitRect");

				Blue = MakeTriangle();
				
				BSAuxFuncs.SetUnlitMaterial(Green, 0.09f, 0.783f, 0.381f);
				BSAuxFuncs.SetUnlitMaterial(Rect, 0.09f, 0.783f, 0.381f);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();

			BSAuxFuncs.SetVisible(0, Blue.gameObject);

			pr_LOOP_CODE = true;
			
			if (MatchCase(1, 2))
			{
				CamPos = new Vector3(0.0f, 4.0f, -8.0f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;

				pr_LOOP_CODE = false;
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(0, CrdSys.gameObject);

				BSAuxFuncs.SetVisible(1, CS_2D.gameObject);
				CS_2D.SetIdentity();

				BSAuxFuncs.SetVisible(1, Green.gameObject);
				Green.SetIdentity();

				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				pr_degBl = 45;
				Blue.transform.position = new Vector2(-0.4f, 0.2f);		// -0.4, 0.2
				Blue.transform.rotation = Quaternion.AngleAxis(pr_degBl, Vector3.forward);
				Blue.transform.localScale = Vector3.one;

				pr_E = new Vector2(-0.6f, 8.1f);
				pr_F = new Vector2(3.4f, 6.2f);
				List<Vector3> vrts = new List<Vector3>{ Vector2.zero, pr_E, pr_F };
				pr_msFltr.mesh.SetVertices(vrts);

				pr_degGr = 0;
				Green.SetScale(3.6f, 2.7f);

				CamPos = new Vector3(3.3f, 2.2f, -16.0f);
				TrgPos = CamPos;
				TrgPos.z = 0.0f;
				uJ = Vector3.up;
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(0, CrdSys.gameObject);

				BSAuxFuncs.SetVisible(1, CS_2D.gameObject);
				CS_2D.SetIdentity();

				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				Rect.SetIdentity();

				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				Triangle = Blue;
				pr_degBl = 50;
				Triangle.transform.position = new Vector3(-9.7f, -6.4f);
				Triangle.transform.rotation = Quaternion.AngleAxis(pr_degBl, Vector3.forward);
				Triangle.transform.localScale = Vector3.one;

				pr_E = new Vector2(-1.7f, 3.9f);
				pr_F = new Vector2(7.3f, 2.1f);
				List<Vector3> vrts = new List<Vector3>{ Vector2.zero, pr_E, pr_F };
				pr_msFltr.mesh.SetVertices(vrts);

				pr_degGr = 0;
				Rect.SetScale(8, 3);

				CamPos = new Vector3(0.0f, 0.0f, -16.0f);
				TrgPos = CamPos;
				TrgPos.z = 0.0f;
				uJ = Vector3.up;
			}


			SetCameraPosition_3D(CamPos, TrgPos, uJ);		
		}

	}

}
