//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec309
{
	public class Sec309 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Signboard1;
		THObject3D Signboard2;
		
		THObject3D Part1;
		THObject3D Part2;
		
		THObject3D AuxCircle;


		// Instance Variables
		int i_degRev = 0;
		
		int i_SB = 1;
		
		readonly Vector3 A = new Vector3(0.5f, 1.0f, 0.0f);
		readonly Vector3 B = new Vector3(0.0f, 1.0f, 0.5f);
		readonly Vector3 C = new Vector3(0.0f, 0.0f, 1.0f);
		readonly Vector3 D = new Vector3(1.0f, 0.0f, 0.0f);
		
		bool pr_TRANSPARENT_CUBE;
		Material[] pr_transparentMtrls;
		Material[] pr_opaqueMtrls;




		void Code1()
		{
			Vector4 P = new Vector4(1.0f, 5.0f, 2.0f, 1.0f);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(4.0f, -2.0f, -3.0f);
			
			Vector3 P1 = T * P;
			Debug.Log(P1);
		}


		void Code2()
		{
			i_SB = 1;
			
			Matrix4x4 T = TH3DMath.GetTranslation4x4(3.0f, 0.0f, 3.0f);
			if (i_SB == 1)
			{
				Signboard1.SetMatrix(T);
			}
			else
			{
				Signboard2.SetMatrix(T);
			}
		}


		void Code3()
		{
			i_SB = 1;
			
			i_degRev++;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(np);
			
			if (i_SB == 1)
			{
				Signboard1.SetMatrix(T);
			}
			else
			{
				Signboard2.SetMatrix(T);
			}
		}



		void Code4()
		{

		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(2))
			{
				if (i_SB == 1)
				{
					Signboard2.transform.localPosition = new Vector3(-1000, 0, 0);
				}
				else
				{
					Signboard1.transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
			else if (MatchCase(3))
			{
				if (i_SB == 1)
				{
					Signboard2.transform.localPosition = new Vector3(-1000, 0, 0);
					AuxCircle.transform.localPosition = Vector3.zero;
				}
				else
				{
					Signboard1.transform.localPosition = new Vector3(-1000, 0, 0);
					AuxCircle.transform.localPosition = new Vector3(2, 0, 1);
				}
			}
			else if (MatchCase(4))
			{
				if (Input.GetKeyDown(KeyCode.T))
				{
					pr_TRANSPARENT_CUBE = !pr_TRANSPARENT_CUBE;
					var mre = Part2.gameObject.GetComponent<MeshRenderer>();
					mre.materials = pr_TRANSPARENT_CUBE ? pr_transparentMtrls : pr_opaqueMtrls;
				}
			}
		}


		static Sec309 pr_instance = new Sec309(4);

		public static Sec309 PRGSI()
		{
			return pr_instance;
		}


		private Sec309(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Signboard1 = CreateObject_Obj3D("QSignboard1");
				Signboard2 = CreateObject_Obj3D("QSignboard2");
				Part1 = CreateObject_Obj3D("CubePart1");
				Part2 = CreateObject_Obj3D("CubePart2");
				
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				
				// Opaque & Transparent Materials
				Material mtrl1 = BSAuxFuncs.GetSemiTransparentMaterial(0.142f, 0.528f, 0.375f, 0.588f);
				Material mtrl2 = BSAuxFuncs.GetSemiTransparentMaterial(0.37f, 0.37f, 0.54f, 0.28f);
				pr_transparentMtrls = new Material[] { mtrl1, mtrl2 };
				
				Shader standard = Shader.Find("Standard");
				pr_opaqueMtrls = new Material[2];
				pr_opaqueMtrls[0] = new Material(standard);
				pr_opaqueMtrls[0].SetColor("_Color", new Color(0.142f, 0.528f, 0.375f));
				pr_opaqueMtrls[1] = new Material(standard);
				pr_opaqueMtrls[1].SetColor("_Color", new Color(0.371f, 0.371f, 0.538f));

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			i_SB = 1;
			i_degRev = 0;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				CamPos = new Vector3(0.0f, 4.0f, -8.0f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Signboard1.gameObject);
				BSAuxFuncs.SetVisible(1, Signboard2.gameObject);
				
				Signboard1.SetMatrix(Matrix4x4.identity);
				Signboard2.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(7.707109f, 4.276533f, -4.359781f);
				TrgPos = new Vector3(3.722867f, 2.470119f, 0.2073335f);
				uJ = new Vector3(-0.1622038f, 0.9577916f, 0.2373289f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Signboard1.gameObject);
				BSAuxFuncs.SetVisible(1, Signboard2.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Signboard1.SetMatrix(Matrix4x4.identity);
				Signboard2.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);  
				
				CamPos = new Vector3(8.564523f, 5.218285f, -9.470554f);
				TrgPos = new Vector3(1.745296f, 2.12653f, -1.653756f);
				uJ = new Vector3(-0.1853223f, 0.9583338f, 0.2173752f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Part1.gameObject);
				BSAuxFuncs.SetVisible(1, Part2.gameObject);
				Part1.SetMatrix(Matrix4x4.identity);
				Part2.SetMatrix(Matrix4x4.identity);
				
				pr_TRANSPARENT_CUBE = true;
				var mre = Part2.gameObject.GetComponent<MeshRenderer>();
				mre.materials = pr_transparentMtrls;
				
				CamPos = new Vector3(2.729024f, 1.204857f, -1.755599f);	
				TrgPos = new Vector3(0.5783834f, 0.4130237f, 0.3567162f);
				uJ = new Vector3(-0.1670647f, 0.9669919f, 0.1923957f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
