//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec310
{
	public class Sec310 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Signboard1;
		THObject3D Signboard2;
		
		THObject3D Ball;
		THObject3D Axis;
		
		THObject3D AuxCircle;
		THObject3D Arc90;
		
		GameObject DLight;


		// Instance Variables
		int i_degRot = 0;
		
		int i_SB = 1;
		
		readonly Vector3 P = new Vector3(8.697056f, 1.697056f, 5.0f);
		readonly Vector3 M = new Vector3(7.0f, 0.0f, 5.0f);




		void Code1()
		{
			Vector4 P = new Vector4(7.0f, 8.0f, -4.0f, 1.0f);
			Matrix4x4 R = TH3DMath.GetRotation4x4(60.0f, Vector3.right);
			Vector3 P1 = R * P;
			
			Debug.Log("(" + P1.x + ", " + P1.y + ", " + P1.z + ")");
		}


		void Code2()
		{
			i_SB = 1;
			
			i_degRot++;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			if (i_SB == 1)
			{
				Signboard1.SetMatrix(R);
			}
			else
			{
				Signboard2.SetMatrix(R);
			}
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Vector3 wp = Ball.GetWorldPosition();
				Matrix4x4 R = TH3DMath.GetRotation4x4(90.0f, Vector3.forward);
				Vector3 pos = R * TH3DMath.ToVector4(wp - M);
				pos += M;
				
				Ball.SetWorldPosition(pos);
				
				i_INITIALIZED = true;
			}
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(2))
			{
				if (i_SB == 1)
				{
					Signboard2.transform.localPosition = new Vector3(-1000, 0, 0);
					AuxCircle.transform.localPosition = new Vector3(-1000, 0, 0);
				}
				else
				{
					Signboard1.transform.localPosition = new Vector3(-1000, 0, 0);
				}
			}
		}


		static Sec310 pr_instance = new Sec310(3);

		public static Sec310 PRGSI()
		{
			return pr_instance;
		}


		private Sec310(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Signboard1 = CreateObject_Obj3D("QSignboard1");
				Signboard2 = CreateObject_Obj3D("QSignboard2");
				Ball = CreateObject_Obj3D("MiniBall");
				Axis = CreateObject_Obj3D("BlueAxis");
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				Arc90 = CreateObject_Obj3D("Arc90");
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			i_SB = 1;
			i_degRot = 0;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (DLight != null)
			{
				DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
			}
			
			if (MatchCase(1))
			{
				CamPos = new Vector3(0.0f, 4.0f, -8.0f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Signboard1.gameObject);
				BSAuxFuncs.SetVisible(1, Signboard2.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Signboard1.SetMatrix(Matrix4x4.identity);
				Signboard2.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				float rad = new Vector3(2, 0, 1).magnitude;
				AuxCircle.transform.localScale = new Vector3(rad, rad, 1);  
				
				CamPos = new Vector3(5.684361f, 4.617079f, -6.335914f);
				TrgPos = new Vector3(1.542588f, 2.739273f, -1.588243f);
				uJ = new Vector3(-0.1951743f, 0.9582936f, 0.2087595f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Axis.gameObject);
				BSAuxFuncs.SetVisible(1, Arc90.gameObject);
				Ball.SetMatrix(TH3DMath.GetTranslation4x4(P));
				Axis.SetMatrix(TH3DMath.GetTranslation4x4(M));
				Arc90.SetMatrix(TH3DMath.GetTranslation4x4(M));
				
				CamPos = new Vector3(7f, 2.821621f, -0.3179965f);	
				TrgPos = new Vector3(7f, 1.935545f, 2.28489f);
				uJ = new Vector3(0f, 0.9466513f, 0.3222595f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(80, -30, 0);
				}
			}
		}

	}

}
