//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec311
{
	public class Sec311 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Trino;


		// Instance Variables
		int i_degRot = 0;




		void Code1()
		{
			Vector3 posR = new Vector3(2.8284f, 2.4495f, -1.4142f);
			Matrix4x4 rotCR = TH3DMath.GetRotation4x4(50.0f, posR); 
			Trino.SetMatrix(rotCR);
		}


		void Code2()
		{
			Vector3 posR = new Vector3(2.8284f, 2.4495f, -1.4142f);
			Vector3 posG = new Vector3(-2.8284f, 2.4495f, -1.4142f);
			Matrix4x4 rotCR = TH3DMath.GetRotation4x4(50.0f, posR);      // CR軸周りの回転
			Vector3 posG1 = rotCR * posG;
			
			i_degRot++;
			Matrix4x4 rotCG = TH3DMath.GetRotation4x4(i_degRot, posG1);  // CG軸周りの回転
			
			// CR軸周りに回転させてからCG軸周りに回転させる
			Matrix4x4 R = rotCG * rotCR;
			Trino.SetMatrix(R);
		}


		void Code3()
		{
			// (1)
			Matrix4x4 R = TH3DMath.GetRotation4x4(100, new Vector3(3, 4, 5));
			// (2)
			//Matrix4x4 R = TH3DMath.GetRotation4x4(85.28f, new Vector3(-11.53f, 28.2f, 40.65f));
			// (3)
			//Matrix4x4 R = TH3DMath.GetRotation4x4(-24.56f, new Vector3(802.22f, -459.51f, -363.84f));
			
			Vector4 c1 = R.GetColumn(0);    // 第1列目
			Vector4 c2 = R.GetColumn(1);    // 第2列目
			Vector4 c3 = R.GetColumn(2);    // 第3列目
			float mag1 = c1.magnitude;
			float mag2 = c2.magnitude;
			float mag3 = c3.magnitude;
			float dot12 = Vector3.Dot(c1, c2);
			float dot13 = Vector3.Dot(c1, c3);
			float dot23 = Vector3.Dot(c2, c3);
			float Det = R.determinant;      // 行列式
			
			Debug.LogFormat("(1)\n(mag1  {0:f3})  (mag2  {1:f3})  (mag3  {2:f3})\n" +
					"(dot12  {3:f3})  (dot13  {4:f3})  (dot23  {5:f3})\n(Det {6:f3})",
					mag1, mag2, mag3, dot12, dot13, dot23, Det);
			
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec311 pr_instance = new Sec311(3);

		public static Sec311 PRGSI()
		{
			return pr_instance;
		}


		private Sec311(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Trino = CreateObject_Obj3D("Trino");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			i_degRot = 0;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Trino.gameObject);
				Trino.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(3.368557f, 4.713309f, -10.74537f);
				TrgPos = new Vector3(0.3408018f, -0.5396502f, -0.961409f);
				uJ = new Vector3(-0.1309574f, 0.8897821f, 0.4371934f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (MatchCase(1))
				{
					pr_LOOP_CODE = false;
				}
			}
			else if (MatchCase(3))
			{
				CamPos = new Vector3(0.0f, 4.0f, -8.0f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(4))
			{
				CamPos = new Vector3(0.0f, 4.0f, -8.0f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
