//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec313
{
	public class Sec313 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Sphere;
		THObject3D AuxCircle;
		
		THObject3D Wheel;
		THObject3D Part1;
		THObject3D Part2;
		
		GameObject DLight;


		// Instance Variables
		int i_degRot = 0;
		int i_degRev = 0;
		int i_shm = 0;
		
		float i_distance;
		float i_degPart2;
		
		readonly float c_WheelLength = 2.0f * Mathf.PI;  
		readonly Vector3 P = new Vector3(1.0f, 0.0f, 0.5f);
		readonly Vector3 Q = new Vector3(0.5f, 0.0f, 1.0f);
		
		bool pr_TRANSPARENT_CUBE;
		Material[] pr_transparentMtrls;
		Material[] pr_opaqueMtrls;




		void Code1()
		{
			Matrix4x4 T = TH3DMath.GetTranslation4x4(4.0f, 1.0f, -2.0f);
			Matrix4x4 R = TH3DMath.GetRotation4x4(120.0f, Vector3.up);
			Matrix4x4 M = T * R;
			Sphere.SetMatrix(M);
		}


		void Code2()
		{
			Matrix4x4 T = TH3DMath.GetTranslation4x4(4.0f, 1.0f, -2.0f);
			Matrix4x4 R = TH3DMath.GetRotation4x4(120.0f, Vector3.up);
			Matrix4x4 M = R * T;
			Sphere.SetMatrix(M);
		}


		void Code3()
		{
			Matrix4x4 T = TH3DMath.GetTranslation4x4(4.0f, 1.0f, -2.0f);
			Matrix4x4 R = TH3DMath.GetRotation4x4(120.0f, Vector3.up);
			Matrix4x4 S = TH3DMath.GetScale4x4(1.0f, 1.0f, 2.0f);
			Matrix4x4 M = T * R * S;
			Sphere.SetMatrix(M);
		}


		void Code4()
		{
			i_degRot += 3;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(2.0f, 0.0f, 2.0f);
			Matrix4x4 M = T * R;
			Sphere.SetMatrix(M);
		}


		void Code5()
		{
			i_shm += 2;
			float min = 1.0f;
			float max = 2.0f;
			float A = max - min;
			float scale = 0.5f * A * (-Mathf.Cos(i_shm * Mathf.Deg2Rad) + 1.0f) + min;
			Matrix4x4 S = TH3DMath.GetScale4x4(scale);
			
			i_degRev++;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 pos = rev * new Vector4(3, 0, 0, 1);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(pos);
			
			Matrix4x4 M = T * S;
			Sphere.SetMatrix(M);
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				i_distance = 0.0f;
				
				Wheel.SetMatrix(TH3DMath.GetTranslation4x4(6.0f, 1.0f, 0.0f));
				
				i_INITIALIZED = true;
			}

			float vel = 0.0f;
			if (Input.GetKey(KeyCode.H))
			{
				vel = -0.05f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				vel = 0.05f;
			}

			Vector3 pos = Wheel.GetWorldPosition();
			pos += vel * Vector3.right;
			Matrix4x4 T = TH3DMath.GetTranslation4x4(pos);

			i_distance += vel;
			float deg = (i_distance / c_WheelLength) * 360.0f;
			Matrix4x4 R = TH3DMath.GetRotation4x4(-deg, Vector3.forward);

			Wheel.SetMatrix(T * R);
		}



		void Code7()
		{

		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(7))
			{
				if (Input.GetKeyDown(KeyCode.T))
				{
					pr_TRANSPARENT_CUBE = !pr_TRANSPARENT_CUBE;
					var mre = Part2.gameObject.GetComponent<MeshRenderer>();
					mre.materials = pr_TRANSPARENT_CUBE ? pr_transparentMtrls : pr_opaqueMtrls;
				}
			}
		}


		static Sec313 pr_instance = new Sec313(7);

		public static Sec313 PRGSI()
		{
			return pr_instance;
		}


		private Sec313(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere = CreateObject_Obj3D("Sphere035");
				Wheel = CreateObject_Obj3D("TriWheel_2");
				Part1 = CreateObject_Obj3D("CubePartA");
				Part2 = CreateObject_Obj3D("CubePartB");
				
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				
				// Opaque & Transparent Materials
				Material mtrl1 = BSAuxFuncs.GetSemiTransparentMaterial(0.142f, 0.528f, 0.375f, 0.588f);
				Material mtrl2 = BSAuxFuncs.GetSemiTransparentMaterial(0.37f, 0.37f, 0.54f, 0.28f);
				pr_transparentMtrls = new Material[] { mtrl1, mtrl2 };
				
				Shader standard = Shader.Find("Standard");
				pr_opaqueMtrls = new Material[2];
				pr_opaqueMtrls[0] = new Material(standard);
				pr_opaqueMtrls[0].SetColor("_Color", new Color(0.142f, 0.528f, 0.375f));
				pr_opaqueMtrls[1] = new Material(standard);
				pr_opaqueMtrls[1].SetColor("_Color", new Color(0.371f, 0.371f, 0.538f));
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (DLight != null)
			{
				DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
			}
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(7.427967f, 5.469105f, -5.937972f);
				TrgPos = new Vector3(0.08466832f, -0.634822f, -0.2308851f);
				uJ = new Vector3(-0.4184785f, 0.8357841f, 0.3554445f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(7.427967f, 5.469105f, -5.937972f);
				TrgPos = new Vector3(0.08466832f, -0.634822f, -0.2308851f);
				uJ = new Vector3(-0.4184785f, 0.8357841f, 0.3554445f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(6.002689f, 2.44384f, -2.812885f);
				TrgPos = new Vector3(1.64407f, 0.2981458f, 0.5745768f);
				uJ = new Vector3(-0.2902042f, 0.932043f, 0.2169735f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				i_degRot = 0;
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localScale = new Vector3(3.0f, 3.0f, 1.0f);
				
				CamPos = new Vector3(7.219291f, 3.749882f, -5.271893f);
				TrgPos = new Vector3(0.818463f, 0.5988352f, -0.2972586f);
				uJ = new Vector3(-0.3008358f, 0.9317909f, 0.2031343f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				i_degRot = 0;
				i_shm = 0;
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Wheel.gameObject);
				Wheel.SetIdentity();
				
				CamPos = new Vector3(6.0f, 3.520312f, -8.608696f);
				TrgPos = new Vector3(6.0f, 1.207557f, -2.229023f);
				uJ = new Vector3(0f, 0.9401302f, 0.3408154f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(7))
			{
				BSAuxFuncs.SetVisible(1, Part1.gameObject);
				BSAuxFuncs.SetVisible(1, Part2.gameObject);
				Part1.SetMatrix(Matrix4x4.identity);
				Part2.SetMatrix(Matrix4x4.identity);

				i_degPart2 = 0.0f;

				pr_TRANSPARENT_CUBE = true;
				var mre = Part2.gameObject.GetComponent<MeshRenderer>();
				mre.materials = pr_transparentMtrls;
				
				CamPos = new Vector3(3.744529f, 1.851316f, -0.7123114f);
				TrgPos = new Vector3(0.8289611f, 0.4485412f, 0.5384117f);
				uJ = new Vector3(-0.3859373f, 0.9139609f, 0.1254113f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -40, 0);
				}
			}
		}

	}

}
