//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using BSBMTextFactory = OQJ_BSBMTextFactory.BSBMTextFactory;



namespace OQJ_Sec314
{
	public class Sec314 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D FinBall;
		THObject3D RTSphere;
		THObject3D AuxSphereGrid;


		// Instance Variables
		int i_eulX = 0;
		int i_eulY = 0;
		int i_eulZ = 0;




		void Code1()
		{
			if (Input.GetKeyDown(KeyCode.X))
			{
				i_eulX += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Z))
			{
				i_eulZ += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			
			Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
			FinBall.SetMatrix(R);
		}


		void Code2()
		{
			if (Input.GetKey(KeyCode.X))
			{
				i_eulX += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			else if (Input.GetKey(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			else if (Input.GetKey(KeyCode.Z))
			{
				i_eulZ += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			
			Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
			FinBall.SetMatrix(R);
		}




		string GetPads(int src_len, int max_len)
		{
			int pad_len = max_len - src_len;
			
			string pads = "";
			if (pad_len == 1)
			{
				pads = " ";
			}
			else if (pad_len == 2)
			{
				pads = "  ";
			}
			else if (pad_len == 3)
			{
				pads = "   ";
			}
			
			return pads;
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				if (i_eulX > 90)
				{
					i_eulX = 90;
					Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
					FinBall.SetMatrix(R);
				}
				else if (i_eulX < -90)
				{
					i_eulX = -90;
					Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
					FinBall.SetMatrix(R);
				}
				
				if (i_eulY > 180) { i_eulY -= 360; }
				else if (i_eulY < -180) { i_eulY += 360; }
				
				if (i_eulZ > 180) { i_eulZ -= 360; }
				else if (i_eulZ < -180) { i_eulZ += 360; }
				
				
				BSText btxX = BSData.AuxTextList[0];
				BSText btxY = BSData.AuxTextList[1];
				BSText btxZ = BSData.AuxTextList[2];
				
				string ss = "" + i_eulX;
				string pad = GetPads(ss.Length, 3);     
				btxX.ChangeText(2, ss + pad);
				
				ss = "" + i_eulY;
				pad = GetPads(ss.Length, 4);            
				btxY.ChangeText(2, ss + pad);
				
				ss = "" + i_eulZ;                       
				pad = GetPads(ss.Length, 4);
				btxZ.ChangeText(2, ss + pad);
			}
		}


		static Sec314 pr_instance = new Sec314(2);

		public static Sec314 PRGSI()
		{
			return pr_instance;
		}


		private Sec314(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				FinBall = CreateObject_Obj3D("FinBall");
				
				RTSphere = CreateObject_Obj3D("RTSphere");
				AuxSphereGrid = CreateObject_Obj3D("AuxSphereGrid");
				
				MeshRenderer mre = RTSphere.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.0f, 0.8f, 0.25f);
				
				
				// Text : Euler Angles (x ##  y ##  z ##)
				//	x [-90, 90]	, y [-180, 180] , z [-180, 180]
				float fov = BSData.CameraController3D.GetFieldOfView();
				Vector2 ltr_size = BSAuxFuncs.GetLetterSize(fov, BSData.c_LetterScale_Regular);
				
				float rp_y = 0.73f;
				float rp_x_eX = -0.43f;
				float rp_x_eY = rp_x_eX + ltr_size.x * 7;
				float rp_x_eZ = rp_x_eY + ltr_size.x * 7;
				
				string text = "x 0  ";      
				BSText btxX = BSBMTextFactory.MakeText(text, ltr_size, "Euler-x");
				btxX.SetRelativePosition3D(rp_x_eX, rp_y);
				BSData.AuxTextList.Add(btxX);
				
				text = "y 0   ";            
				BSText btxY = BSBMTextFactory.MakeText(text, ltr_size, "Euler-y");
				btxY.SetRelativePosition3D(rp_x_eY, rp_y);
				BSData.AuxTextList.Add(btxY);
				
				text = "z 0   ";            
				BSText btxZ = BSBMTextFactory.MakeText(text, ltr_size, "Euler-z");
				btxZ.SetRelativePosition3D(rp_x_eZ, rp_y);
				BSData.AuxTextList.Add(btxZ);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			i_eulX = 0;
			i_eulY = 0;
			i_eulZ = 0;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, FinBall.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				
				FinBall.SetMatrix(Matrix4x4.identity);
				RTSphere.SetMatrix(Matrix4x4.identity);
				AuxSphereGrid.SetMatrix(TH3DMath.GetScale4x4(1.002f));
				
				CamPos = new Vector3(-0.7102963f, 0.6938158f, -2.321411f);
				TrgPos = new Vector3(-0.1802267f, 0.1286383f, -0.3202901f);
				uJ = new Vector3(0.07623807f, 0.9646534f, 0.2522531f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
