//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using BSBMTextFactory = OQJ_BSBMTextFactory.BSBMTextFactory;



namespace OQJ_Sec315
{
	public class Sec315 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		THObject3D XYZ_Axes;
		
		THObject3D MiniPlane;
		THObject3D RTSphere;
		THObject3D AuxSphereGrid;
		THObject3D AuxOrigPoint;	

		THObject3D MP_mz;
		THObject3D MP_zyx;
		THObject3D MP_zyz;

		GameObject DLight;


		// Instance Variables
		int i_eulX = 0;
		int i_eulY = 0;
		int i_eulZ = 0;

		int i_eulZ1 = 0;
		int i_eulZ2 = 0;



		// // extrinsic ZXY
		void Code1()
		{
			if (Input.GetKeyDown(KeyCode.X))
			{
				i_eulX += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Z))
			{
				i_eulZ += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			
			Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
			MiniPlane.SetMatrix(R);
		}



		// // extrinsic ZXY
		void Code2()
		{
			if (Input.GetKey(KeyCode.X))
			{
				i_eulX += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			else if (Input.GetKey(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			else if (Input.GetKey(KeyCode.Z))
			{
				i_eulZ += (THUtil.IsShiftDown()) ? -1 : 1;
			}
			
			Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
			MiniPlane.SetMatrix(R);
		}



		// // intrinsic zyx
		void Code3()
		{
			if (Input.GetKeyDown(KeyCode.X))
			{
				i_eulX += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Z))
			{
				i_eulZ += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			
			Matrix4x4 Rx = TH3DMath.GetRotation4x4(i_eulX, Vector3.right);
			Matrix4x4 Ry = TH3DMath.GetRotation4x4(i_eulY, Vector3.up);
			Matrix4x4 Rz = TH3DMath.GetRotation4x4(i_eulZ, Vector3.forward);

			Matrix4x4 R = Rz * Ry * Rx;
			MiniPlane.SetMatrix(R);
		}



		// // intrinsic zyz
		void Code4()
		{
			if (Input.GetKeyDown(KeyCode.X))
			{
				i_eulZ1 += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Y))
			{
				i_eulY += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			else if (Input.GetKeyDown(KeyCode.Z))
			{
				i_eulZ2 += (THUtil.IsShiftDown()) ? -10 : 10;
			}
			
			Matrix4x4 Rz1 = TH3DMath.GetRotation4x4(i_eulZ1, Vector3.forward);
			Matrix4x4 Ry  = TH3DMath.GetRotation4x4(i_eulY, Vector3.up);
			Matrix4x4 Rz2 = TH3DMath.GetRotation4x4(i_eulZ2, Vector3.forward);

			Matrix4x4 R = Rz1 * Ry * Rz2;
			MiniPlane.SetMatrix(R);
		}







///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////



		string GetPads(int src_len, int max_len)
		{
			int pad_len = max_len - src_len;
			
			string pads = "";
			if (pad_len == 1)
			{
				pads = " ";
			}
			else if (pad_len == 2)
			{
				pads = "  ";
			}
			else if (pad_len == 3)
			{
				pads = "   ";
			}
			else if (pad_len == 4)
			{
				pads = "    ";
			}
			
			return pads;
		}



		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;
			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				if (i_eulX < -90 || 90 < i_eulX)
				{
					i_eulX = (i_eulX < 0) ? -90 : 90;

					Matrix4x4 R = TH3DMath.GetEulerRotation4x4(i_eulX, i_eulY, i_eulZ);
					MiniPlane.SetMatrix(R);
				}
				
				if (i_eulY > 180) { i_eulY -= 360; }
				else if (i_eulY < -180) { i_eulY += 360; }
				
				if (i_eulZ > 180) { i_eulZ -= 360; }
				else if (i_eulZ < -180) { i_eulZ += 360; }
				
				
				BSText btxX = BSData.AuxTextList[0];
				BSText btxY = BSData.AuxTextList[1];
				BSText btxZ = BSData.AuxTextList[2];
				
				string ss = "x " + i_eulX;
				string pad = GetPads(ss.Length, 7);     
				btxX.ChangeText(0, ss + pad);
				
				ss = "" + i_eulY;
				pad = GetPads(ss.Length, 5);            
				btxY.ChangeText(2, ss + pad);
				
				ss = "z " + i_eulZ;                       
				pad = GetPads(ss.Length, 7);
				btxZ.ChangeText(0, ss + pad);
			}
			else if (MatchCase(3))
			{
				if (i_eulY < -90 || 90 < i_eulY)
				{
					i_eulY = (i_eulY < 0) ? -90 : 90;

					Matrix4x4 Rx = TH3DMath.GetRotation4x4(i_eulX, Vector3.right);
					Matrix4x4 Ry = TH3DMath.GetRotation4x4(i_eulY, Vector3.up);
					Matrix4x4 Rz = TH3DMath.GetRotation4x4(i_eulZ, Vector3.forward);

					Matrix4x4 R = Rz * Ry * Rx;
					MiniPlane.SetMatrix(R);
				}
				
				if (i_eulX > 180) { i_eulX -= 360; }
				else if (i_eulX < -180) { i_eulX += 360; }
				
				if (i_eulZ > 180) { i_eulZ -= 360; }
				else if (i_eulZ < -180) { i_eulZ += 360; }
				
				
				BSText btxX = BSData.AuxTextList[0];
				BSText btxY = BSData.AuxTextList[1];
				BSText btxZ = BSData.AuxTextList[2];
				
				string ss = "x " + i_eulX;
				string pad = GetPads(ss.Length, 7);     
				btxX.ChangeText(0, ss + pad);
				
				ss = "" + i_eulY;
				pad = GetPads(ss.Length, 5);            
				btxY.ChangeText(2, ss + pad);
				
				ss = "z " + i_eulZ;                       
				pad = GetPads(ss.Length, 7);
				btxZ.ChangeText(0, ss + pad);
			}
			else if (MatchCase(4))
			{
				if (i_eulY < 0 || 180 < i_eulY)
				{
					i_eulY = (i_eulY < 0) ? 0 : 180;

					Matrix4x4 Rz1 = TH3DMath.GetRotation4x4(i_eulZ1, Vector3.forward);
					Matrix4x4 Ry  = TH3DMath.GetRotation4x4(i_eulY, Vector3.up);
					Matrix4x4 Rz2 = TH3DMath.GetRotation4x4(i_eulZ2, Vector3.forward);

					Matrix4x4 R = Rz1 * Ry * Rz2;
					MiniPlane.SetMatrix(R);
				}
				
				if (i_eulZ1 > 180) { i_eulZ1 -= 360; }
				else if (i_eulZ1 < -180) { i_eulZ1 += 360; }
				
				if (i_eulZ2 > 180) { i_eulZ2 -= 360; }
				else if (i_eulZ2 < -180) { i_eulZ2 += 360; }
				
				
				BSText btxX = BSData.AuxTextList[0];
				BSText btxY = BSData.AuxTextList[1];
				BSText btxZ = BSData.AuxTextList[2];
				
				string ss = "z1 " + i_eulZ1;
				string pad = GetPads(ss.Length, 7);     
				btxX.ChangeText(0, ss + pad);
				
				ss = " " + i_eulY;
				pad = GetPads(ss.Length, 5);            
				btxY.ChangeText(2, ss + pad);
				
				ss = "z2 " + i_eulZ2;                       
				pad = GetPads(ss.Length, 7);
				btxZ.ChangeText(0, ss + pad);
			}

			BSData.CameraController3D.UpdateCameraData();

		}


		static Sec315 pr_instance = new Sec315(4);

		public static Sec315 PRGSI()
		{
			return pr_instance;
		}


		private Sec315(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				XYZ_Axes = CreateObject_Obj3D("XYZ_Axes_2");
				MP_mz = CreateObject_Obj3D("MiniPlane_mZ");
				MP_zyx = CreateObject_Obj3D("MiniPlane_zyx");
				MP_zyz = CreateObject_Obj3D("MiniPlane_zyz");
				
				RTSphere = CreateObject_Obj3D("RTSphere");
				AuxSphereGrid = CreateObject_Obj3D("AuxSphereGrid");
				AuxOrigPoint = CreateObject_Obj3D("PointB");

				MeshRenderer mre = RTSphere.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.0f, 0.8f, 0.25f);
				BSAuxFuncs.SetStandardMaterial(AuxOrigPoint, 0.8f, 0.8f, 0.0f);

				DLight = BSAuxFuncs.GetDirectionalLight();


				// // Text : Euler Angles (x ##  y ##  z ##)
				// zyx	:	x [-180, 180] , y [-90, 90]	, z [-180, 180]	
				// zyz	:	z1[-180, 180] , y [0, 180]	, z2[-180, 180]	
				float fov = BSData.CameraController3D.GetFieldOfView();
				Vector2 ltr_size = BSAuxFuncs.GetLetterSize(fov, BSData.c_LetterScale_Regular * 1.3f);

				float rp_x = -0.6f;
				float rp_y_eX = 0.65f;
				float rp_y_eY = rp_y_eX - ltr_size.y * 1.5f;
				float rp_y_eZ = rp_y_eY - ltr_size.y * 1.5f;
				
				string text = "x 0    ";      
				BSText btxX = BSBMTextFactory.MakeText(text, ltr_size, "Euler-x", BSText.c_ROBOT);
				btxX.SetRelativePosition3D(rp_x, rp_y_eX);
				BSData.AuxTextList.Add(btxX);
				
				text = "y 0    ";            
				BSText btxY = BSBMTextFactory.MakeText(text, ltr_size, "Euler-y", BSText.c_ROBOT);
				btxY.SetRelativePosition3D(rp_x, rp_y_eY);
				BSData.AuxTextList.Add(btxY);
				
				text = "z 0    ";            
				BSText btxZ = BSBMTextFactory.MakeText(text, ltr_size, "Euler-z", BSText.c_ROBOT);
				btxZ.SetRelativePosition3D(rp_x, rp_y_eZ);
				BSData.AuxTextList.Add(btxZ);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			i_eulX = 0;
			i_eulY = 0;
			i_eulZ = 0;
			i_eulZ1 = 0;
			i_eulZ2 = 0;
			
			//BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			//CrdSys.SetIdentity();

			BSAuxFuncs.SetVisible(0, CrdSys.gameObject);
			BSAuxFuncs.SetVisible(1, XYZ_Axes.gameObject);
			
			if (MatchCase(1, 2))
			{
				MiniPlane = MP_mz;

				BSAuxFuncs.SetVisible(1, MiniPlane.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				BSAuxFuncs.SetVisible(1, AuxOrigPoint.gameObject);
				
				MiniPlane.SetIdentity();
				RTSphere.SetIdentity();
				AuxSphereGrid.SetMatrix(TH3DMath.GetScale4x4(1.002f));
				AuxOrigPoint.SetLocalPosition(0.0f, 0.0f, -1.008f);
				AuxOrigPoint.SetLocalScale(0.018f);
				
				CamPos = new Vector3(-1.607496f, 1.419979f, -1.95929f);
				TrgPos = new Vector3(0.2334476f, -0.2610493f, 0.2578177f);
				uJ = new Vector3(0.3262999f, 0.8637658f, 0.3839753f);

				if(DLight != null)
				{
					DLight.transform.rotation = Quaternion.Euler(50, -30, 0);
				}
			}
			else if (MatchCase(3))
			{
				MiniPlane = MP_zyx;

				BSAuxFuncs.SetVisible(1, MiniPlane.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				BSAuxFuncs.SetVisible(1, AuxOrigPoint.gameObject);

				MiniPlane.SetIdentity();
				RTSphere.SetIdentity();
				AuxSphereGrid.SetMatrix(TH3DMath.GetRotation4x4(90, Vector3.right) * TH3DMath.GetScale4x4(1.002f));
				AuxOrigPoint.SetLocalPosition(1.008f, 0f, 0f);
				AuxOrigPoint.SetLocalScale(0.018f);

				CamPos = new Vector3(1.95929f, 1.607496f, 1.419979f);
				TrgPos = new Vector3(-0.2578192f, -0.2334487f, -0.2610505f);
				uJ = new Vector3(-0.3839753f, -0.3263f, 0.8637658f);

				if(DLight != null)
				{
					DLight.transform.rotation = Quaternion.Euler(40, -160, 0);
				}
			}
			else if (MatchCase(4))
			{
				MiniPlane = MP_zyz;

				BSAuxFuncs.SetVisible(1, MiniPlane.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				BSAuxFuncs.SetVisible(1, AuxOrigPoint.gameObject);

				MiniPlane.SetIdentity();
				RTSphere.SetIdentity();
				AuxSphereGrid.SetMatrix(TH3DMath.GetRotation4x4(-90, Vector3.right) * TH3DMath.GetScale4x4(1.002f));
				AuxOrigPoint.SetLocalPosition(1.008f, 0f, 0f);
				AuxOrigPoint.SetLocalScale(0.018f);
				
				CamPos = new Vector3(1.95929f, 1.607496f, 1.419979f);
				TrgPos = new Vector3(-0.2578192f, -0.2334487f, -0.2610505f);
				uJ = new Vector3(-0.3839753f, -0.3263f, 0.8637658f);

				if(DLight != null)
				{
					DLight.transform.rotation = Quaternion.Euler(40, -160, 0);
				}
			}


			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}
	}

}
