//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec316
{
	public class Sec316 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Octah;
		THObject3D Handle;
		THObject3D Stand;


		// Instance Variables
		float i_shmOctah;
		float i_degHandle;
		float i_shmStand;
		
		readonly Vector3 c_attachPos_Octah = new Vector3(4.0f, 3.6546f, 0.0f);
		readonly Vector3 c_attachPos_Handle = new Vector3(0.0f, 3.0f, 1.8f);




		void Code1()
		{
			Matrix4x4 localOctah = TH3DMath.GetTranslation4x4(c_attachPos_Octah);
			Matrix4x4 localHandle = TH3DMath.GetTranslation4x4(c_attachPos_Handle);
			Matrix4x4 localStand = Matrix4x4.identity;
			
			Matrix4x4 worldOctah = localStand * localHandle * localOctah;
			Matrix4x4 worldHandle = localStand * localHandle;
			Matrix4x4 worldStand = localStand;
			
			Octah.SetMatrix(worldOctah);
			Handle.SetMatrix(worldHandle);
			Stand.SetMatrix(worldStand);
		}


		void Code2()
		{
			// Octah
			i_shmOctah += 4.0f;
			float scale = 0.5f * Mathf.Sin(i_shmOctah * Mathf.Deg2Rad) + 1.5f;  // 1.0 -- 2.0
			Matrix4x4 S = TH3DMath.GetScale4x4(scale, 1.0f, scale);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(c_attachPos_Octah);
			Matrix4x4 localOctah = T * S;
			
			// Handle
			i_degHandle++;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degHandle, Vector3.up);
			Matrix4x4 localHandle = R;
			
			Matrix4x4 worldOctah = localHandle * localOctah;
			Matrix4x4 worldHandle = localHandle;
			
			Octah.SetMatrix(worldOctah);
			Handle.SetMatrix(worldHandle);
		}


		void Code3()
		{
			// Octah
			i_shmOctah += 4.0f;
			float scale = 0.5f * Mathf.Sin(i_shmOctah * Mathf.Deg2Rad) + 1.5f;  // 1.0 -- 2.0
			Matrix4x4 S = TH3DMath.GetScale4x4(scale, 1.0f, scale);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(c_attachPos_Octah);
			Matrix4x4 localOctah = T * S;
			
			// Handle
			i_degHandle++;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degHandle, Vector3.up);
			T = TH3DMath.GetTranslation4x4(c_attachPos_Handle);
			Matrix4x4 localHandle = T * R;
			
			// Stand
			i_shmStand += 1.0f;
			float mv = 2.0f * Mathf.Sin(i_shmStand * Mathf.Deg2Rad) + 2.0f;  // 0.0 -- 4.0
			Matrix4x4 localStand = TH3DMath.GetTranslation4x4(0.0f, 0.0f, mv);
			
			
			Matrix4x4 worldOctah = localStand * localHandle * localOctah;
			Matrix4x4 worldHandle = localStand * localHandle;
			Matrix4x4 worldStand = localStand;
			
			Octah.SetMatrix(worldOctah);
			Handle.SetMatrix(worldHandle);
			Stand.SetMatrix(worldStand);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec316 pr_instance = new Sec316(3);

		public static Sec316 PRGSI()
		{
			return pr_instance;
		}


		private Sec316(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Octah = CreateObject_Obj3D("Octah");
				Handle = CreateObject_Obj3D("Handle");
				Stand = CreateObject_Obj3D("Stand");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			i_shmOctah = 0.0f;
			i_degHandle = 0.0f;
			i_shmStand = 0.0f;
			
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Octah.gameObject);
				BSAuxFuncs.SetVisible(1, Handle.gameObject);
				BSAuxFuncs.SetVisible(1, Stand.gameObject);
				
				Octah.SetMatrix(Matrix4x4.identity);
				Handle.SetMatrix(Matrix4x4.identity);
				Stand.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(12.40006f, 7.846384f, -11.04819f);
				TrgPos = new Vector3(0.700168f, 3.628634f, 1.075029f);
				uJ = new Vector3(-0.1839916f, 0.9698431f, 0.159848f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Octah.gameObject);
				BSAuxFuncs.SetVisible(1, Handle.gameObject);
				
				Octah.SetMatrix(Matrix4x4.identity);
				Handle.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(11.94354f, 9.522049f, -12.79614f);
				TrgPos = new Vector3(0.2437019f, 1.954314f, -0.6728849f);
				uJ = new Vector3(-0.2684591f, 0.9119778f, 0.3102037f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Octah.gameObject);
				BSAuxFuncs.SetVisible(1, Handle.gameObject);
				BSAuxFuncs.SetVisible(1, Stand.gameObject);
				
				Octah.SetMatrix(Matrix4x4.identity);
				Handle.SetMatrix(Matrix4x4.identity);
				Stand.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(12.90278f, 10.06772f, -10.29913f);
				TrgPos = new Vector3(1.086972f, 4.69728f, 1.944145f);
				uJ = new Vector3(-0.236805f, 0.9529073f, 0.1894497f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
