//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec317
{
	public class Sec317 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D[] Cube = new THObject3D[26];


		// Instance Variables
		bool i_ROTATE;
		int i_degX;
		int i_addDeg;
		Vector3 i_rotAxis;
		
		int i_rotCounter;
		int i_rotMaxCount;
		int[] i_rotCubeIndices;
		
		
		Vector3[] i_curCubePos = new Vector3[26];
		
		readonly Vector3[] c_initCubePos =
		{
		new Vector3(-2, 2, -2),		
		new Vector3(0, 2, -2),
		new Vector3(2, 2, -2),
		
		new Vector3(-2, 0, -2),		
		new Vector3(0, 0, -2),
		new Vector3(2, 0, -2),
		
		new Vector3(-2, -2, -2),	
		new Vector3(0, -2, -2),
		new Vector3(2, -2, -2),
		
		
		new Vector3(-2, 2, 0),		
		new Vector3(0, 2, 0),
		new Vector3(2, 2, 0),
		
		new Vector3(-2, 0, 0),		
		new Vector3(2, 0, 0),		
		
		new Vector3(-2, -2, 0),		
		new Vector3(0, -2, 0),
		new Vector3(2, -2, 0),
		
		
		new Vector3(-2, 2, 2),		
		new Vector3(0, 2, 2),
		new Vector3(2, 2, 2),
		
		new Vector3(-2, 0, 2),		
		new Vector3(0, 0, 2),
		new Vector3(2, 0, 2),
		
		new Vector3(-2, -2, 2),	
		new Vector3(0, -2, 2),
		new Vector3(2, -2, 2)
		};




		void Code1()
		{
			i_degX += 1;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degX, Vector3.right);
			
			// 初期状態における右列のCubeのインデックス (9個のCube)
			int[] idcs = { 2, 5, 8, 11, 13, 16, 19, 22, 25 };
			foreach (int i in idcs)
			{
				Cube[i].SetMatrix(R);
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				i_ROTATE = false;
				i_rotMaxCount = 30;
				i_rotCounter = 0;
				i_rotCubeIndices = new int[0];
				
				for (int i = 0; i < c_initCubePos.Length; i++)
				{
					i_curCubePos[i] = c_initCubePos[i];
				}
				
				i_INITIALIZED = true;
			}
			
			
			if (i_ROTATE)
			{
				Matrix4x4 R = TH3DMath.GetRotation4x4(i_addDeg, i_rotAxis);
				
				foreach (int idx in i_rotCubeIndices)
				{
					Matrix4x4 curM = Cube[idx].GetMatrix();
					Matrix4x4 M = R * curM;
					Cube[idx].SetMatrix(M);
				}
				
				i_rotCounter++;
				if (i_rotCounter >= i_rotMaxCount)
				{
					i_ROTATE = false;
					
					foreach (int idx in i_rotCubeIndices)
					{
						i_curCubePos[idx] = Cube[idx].GetMatrix() * TH3DMath.ToVector4(c_initCubePos[idx]);
					}
				}
				
				return;
			}
			
			
			List<int> wL = new List<int>();
			
			if (Input.GetKeyDown(KeyCode.U))
			{
				for (int i = 0; i < i_curCubePos.Length; i++)
				{
					if (i_curCubePos[i].y >= 2.0f - 0.01f)
					{
						wL.Add(i);
					}
				}
				
				i_rotAxis = Vector3.up;
			}
			else if (Input.GetKeyDown(KeyCode.R))
			{
				for (int i = 0; i < i_curCubePos.Length; i++)
				{
					if (i_curCubePos[i].x >= 2.0f - 0.01f)
					{
						wL.Add(i);
					}
				}
				
				i_rotAxis = Vector3.right;
			}
			else
			{
				return;
			}
			
			
			i_rotCubeIndices = wL.ToArray();
			i_ROTATE = true;
			i_rotCounter = 0;
			i_addDeg = THUtil.IsShiftDown() ? (-90 / i_rotMaxCount) : (90 / i_rotMaxCount);
		}



		void Code3()
		{
			if (!i_INITIALIZED)
			{
				
				i_INITIALIZED = true;
			}
			

		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec317 pr_instance = new Sec317(3);

		public static Sec317 PRGSI()
		{
			return pr_instance;
		}


		private Sec317(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				
				for (int i = 0; i < Cube.Length; i++)
				{
					string name = "C" + ((i < 10) ? "0" + i : "" + i);
					Cube[i] = CreateObject_Obj3D(name);
				}

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			if (MatchCase(1, 2, 3))
			{
				for (int i = 0; i < Cube.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Cube[i].gameObject);
					Cube[i].SetMatrix(Matrix4x4.identity);
				}
				
				CamPos = new Vector3(12.40184f, 6.712972f, -14.77941f);
				TrgPos = new Vector3(0.05304391f, -0.02162878f, -0.08771291f);
				uJ = new Vector3(-0.2094753f, 0.9435819f, 0.2564632f);
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				i_degX = 0;
			}
		}

	}

}
