//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using ClReel = OQJ_Main03.ClReel;



namespace OQJ_Sec318
{
	public class Sec318 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		ClReel Reel_src;
		ClReel[] Reel = new ClReel[3];


		// Instance Variables
		System.Random i_random = new System.Random();
		
		float i_v0;
		float i_g;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				i_v0 = 400.0f;
				i_g = 40.0f;
				Reel[0].rotDeg = 0.0f;
				Reel[0].ROTATE = false;
				Reel[0].SetWorldPosition(0.0f, 3.2f, 0.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.S) && !Reel[0].ROTATE)
			{
				Reel[0].angVel = i_v0;
				Reel[0].ROTATE = true;
			}
			
			if (!Reel[0].ROTATE) { return; }
			
			float dt = Time.deltaTime;
			Reel[0].angVel -= i_g * dt;
			
			if (Reel[0].angVel <= 0.0f)
			{
				Reel[0].ROTATE = false;
				return;
			}
			
			float dr = Reel[0].angVel * dt;
			Reel[0].rotDeg += dr;
			Matrix4x4 R = TH3DMath.GetRotation4x4(Reel[0].rotDeg, Vector3.left);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(0.0f, 3.2f, 0.0f);
			Matrix4x4 M = T * R;
			Reel[0].SetMatrix(M);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				i_v0 = 500.0f;
				i_g = 40.0f;
				Reel[0].rotDeg = 0.0f;
				Reel[0].ROTATE = false;
				Reel[0].SetWorldPosition(0.0f, 3.2f, 0.0f);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.S) && !Reel[0].ROTATE)
			{
				float coef = (float)i_random.NextDouble();     // [0.0, 1.0)
				Reel[0].angVel = i_v0 + coef * 360;
				Reel[0].nextBaseline = 0;
				Reel[0].ROTATE = true;
			}
			
			if (!Reel[0].ROTATE)
			{
				return;
			}
			
			float dt = Time.deltaTime;
			
			if (Reel[0].nextBaseline == 0)
			{
				Reel[0].angVel -= i_g * dt;
				if (Reel[0].angVel < 16.0f)
				{
					Reel[0].angVel = 16.0f;
					Reel[0].nextBaseline = Mathf.CeilToInt(Reel[0].rotDeg / 60.0f) * 60;
				}
			}
			
			float dr = Reel[0].angVel * dt;
			Reel[0].rotDeg += dr;
			
			if (Reel[0].nextBaseline > 0 && Reel[0].nextBaseline <= Reel[0].rotDeg)
			{
				Reel[0].rotDeg = Reel[0].nextBaseline % 360;  
				Reel[0].ROTATE = false;
			}
			
			Matrix4x4 R = TH3DMath.GetRotation4x4(Reel[0].rotDeg, Vector3.left);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(0.0f, 3.2f, 0.0f);
			Matrix4x4 M = T * R;
			Reel[0].SetMatrix(M);
		}



		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Reel[0].SetWorldPosition(-2.05f, 3.2f, 0.0f);
				Reel[1].SetWorldPosition(0.0f, 3.2f, 0.0f);
				Reel[2].SetWorldPosition(2.05f, 3.2f, 0.0f);
				
				i_INITIALIZED = true;
			}
			
			
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec318 pr_instance = new Sec318(3);

		public static Sec318 PRGSI()
		{
			return pr_instance;
		}


		private Sec318(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Reel_src = (ClReel)CreateObject_Others("GrReel");
				
				Texture2D greekTex = Resources.Load<Texture2D>("images/GreekLetters");
				var mre = Reel_src.gameObject.GetComponent<MeshRenderer>();
				mre.material.SetTexture("_MainTex", greekTex);
				mre.material.SetColor("_Color", new Color(0.8f, 0.8f, 0.8f, 1.0f));
				
				Reel[0] = Reel_src;
				Reel[1] = (ClReel)CreateObject_InsOthers3D(Reel_src.gameObject, "GrReel");
				Reel[2] = (ClReel)CreateObject_InsOthers3D(Reel_src.gameObject, "GrReel");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Reel[0].gameObject);
				Reel[0].SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(0f, 3.15f, -11.22519f);
				TrgPos = new Vector3(0f, 3.15f, -1.898025f);
				uJ = new Vector3(0f, 1f, 0f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				for (int i = 0; i < Reel.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Reel[i].gameObject);
					Reel[i].SetMatrix(Matrix4x4.identity);
				}
				
				CamPos = new Vector3(0f, 3.15f, -11.22519f);
				TrgPos = new Vector3(0f, 3.15f, -1.898025f);
				uJ = new Vector3(0f, 1f, 0f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
