//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec320
{
	public class Sec320 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		THObject3D RZ_CS;
		
		THObject3D Blue;
		THObject3D Green;
		
		THObject3D AuxCircleL;
		THObject3D AuxCircleR;
		
		GameObject MainCamera;
		Camera pr_COMP_Main;
		
		GameObject SubCamera;
		Camera pr_COMP_SubCamera;
		
		GameObject BackgroundCamera;
		Camera pr_COMP_BackgroundCamera;
		
		GameObject DLight;


		// Instance Variables
		float pr_rotDeg;
		Vector3 pr_posBlueR;






		void Code1()
		{
			Vector3 posB = GetBluePosition();

		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////



		Vector3 GetBluePosition()
		{
			return pr_posBlueR;
		}



		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			if (MatchCase(1))
			{
				pr_rotDeg++;
				Vector3 pos = TH3DMath.GetRotation4x4(pr_rotDeg, -Vector3.forward) * new Vector3(3, 0, 0);
				pos = TH3DMath.GetEulerRotation4x4(60, -50, 0) * pos;
				pos += new Vector3(100 + 4, 3, 6);
				Blue.SetWorldPosition(pos);
				
				Vector3 wp = pos - 100 * Vector3.right;
				wp.z = -wp.z;
				pr_posBlueR = TH3DMath.GetRotation4x4(90.0f, Vector3.right) * wp;
			}
		}


		void PostProcess()
		{
			Blue.SetLocalScale(0.35f);
			Green.SetLocalScale(0.35f);
		}


		static Sec320 pr_instance = new Sec320(1);

		public static Sec320 PRGSI()
		{
			return pr_instance;
		}


		private Sec320(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				RZ_CS = CreateObject_Obj3D("RZ_CS");
				Blue = CreateObject_Obj3D("PointB");
				Green = CreateObject_Obj3D("PointG");
				AuxCircleL = CreateObject_Obj3D("AuxCircle");
				AuxCircleR = CreateObject_InsObj3D(AuxCircleL.gameObject);
				
				
				// Main Camera	
				MainCamera = BSAuxFuncs.GetMainCamera();
				pr_COMP_Main = MainCamera.GetComponent<Camera>();
				pr_COMP_Main.nearClipPlane = 0.1f;
				pr_COMP_Main.farClipPlane = 100.0f;
				pr_COMP_Main.depth = 0;
				pr_COMP_Main.rect = new Rect(0.01f, 0.014f, 0.486f, 0.972f);
				
				// Sub Camera
				SubCamera = new GameObject();
				SubCamera.name = "Camera_2";
				pr_COMP_SubCamera = SubCamera.AddComponent<Camera>();
				pr_COMP_SubCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_COMP_SubCamera.backgroundColor = BSData.BackgroundColor;
				pr_COMP_SubCamera.orthographic = false;  
				pr_COMP_SubCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_COMP_SubCamera.depth = 2;
				pr_COMP_SubCamera.rect = new Rect(0.504f, 0.014f, 0.486f, 0.972f);
				pr_COMP_SubCamera.enabled = true;
				
				// 'Background-Camera'
				BackgroundCamera = new GameObject();
				BackgroundCamera.name = "Background-Camera";
				pr_COMP_BackgroundCamera = BackgroundCamera.AddComponent<Camera>();
				pr_COMP_BackgroundCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_COMP_BackgroundCamera.backgroundColor = new Color(0.38f, 0.38f, 0.38f);
				pr_COMP_BackgroundCamera.orthographic = false;  
				pr_COMP_BackgroundCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_COMP_BackgroundCamera.depth = -1;
				pr_COMP_BackgroundCamera.rect = new Rect(0.0f, 0.0f, 1.0f, 1.0f);
				pr_COMP_BackgroundCamera.enabled = true;
				BackgroundCamera.transform.position = new Vector3(0, 1000, 0);
				
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, RZ_CS.gameObject);
				RZ_CS.SetIdentity();
				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				Blue.SetIdentity();
				BSAuxFuncs.SetVisible(1, Green.gameObject);
				Green.SetIdentity();
				BSAuxFuncs.SetVisible(1, AuxCircleL.gameObject);
				AuxCircleL.SetIdentity();
				BSAuxFuncs.SetVisible(1, AuxCircleR.gameObject);
				AuxCircleR.SetIdentity();
				
				RZ_CS.SetWorldPosition(100f, 0, 0);
				
				Blue.SetLocalScale(0.35f);
				Green.SetLocalScale(0.35f);
				
				AuxCircleL.SetWorldPosition(4, 3, 6);
				AuxCircleL.SetWorldRotation(60, -50, 0);
				AuxCircleL.SetLocalScale(3.0f);
				AuxCircleR.SetWorldPosition(100 + 4, 3, 6);
				AuxCircleR.SetWorldRotation(60, -50, 0);
				AuxCircleR.SetLocalScale(3.0f);
				
				CamPos = new Vector3(4.634171f, 5.193913f, -10.97013f);
				TrgPos = new Vector3(4.314411f, 3.981814f, 6.250083f);
				uJ = new Vector3(-0.0228997f, 0.9973001f, 0.06977289f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				
				Vector3 p2 = CamPos;
				p2.x += 100;
				SubCamera.transform.position = p2;
				SubCamera.transform.rotation = MainCamera.transform.rotation;;
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
				}
			}
		}

	}

}
