//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec321
{
	public class Sec321 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		THObject3D CameraObject;
		THObject3D Grid;
		
		GameObject DLight;


		// Instance Variables
		Matrix4x4 T, R;






		void Code1()
		{
			if (Input.GetKeyDown(KeyCode.C))
			{
				Reset();
			}
			
			
			if (Input.GetKeyDown(KeyCode.V))
			{
				Matrix4x4 V = Matrix4x4.identity;
				MoveCamera(V);
			}
		}








///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////



		void MoveCamera(Matrix4x4 mtx)
		{
			Matrix4x4 curMtx = CameraObject.GetMatrix();
			Vector3 c0 = curMtx.GetColumn(0);
			Vector3 c1 = curMtx.GetColumn(1);
			Vector3 c2 = curMtx.GetColumn(2);
			Vector3 c3 = curMtx.GetColumn(3);
			if (THUtil.IsZero((c0 - Vector3.right).sqrMagnitude) &&
			THUtil.IsZero((c1 - Vector3.up).sqrMagnitude) &&
			THUtil.IsZero((c2 - Vector3.forward).sqrMagnitude) &&
			THUtil.IsZero((c3 - Vector3.zero).sqrMagnitude))
			{
				return;
			}
			
			
			Matrix4x4 M = mtx * curMtx;
			CameraObject.SetMatrix(M);
			Grid.SetMatrix(M);
		}
		
		

		void Reset()
		{
			T = TH3DMath.GetTranslation4x4(3, 7, 12);
			R = TH3DMath.GetEulerRotation4x4(30, 120, 0);
			Matrix4x4 M = T * R;
			
			CameraObject.SetMatrix(M);
			Grid.SetMatrix(M);
		}



		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec321 pr_instance = new Sec321(1);

		public static Sec321 PRGSI()
		{
			return pr_instance;
		}


		private Sec321(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				CameraObject = CreateObject_Obj3D("CamAx");
				Grid = CreateObject_Obj3D("Quater_Grid");
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, CameraObject.gameObject);
				CameraObject.SetIdentity();
				BSAuxFuncs.SetVisible(1, Grid.gameObject);
				Grid.SetIdentity();
				
				CamPos = new Vector3(16.67605f, 10.01993f, 10.31064f);
				TrgPos = new Vector3(4.212205f, 1.841582f, 8.757264f);
				uJ = new Vector3(-0.5289754f, 0.8349715f, -0.1516828f);
				
				Reset();
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -80, 0);
				}
			}
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}

	}

}
