//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec322
{
	public class Sec322 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Ball;
		THObject3D Path;
		
		THObject3D Sp1;
		THObject3D Sp2;
		THObject3D Pa1;
		THObject3D Pa2;
		
		GameObject MainCamera;
		
		THObject3D BackPlane;	
		GameObject SubCamera;
		
		GameObject DLight;


		// Instance Variables
		
		Vector3 P0;    

		readonly float c_Circumference = 2.0f * Mathf.PI;
		
		
		Vector3 pr_difVtr;	
		Vector3 pr_difVtr_Sub;	
		
		Camera pr_COMP_SubCamera;
		MeshRenderer pr_MRE_BackPlane;





		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Matrix4x4 mtx = TH3DMath.GetTranslation4x4(P0 + Vector3.up);
				Ball.SetMatrix(mtx);	
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S))
			{
			}
			
		}




		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Matrix4x4 mtx = TH3DMath.GetTranslation4x4(P0 + Vector3.up) *
				TH3DMath.GetRotation4x4(-15.0f, Vector3.up);
				Ball.SetMatrix(mtx);	
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S))
			{
			}
			
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				MainCamera.transform.position = Ball.GetWorldPosition() + pr_difVtr;
				
				SubCamera.transform.position = Ball.GetWorldPosition() + 8.0f * Vector3.up;
				BackPlane.SetWorldPosition(MainCamera.transform.position + pr_difVtr_Sub);
				
				BSData.CameraController3D.UpdateCameraData();
			}
		}


		static Sec322 pr_instance = new Sec322(2);

		public static Sec322 PRGSI()
		{
			return pr_instance;
		}


		private Sec322(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sp1 = CreateObject_Obj3D("RBSphere_1" , true);
				Sp2 = CreateObject_Obj3D("RBSphere_2" , true);
				Pa1 = CreateObject_Obj3D("ShortPath_1", true);
				Pa2 = CreateObject_Obj3D("ShortPath_2", true);
				
				
				// Main Camera	
				MainCamera = BSAuxFuncs.GetMainCamera();
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();
				DLight.GetComponent<Light>().shadows = LightShadows.Soft;
				
				// Sub Camera
				GameObject go = new GameObject();
				go.name = "Camera_2";
				pr_COMP_SubCamera = go.AddComponent<Camera>();
				pr_COMP_SubCamera.nearClipPlane = 0.01f;
				pr_COMP_SubCamera.farClipPlane = 50.0f;
				pr_COMP_SubCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_COMP_SubCamera.backgroundColor = BSData.BackgroundColor;
				pr_COMP_SubCamera.orthographic = true;  
				pr_COMP_SubCamera.orthographicSize = 4.0f;
				pr_COMP_SubCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_COMP_SubCamera.enabled = false;
				SubCamera = go;  
				
				// BackPlane (init XY 1x1)
				BackPlane = CreateObject_Obj3D("UnitRect");
				Material objMtrl = BSAuxFuncs.GetUnlitMaterial(0.0f, 0.447f, 0.545f);
				if(objMtrl == null) { BSAuxFuncs.GetStandardMaterial(0.0f, 0.447f, 0.545f); }
				pr_MRE_BackPlane = BackPlane.gameObject.GetComponent<MeshRenderer>();
				pr_MRE_BackPlane.material = objMtrl;

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (MatchCase(1))    
			{
				BSAuxFuncs.SetVisible(1, Sp1.gameObject);
				Sp1.SetIdentity();
				BSAuxFuncs.SetVisible(1, Pa1.gameObject);
				Pa1.SetIdentity();
				Ball = Sp1;
				Path = Pa1;
				
				Path.SetWorldPosition(6.0f, 1.0f, 21.0f);
				
				Vector3 wp = Path.GetWorldPosition();
				P0 = wp + 20.0f * Vector3.right;
				P0.y = 0.6f;
				
				CamPos = new Vector3(21.18075f, 6.882576f, 16.84037f);
				TrgPos = new Vector3(24.94294f, -0.3180068f, 21.46993f);
				uJ = new Vector3(0.1129106f, 0.6335524f, 0.7654167f);
				
				pr_difVtr = CamPos - P0;
				
				if (DLight != null)
				{
					DLight.transform.rotation = Quaternion.Euler(50, -30, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Sp2.gameObject);
				Sp2.SetIdentity();
				BSAuxFuncs.SetVisible(1, Pa2.gameObject);
				Pa2.SetIdentity();
				Ball = Sp2;
				Path = Pa2;
				
				Path.SetWorldPosition(10.0f, 1.0f, 21.0f);
				Path.SetWorldRotation(-15.0f, Vector3.up);
				
				Vector3 wp = Path.GetWorldPosition();
				P0 = wp + 20.0f * (Quaternion.AngleAxis(-15.0f, Vector3.up) * Vector3.right);
				P0.y = 0.6f;
				
				CamPos = new Vector3(24.60251f, 5.490827f, 21.57397f);
				TrgPos = new Vector3(26.29931f, 3.850806f, 23.17945f);
				uJ = new Vector3(0.3932086f, 0.817936f, 0.4199616f);
				
				pr_difVtr = CamPos - P0;
				
				if (DLight != null)
				{
					DLight.transform.rotation = Quaternion.Euler(50, 60, 0);
				}
			}	
			
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
			
			// Sub Camera
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, BackPlane.gameObject);
				BackPlane.SetIdentity();
				
				pr_COMP_SubCamera.enabled = true;
				pr_COMP_SubCamera.orthographicSize = 2.2f;
				
				SubCamera.transform.position = P0 + 10.0f * Vector3.up;
				SubCamera.transform.rotation = Quaternion.AngleAxis(90, Vector3.right);
				
				Rect rectBounds = new Rect(0.66f, 0.66f, 0.32f, 0.32f);
				pr_COMP_SubCamera.depth = 2;
				pr_COMP_SubCamera.rect = rectBounds;
				
				float[] rbds2 = { rectBounds.x+rectBounds.width/2.0f,
				rectBounds.y+rectBounds.height/2.0f, rectBounds.width * 1.04f, rectBounds.height * 1.057f   };
				
				float[] iplaneData = BSAuxFuncs.CalcImagePlaneData(); 
				object[] trs = BSAuxFuncs.CalcUIRectTransformN_LowerLeft(rbds2, iplaneData);
				Vector3 posBP = (Vector3)trs[0];
				Quaternion rot = (Quaternion)trs[1];
				Vector3 scale = (Vector3)trs[2];
				BackPlane.transform.position = posBP;
				BackPlane.transform.rotation = rot;
				BackPlane.transform.localScale = scale;
				
				pr_difVtr_Sub = posBP - CamPos;
				
				int LAYER_UI = 5;
				BackPlane.gameObject.layer = LAYER_UI;          
				int layerMask = 1 << LAYER_UI;
				pr_COMP_SubCamera.cullingMask = ~layerMask;		
			}
		}

	}

}
