//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections.Generic;
using UnityEngine;

using OQJ_KeyframeData;


namespace OQJ_RGAuxFuncs04
{
	public static class RGAuxFuncs04
	{
		static THObject3D Airplane;

		static float pr_degAirplane;

		static float pr_vy = 0.0f;
		static float pr_fctrMoveY = 0.16f;
		static Vector3 pr_saveVtr;

		static Vector3 pr_moveVtr = new Vector3(-0.05f, 0.0f, 0.0f);

		static Matrix4x4 pr_tmpSaveMtx;


		readonly static Vector3 pr_acclVtr = new Vector3(-0.0015f, 0.0f, 0.0f);

		readonly static float pr_ay = 0.0003f;

		readonly static float pr_rgLandY = 0.3579359f;


		readonly static float pr_turningSpeed = (Mathf.PI * 416.8141f) / 1200.0f;

		readonly static float pr_lastFlyingVel = 1.091217f;

		readonly static Vector4 pr_WheelLandingPos = new Vector4(0.24039f, 0.0f, 1.0f, 1.0f);

		readonly static Matrix4x4 pr_tmpM = TH3DMath.GetTranslation4x4(0.328351f, 0.150820f, -0.118694f) *
						TH3DMath.GetRotation4x4(90, Vector3.forward) *
						TH3DMath.GetTranslation4x4(-0.155461f, 0.0f, 0.0f) *
						TH3DMath.GetRotation4x4(-180, Vector3.forward);


		readonly static Vector3[] c_attachPos_UpperLeg =
		{
			new Vector3(-1.036088f, 0.152180f, 0.0f),
			new Vector3(0.328351f,  0.150820f,  0.118694f),
			new Vector3(0.328351f,  0.150820f, -0.118694f)
		};

		readonly static Vector3 c_attachPos_LowerLeg = new Vector3(-0.155461f, 0.0f, 0.0f);

		readonly static Vector3[] c_attachPos_DoorA =
		{
			new Vector3(-1.2530000f, 0.0018442f, -0.0761302f),
			new Vector3(0.1114389f, 0.0004838f,  0.0425581f),
			new Vector3(0.1114389f, 0.0004838f, -0.1948225f)
		};

		readonly static Vector3[] c_attachPos_DoorB =
		{
			new Vector3(-0.9329223f, 0.0018442f, -0.0761302f),
			new Vector3(0.4315166f, 0.0004838f, 0.0425596f),
			new Vector3(0.4315166f, 0.0004838f, -0.1948291f)
		};




		public static void SetAirplane(THObject3D obj)
		{
			Airplane = obj;
		}



		public static void Standby(THObject3D[] UpperLeg, THObject3D[] LowerLeg,
					THObject3D[] DoorA, THObject3D[] DoorB, Vector3 start_pos, int nr = 0)
		{
			BSAuxFuncs.SetVisible(1, Airplane.gameObject);

			pr_degAirplane = 0.0f;
			pr_vy = 0.0f;
			pr_fctrMoveY = 0.16f;

			pr_saveVtr = new Vector3();
			pr_moveVtr = new Vector3(-0.05f, 0.0f, 0.0f);

			Matrix4x4 localAirplane = Matrix4x4.identity;
			float degA = -180.0f;
			float degB =  180.0f;
			float degU =  90.0f;
			float degL = -180.0f;
			if (nr == 0)
			{
				localAirplane = TH3DMath.GetTranslation4x4(start_pos);
				pr_tmpSaveMtx = localAirplane;
			}
			else if (nr == 1)
			{
				Vector4 wp = new Vector4(665.3359f, 5.042597f, -480.0f, 1.0f);
				Quaternion wq = new Quaternion(0.000000f, 0.000000f, -0.092414f, 0.995721f);
				localAirplane = Matrix4x4.Rotate(wq);
				localAirplane.SetColumn(3, wp);

				wp = new Vector4(664.257700f, 5.074990f, -480.0f, 1.0f);
				wq = new Quaternion(0.0f, 0.0f, -0.092678f, 0.995696f);
				pr_tmpSaveMtx = Matrix4x4.Rotate(wq);
				pr_tmpSaveMtx.SetColumn(3, wp);

				pr_degAirplane = -10.63532f;
				pr_vy = 0.03029999f;
				pr_fctrMoveY = 0.16f;
				pr_saveVtr = new Vector3(-1.097006f, 0.0f, 0.0f);
				pr_moveVtr = new Vector3(-1.078161f, 0.032394f, 0.0f);
			}
			else if (nr == 2)
			{
				pr_saveVtr = new Vector3(-1.097006f, 0.0f, 0.0f);
				pr_moveVtr = new Vector3(-1.091217f, -0.2f, 0.0f);

				Vector4 wp = new Vector4(145.650400f, 171.214600f, -480.0f, 1.0f);
				Quaternion wq = Quaternion.identity;
				localAirplane = Matrix4x4.Rotate(wq);
				pr_tmpSaveMtx = localAirplane;

				pr_tmpSaveMtx.SetColumn(3, wp + TH3DMath.ToVector4(pr_moveVtr));
				localAirplane.SetColumn(3, wp);

				degA = 0.0f;
				degB = 0.0f;
				degU = 0.0f;
				degL = 0.0f;
			}

			Matrix4x4 rotDoorA = TH3DMath.GetRotation4x4(degA, Vector3.forward);
			Matrix4x4 rotDoorB = TH3DMath.GetRotation4x4(degB, Vector3.forward);
			Matrix4x4 rotUpperLeg = TH3DMath.GetRotation4x4(degU, Vector3.forward);
			Matrix4x4 rotLowerLeg = TH3DMath.GetRotation4x4(degL, Vector3.forward);

			Matrix4x4[] localUpperLeg = new Matrix4x4[3];
			Matrix4x4[] localLowerLeg = new Matrix4x4[3];
			Matrix4x4[] localDoorA = new Matrix4x4[3];
			Matrix4x4[] localDoorB = new Matrix4x4[3];
			for (int i = 0; i < 3; i++)
			{
				Matrix4x4 traUpperLeg = TH3DMath.GetTranslation4x4(c_attachPos_UpperLeg[i]);
				localUpperLeg[i] = traUpperLeg * rotUpperLeg;

				Matrix4x4 traLowerLeg = TH3DMath.GetTranslation4x4(c_attachPos_LowerLeg);
				localLowerLeg[i] = traLowerLeg * rotLowerLeg;

				Matrix4x4 traDoorA = TH3DMath.GetTranslation4x4(c_attachPos_DoorA[i]);
				localDoorA[i] = traDoorA * rotDoorA;

				Matrix4x4 traDoorB = TH3DMath.GetTranslation4x4(c_attachPos_DoorB[i]);
				localDoorB[i] = traDoorB * rotDoorB;
			}


			Matrix4x4[] worldUpperLeg = new Matrix4x4[3];
			Matrix4x4[] worldLowerLeg = new Matrix4x4[3];
			Matrix4x4[] worldDoorA = new Matrix4x4[3];
			Matrix4x4[] worldDoorB = new Matrix4x4[3];
			for (int i = 0; i < 3; i++)
			{
				worldDoorA[i] = localAirplane * localDoorA[i];
				worldDoorB[i] = localAirplane * localDoorB[i];
				worldUpperLeg[i] = localAirplane * localUpperLeg[i];
				worldLowerLeg[i] = localAirplane * localUpperLeg[i] * localLowerLeg[i];

				DoorA[i].SetMatrix(worldDoorA[i]);
				DoorB[i].SetMatrix(worldDoorB[i]);
				UpperLeg[i].SetMatrix(worldUpperLeg[i]);
				LowerLeg[i].SetMatrix(worldLowerLeg[i]);
			}

			Airplane.SetMatrix(localAirplane);
		}





		public static void AirplaneMotion(int frame)
		{
			Vector3 wp = Airplane.GetLocalPosition(); 
			Matrix4x4 rotAirplane = Matrix4x4.identity;

			if (frame <= 698)       
			{
				pr_moveVtr += pr_acclVtr;
				pr_saveVtr = pr_moveVtr;
			}
			else if (frame <= 1560)
			{
				if (frame <= 1343)  
				{
					if (frame <= 799)   
					{
						pr_vy += pr_ay;
					}

					pr_degAirplane -= pr_vy;
				}


				rotAirplane = TH3DMath.GetRotation4x4(pr_degAirplane, Vector3.forward);


				if (frame <= 913)   
				{
					pr_moveVtr = pr_saveVtr;

					Matrix4x4 lcA = TH3DMath.GetTranslation4x4(wp) * rotAirplane;
					Matrix4x4 wldLL = lcA * pr_tmpM;
					Vector3 tp = wldLL * pr_WheelLandingPos;
					pr_moveVtr.y = 0.355f - tp.y;
				}
				else                        
				{
					pr_moveVtr = rotAirplane * pr_saveVtr;

					if (frame <= 1099)
					{
						pr_moveVtr.y *= 0.16f;
					}
					else
					{   
						pr_fctrMoveY += 0.0025f;
						if (pr_fctrMoveY > 1.0f)
						{
							pr_fctrMoveY = 1.0f;
						}

						pr_moveVtr.y *= pr_fctrMoveY;
					}
				}
			}
			else if (frame <= 2160)     
			{
				Quaternion curRot = Quaternion.AngleAxis(pr_degAirplane, Vector3.forward);
				Quaternion rot90 = Quaternion.Euler(40.0f, 90.0f, 0.0f);

				float t = (frame - 1560) / 600.0f;
				Quaternion rot = Quaternion.Slerp(curRot, rot90, t);
				rotAirplane = Matrix4x4.Rotate(rot);

				pr_moveVtr = rotAirplane * pr_saveVtr;
			}
			else if (frame <= 3000)     
			{
				Quaternion rot90 = Quaternion.Euler(40.0f, 90.0f, 0.0f);
				Quaternion rot180 = Quaternion.Euler(0.0f, 180.0f, 0.0f);

				float t = (frame - 2160) / 600.0f;
				Quaternion rot = Quaternion.Slerp(rot90, rot180, t);
				rotAirplane = Matrix4x4.Rotate(rot);

				pr_moveVtr = rotAirplane * pr_saveVtr;

				if (2781 <= frame && frame <= 2880) 
				{									
					t = (frame - 2780) / 100.0f;
					pr_moveVtr.y = -(wp.y - 206.8141f) * t;
				}
			}
			else if (frame <= 4200)     
			{
				float deg = (frame - 3000) / 1200.0f * 180.0f;  
				Quaternion q = Quaternion.AngleAxis(deg + 180.0f, Vector3.up);
				pr_moveVtr = q * new Vector3(-pr_turningSpeed, 0.0f, 0.0f);

				if (frame <= 3100)  
				{
					float t = (frame - 3000) / 100.0f;
					q = Quaternion.Slerp(Quaternion.Euler(0, 180, 0), Quaternion.Euler(32, 195, 0), t);
				}
				else if (frame <= 3600)
				{   
					float t = (frame - 3100) / 500.0f;
					q = Quaternion.Slerp(Quaternion.Euler(32, 195, 0), Quaternion.Euler(32, 270, 0), t);
				}
				else
				{   
					float t = (frame - 3600) / 600.0f;
					q = Quaternion.Slerp(Quaternion.Euler(32, 270, 0), Quaternion.Euler(0, 0, 0), t);

					if (frame == 4200)
					{
						pr_moveVtr.z = -480.0f - wp.z;
					}
				}

				rotAirplane = Matrix4x4.Rotate(q);
			}
			else if (frame <= 4320)     
			{
				pr_moveVtr = new Vector3(-pr_lastFlyingVel, 0.0f, 0.0f);
				rotAirplane = Matrix4x4.identity;
			}
			else if (frame <= 4980)     
			{
				pr_moveVtr = new Vector3(-pr_lastFlyingVel, -0.2f, 0.0f);
				rotAirplane = Matrix4x4.identity;
			}
			else if (frame <= 6200)     
			{
				float t = (frame - 4980) / 600.0f;
				float dy = -0.2f + 0.16f * t;
				if (dy > -0.04f) dy = -0.04f;
				pr_moveVtr = new Vector3(-pr_lastFlyingVel, dy, 0.0f);

				if (frame <= 5637)
				{
					float t2 = (frame - 4980) / 658.0f;
					var q = Quaternion.Slerp(Quaternion.identity, Quaternion.AngleAxis(-10.0f, Vector3.forward), t2);
					rotAirplane = Matrix4x4.Rotate(q);

					if (frame == 5637)  
					{
						pr_moveVtr.y = 0.355f - pr_rgLandY;
					}
				}
				else 
				{
					float t3 = (frame - 5637) / 120.0f;
					var q = Quaternion.Slerp(Quaternion.AngleAxis(-10.0f, Vector3.forward), Quaternion.identity, t3);
					rotAirplane = Matrix4x4.Rotate(q);

					pr_moveVtr.x = -0.7f;

					Matrix4x4 lcA = TH3DMath.GetTranslation4x4(wp) * rotAirplane;
					Matrix4x4 wldLL = lcA * pr_tmpM;
					Vector3 tp = wldLL * pr_WheelLandingPos; 
					pr_moveVtr.y = 0.355f - tp.y;
				}
			}
			else
			{   
				float t = (frame - 6200) / 200.0f;
				pr_moveVtr = new Vector3(-0.7f * (1.0f - t), 0.0f, 0.0f);
				if (pr_moveVtr.x > 0.0f) pr_moveVtr.x = 0.0f;
				rotAirplane = Matrix4x4.identity;
			}

			Vector3 np = wp + pr_moveVtr;
			Matrix4x4 traAirplane = TH3DMath.GetTranslation4x4(np);
			pr_tmpSaveMtx = traAirplane * rotAirplane;

		}



		public static Matrix4x4 GetAirplaneMatrix()
		{
			Airplane.SetMatrix(pr_tmpSaveMtx);
			return Airplane.GetMatrix();
		}



		public static Vector3 GetAirplanePosition()
		{
			return Airplane.GetLocalPosition();
		}



		public static Quaternion GetAirplaneRotation()
		{
			return Airplane.transform.localRotation;
		}



		public static Vector3 GetAirplaneMotionVector()
		{
			return pr_moveVtr;
		}




		public static void CreateLargeCoordinateSystem()
		{
			GameObject goCS = new GameObject("CSvL");
			goCS.AddComponent<MeshFilter>();
			goCS.AddComponent<MeshRenderer>();

			List<Vector3> verts = new List<Vector3>();
			List<Color> colors = new List<Color>();

			float gridGray = 0.0f;
			float bkgrndGray = 0.0f;
			Color clrNegX = new Color();
			Color clrNegY = new Color();
			Color clrNegZ = new Color();
			Color clrPlsX = new Color();
			Color clrPlsY = new Color();
			Color clrPlsZ = new Color();

			gridGray = 0.56f; 
			bkgrndGray = BSData.BackgroundColor.r;

			float alpha = 0.25f;
			float semiVal = alpha * 0.73f + (1.0f - alpha) * bkgrndGray;
			float semiBgr = alpha * 0.0f + (1.0f - alpha) * bkgrndGray;
			clrNegX = new Color(semiVal, semiBgr, semiBgr);
			clrNegY = new Color(semiBgr, semiVal, semiBgr);
			clrNegZ = new Color(semiBgr, semiBgr, semiVal);
			clrPlsX = new Color(0.73f, 0.0f, 0.0f);
			clrPlsY = new Color(0.0f, 0.73f, 0.0f);
			clrPlsZ = new Color(0.0f, 0.0f, 0.73f);


			Mesh wMesh = new Mesh();
			Color gridColor = new Color(gridGray, gridGray, gridGray);
			const int gridMax = 200;
			int[] idcs = new int[gridMax * 2 * 4 + 12];  
			int vi = 0;
			for (int i = -gridMax; i <= gridMax; i++)
			{
				if (i == 0) { continue; }

				float iL = i * 20.0f;

				float cx = gridMax * 20;
				verts.Add(new Vector3((float)-cx, -0.001f, (float)iL));
				colors.Add(gridColor);
				verts.Add(new Vector3((float)cx, -0.001f, (float)iL));
				colors.Add(gridColor);

				float cz = gridMax * 20;
				verts.Add(new Vector3((float)iL, -0.001f, (float)-cz));
				colors.Add(gridColor);
				verts.Add(new Vector3((float)iL, -0.001f, (float)cz));
				colors.Add(gridColor);

				idcs[vi] = vi;
				idcs[vi + 1] = vi + 1;
				idcs[vi + 2] = vi + 2;
				idcs[vi + 3] = vi + 3;
				vi += 4;
			}

			verts.Add(new Vector3((float)-gridMax * 20, 0.0f, 0.0f));
			colors.Add(clrNegX);
			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrNegX);

			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsX);
			verts.Add(new Vector3((float)gridMax * 20, 0.0f, 0.0f));
			colors.Add(clrPlsX);

			verts.Add(new Vector3(0.0f, (float)-gridMax * 20, 0.0f));
			colors.Add(clrNegY);
			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrNegY);

			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsY);
			verts.Add(new Vector3(0.0f, (float)gridMax * 20, 0.0f));
			colors.Add(clrPlsY);

			verts.Add(new Vector3(0.0f, 0.0f, (float)-gridMax * 20));
			colors.Add(clrNegZ);
			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrNegZ);

			verts.Add(new Vector3((float)0.0f, 0.0f, 0.0f));
			colors.Add(clrPlsZ);
			verts.Add(new Vector3(0.0f, 0.0f, (float)gridMax * 20));
			colors.Add(clrPlsZ);

			idcs[vi] = vi;
			idcs[vi + 1] = vi + 1;
			idcs[vi + 2] = vi + 2;
			idcs[vi + 3] = vi + 3;
			idcs[vi + 4] = vi + 4;
			idcs[vi + 5] = vi + 5;
			idcs[vi + 6] = vi + 6;
			idcs[vi + 7] = vi + 7;
			idcs[vi + 8] = vi + 8;
			idcs[vi + 9] = vi + 9;
			idcs[vi + 10] = vi + 10;
			idcs[vi + 11] = vi + 11;

			wMesh.SetVertices(verts);
			wMesh.SetColors(colors);

			wMesh.SetIndices(idcs, MeshTopology.Lines, 0);

			MeshFilter mshFltr = goCS.GetComponent<MeshFilter>();
			mshFltr.mesh = wMesh;

			goCS.transform.localPosition = Vector3.zero;
			goCS.transform.localRotation = Quaternion.identity;
			goCS.transform.localScale = Vector3.one;

			MeshRenderer mshRndr = goCS.GetComponent<MeshRenderer>();
			mshRndr.material = BSData.LineMaterial;
			BSAuxFuncs.MeshRendererShadowOff(mshRndr);

			BSData.RunObjectDict.Add(goCS.name, goCS);
		}



		public static Matrix4x4 CalcTransform(float cur_frame, KeyframeData[] key_frames)
		{
			Vector3 pos = Vector3.zero;
			Quaternion rot = Quaternion.identity;

			for (int i = 0; i < key_frames.Length - 1; i++)
			{
				KeyframeData kfA = key_frames[i];
				KeyframeData kfB = key_frames[i + 1];

				if (cur_frame <= kfB.frameNumber)
				{
					float interval = kfB.frameNumber - kfA.frameNumber;
					float t = (cur_frame - kfA.frameNumber) / interval;

					if (kfA.type == KeyframeData.LINE)
					{
						pos = (1 - t) * kfA.position + t * kfB.position;
					}
					else if (kfA.type == KeyframeData.CIRCLE)
					{
						float degB = kfB.anotherCircle ? kfB.deg2 : kfB.degCircle;

						float deg = t * (degB - kfA.degCircle) + kfA.degCircle;
						pos = Quaternion.AngleAxis(deg, kfA.axis) * (kfA.startPosCircle - kfA.center) + kfA.center;
					}
					else if (kfA.type == KeyframeData.SPIRAL)
					{
						float deg = t * (kfB.degHelix - kfA.degHelix) + kfA.degHelix;
						pos = Quaternion.AngleAxis(deg, kfA.centerAxis) * (kfA.startPosHelix - kfA.axisStart) +
										kfA.axisStart + (-deg / 360.0f * kfA.primaryLength) * kfA.centerAxis;
					}

					rot = Quaternion.Slerp(kfA.rotation, kfB.rotation, t);

					break;
				}
			}

			Matrix4x4 M = Matrix4x4.Rotate(rot);
			M.SetColumn(3, TH3DMath.ToVector4(pos));

			return M;
		}



		public static KeyframeData[] InitKeyframes_19()
		{
			const float c_BitW = 0.234108f;
			const float c_TrackRad = 0.588471f;		

			float hcF = (c_TrackRad * Mathf.PI) * 50.0f; // ((c_TrackRad * 2.0f * Mathf.PI * 0.5f) / 0.01f) * 0.5f;
			float F1 = c_BitW * 21 * 100.0f;
			float F2 = F1 + hcF;
			float F3 = F2 + hcF;
			float F4 = F3 + F1;
			float F5 = F4 + hcF;
			float F6 = F5 + hcF;
			////i_totalFrames = F6;

			float mvX = c_BitW * 10.5f;
			Vector3 P0 = new Vector3(-mvX,  c_TrackRad, 0.0f);	
			Vector3 P1 = new Vector3( mvX,  c_TrackRad, 0.0f);	
			Vector3 P2 = new Vector3( mvX, -c_TrackRad, 0.0f);	
			Vector3 P3 = new Vector3(-mvX, -c_TrackRad, 0.0f);
			Vector3 C1 = new Vector3(mvX, 0.0f, 0.0f);	
			Vector3 C2 = new Vector3(-mvX, 0.0f, 0.0f);	
			Vector3 mid1 = new Vector3(mvX + c_TrackRad, 0.0f, 0.0f);	
			Vector3 mid2 = new Vector3(-mvX - c_TrackRad, 0.0f, 0.0f);	


			int LINE = KeyframeData.LINE;
			int CIRCLE = KeyframeData.CIRCLE;
			int END = 0;

			KeyframeData[] kfs = new KeyframeData[]
			{
				new KeyframeData(LINE, P0, Quaternion.identity),
				new KeyframeData(F1, CIRCLE, P1, Quaternion.identity, C1, -Vector3.forward),
				new KeyframeData(F2, CIRCLE, mid1, Quaternion.AngleAxis(90.0f, -Vector3.forward)),
				new KeyframeData(F3, LINE,   P2, Quaternion.AngleAxis(180.0f, -Vector3.forward)),
				new KeyframeData(F4, CIRCLE, P3, Quaternion.AngleAxis(180.0f, -Vector3.forward), C2, -Vector3.forward),
				new KeyframeData(F5, CIRCLE, mid2, Quaternion.AngleAxis(270.0f, -Vector3.forward)),
				new KeyframeData(F6, END, P0, Quaternion.identity)
			};

			return kfs;
		}



		public static KeyframeData[] InitKeyframes_24(int dn, bool B=false)
		{
			float cy = 1.5f;

			Vector3 P0 = new Vector3(0, cy, 0); 
			Vector3 P1 = new Vector3(0, cy, 6); 
			Vector3 P2 = new Vector3(0, cy, 12);

			Quaternion r45 = Quaternion.AngleAxis(45, Vector3.up);
			Vector3 P3 = r45 * P2;
			Vector3 P4 = r45 * P1;

			Vector3 C0 = new Vector3(-3, cy, 3);
			Vector3 C1 = new Vector3( 3, cy, 9);
			Vector3 C2 = new Vector3( 3.514718f, cy, 8.485281f);		// rad 4.970563
			Vector3 C3 = r45 * new Vector3(-3, cy, 9);
			Vector3 C4 = r45 * new Vector3( 3, cy, 3);

			Quaternion q0 = r45;
			Quaternion q1 = Quaternion.AngleAxis(-45, Vector3.up);
			Quaternion q2 = q0;
			Quaternion q3 = Quaternion.AngleAxis(180, Vector3.up);
			Quaternion q4 = Quaternion.AngleAxis(-90, Vector3.up);
			Quaternion q5 = q3;

			float tmpScl = 1.5f;
			P0 = new Vector3(P0.x * tmpScl, cy, P0.z * tmpScl);
			P1 = new Vector3(P1.x * tmpScl, cy, P1.z * tmpScl);
			P2 = new Vector3(P2.x * tmpScl, cy, P2.z * tmpScl);
			P3 = new Vector3(P3.x * tmpScl, cy, P3.z * tmpScl);
			P4 = new Vector3(P4.x * tmpScl, cy, P4.z * tmpScl);
			C0 = new Vector3(C0.x * tmpScl, cy, C0.z * tmpScl);
			C1 = new Vector3(C1.x * tmpScl, cy, C1.z * tmpScl);
			C2 = new Vector3(C2.x * tmpScl, cy, C2.z * tmpScl);
			C3 = new Vector3(C3.x * tmpScl, cy, C3.z * tmpScl);
			C4 = new Vector3(C4.x * tmpScl, cy, C4.z * tmpScl);

			float jdg = 0; 
			if (B)
			{
				jdg = 90;
				P3.x = -P3.x;
				P4.x = -P4.x;

				C0.x = -C0.x;
				C1.x = -C1.x;
				C2.x = -C2.x;
				C3.x = -C3.x;
				C4.x = -C4.x;

				q0 = Quaternion.AngleAxis(-45, Vector3.up);
				q1 = r45;	
				q2 = q0;	
				//q3 = ..;	
				q4 = Quaternion.AngleAxis(90, Vector3.up);	
				//q5 = ..;	
			}

			Quaternion rr = Quaternion.AngleAxis(jdg + 45 * (dn + 1), Vector3.up);
			Vector3[] Prr = { P0, rr * P1, rr * P2, rr * P3, rr * P4, P0 };
			Vector3[] Crr = { rr * C0, rr * C1, rr * C2, rr * C3, rr * C4 };
			Quaternion[] qrr = { rr * q0, rr * q1, rr * q2, rr * q3, rr * q4, rr * q5 };

			int LINE = KeyframeData.LINE;
			int CIRCLE = KeyframeData.CIRCLE;
			int END = 0;

			float r1 = 4.242640f * tmpScl;
			float r2 = 4.970563f * tmpScl;
			float baseArc = 2.0f * Mathf.PI * r1 * 0.25f;

			float bsf = baseArc *10 ; 
			float F1 = bsf;
			float F2 = F1 + bsf;
			float jj = r2 / r1;			
			float F3 = F2 + bsf * jj * 1.5f;  
			float F4 = F3 + bsf;
			float F5 = F4 + bsf;

			KeyframeData[] arr = new KeyframeData[]
			{
				new KeyframeData(    CIRCLE, Prr[0], qrr[0], Crr[0], Vector3.up),
				new KeyframeData(F1, CIRCLE, Prr[1], qrr[1], Crr[1], Vector3.up),
				new KeyframeData(F2, CIRCLE, Prr[2], qrr[2], Crr[2], Vector3.up),
				new KeyframeData(F3, CIRCLE, Prr[3], qrr[3], Crr[3], Vector3.up),
				new KeyframeData(F4, CIRCLE, Prr[4], qrr[4], Crr[4], Vector3.up),
				new KeyframeData(F5, END,    Prr[5], qrr[5])
			};

			return arr;
		}
	}

}

