//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec401
{
	public class Sec401 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Sphere;
		THObject3D PointB;
		
		THObject3D AuxCircle;
		THObject3D AuxLine;


		// Instance Variables
		float i_degRev = 0.0f;
		float i_degRot = 0.0f;
		
		Vector3 c_centerPos = new Vector3(5, 3, -5);




		void Code1()
		{
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(np);
			
			Sphere.SetMatrix(T);
		}


		void Code2()
		{
			i_degRot += 4.0f;
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(np);
			
			Matrix4x4 M = T * R;
			Sphere.SetMatrix(M);
		}


		void Code3()
		{
			i_degRot += 4.0f;
			Matrix4x4 rotY = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			Matrix4x4 rotZ = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
			Matrix4x4 R = rotZ * rotY;
			
			Sphere.SetMatrix(R);
		}


		void Code4()
		{
			i_degRot += 4.0f;
			Matrix4x4 rotY = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			Matrix4x4 rotZ = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
			Matrix4x4 R = rotZ * rotY;
			
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Matrix4x4 T = TH3DMath.GetTranslation4x4(np);
			
			Matrix4x4 M = T * R;
			Sphere.SetMatrix(M);
		}


		void Code5()
		{
			i_degRot += 4.0f;
			Matrix4x4 rotY = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			Matrix4x4 rotZ = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
			Matrix4x4 R = rotZ * rotY;
			
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			np += c_centerPos;
			Matrix4x4 T = TH3DMath.GetTranslation4x4(np);
			
			Matrix4x4 M = T * R;
			Sphere.SetMatrix(M);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(2))
			{
				var scale = Matrix4x4.Scale(new Vector3(1.0f, 2.0f, 1.0f));
				Matrix4x4 tra = TH3DMath.GetTranslation4x4(Sphere.GetLocalPosition());
				AuxLine.SetMatrix(tra * scale);
			}
			else if (MatchCase(4, 5))
			{
				Matrix4x4 S = Matrix4x4.Scale(new Vector3(1.0f, 2.0f, 1.0f));
				Matrix4x4 R = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
				Matrix4x4 T = TH3DMath.GetTranslation4x4(Sphere.GetLocalPosition());
				AuxLine.SetMatrix(T * R * S);
			}
		}


		static Sec401 pr_instance = new Sec401(5);

		public static Sec401 PRGSI()
		{
			return pr_instance;
		}


		private Sec401(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere = CreateObject_Obj3D("Sphere_half");
				PointB = CreateObject_Obj3D("PointB");
				
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				AuxLine = CreateObject_Obj3D("AuxLine");
				
				
				Mesh mesh = AuxLine.gameObject.GetComponent<MeshFilter>().mesh;
				List<Vector3> verts = new List<Vector3>();
				verts.Add(new Vector3(0.0f, -0.5f, 0.0f));
				verts.Add(new Vector3(0.0f, 0.5f, 0.0f));
				mesh.SetVertices(verts);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			i_degRev = 0.0f;
			i_degRot = 0.0f;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				
				CamPos = new Vector3(0.0f, 6.0f, -8.75f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				AuxLine.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				
				CamPos = new Vector3(0.0f, 6.0f, -8.75f);
				TrgPos = Vector3.zero;
				uJ = Vector3.up;
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxLine.SetMatrix(Matrix4x4.identity);
				
				Matrix4x4 scale = Matrix4x4.Scale(new Vector3(1.0f, 2.0f, 1.0f));
				Matrix4x4 rot = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
				AuxLine.SetMatrix(rot * scale);
				
				CamPos = new Vector3(0f, 1.596265f, -6.207716f);     
				TrgPos = Vector3.zero;
				uJ = new Vector3(0f, 0.9684931f, 0.2490404f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				AuxLine.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				
				CamPos = new Vector3(0f, 4.961532f, -9.878842f);
				TrgPos = new Vector3(0f, -0.4997162f, -0.2844447f);
				uJ = new Vector3(0f, 0.8690717f, 0.4946862f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				BSAuxFuncs.SetVisible(1, PointB.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				AuxLine.SetMatrix(Matrix4x4.identity);
				PointB.SetMatrix(Matrix4x4.identity);
				
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(5, 3, -5);
				
				PointB.transform.localScale = new Vector3(0.2f, 0.2f, 0.2f);
				PointB.transform.localPosition = new Vector3(5, 3, -5);
				
				MeshRenderer mre = PointB.transform.gameObject.GetComponent<MeshRenderer>();
				Material wmtrl = mre.material;
				wmtrl.SetColor("_Color", new Color(0.6792f, 0.6665f, 0.4773f, 1.0f));
				
				
				CamPos = new Vector3(8.704226f, 6.571349f, -14.06188f);
				TrgPos = new Vector3(3.191605f, 0.5071556f, -0.02890308f);
				uJ = new Vector3(-0.1914269f, 0.9261311f, 0.3250184f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
		}

	}

}
