//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec402
{
	public class Sec402 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Body;
		THObject3D Propeller;
		
		THObject3D RTSphere;
		THObject3D AuxSphereGrid;
		THObject3D AuxCircle;


		// Instance Variables
		float i_eulerX = 0;
		float i_eulerY = 0;
		float i_eulerZ = 0;
		Vector3 i_forwardDir = new Vector3(0.0f, 0.0f, 1.0f);
		
		int i_degPropeller = 0;
		int i_degBody = 0;
		
		readonly Vector3 c_attachPos_Propeller = new Vector3(0.0f, 0.0f, 2.5f);
		readonly Vector4 c_initDir = new Vector4(0.0f, 0.0f, 1.0f, 1.0f);




		void Code1()
		{
			i_degPropeller += 15;
			Matrix4x4 rotProp = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traProp = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traProp * rotProp;
			
			Matrix4x4 localBody = Matrix4x4.identity;
			
			Matrix4x4 worldPropeller = localBody * localPropeller;
			Matrix4x4 worldBody = localBody;
			
			Propeller.SetMatrix(worldPropeller);
			Body.SetMatrix(worldBody);
		}


		void Code2()
		{
			i_degPropeller += 15;
			Matrix4x4 rotProp = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traProp = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traProp * rotProp;
			
			i_degBody -= 2;
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 np = rotBody * new Vector4(8.0f, 10.0f, 0.0f, 1.0f);
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(np);
			Matrix4x4 localBody = traBody * rotBody;
			
			Matrix4x4 worldPropeller = localBody * localPropeller;
			Matrix4x4 worldBody = localBody;
			
			Propeller.SetMatrix(worldPropeller);
			Body.SetMatrix(worldBody);
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.J))
			{
				i_eulerX += 1;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_eulerX -= 1;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_eulerY -= 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_eulerY += 1;
			}
			
			Matrix4x4 traProp = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traProp;
			
			Matrix4x4 rotBody = TH3DMath.GetEulerRotation4x4(i_eulerX, i_eulerY, 0.0f);
			Matrix4x4 localBody = rotBody;
			i_forwardDir = rotBody * c_initDir;
			
			Matrix4x4 worldPropeller = localBody * localPropeller;
			Matrix4x4 worldBody = localBody;
			
			Propeller.SetMatrix(worldPropeller);
			Body.SetMatrix(worldBody);
		}


		void Code4()
		{
			if (Input.GetKey(KeyCode.J))
			{
				i_eulerX += 1.0f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_eulerX -= 1.0f;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_eulerY -= 2.0f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_eulerY += 2.0f;
			}
			
			if (Input.GetKeyDown(KeyCode.S))  // 移動/停止
			{
				i_MOVE = !i_MOVE;
			}
			
			// localPropeller
			i_degPropeller += 15;
			Matrix4x4 rotProp = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traProp = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traProp * rotProp;
			
			// localBody
			Matrix4x4 rotBody = TH3DMath.GetEulerRotation4x4(i_eulerX, i_eulerY, 0.0f);
			i_forwardDir = rotBody * c_initDir;
			Vector3 curPos = Body.GetWorldPosition();
			Vector3 newPos = (i_MOVE) ? curPos + 0.1f * i_forwardDir : curPos;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(newPos);
			Matrix4x4 localBody = traBody * rotBody;
			
			// world matrix	
			Matrix4x4 worldPropeller = localBody * localPropeller;
			Matrix4x4 worldBody = localBody;
			
			Propeller.SetMatrix(worldPropeller);
			Body.SetMatrix(worldBody);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec402 pr_instance = new Sec402(4);

		public static Sec402 PRGSI()
		{
			return pr_instance;
		}


		private Sec402(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Body = CreateObject_Obj3D("PlaneBody");
				Propeller = CreateObject_Obj3D("Propeller");
				RTSphere = CreateObject_Obj3D("RTSphere");
				AuxSphereGrid = CreateObject_Obj3D("AuxSphereGrid");
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				
				MeshRenderer mre = RTSphere.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.0f, 0.8f, 0.25f);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_eulerX = 0.0f;
			i_eulerY = 0.0f;
			i_degPropeller = 0;
			i_degBody = 0;
			i_MOVE = false;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Propeller.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(4.587923f, 4.987331f, 9.367083f);          
				TrgPos = new Vector3(0.06526981f, -0.2891484f, 0.1332592f);
				uJ = new Vector3(-0.216473f, 0.8895519f, -0.4022895f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Propeller.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localScale = new Vector3(8, 8, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(0, 10, 0);
				
				CamPos = new Vector3(12.62591f, 25.24535f, -19.98914f);
				TrgPos = new Vector3(0.6450775f, 7.977334f, -0.9672379f);
				uJ = new Vector3(-0.3318465f, 0.7930158f, 0.5108854f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Propeller.SetMatrix(Matrix4x4.identity);
				RTSphere.SetMatrix(Matrix4x4.identity);
				AuxSphereGrid.SetMatrix(Matrix4x4.identity);
				
				RTSphere.transform.localScale = new Vector3(3.2f, 3.2f, 3.2f); 
				AuxSphereGrid.transform.localScale = new Vector3(3.202f, 3.202f, 3.202f); 
				
				CamPos = new Vector3(7.229473f, 6.283071f, 6.375564f);          
				TrgPos = new Vector3(-0.05147453f, 0.09195375f, 0.07354215f);
				uJ = new Vector3(-0.414159f, 0.841123f, -0.3478282f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Propeller.SetMatrix(Matrix4x4.identity);
				
				Body.SetWorldPosition(0.0f, 15.0f, -12.0f);
				
				CamPos = new Vector3(0f, 27.312f, -28.73146f);
				TrgPos = new Vector3(0f, 5.079151f, 0.06111623f);
				uJ = new Vector3(0f, 0.7914972f, 0.6111727f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
		}

	}

}
