//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec403
{
	public class Sec403 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Body;
		THObject3D WingL;
		THObject3D WingR;
		
		THObject3D RTSphere;
		THObject3D AuxSphereGrid;


		// Instance Variables
		float i_eulerX = 0;
		float i_eulerY = 0;
		float i_eulerZ = 0;
		Vector3 i_forwardDir = new Vector3(0.0f, 0.0f, 1.0f);
		
		int i_shm = 0;
		readonly Vector3 c_attachPos_wingL = new Vector3(-0.465f, 0.414f, 0.022f);
		readonly Vector3 c_attachPos_wingR = new Vector3(0.465f, 0.414f, 0.022f);
		readonly Vector4 c_initDir = new Vector4(0.0f, 0.0f, 1.0f, 1.0f);




		void Code1()
		{
			Matrix4x4 traWingL = TH3DMath.GetTranslation4x4(c_attachPos_wingL);
			Matrix4x4 localWingL = traWingL;
			Matrix4x4 traWingR = TH3DMath.GetTranslation4x4(c_attachPos_wingR);
			Matrix4x4 localWingR = traWingR;
			
			Matrix4x4 localBody = Matrix4x4.identity;
			
			Matrix4x4 worldWingL = localBody * localWingL;
			Matrix4x4 worldWingR = localBody * localWingR;
			Matrix4x4 worldBody = localBody;
			
			WingL.SetMatrix(worldWingL);
			WingR.SetMatrix(worldWingR);
			Body.SetMatrix(worldBody);
		}


		void Code2()
		{
			i_shm += 3;
			float degWing = 45.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			
			// localWingL
			Matrix4x4 rotWingL = TH3DMath.GetRotation4x4(-degWing, Vector3.forward);
			Matrix4x4 traWingL = TH3DMath.GetTranslation4x4(c_attachPos_wingL);
			Matrix4x4 localWingL = traWingL * rotWingL;
			
			// localWingR
			Matrix4x4 rotWingR = TH3DMath.GetRotation4x4(degWing, Vector3.forward);
			Matrix4x4 traWingR = TH3DMath.GetTranslation4x4(c_attachPos_wingR);
			Matrix4x4 localWingR = traWingR * rotWingR;
			
			// localBody
			Matrix4x4 localBody = Matrix4x4.identity;
			
			// world matrix
			Matrix4x4 worldWingL = localBody * localWingL;
			Matrix4x4 worldWingR = localBody * localWingR;
			Matrix4x4 worldBody = localBody;
			
			WingL.SetMatrix(worldWingL);
			WingR.SetMatrix(worldWingR);
			Body.SetMatrix(worldBody);
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_eulerY -= 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_eulerY += 1;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				i_eulerX += 1;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_eulerX -= 1;
			}
			
			Matrix4x4 traWingL = TH3DMath.GetTranslation4x4(c_attachPos_wingL);
			Matrix4x4 localWingL = traWingL;
			Matrix4x4 traWingR = TH3DMath.GetTranslation4x4(c_attachPos_wingR);
			Matrix4x4 localWingR = traWingR;
			
			Matrix4x4 rotBody = TH3DMath.GetEulerRotation4x4(i_eulerX, i_eulerY, 0);
			Matrix4x4 localBody = rotBody;
			i_forwardDir = rotBody * c_initDir;
			
			Matrix4x4 worldWingL = localBody * localWingL;
			Matrix4x4 worldWingR = localBody * localWingR;
			Matrix4x4 worldBody = localBody;
			
			WingL.SetMatrix(worldWingL);
			WingR.SetMatrix(worldWingR);
			Body.SetMatrix(worldBody);
		}


		void Code4()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_eulerY -= 1;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_eulerY += 1;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				i_eulerX += 0.5f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_eulerX -= 0.5f;
			}
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			i_shm += 3;
			float degWing = 45.0f * Mathf.Sin(i_shm * Mathf.Deg2Rad);
			
			// localWingL
			Matrix4x4 rotWingL = TH3DMath.GetRotation4x4(-degWing, Vector3.forward);
			Matrix4x4 traWingL = TH3DMath.GetTranslation4x4(c_attachPos_wingL);
			Matrix4x4 localWingL = traWingL * rotWingL;
			
			// localWingR
			Matrix4x4 rotWingR = TH3DMath.GetRotation4x4(degWing, Vector3.forward);
			Matrix4x4 traWingR = TH3DMath.GetTranslation4x4(c_attachPos_wingR);
			Matrix4x4 localWingR = traWingR * rotWingR;
			
			// localBody
			Matrix4x4 rotBody = TH3DMath.GetEulerRotation4x4(i_eulerX, i_eulerY, 0.0f);
			i_forwardDir = rotBody * c_initDir;
			Vector3 curPos = Body.GetWorldPosition();
			Vector3 newPos = (i_MOVE) ? curPos + 0.1f * i_forwardDir : curPos;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(newPos);
			Matrix4x4 localBody = traBody * rotBody;
			
			// world matrix
			Matrix4x4 worldWingL = localBody * localWingL;
			Matrix4x4 worldWingR = localBody * localWingR;
			Matrix4x4 worldBody = localBody;
			
			WingL.SetMatrix(worldWingL);
			WingR.SetMatrix(worldWingR);
			Body.SetMatrix(worldBody);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec403 pr_instance = new Sec403(4);

		public static Sec403 PRGSI()
		{
			return pr_instance;
		}


		private Sec403(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				RTSphere = CreateObject_Obj3D("RTSphere");
				AuxSphereGrid = CreateObject_Obj3D("AuxSphereGrid");
				Body = CreateObject_Obj3D("RBird_body");
				WingL = CreateObject_Obj3D("RBird_wingL");
				WingR = CreateObject_Obj3D("RBird_wingR");
				
				MeshRenderer mre = RTSphere.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.0f, 0.8f, 0.25f);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_shm = 0;
			i_eulerX = 0;
			i_eulerY = 0;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, WingL.gameObject);
				BSAuxFuncs.SetVisible(1, WingR.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				WingL.SetMatrix(Matrix4x4.identity);
				WingR.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(7.758681f, 8.403976f, 7.458959f);
				TrgPos = new Vector3(0.06028552f, -1.059821f, 0.05334377f);
				uJ = new Vector3(-0.4756194f, 0.7484986f, -0.4620997f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, WingL.gameObject);
				BSAuxFuncs.SetVisible(1, WingR.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				WingL.SetMatrix(Matrix4x4.identity);
				WingR.SetMatrix(Matrix4x4.identity);
				RTSphere.SetMatrix(Matrix4x4.identity);
				AuxSphereGrid.SetMatrix(Matrix4x4.identity);
				
				RTSphere.transform.localScale = new Vector3(4.5f, 4.5f, 4.5f); 
				AuxSphereGrid.transform.localScale = new Vector3(4.502f, 4.502f, 4.502f); 
				
				CamPos = new Vector3(10.02361f, 10.2423f, 9.640756f);            
				TrgPos = new Vector3(0.02223966f, -0.04016363f, 0.01980611f);
				uJ = new Vector3(-0.4447935f, 0.8032721f, -0.396122f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, WingL.gameObject);
				BSAuxFuncs.SetVisible(1, WingR.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				WingL.SetMatrix(Matrix4x4.identity);
				WingR.SetMatrix(Matrix4x4.identity);
				
				Body.SetWorldPosition(-11.5f, 18.0f, -16.5f);
				i_eulerX = 34;
				i_eulerY = 21;
				
				CamPos = new Vector3(0f, 32.21326f, -34.83307f);
				TrgPos = new Vector3(0f, 5.197875f, 0.1527924f);
				uJ = new Vector3(0f, 0.7914945f, 0.6111763f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
