//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec404
{
	public class Sec404 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Body;
		THObject3D MainRotor;
		THObject3D TailRotor;
		
		THObject3D RTSphere;
		THObject3D AuxSphereGrid;
		
		GameObject DLight;


		// Instance Variables
		float i_eulerX = 0;
		float i_eulerY = 0;
		float i_eulerZ = 0;
		
		float i_degMainRotor = 0.0f;
		float i_degTailRotor = 0.0f;
		
		Vector3 c_initDir = new Vector3(0.0f, 0.0f, 1.0f);
		readonly Vector3 c_attachPos_MainRotor = new Vector3(0.0f, 0.50f, 0.0f);        
		readonly Vector3 c_attachPos_TailRotor = new Vector3(-0.03152f, 0.40674f, -4.81687f);




		void Code1()
		{

		}



		void Code2()
		{

		}



		void Code3()
		{

		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec404 pr_instance = new Sec404(3);

		public static Sec404 PRGSI()
		{
			return pr_instance;
		}


		private Sec404(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Body = CreateObject_Obj3D("HeliBody");
				MainRotor = CreateObject_Obj3D("MainRotor");
				TailRotor = CreateObject_Obj3D("TailRotor");
				RTSphere = CreateObject_Obj3D("RTSphere");
				AuxSphereGrid = CreateObject_Obj3D("AuxSphereGrid");
				
				MeshRenderer mre = RTSphere.gameObject.GetComponent<MeshRenderer>();
				mre.material = BSAuxFuncs.GetSemiTransparentMaterial(0.0f, 0.0f, 0.8f, 0.25f);
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_eulerX = 0.0f;
			i_eulerY = 0.0f;
			i_degMainRotor = 0.0f;
			i_degTailRotor = 0.0f;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, MainRotor.gameObject);
				BSAuxFuncs.SetVisible(1, TailRotor.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				MainRotor.SetMatrix(Matrix4x4.identity);
				TailRotor.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(-7.16279f, 6.072344f, 6.59898f);
				TrgPos = new Vector3(-0.4540634f, 0.898286f, 0.7280643f);
				uJ = new Vector3(0.3645279f, 0.864714f, -0.3455273f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 0, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, MainRotor.gameObject);
				BSAuxFuncs.SetVisible(1, TailRotor.gameObject);
				BSAuxFuncs.SetVisible(1, RTSphere.gameObject);
				BSAuxFuncs.SetVisible(1, AuxSphereGrid.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				MainRotor.SetMatrix(Matrix4x4.identity);
				TailRotor.SetMatrix(Matrix4x4.identity);
				RTSphere.SetMatrix(Matrix4x4.identity);
				AuxSphereGrid.SetMatrix(Matrix4x4.identity);
				
				RTSphere.transform.localScale = new Vector3(3.0f, 3.0f, 3.0f);
				AuxSphereGrid.transform.localScale = new Vector3(3.002f, 3.002f, 3.002f);
				
				
				CamPos = new Vector3(-9.026495f, 4.722257f, 5.134228f);
				TrgPos = new Vector3(-0.4460388f, 0.02302136f, 0.23276f);
				uJ = new Vector3(0.3762683f, 0.9030648f, -0.2071137f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(75, 20, 0);
				}
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, MainRotor.gameObject);
				BSAuxFuncs.SetVisible(1, TailRotor.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				MainRotor.SetMatrix(Matrix4x4.identity);
				TailRotor.SetMatrix(Matrix4x4.identity);
				
				i_MOVE = false;
				
				CamPos = new Vector3(-26.70729f, 14.35613f, 19.81712f);
				TrgPos = new Vector3(-0.2580916f, 5.238884f, 0.134412f);
				uJ = new Vector3(0.214192f, 0.9638252f, -0.1586278f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 80, 0);
				}
			}
		}

	}

}
