//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec405
{
	public class Sec405 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Ball;
		THObject3D Ring;
		THObject3D Grabber;
		
		THObject3D AuxCircle;


		// Instance Variables
		float i_shmBall;
		float i_degrevBall;
		float i_degrotBall;
		
		float i_degRing;
		float i_degGrabber;
		
		bool i_switch1_Ball = true;     
		bool i_switch2_Ball = true;     
		
		bool i_switch_Ring = true;
		bool i_switch_Grabber = true;
		
		readonly Vector3 c_attachPos_Ball = new Vector3(2.0f, 0.0f, 0.0f);
		readonly Vector3 c_attachPos_Ring = new Vector3(-11.0f, 0.0f, 0.0f);
		readonly Vector3 c_attachPos_Grabber = new Vector3(0.0f, 8.0f, 0.0f);




		void Code1()
		{
			// Ball ; c_attachPos_Ball (2, 0, 0)
			Matrix4x4 localBall = TH3DMath.GetTranslation4x4(c_attachPos_Ball);
			
			// Ring ; c_attachPos_Ring (-11, 0, 0)
			Matrix4x4 localRing = TH3DMath.GetTranslation4x4(c_attachPos_Ring);
			
			// Grabber
			Matrix4x4 localGrabber = TH3DMath.GetTranslation4x4(0, 8, 0);
			
			Matrix4x4 worldBall = localGrabber * localRing * localBall;
			Matrix4x4 worldRing = localGrabber * localRing;
			Matrix4x4 worldGrabber = localGrabber;
			
			Ball.SetMatrix(worldBall);
			Ring.SetMatrix(worldRing);
			Grabber.SetMatrix(worldGrabber);
		}


		void Code2()
		{
			// Ball ; c_attachPos_Ball (2, 0, 0)
			i_shmBall += 1.5f;
			float scale = 0.25f * Mathf.Sin(i_shmBall * Mathf.Deg2Rad) + 1.25f; 
			Matrix4x4 sclBall = TH3DMath.GetScale4x4(scale);
			i_degrotBall += 1;
			Matrix4x4 rotBall = TH3DMath.GetRotation4x4(i_degrotBall, Vector3.up);
			i_degrevBall += 2;
			Vector3 pos = TH3DMath.GetRotation4x4(i_degrevBall, Vector3.forward) * new Vector4(2, 0, 0, 1);
			Matrix4x4 traBall = TH3DMath.GetTranslation4x4(pos);
			Matrix4x4 localBall = traBall * rotBall * sclBall;
			
			// Ring
			Matrix4x4 localRing = Matrix4x4.identity;
			
			// world matrix
			Matrix4x4 worldBall = localRing * localBall;
			Matrix4x4 worldRing = localRing;
			
			Ball.SetMatrix(worldBall);
			Ring.SetMatrix(worldRing);
		}


		void Code3()
		{
			// Ring
			i_degRing += 1;
			Matrix4x4 rotRing = TH3DMath.GetRotation4x4(i_degRing, Vector3.up);
			Matrix4x4 traRing = TH3DMath.GetTranslation4x4(c_attachPos_Ring);  // c_attachPos_Ring (-11, 0, 0)
			Matrix4x4 localRing = traRing * rotRing;
			
			// Grabber
			Matrix4x4 localGrabber = Matrix4x4.identity;
			
			// world matrix
			Matrix4x4 worldRing = localGrabber * localRing;
			Matrix4x4 worldGrabber = localGrabber;
			
			Ring.SetMatrix(worldRing);
			Grabber.SetMatrix(worldGrabber);
		}


		void Code4()
		{
			// Grabber
			i_degGrabber += 0.5f;
			Matrix4x4 rotGrabber = TH3DMath.GetRotation4x4(i_degGrabber, Vector3.up) *
			TH3DMath.GetRotation4x4(i_degGrabber + 90, Vector3.right);
			Matrix4x4 traGrabber = TH3DMath.GetTranslation4x4(0, 8, 0);
			Matrix4x4 localGrabber = traGrabber * rotGrabber;
			
			Matrix4x4 worldGrabber = localGrabber;
			Grabber.SetMatrix(worldGrabber);
		}



		void Code5()
		{

		}



		void Code6()
		{
			if (!i_INITIALIZED)
			{

				i_INITIALIZED = true;
			}


		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec405 pr_instance = new Sec405(6);

		public static Sec405 PRGSI()
		{
			return pr_instance;
		}


		private Sec405(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Ball = CreateObject_Obj3D("WhBall");
				Ring = CreateObject_Obj3D("Ring");
				Grabber = CreateObject_Obj3D("Grabber");
				AuxCircle = CreateObject_Obj3D("AuxCircle");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_shmBall = 0.0f;
			i_degrotBall = 0.0f;
			i_degrevBall = 0.0f;
			i_degRing = 0.0f;
			i_degGrabber = 0.0f;
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				BSAuxFuncs.SetVisible(1, Grabber.gameObject);
				Ball.SetMatrix(Matrix4x4.identity);
				Ring.SetMatrix(Matrix4x4.identity);
				Grabber.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(2.216967f, 16.10848f, -13.57405f);
				TrgPos = new Vector3(-7.425043f, 4.623932f, 3.565045f);
				uJ = new Vector3(-0.09373581f, 0.850667f, 0.5172807f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				Ball.SetMatrix(Matrix4x4.identity);
				Ring.SetMatrix(Matrix4x4.identity);
				AuxCircle.SetMatrix(Matrix4x4.identity);
				
				AuxCircle.transform.localScale = new Vector3(2.0f, 2.0f, 1.0f);
				
				CamPos = new Vector3(5.682378f, 3.976736f, -7.813802f);
				TrgPos = new Vector3(-1.365607f, -0.9932289f, 2.251051f);
				uJ = new Vector3(-0.2169545f, 0.9270343f, 0.3058401f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				BSAuxFuncs.SetVisible(1, Grabber.gameObject);
				Ring.SetMatrix(Matrix4x4.identity);
				Grabber.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(2.881164f, 9.600407f, -15.29634f);
				TrgPos = new Vector3(-5.822847f, -1.262236f, 0.1755343f);
				uJ = new Vector3(-0.1688612f, 0.8488359f, 0.5009626f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Grabber.gameObject);
				Grabber.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(18.78152f, 18.37385f, 18.10335f);
				TrgPos = new Vector3(-1.71418f, 4.350049f, -1.612257f);
				uJ = new Vector3(-0.3103795f, 0.8968176f, -0.3152502f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(5, 6))
			{
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				BSAuxFuncs.SetVisible(1, Grabber.gameObject);
				Ball.SetMatrix(Matrix4x4.identity);
				Ring.SetMatrix(Matrix4x4.identity);
				Grabber.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(18.78152f, 18.37385f, 18.10335f);
				TrgPos = new Vector3(-1.71418f, 4.350049f, -1.612257f);
				uJ = new Vector3(-0.3103795f, 0.8968176f, -0.3152502f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
