//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec408
{
	public class Sec408 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Sphere;
		THObject3D Arm;


		// Instance Variables
		float i_shmSphere = 0;
		float i_degArm = 0;
		
		readonly Vector3 c_attachPos_Sphere = new Vector3(5, 4, 0);




		void Code1()
		{
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere;
			
			Matrix4x4 localArm = Matrix4x4.identity;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}


		void Code2()
		{
			Matrix4x4 sclSphere = TH3DMath.GetScale4x4(2.0f);
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere * sclSphere;
			
			Matrix4x4 localArm = Matrix4x4.identity;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}


		void Code3()
		{
			i_shmSphere += 1.0f;
			float scale = 0.5f * Mathf.Sin(i_shmSphere * Mathf.Deg2Rad) + 1.5f; 
			Matrix4x4 sclSphere = TH3DMath.GetScale4x4(scale);
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere * sclSphere;
			
			Matrix4x4 localArm = Matrix4x4.identity;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec408 pr_instance = new Sec408(3);

		public static Sec408 PRGSI()
		{
			return pr_instance;
		}


		private Sec408(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere = CreateObject_Obj3D("R2Sphere");
				Arm = CreateObject_Obj3D("DBrwnArm");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_shmSphere = 0.0f;
			
			if (MatchCase(1, 2, 3))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				Arm.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(2.624317f, 6.877073f, -11.95993f);      
				TrgPos = new Vector3(3.824774f, 3.070199f, 0.3419875f);
				uJ = new Vector3(-0.005258686f, 0.9551466f, 0.2960868f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
