//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec409
{
	public class Sec409 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Sphere;
		THObject3D Arm;


		// Instance Variables
		float i_shmSphere = 0;
		float i_degArm = 0;
		
		readonly Vector3 c_attachPos_Sphere = new Vector3(5, 4, 0);




		void Code1()
		{
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere;
			
			Matrix4x4 localArm = TH3DMath.GetTranslation4x4(3, 0, -3);
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}


		void Code2()
		{
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere;
			
			Matrix4x4 rotArm = TH3DMath.GetRotation4x4(-60, Vector3.up);
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(3, 0, -3);
			Matrix4x4 localArm = traArm * rotArm;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}


		void Code3()
		{
			Matrix4x4 sclSphere = TH3DMath.GetScale4x4(2.0f);
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere * sclSphere;
			
			Matrix4x4 rotArm = TH3DMath.GetRotation4x4(-60, Vector3.up);
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(3, 0, -3);
			Matrix4x4 localArm = traArm * rotArm;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}


		void Code4()
		{
			i_shmSphere += 1.0f;
			float scale = 0.5f * Mathf.Sin(i_shmSphere * Mathf.Deg2Rad) + 1.5f; // 1 -- 2
			Matrix4x4 sclSphere = TH3DMath.GetScale4x4(scale);
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere * sclSphere;
			
			i_degArm -= 1.0f;
			Matrix4x4 rotArm = TH3DMath.GetRotation4x4(i_degArm, Vector3.up);
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(3, 0, -3);
			Matrix4x4 localArm = traArm * rotArm;
			
			Matrix4x4 worldSphere = localArm * localSphere;
			Matrix4x4 worldArm = localArm;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec409 pr_instance = new Sec409(4);

		public static Sec409 PRGSI()
		{
			return pr_instance;
		}


		private Sec409(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere = CreateObject_Obj3D("R2Sphere");
				Arm = CreateObject_Obj3D("DBrwnArm");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			if (MatchCase(1, 2, 3, 4))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				Arm.SetMatrix(Matrix4x4.identity);
				
				if (MatchCase(4))
				{
					CamPos = new Vector3(6.544559f, 7.675213f, -15.73859f);
					TrgPos = new Vector3(2.925701f, 1.955443f, -2.572361f);
					uJ = new Vector3(-0.1030754f, 0.9223462f, 0.3723613f);
					
					i_shmSphere = -45.0f;
					i_degArm = 0.0f;
				}
				else
				{
					CamPos = new Vector3(6.328471f, 6.239554f, -14.51427f);
					TrgPos = new Vector3(6.328471f, 1.526844f, -2.911859f);
					uJ = new Vector3(0f, 0.926488f, 0.3763243f);
				}
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
