//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec410
{
	public class Sec410 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Sphere;
		THObject3D Arm;
		THObject3D Disk;


		// Instance Variables
		float i_shmSphere = 0;
		float i_degArm = 0;
		float i_shmDisk = 0;
		
		readonly Vector3 c_attachPos_Sphere = new Vector3(5, 4, 0);
		readonly Vector3 c_attachPos_Arm = new Vector3(2, 0, 0);




		void Code1()
		{
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere;
			
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(c_attachPos_Arm);
			Matrix4x4 localArm = traArm;
			
			Matrix4x4 localDisk = Matrix4x4.identity;
			
			Matrix4x4 worldSphere = localDisk * localArm * localSphere;
			Matrix4x4 worldArm = localDisk * localArm;
			Matrix4x4 worldDisk = localDisk;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
			Disk.SetMatrix(worldDisk);
		}


		void Code2()
		{
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12の間を往復
			Matrix4x4 traDisk = TH3DMath.GetTranslation4x4(0.0f, posY, 0.0f);
			Matrix4x4 localDisk = traDisk;
			
			Matrix4x4 worldDisk = localDisk;
			
			Disk.SetMatrix(worldDisk);
		}


		void Code3()
		{
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12の間を往復
			Matrix4x4 rotDisk = TH3DMath.GetRotation4x4(-15.0f, Vector3.right);
			Matrix4x4 traDisk = TH3DMath.GetTranslation4x4(0.0f, posY, 0.0f);
			Matrix4x4 localDisk = traDisk * rotDisk;
			
			Matrix4x4 worldDisk = localDisk;
			
			Disk.SetMatrix(worldDisk);
		}


		void Code4()
		{
			// c_attachPos_Sphere : (5, 4, 0)
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere;
			
			// c_attachPos_Arm : (2, 0, 0)
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(c_attachPos_Arm);
			Matrix4x4 localArm = traArm;
			
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12の間を往復
			Matrix4x4 rotDisk = TH3DMath.GetRotation4x4(-15.0f, Vector3.right);
			Matrix4x4 traDisk = TH3DMath.GetTranslation4x4(0.0f, posY, 0.0f);
			Matrix4x4 localDisk = traDisk * rotDisk;
			
			Matrix4x4 worldSphere = localDisk * localArm * localSphere;
			Matrix4x4 worldArm = localDisk * localArm;
			Matrix4x4 worldDisk = localDisk;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
			Disk.SetMatrix(worldDisk);
		}


		void Code5()
		{
			i_shmSphere += 1.0f;
			float scale = 0.5f * Mathf.Sin(i_shmSphere * Mathf.Deg2Rad) + 1.5f; // 1 -- 2
			Matrix4x4 sclSphere = TH3DMath.GetScale4x4(scale, scale, scale);
			Matrix4x4 traSphere = TH3DMath.GetTranslation4x4(c_attachPos_Sphere);
			Matrix4x4 localSphere = traSphere * sclSphere;
			
			i_degArm--;
			Matrix4x4 rotArm = TH3DMath.GetRotation4x4(i_degArm, Vector3.up);
			Matrix4x4 traArm = TH3DMath.GetTranslation4x4(c_attachPos_Arm);
			Matrix4x4 localArm = traArm * rotArm;
			
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12
			Matrix4x4 rotDisk = TH3DMath.GetRotation4x4(-15.0f, Vector3.right);
			Matrix4x4 traDisk = TH3DMath.GetTranslation4x4(0.0f, posY, 0.0f);
			Matrix4x4 localDisk = traDisk * rotDisk;
			
			
			Matrix4x4 worldSphere = localDisk * localArm * localSphere;
			Matrix4x4 worldArm = localDisk * localArm;
			Matrix4x4 worldDisk = localDisk;
			
			Sphere.SetMatrix(worldSphere);
			Arm.SetMatrix(worldArm);
			Disk.SetMatrix(worldDisk);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec410 pr_instance = new Sec410(5);

		public static Sec410 PRGSI()
		{
			return pr_instance;
		}


		private Sec410(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere = CreateObject_Obj3D("R2Sphere");
				Arm = CreateObject_Obj3D("DBrwnArm");
				Disk = CreateObject_Obj3D("GapDisk");

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				Arm.SetMatrix(Matrix4x4.identity);
				Disk.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(8.480182f, 9.015744f, -16.3627f);
				TrgPos = new Vector3(3.084925f, 0.3441896f, -3.23361f);
				uJ = new Vector3(-0.02867809f, 0.8394597f, 0.5426648f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2, 3))
			{
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Disk.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(12.2925f, 12.65544f, -24.1966f);
				TrgPos = new Vector3(-0.90039f, 9.064087f, 1.753844f);
				uJ = new Vector3(-0.04559816f, 0.9924152f, 0.1141616f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Sphere.SetMatrix(Matrix4x4.identity);
				Arm.SetMatrix(Matrix4x4.identity);
				Disk.SetMatrix(Matrix4x4.identity);
				
				i_shmSphere = 270;
				i_degArm = 0;
				
				CamPos = new Vector3(12.2925f, 12.65544f, -24.1966f);
				TrgPos = new Vector3(-0.90039f, 9.064087f, 1.753844f);
				uJ = new Vector3(-0.04559816f, 0.9924152f, 0.1141616f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
