//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec412
{
	public class Sec412 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Tent1;
		THObject3D Tent2;
		THObject3D Tent3;
		
		GameObject DLight;




		void Code1()
		{
			Matrix4x4 localTent3 = TH3DMath.GetTranslation4x4(0, 0, 8);
			Matrix4x4 localTent2 = TH3DMath.GetTranslation4x4(0, 8, 0);
			Matrix4x4 localTent1 = TH3DMath.GetTranslation4x4(8, 0, 0);
			
			Matrix4x4 worldTent3 = localTent1 * localTent2 * localTent3;
			Matrix4x4 worldTent2 = localTent1 * localTent2;
			Matrix4x4 worldTent1 = localTent1;
			
			Tent3.SetMatrix(worldTent3);
			Tent2.SetMatrix(worldTent2);
			Tent1.SetMatrix(worldTent1);
		}


		void Code2()
		{
			Matrix4x4 rotTent3 = TH3DMath.GetRotation4x4(90, Vector3.up);
			Matrix4x4 traTent3 = TH3DMath.GetTranslation4x4(0, 0, 8);
			Matrix4x4 localTent3 = traTent3 * rotTent3;
			
			Matrix4x4 rotTent2 = TH3DMath.GetRotation4x4(90, Vector3.up) *
			TH3DMath.GetRotation4x4(90, Vector3.forward);
			Matrix4x4 traTent2 = TH3DMath.GetTranslation4x4(0, 8, 0);
			Matrix4x4 localTent2 = traTent2 * rotTent2;
			
			Matrix4x4 rotTent1 = TH3DMath.GetRotation4x4(-90, Vector3.right);
			Matrix4x4 traTent1 = TH3DMath.GetTranslation4x4(8, 0, 0);
			Matrix4x4 localTent1 = traTent1 * rotTent1;
			
			Matrix4x4 worldTent3 = localTent1 * localTent2 * localTent3;
			Matrix4x4 worldTent2 = localTent1 * localTent2;
			Matrix4x4 worldTent1 = localTent1;
			
			Tent3.SetMatrix(worldTent3);
			Tent2.SetMatrix(worldTent2);
			Tent1.SetMatrix(worldTent1);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec412 pr_instance = new Sec412(2);

		public static Sec412 PRGSI()
		{
			return pr_instance;
		}


		private Sec412(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Tent1 = CreateObject_Obj3D("Tent_R");
				Tent2 = CreateObject_Obj3D("Tent_G");
				Tent3 = CreateObject_Obj3D("Tent_B");
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Tent1.gameObject);
				BSAuxFuncs.SetVisible(1, Tent2.gameObject);
				BSAuxFuncs.SetVisible(1, Tent3.gameObject);
				Tent1.SetMatrix(Matrix4x4.identity);
				Tent2.SetMatrix(Matrix4x4.identity);
				Tent3.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(19.50761f, 15.19451f, 16.98243f);
				TrgPos = new Vector3(1.923333f, 2.548301f, -0.1881376f);
				uJ = new Vector3(-0.3477192f, 0.8888084f, -0.2985145f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -70, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Tent1.gameObject);
				BSAuxFuncs.SetVisible(1, Tent2.gameObject);
				BSAuxFuncs.SetVisible(1, Tent3.gameObject);
				Tent1.SetMatrix(Matrix4x4.identity);
				Tent2.SetMatrix(Matrix4x4.identity);
				Tent3.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(21.20616f, 10.25115f, -16.74528f);
				TrgPos = new Vector3(6.416293f, -7.683714f, 2.097193f);
				uJ = new Vector3(-0.2683342f, 0.7941487f, 0.5452749f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
				}
			}
		}

	}

}
