//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;


using OQJ_THShell3D;


namespace OQJ_Sec413
{
	public class Sec413 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Green;
		THObject3D Box;
		THObject3D Base;
		THObject3D Beam;
		THObject3D BallB;
		THObject3D BallR;
		THObject3D[] MiniBall;
		
		THObject3D Cannon;
		THObject3D Battery;
		THShell3D Shell;
		THShell3D Shell_A;
		THShell3D Shell_B;
		
		THObject3D Trj;
		
		
		
		GameObject DLight;


		// Instance Variables
		float i_shmBeam;
		float i_shmGreen;
		float i_degBattery;
		
		readonly Vector3 c_initV1 = new Vector3(0.0f, 2.1f, 0.0f);
		readonly Vector3 c_initV2 = new Vector3(0.9f, 0.0f, -0.9f);
		
		readonly Vector3 c_attachPos_Beam = new Vector3(0.0f, 1.982f, 0.0f);
		readonly Vector3 c_attachPos_Green = new Vector3(-4.24f, 0.302f, 0.0f);
		readonly Vector3 c_attachPos_Cannon = new Vector3(0.0f, 3.21714f, 0.80772f);
		
		readonly Vector3 c_initShootPosition = new Vector3(0.0f, 1.3f, 2.6f);
		readonly Vector3 c_initShootDirection = new Vector3(0, 0, 1);
		
		readonly Vector3[] c_initBoxVerts =
		{
		new Vector3( 0.5f,  0.5f, -0.5f),	
		new Vector3( 0.5f, -0.5f, -0.5f),
		new Vector3(-0.5f, -0.5f, -0.5f),	
		new Vector3(-0.5f,  0.5f, -0.5f),
		
		new Vector3( 0.5f,  0.5f,  0.5f),	
		new Vector3( 0.5f, -0.5f,  0.5f),
		new Vector3(-0.5f, -0.5f,  0.5f),	
		new Vector3(-0.5f,  0.5f,  0.5f)
		};
		
		
		float pr_eulerX;
		float pr_eulerY;
		float pr_eulerZ;
		float pr_shmW;
		float pr_shmD;
		
		readonly Vector3 pr_initStartTrj = new Vector3(0.0f, 1.3f, 2.0f);




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Matrix4x4 T = TH3DMath.GetTranslation4x4(4, 5, -10);
				Matrix4x4 R = TH3DMath.GetEulerRotation4x4(30, 80, -20);
				Matrix4x4 S = TH3DMath.GetScale4x4(1.0f, 2.2f, 1.5f);
				
				Matrix4x4 M = T * R * S;
				Green.SetMatrix(M);
				
				Vector3 V1 = M * TH3DMath.ToVector4(c_initV1);
				Vector3 V2 = M * TH3DMath.ToVector4(c_initV2);
				BallB.SetWorldPosition(V1);
				BallR.SetWorldPosition(V2);
				
				i_INITIALIZED = true;
			}
		}


		void Code2()
		{
			BoxMotion();
			
			Matrix4x4 M = Box.GetMatrix();
			
			for (int i = 0; i < MiniBall.Length; i++)
			{
				Vector3 pos = M * TH3DMath.ToVector4(c_initBoxVerts[i]);
				MiniBall[i].SetWorldPosition(pos);
			}
		}


		void Code3()
		{
			Matrix4x4[] mtrcs = CalcLocalMatrices();
			Matrix4x4 localGreen = mtrcs[0];
			Matrix4x4 localBeam  = mtrcs[1];
			Matrix4x4 localBase  = mtrcs[2];
			
			Matrix4x4 worldGreen = localBase * localBeam * localGreen;
			Matrix4x4 worldBeam  = localBase * localBeam;
			Matrix4x4 worldBase  = localBase;
			
			Green.SetMatrix(worldGreen);
			Beam.SetMatrix(worldBeam);
			Base.SetMatrix(worldBase);
			
			Vector3 V1 = worldGreen * TH3DMath.ToVector4(c_initV1);
			Vector3 V2 = worldGreen * TH3DMath.ToVector4(c_initV2);
			BallB.SetWorldPosition(V1);
			BallR.SetWorldPosition(V2);
		}


		void Code4()
		{
			if (Input.GetKeyDown(KeyCode.A))
			{
				Matrix4x4 mtx = Cannon.GetMatrix();
				Vector3 shootPos = mtx * TH3DMath.ToVector4(c_initShootPosition);
				Vector3 shootDir = mtx * c_initShootDirection;
				
				Shell.direction = shootDir;
				Shell.speed = 0.094f;
				
				Matrix4x4 T = TH3DMath.GetTranslation4x4(shootPos);
				Shell.SetMatrix(T);
				
				return;
			}
			
			
			Vector3 pos = Shell.position + Shell.speed * Shell.direction;
			Matrix4x4 M = TH3DMath.GetTranslation4x4(pos);
			Shell.SetMatrix(M);
		}


		void Code5()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L) )
			{
				ChangeCannonDirection();
			}
			
			
			if (Input.GetKeyDown(KeyCode.A))
			{
				Matrix4x4 mtx = Cannon.GetMatrix();
				Vector3 shootPos = mtx * TH3DMath.ToVector4(c_initShootPosition);
				Vector3 shootDir = mtx * c_initShootDirection;
				
				Shell.direction = shootDir;
				Shell.speed = 0.094f;
				
				mtx.SetColumn(3, new Vector4(0, 0, 0, 1));
				Shell.rotMatrix = mtx;
				
				Matrix4x4 T = TH3DMath.GetTranslation4x4(shootPos);
				Shell.SetMatrix(T * Shell.rotMatrix);
				
				return;
			}
			
			
			Vector3 pos = Shell.position + Shell.speed * Shell.direction;
			Matrix4x4 M = TH3DMath.GetTranslation4x4(pos) * Shell.rotMatrix;
			Shell.SetMatrix(M);
		}


		void Code6()
		{
			if (Input.GetKey(KeyCode.H))
			{
				i_degBattery -= 0.5f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBattery += 0.5f;
			}
			
			Matrix4x4 traBattery = TH3DMath.GetTranslation4x4(-8, 0, -22);
			Matrix4x4 rotBattery = TH3DMath.GetRotation4x4(i_degBattery, Vector3.up);
			Matrix4x4 localBattery = traBattery * rotBattery;
			
			Matrix4x4 localCannon = TH3DMath.GetTranslation4x4(c_attachPos_Cannon);
			
			Matrix4x4 worldCannon  = localBattery * localCannon;
			Matrix4x4 worldBattery = localBattery;
			
			Cannon.SetMatrix(worldCannon);
			Battery.SetMatrix(worldBattery);
			
			
			
			if (Input.GetKeyDown(KeyCode.A))
			{
				Matrix4x4 mtx = Cannon.GetMatrix();
				Vector3 shootPos = mtx * TH3DMath.ToVector4(c_initShootPosition);
				Vector3 shootDir = mtx * c_initShootDirection;
				
				Shell.direction = shootDir;
				Shell.speed = 0.094f;
				
				mtx.SetColumn(3, new Vector4(0, 0, 0, 1));
				Shell.rotMatrix = mtx;
				
				Matrix4x4 T = TH3DMath.GetTranslation4x4(shootPos);
				Shell.SetMatrix(T * Shell.rotMatrix);
				
				return;
			}
			
			
			Vector3 pos = Shell.position + Shell.speed * Shell.direction;
			Matrix4x4 M = TH3DMath.GetTranslation4x4(pos) * Shell.rotMatrix;
			Shell.SetMatrix(M);
		}




		void BoxMotion()
		{
			pr_eulerX += 0.09f;
			pr_eulerY -= 0.13f;
			pr_eulerZ -= 0.04f;
			Matrix4x4 R = TH3DMath.GetEulerRotation4x4(pr_eulerX, pr_eulerY, pr_eulerZ);
			
			pr_shmW += 0.55f;	
			float min = 4.0f;
			float max = 10.0f;
			float A = max - min;
			float scaleW = 0.5f * A * (-Mathf.Cos(pr_shmW * Mathf.Deg2Rad) + 1.0f) + min;
			
			pr_shmD += 0.44f;	
			min = 3.0f;
			max = 9.0f;
			A = max - min;
			float scaleD = 0.5f * A * (-Mathf.Cos(pr_shmD * Mathf.Deg2Rad) + 1.0f) + min;
			
			Matrix4x4 S = TH3DMath.GetScale4x4(scaleW, 3.6f, scaleD);
			
			Matrix4x4 M = TH3DMath.GetTranslation4x4(12, 8, 8) * R * S;
			Box.SetMatrix(M);
		}
		
		
		
		Matrix4x4[] CalcLocalMatrices()
		{
			i_shmGreen += 0.6f;
			float min = 0.8f;
			float max = 1.8f;
			float A = max - min;
			float scaleXZ = 0.5f * A * (-Mathf.Cos(i_shmGreen * Mathf.Deg2Rad) + 1.0f) + min;
			float scaleY = max - (scaleXZ - min);
			
			i_shmBeam += 0.4f;
			min = -12f;
			max = 12f;
			A = max - min;
			float degBeam = 0.5f * A * (-Mathf.Cos(i_shmBeam * Mathf.Deg2Rad) + 1.0f) + min;
			
			Matrix4x4 sclGreen = TH3DMath.GetScale4x4(scaleXZ, scaleY, scaleXZ);
			Matrix4x4 traGreen = TH3DMath.GetTranslation4x4(c_attachPos_Green);
			Matrix4x4 rotGreen = TH3DMath.GetRotation4x4(30, Vector3.up);
			Matrix4x4 localGreen = traGreen * rotGreen * sclGreen;
			
			Matrix4x4 traBeam = TH3DMath.GetTranslation4x4(c_attachPos_Beam);
			Matrix4x4 rotBeam = TH3DMath.GetRotation4x4(degBeam, Vector3.forward);
			Matrix4x4 localBeam = traBeam * rotBeam;
			
			Matrix4x4 traBase = TH3DMath.GetTranslation4x4(1, 5, 6);
			Matrix4x4 rotBase = TH3DMath.GetRotation4x4(35.0f, new Vector3(2, 3, 4));
			Matrix4x4 localBase = traBase * rotBase;
			
			return new Matrix4x4[] { localGreen, localBeam, localBase };
		}
		
		
		
		void ChangeCannonDirection()
		{
			if (Input.GetKey(KeyCode.H))
			{
				pr_eulerY += 0.2f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				pr_eulerY -= 0.2f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				pr_eulerX += 0.2f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				pr_eulerX -= 0.2f;
			}
			
			Cannon.SetWorldRotation(pr_eulerX, pr_eulerY, 20);
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(4, 5, 6))
			{
				Matrix4x4 M = Cannon.GetMatrix();
				Vector3 pos = M * TH3DMath.ToVector4(pr_initStartTrj);
				Quaternion rot = Cannon.GetWorldRotation();
				
				Trj.SetWorldPosition(pos);
				Trj.SetWorldRotation(rot);
			}
		}


		static Sec413 pr_instance = new Sec413(6);

		public static Sec413 PRGSI()
		{
			return pr_instance;
		}


		private Sec413(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Green = CreateObject_Obj3D("PrsGreen");
				Base = CreateObject_Obj3D("BlBase");
				Beam = CreateObject_Obj3D("BlBeam");
				Box = CreateObject_Obj3D("UnitCube01");
				BallB = CreateObject_Obj3D("BallB");
				BallR = CreateObject_Obj3D("BallR");
				Cannon = CreateObject_Obj3D("MlCannon");
				Battery = CreateObject_Obj3D("Battery");
				Shell_A = CreateObject_Shell3D("MlShell_A");
				Shell_B = CreateObject_Shell3D("MlShell_B");
				
				Trj = CreateObject_Obj3D("AuxTrj");
				
				var src_Mini = CreateObject_InsObj3D(BallB.gameObject);
				src_Mini.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetStandardMaterial(0.74f, 0.74f, 0.74f);
				MiniBall = new THObject3D[8];
				MiniBall[0] = src_Mini;
				for (int i = 1; i < MiniBall.Length; i++)
				{
					MiniBall[i] = CreateObject_InsObj3D(MiniBall[0].gameObject);
				}
				
				Mesh mesh = Trj.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { Vector3.zero, Vector3.forward });
				
				
				
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			if (DLight != null)
			{
				DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
			}
			
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Green.gameObject);
				Green.SetIdentity();
				BSAuxFuncs.SetVisible(1, BallB.gameObject);
				BallB.SetIdentity();
				BSAuxFuncs.SetVisible(1, BallR.gameObject);
				BallR.SetIdentity();
				
				CamPos = new Vector3(3.747361f, 10.99211f, -20.27189f);     
				TrgPos = new Vector3(5.28458f, -0.04137656f, 1.305836f);
				uJ = new Vector3(-0.02290531f, 0.8894573f, 0.4564439f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 10, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Box.gameObject);
				Box.SetIdentity();
				
				for (int i = 0; i < MiniBall.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, MiniBall[i].gameObject);
					MiniBall[i].SetIdentity();
				}
				
				pr_eulerX = 30;
				pr_eulerY = 40;
				pr_eulerZ = 50;
				
				pr_shmW = 90;
				pr_shmD = -45;
				
				CamPos = new Vector3(12.63488f, 14.48518f, -13.58869f);
				TrgPos = new Vector3(11.9942f, 7.985121f, 6.216547f);
				uJ = new Vector3(0.0059135f, 0.9500633f, 0.3120013f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, -30, 0);
				}
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Green.gameObject);
				Green.SetIdentity();
				BSAuxFuncs.SetVisible(1, Base.gameObject);
				Base.SetIdentity();
				BSAuxFuncs.SetVisible(1, Beam.gameObject);
				Beam.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, BallB.gameObject);
				BallB.SetIdentity();
				BSAuxFuncs.SetVisible(1, BallR.gameObject);
				BallR.SetIdentity();
				
				i_shmGreen = 0.0f;
				i_shmBeam = 0.0f;
				
				CamPos = new Vector3(-12.4819f, 9.324506f, -4.686748f);   
				TrgPos = new Vector3(-1.234302f, 6.50392f, 6.243649f);
				uJ = new Vector3(0.1454537f, 0.9838615f, 0.1042104f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 30, 0);
				}
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				BSAuxFuncs.SetVisible(1, Shell_A.gameObject);
				Shell_A.SetIdentity();
				BSAuxFuncs.SetVisible(1, Shell_B.gameObject);
				Shell_B.SetIdentity();
				BSAuxFuncs.SetVisible(1, Trj.gameObject);
				Trj.SetIdentity();
				Trj.SetLocalScale(1, 1, 50);
				
				Shell_A.SetWorldPosition(-1000, 0, 0);
				Shell_B.SetWorldPosition(-1000, 0, 0);
				Shell = MatchCase(4) ? Shell_A : Shell_B;
				
				pr_eulerX = 10;
				pr_eulerY = 120;
				Cannon.SetWorldPosition(4, 7, 3);
				Cannon.SetWorldRotation(pr_eulerX, pr_eulerY, 20);
				
				CamPos = new Vector3(9.025776f, 10.91035f, -10.07427f);		
				TrgPos = new Vector3(6.277955f, 5.511251f, 7.090729f);
				uJ = new Vector3(0.04707637f, 0.9506847f, 0.3065657f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(60, -10, 0);
				}
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				BSAuxFuncs.SetVisible(1, Battery.gameObject);
				Battery.SetIdentity();
				BSAuxFuncs.SetVisible(1, Shell_B.gameObject);
				Shell_B.SetIdentity();
				BSAuxFuncs.SetVisible(1, Trj.gameObject);
				Trj.SetIdentity();
				Trj.SetLocalScale(1, 1, 50);
				
				Shell_B.SetWorldPosition(-1000, 0, 0);
				Shell = Shell_B;
				
				CamPos = new Vector3(6.556164f, 10.58475f, -35.88346f);
				TrgPos = new Vector3(-18.05704f, -7.066814f, -2.488615f);
				uJ = new Vector3(-0.2524975f, 0.9198763f, 0.3001208f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(60, -10, 0);
				}
			}		
			
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}

	}

}
