//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec414
{
	public class Sec414 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Pole;
		THObject3D[] Arm = new THObject3D[3];
		THObject3D[] Box = new THObject3D[3];
		THObject3D[] Propeller = new THObject3D[3];
		THObject3D YLine;


		// Instance Variables
		int i_degPropeller = 0;
		int i_shmArm = 0;
		int i_degPole = 0;
		
		int i_degBox0 = 0;
		
		bool i_switch_Pole = true;
		bool i_switch_Arm = true;
		
		readonly Vector3 c_attachPos_Propeller = new Vector3(0, 0, -1);
		readonly Vector3 c_attachPos_Box = new Vector3(4.75641f, 10.0f - 0.5f, 0.0f);




		void Code1()
		{
			i_degPropeller -= 6;
			Matrix4x4 rotPropeller = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traPropeller = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traPropeller * rotPropeller;
			
			Matrix4x4 localBox = TH3DMath.GetTranslation4x4(c_attachPos_Box);
			Matrix4x4 localArm = Matrix4x4.identity;
			Matrix4x4 localPole = Matrix4x4.identity;
			
			Matrix4x4 worldPropeller = localPole * localArm * localBox * localPropeller;
			Matrix4x4 worldBox = localPole * localArm * localBox;
			Matrix4x4 worldArm = localPole * localArm;
			Matrix4x4 worldPole = localPole;
			
			Propeller[0].SetMatrix(worldPropeller);
			Box[0].SetMatrix(worldBox);
			Arm[0].SetMatrix(worldArm);
			Pole.SetMatrix(worldPole);
		}


		void Code2()
		{
			// Propeller
			i_degPropeller -= 6;
			Matrix4x4 rotPropeller = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traPropeller = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traPropeller * rotPropeller;
			
			// Box
			Matrix4x4 localBox = TH3DMath.GetTranslation4x4(c_attachPos_Box);
			
			// Arm
			i_shmArm += 1;
			float my = 3.0f * Mathf.Sin(i_shmArm * Mathf.Deg2Rad);    
			Matrix4x4 localArm = TH3DMath.GetTranslation4x4(0.0f, my, 0.0f);
			
			// Pole
			i_degPole += 1;
			Matrix4x4 localPole = TH3DMath.GetRotation4x4(i_degPole, Vector3.up);
			
			// world matrix
			Matrix4x4 worldPropeller = localPole * localArm * localBox * localPropeller;
			Matrix4x4 worldBox = localPole * localArm * localBox;
			Matrix4x4 worldArm = localPole * localArm;
			Matrix4x4 worldPole = localPole;
			
			Propeller[0].SetMatrix(worldPropeller);
			Box[0].SetMatrix(worldBox);
			Arm[0].SetMatrix(worldArm);
			Pole.SetMatrix(worldPole);
		}


		void Code3()
		{
			Matrix4x4 localPropeller = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localBox = TH3DMath.GetTranslation4x4(c_attachPos_Box);
			Matrix4x4 localPole = Matrix4x4.identity;
			
			for (int i = 0; i < 3; i++)
			{
				Matrix4x4 localArm = TH3DMath.GetRotation4x4(i * 120, Vector3.up);
				
				Matrix4x4 worldPropeller = localPole * localArm * localBox * localPropeller;
				Matrix4x4 worldBox = localPole * localArm * localBox;
				Matrix4x4 worldArm = localPole * localArm;
				
				Propeller[i].SetMatrix(worldPropeller);
				Box[i].SetMatrix(worldBox);
				Arm[i].SetMatrix(worldArm);
			}
			
			Matrix4x4 worldPole = localPole;
			Pole.SetMatrix(worldPole);
		}


		void Code4()
		{
			// Propeller
			i_degPropeller -= 6;
			Matrix4x4 rotPropeller = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traPropeller = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traPropeller * rotPropeller;
			
			// Box
			Matrix4x4 localBox = TH3DMath.GetTranslation4x4(c_attachPos_Box);
			
			// Pole
			i_degPole += 1;
			Matrix4x4 localPole = TH3DMath.GetRotation4x4(i_degPole, Vector3.up);
			
			i_shmArm += 1;
			for (int i = 0; i < 3; i++)
			{
				// Arm
				float my = 3.0f * Mathf.Sin((i_shmArm + i * 60) * Mathf.Deg2Rad);    
				Matrix4x4 traArm = TH3DMath.GetTranslation4x4(0.0f, my, 0.0f);
				Matrix4x4 rotArm = TH3DMath.GetRotation4x4(i * 120, Vector3.up);
				Matrix4x4 localArm = traArm * rotArm;
				
				// world matrix : Propeller, Box, Arm
				Matrix4x4 worldPropeller = localPole * localArm * localBox * localPropeller;
				Matrix4x4 worldBox = localPole * localArm * localBox;
				Matrix4x4 worldArm = localPole * localArm;
				
				Propeller[i].SetMatrix(worldPropeller);
				Box[i].SetMatrix(worldBox);
				Arm[i].SetMatrix(worldArm);
			}
			
			// world matrix : Pole
			Matrix4x4 worldPole = localPole;
			Pole.SetMatrix(worldPole);
		}


		void Code5()
		{
			if (Input.GetKeyDown(KeyCode.J))
			{
				i_switch_Pole = !i_switch_Pole;
			}
			else if (Input.GetKeyDown(KeyCode.K))
			{
				i_switch_Arm = !i_switch_Arm;
			}
			else if (Input.GetKey(KeyCode.L))  // 長押しに対応 
			{
				i_degBox0 = THUtil.IsShiftDown() ? i_degBox0 - 2 : i_degBox0 + 2;
			}
			
			
			// Propeller
			i_degPropeller -= 6;
			Matrix4x4 rotPropeller = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			Matrix4x4 traPropeller = TH3DMath.GetTranslation4x4(c_attachPos_Propeller);
			Matrix4x4 localPropeller = traPropeller * rotPropeller;
			
			// Box
			Matrix4x4 traBox = TH3DMath.GetTranslation4x4(c_attachPos_Box);
			Matrix4x4 lcBox = traBox;             // Box1, Box2用のローカル行列
			
			Matrix4x4 rotBox0 = TH3DMath.GetRotation4x4(i_degBox0, Vector3.up);
			Matrix4x4 lcBox0 = traBox * rotBox0;  // Box0用のローカル行列
			
			// Pole
			i_degPole = (i_switch_Pole) ? i_degPole + 1 : i_degPole;
			Matrix4x4 localPole = TH3DMath.GetRotation4x4(i_degPole, Vector3.up);
			
			
			i_shmArm = (i_switch_Arm) ? i_shmArm + 1 : i_shmArm;
			for (int i = 0; i < 3; i++)
			{
				// Box
				Matrix4x4 localBox = (i == 0) ? lcBox0 : lcBox;
				
				// Arm
				float my = 3.0f * Mathf.Sin((i_shmArm + i * 60) * Mathf.Deg2Rad);    // -3.0 -- 3.0
				Matrix4x4 traArm = TH3DMath.GetTranslation4x4(0.0f, my, 0.0f);
				Matrix4x4 rotArm = TH3DMath.GetRotation4x4(i * 120, Vector3.up);
				Matrix4x4 localArm = traArm * rotArm;
				
				// world matrix : Propeller, Box, Arm
				Matrix4x4 worldPropeller = localPole * localArm * localBox * localPropeller;
				Matrix4x4 worldBox = localPole * localArm * localBox;
				Matrix4x4 worldArm = localPole * localArm;
				
				Propeller[i].SetMatrix(worldPropeller);
				Box[i].SetMatrix(worldBox);
				Arm[i].SetMatrix(worldArm);
			}
			
			// world matrix : Pole
			Matrix4x4 worldPole = localPole;
			Pole.SetMatrix(worldPole);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(5))  
			{
				Matrix4x4 lBox0 = TH3DMath.GetTranslation4x4(c_attachPos_Box) *
				TH3DMath.GetRotation4x4(i_degBox0, Vector3.up);
				
				float my = 3.0f * Mathf.Sin(i_shmArm * Mathf.Deg2Rad);
				Matrix4x4 lArm0 = TH3DMath.GetTranslation4x4(0.0f, my, 0.0f);
				
				Matrix4x4 lPole = TH3DMath.GetRotation4x4(i_degPole, Vector3.up);
				
				Matrix4x4 M = lPole * lArm0 * lBox0;
				YLine.SetMatrix(M);
			}
		}


		static Sec414 pr_instance = new Sec414(5);

		public static Sec414 PRGSI()
		{
			return pr_instance;
		}


		private Sec414(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Pole = CreateObject_Obj3D("MBlPole");
				Arm[0] = CreateObject_Obj3D("MBlArm");
				Box[0] = CreateObject_Obj3D("SkyBox");
				Propeller[0] = CreateObject_Obj3D("BrProp");
				YLine = CreateObject_Obj3D("YLine");
				
				
				for (int i = 1; i < Arm.Length; i++)
				{
					Arm[i] = CreateObject_InsObj3D(Arm[0].gameObject);
				}
				
				
				for (int i = 1; i < Box.Length; i++)
				{
					Box[i] = CreateObject_InsObj3D(Box[0].gameObject);
				}
				
				
				for (int i = 1; i < Propeller.Length; i++)
				{
					Propeller[i] = CreateObject_InsObj3D(Propeller[0].gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			i_degPropeller = 0;
			i_shmArm = 0;
			i_degPole = 0;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Propeller[0].gameObject);
				BSAuxFuncs.SetVisible(1, Box[0].gameObject);
				BSAuxFuncs.SetVisible(1, Arm[0].gameObject);
				BSAuxFuncs.SetVisible(1, Pole.gameObject);
				
				Propeller[0].SetIdentity();
				Box[0].SetIdentity();
				Arm[0].SetIdentity();
				Pole.SetIdentity();
			}
			else if (MatchCase(3, 4, 5)) 
			{
				
				for (int i = 0; i < 3; i++)
				{
					BSAuxFuncs.SetVisible(1, Propeller[i].gameObject);
					BSAuxFuncs.SetVisible(1, Box[i].gameObject);
					BSAuxFuncs.SetVisible(1, Arm[i].gameObject);
					
					Propeller[i].SetIdentity();
					Box[i].SetIdentity();
					Arm[i].SetIdentity();
				}
				
				BSAuxFuncs.SetVisible(1, Pole.gameObject);
				Pole.SetIdentity();
				
				if (MatchCase(5))	
				{
					
					i_switch_Pole = false;
					i_switch_Arm = true;
					i_degBox0 = 0;
					
					BSAuxFuncs.SetVisible(1, YLine.gameObject);
					YLine.SetIdentity();
				}
			}
			
			CamPos = new Vector3(12.0253f, 20.00275f, -13.07123f);
			TrgPos = new Vector3(-0.3658276f, 7.516417f, 0.8944302f);
			uJ = new Vector3(-0.3822502f, 0.8311056f, 0.403916f);
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}

	}

}
