//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec415
{
	public class Sec415 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D[] Face;
		THObject3D[] PrismSide;
		THObject3D PrismBottom;
		THObject3D PrismTop;
		THObject3D[] CubeSide;
		THObject3D CubeBottom;
		THObject3D[] CylinderSide;
		THObject3D CylinderBottom;
		THObject3D CylinderTop;


		// Instance Variables
		float i_degPrimarySide = 0.0f;
		float i_degSide = 0.0f;
		float i_degTop = 0.0f;
		
		int i_motionCounter;
		
		readonly float c_CylinderSideW = 0.09801714f;




		void Code1()
		{
			Matrix4x4[] localFace = new Matrix4x4[5];
			localFace[4] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.2f);
			localFace[3] = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[2] = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[1] = Matrix4x4.identity;
			localFace[0] = Matrix4x4.identity;
			
			for (int i = 4; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}


		void Code2()
		{
			Matrix4x4[] localFace = new Matrix4x4[5];
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(120.0f, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = Matrix4x4.identity;
			
			Matrix4x4 worldFace3 = localFace[1] * localFace[2] * localFace[3];
			Matrix4x4 worldFace2 = localFace[1] * localFace[2];
			Matrix4x4 worldFace1 = localFace[1];
			Face[3].SetMatrix(worldFace3);
			Face[2].SetMatrix(worldFace2);
			Face[1].SetMatrix(worldFace1);
		}


		void Code3()
		{
			Matrix4x4[] localFace = new Matrix4x4[5];
			localFace[4] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.2f) *
			TH3DMath.GetRotation4x4(-90.0f, Vector3.right);
			
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(120.0f, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = TH3DMath.GetRotation4x4(-90.0f, Vector3.right);
			
			localFace[0] = Matrix4x4.identity;
			
			for (int i = 4; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_degPrimarySide = 0.0f;
				i_degSide = 0.0f;
				i_degTop = 0.0f;
				i_motionCounter = 0;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.J))
			{
				i_motionCounter++;
				if (i_motionCounter > 180) { i_motionCounter = 180; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_motionCounter--;
				if (i_motionCounter < 0) { i_motionCounter = 0; }
			}
			
			
			float t = i_motionCounter / 180.0f;
			i_degPrimarySide = -90.0f * t;
			i_degSide = 120.0f * t;
			i_degTop = -90.0f * t;
			
			Matrix4x4[] localFace = new Matrix4x4[5];
			localFace[4] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.2f) *
			TH3DMath.GetRotation4x4(i_degTop, Vector3.right);
			
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(i_degSide, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = TH3DMath.GetRotation4x4(i_degPrimarySide, Vector3.right);
			
			localFace[0] = Matrix4x4.identity;
			
			
			for (int i = 4; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				i_degPrimarySide = 0.0f;
				i_degSide = 0.0f;
				i_degTop = 0.0f;
				i_motionCounter = 0;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.J))
			{
				i_motionCounter++;
				if (i_motionCounter > 240) { i_motionCounter = 240; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_motionCounter--;
				if (i_motionCounter < 0) { i_motionCounter = 0; }
			}
			
			
			if (0 <= i_motionCounter && i_motionCounter <= 60)
			{
				float t = i_motionCounter / 60.0f;
				i_degPrimarySide = -90.0f * t;
			}
			if (60 <= i_motionCounter && i_motionCounter <= 180)
			{
				float t = (i_motionCounter - 60) / 120.0f;
				i_degSide = 120.0f * t;
			}
			if (180 <= i_motionCounter && i_motionCounter <= 240)
			{
				float t = (i_motionCounter - 180) / 60.0f;
				i_degTop = -90.0f * t;
			}
			
			
			Matrix4x4[] localFace = new Matrix4x4[5];
			localFace[4] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.2f) *
			TH3DMath.GetRotation4x4(i_degTop, Vector3.right);
			
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(i_degSide, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = TH3DMath.GetRotation4x4(i_degPrimarySide, Vector3.right);
			
			localFace[0] = Matrix4x4.identity;
			
			
			for (int i = 4; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}


		void Code6()
		{
			Matrix4x4[] localFace = new Matrix4x4[6];
			localFace[5] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.0f) *     
			TH3DMath.GetRotation4x4(-90.0f, Vector3.right);
			
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(90.0f, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[4] = traSide * rotSide;
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = TH3DMath.GetRotation4x4(-90.0f, Vector3.right);
			
			localFace[0] = Matrix4x4.identity;      
			
			for (int i = 5; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}


		void Code7()
		{
			if (!i_INITIALIZED)
			{
				i_degPrimarySide = 0.0f;
				i_degSide = 0.0f;
				i_degTop = 0.0f;
				i_motionCounter = 0;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.J))
			{
				i_motionCounter++;
				if (i_motionCounter > 240) { i_motionCounter = 240; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_motionCounter--;
				if (i_motionCounter < 0) { i_motionCounter = 0; }
			}
			
			
			if (0 <= i_motionCounter && i_motionCounter <= 60)
			{
				float t = i_motionCounter / 60.0f;
				i_degPrimarySide = -90.0f * t;
			}
			if (60 <= i_motionCounter && i_motionCounter <= 180)
			{
				float t = (i_motionCounter - 60) / 120.0f;
				i_degSide = 90.0f * t;
			}
			if (180 <= i_motionCounter && i_motionCounter <= 240)
			{
				float t = (i_motionCounter - 180) / 60.0f;
				i_degTop = -90.0f * t;
			}
			
			Matrix4x4[] localFace = new Matrix4x4[6];
			localFace[5] = TH3DMath.GetTranslation4x4(0.0f, 0.0f, 1.0f) *     
			TH3DMath.GetRotation4x4(i_degTop, Vector3.right);
			
			Matrix4x4 rotSide = TH3DMath.GetRotation4x4(i_degSide, Vector3.forward);
			Matrix4x4 traSide = TH3DMath.GetTranslation4x4(1.0f, 0.0f, 0.0f);
			localFace[4] = traSide * rotSide;
			localFace[3] = traSide * rotSide;
			localFace[2] = traSide * rotSide;
			localFace[1] = TH3DMath.GetRotation4x4(i_degPrimarySide, Vector3.right);
			
			localFace[0] = Matrix4x4.identity;      
			
			for (int i = 5; i >= 0; i--)
			{
				Matrix4x4 worldM = localFace[0];
				for (int j = 1; j <= i; j++)
				{
					worldM *= localFace[j];
				}
				
				Face[i].SetMatrix(worldM);
			}
		}



		void Code8()
		{
			if (!i_INITIALIZED)
			{
				
				i_INITIALIZED = true;
			}
			
			
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

				case 8 :	Code8();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec415 pr_instance = new Sec415(8);

		public static Sec415 PRGSI()
		{
			return pr_instance;
		}


		private Sec415(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				PrismBottom = CreateObject_Obj3D("PrismBottom");
				PrismTop = CreateObject_Obj3D("PrismTop");
				CubeBottom = CreateObject_Obj3D("CubeBottom");
				CylinderBottom = CreateObject_Obj3D("CylinderBottom");
				CylinderTop = CreateObject_Obj3D("CylinderTop");
				
				var side_src = CreateObject_Obj3D("PrismSide");
				PrismSide = new THObject3D[3];
				for (int i = 0; i < PrismSide.Length; i++)
				{
					PrismSide[i] = CreateObject_InsObj3D(side_src.gameObject);
				}
				
				side_src = CreateObject_Obj3D("CubeSide");
				CubeSide = new THObject3D[5];
				for (int i = 0; i < CubeSide.Length; i++)
				{
					CubeSide[i] = CreateObject_InsObj3D(side_src.gameObject);
				}
				
				side_src = CreateObject_Obj3D("CylinderSide");
				CylinderSide = new THObject3D[32];
				for (int i = 0; i < CylinderSide.Length; i++)
				{
					CylinderSide[i] = CreateObject_InsObj3D(side_src.gameObject);
				}

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			CrdSys.SetLocalPosition(0.0f, -0.005f, 0.0f);	
			
			
			if (MatchCase(1, 2, 3, 4, 5))
			{
				int vis = 1;
				if (MatchCase(2)) { vis = 0; }
				
				BSAuxFuncs.SetVisible(vis, PrismBottom.gameObject);
				BSAuxFuncs.SetVisible(vis, PrismTop.gameObject);
				
				for (int i = 0; i < 3; i++)
				{
					BSAuxFuncs.SetVisible(1, PrismSide[i].gameObject);
				}
				
				Face = new THObject3D[5];
				Face[0] = PrismBottom;
				Face[1] = PrismSide[0];
				Face[2] = PrismSide[1];
				Face[3] = PrismSide[2];
				Face[4] = PrismTop;
				
				CamPos = new Vector3(3.149546f, 2.770397f, -2.378175f);
				TrgPos = new Vector3(1.918452f, 0.9976878f, -0.5986918f);
				uJ = new Vector3(-0.3113635f, 0.7722035f, 0.5538542f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(6, 7))
			{
				BSAuxFuncs.SetVisible(1, CubeBottom.gameObject);
				CubeBottom.SetIdentity();
				
				for (int i = 0; i < CubeSide.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, CubeSide[i].gameObject);
					CubeSide[i].SetIdentity();
				}
				
				Face = new THObject3D[6];
				Face[0] = CubeBottom;
				Face[1] = CubeSide[0];
				Face[2] = CubeSide[1];
				Face[3] = CubeSide[2];
				Face[4] = CubeSide[3];
				Face[5] = CubeSide[4];  
				
				CamPos = new Vector3(4.030374f, 3.147966f, -3.108043f);
				TrgPos = new Vector3(2.186804f, 0.5344113f, -0.2183516f);
				uJ = new Vector3(-0.3673171f, 0.7942415f, 0.4840027f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(8))
			{
				BSAuxFuncs.SetVisible(1, CylinderBottom.gameObject);
				BSAuxFuncs.SetVisible(1, CylinderTop.gameObject);
				
				for (int i = 0; i < CylinderSide.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, CylinderSide[i].gameObject);
				}
				
				Face = new THObject3D[34];
				Face[0] = CylinderBottom;
				for (int i = 0; i < CylinderSide.Length; i++)
				{
					Face[i + 1] = CylinderSide[i];
				}
				Face[33] = CylinderTop;
				
				CamPos = new Vector3(3.475722f, 3.80267f, -3.465006f);
				TrgPos = new Vector3(1.453926f, 1.306283f, -0.5426088f);
				uJ = new Vector3(-0.4093182f, 0.8139936f, 0.4121566f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
