//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec416
{
	public class Sec416 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Plinth;
		THObject3D Platter;
		THObject3D Record;
		THObject3D Tonearm;
		THObject3D Adjuster;
		THObject3D Stand;
		
		GameObject DLight;


		// Instance Variables
		float i_degPlatter;
		float i_degTonearm;
		float i_degAdjuster;
		bool i_PLAY;
		float i_degRot;     
		
		int i_motionCounter;
		
		readonly Vector3 c_attachPos_Platter = new Vector3(-0.34400f, 1.12536f, 0.0f);
		readonly Vector3 c_attachPos_Record = new Vector3(0.0f, 0.31264f, 0.0f);
		readonly Vector3 c_attachPos_Adjuster = new Vector3(2.95976f, 1.10992f, 2.58188f);
		readonly Vector3 c_attachPos_Tonearm = new Vector3(0.0f, 0.44700f, 0.0f);




		void Code1()
		{
			Matrix4x4 localTonearm = TH3DMath.GetTranslation4x4(c_attachPos_Tonearm);
			Matrix4x4 localAdjuster = TH3DMath.GetTranslation4x4(c_attachPos_Adjuster);
			Matrix4x4 localRecord = TH3DMath.GetTranslation4x4(c_attachPos_Record);
			Matrix4x4 localPlatter = TH3DMath.GetTranslation4x4(c_attachPos_Platter);
			Matrix4x4 localPlinth = TH3DMath.GetTranslation4x4(32.0f, 16.0f, -8.0f);
			
			Matrix4x4 worldTonearm = localPlinth * localAdjuster * localTonearm;
			Matrix4x4 worldAdjuster = localPlinth * localAdjuster;
			Matrix4x4 worldRecord = localPlinth * localPlatter * localRecord;
			Matrix4x4 worldPlatter = localPlinth * localPlatter;
			Matrix4x4 worldPlinth = localPlinth;
			
			Tonearm.SetMatrix(worldTonearm);
			Adjuster.SetMatrix(worldAdjuster);
			Record.SetMatrix(worldRecord);
			Platter.SetMatrix(worldPlatter);
			Plinth.SetMatrix(worldPlinth);
		}


		void Code2()
		{
			i_degPlatter += 3.0f;
			Matrix4x4 traPlatter = TH3DMath.GetTranslation4x4(c_attachPos_Platter);
			Matrix4x4 rotPlatter = TH3DMath.GetRotation4x4(i_degPlatter, Vector3.up);
			Matrix4x4 localPlatter = traPlatter * rotPlatter;
			
			Matrix4x4 localRecord = TH3DMath.GetTranslation4x4(c_attachPos_Record);
			
			Matrix4x4 localPlinth = TH3DMath.GetTranslation4x4(32.0f, 16.0f, -8.0f);
			
			Matrix4x4 worldRecord = localPlinth * localPlatter * localRecord;
			Matrix4x4 worldPlatter = localPlinth * localPlatter;
			Matrix4x4 worldPlinth = localPlinth;
			
			Record.SetMatrix(worldRecord);
			Platter.SetMatrix(worldPlatter);
			Plinth.SetMatrix(worldPlinth);
		}


		void Code3()
		{
			Matrix4x4 rotTonearm = TH3DMath.GetRotation4x4(2, Vector3.right);
			Matrix4x4 traTonearm = TH3DMath.GetTranslation4x4(c_attachPos_Tonearm);
			Matrix4x4 localTonearm = traTonearm * rotTonearm;
			
			Matrix4x4 rotAdjuster = TH3DMath.GetRotation4x4(20, Vector3.up);
			Matrix4x4 traAdjuster = TH3DMath.GetTranslation4x4(c_attachPos_Adjuster);
			Matrix4x4 localAdjuster = traAdjuster * rotAdjuster;
			
			Matrix4x4 localRecord = TH3DMath.GetTranslation4x4(c_attachPos_Record);
			Matrix4x4 localPlatter = TH3DMath.GetTranslation4x4(c_attachPos_Platter);
			Matrix4x4 localPlinth = TH3DMath.GetTranslation4x4(32.0f, 16.0f, -8.0f);
			
			Matrix4x4 worldTonearm = localPlinth * localAdjuster * localTonearm;
			Matrix4x4 worldAdjuster = localPlinth * localAdjuster;
			Matrix4x4 worldRecord = localPlinth * localPlatter * localRecord;
			Matrix4x4 worldPlatter = localPlinth * localPlatter;
			Matrix4x4 worldPlinth = localPlinth;
			
			Tonearm.SetMatrix(worldTonearm);
			Adjuster.SetMatrix(worldAdjuster);
			Record.SetMatrix(worldRecord);
			Platter.SetMatrix(worldPlatter);
			Plinth.SetMatrix(worldPlinth);
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_motionCounter = 1000;
				i_degTonearm = 0.0f;
				i_degAdjuster = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S) && i_motionCounter > 240)
			{
				i_motionCounter = 0;
				i_degTonearm = 0.0f;
				i_degAdjuster = 0.0f;
			}
			
			i_motionCounter++;
			
			if (i_motionCounter <= 60)
			{
				float t = i_motionCounter / 60.0f;
				i_degTonearm = 8.0f * t;
			}
			else if (i_motionCounter <= 180)
			{
				float t = (i_motionCounter - 60) / 120.0f;
				i_degAdjuster = 20.0f * t;
			}
			else if (i_motionCounter <= 240)
			{
				float t = (i_motionCounter - 180) / 60.0f;
				i_degTonearm = 8.0f - 6.0f * t;
			}
			
			Matrix4x4 rotTonearm = TH3DMath.GetRotation4x4(i_degTonearm, Vector3.right);
			Matrix4x4 traTonearm = TH3DMath.GetTranslation4x4(c_attachPos_Tonearm);
			Matrix4x4 localTonearm = traTonearm * rotTonearm;
			
			Matrix4x4 rotAdjuster = TH3DMath.GetRotation4x4(i_degAdjuster, Vector3.up);
			Matrix4x4 traAdjuster = TH3DMath.GetTranslation4x4(c_attachPos_Adjuster);
			Matrix4x4 localAdjuster = traAdjuster * rotAdjuster;
			
			Matrix4x4 localPlatter = TH3DMath.GetTranslation4x4(c_attachPos_Platter);
			Matrix4x4 localRecord = TH3DMath.GetTranslation4x4(c_attachPos_Record);
			Matrix4x4 localPlinth = TH3DMath.GetTranslation4x4(32.0f, 16.0f, -8.0f);
			
			Matrix4x4 worldTonearm = localPlinth * localAdjuster * localTonearm;
			Matrix4x4 worldAdjuster = localPlinth * localAdjuster;
			Matrix4x4 worldRecord = localPlinth * localPlatter * localRecord;
			Matrix4x4 worldPlatter = localPlinth * localPlatter;
			Matrix4x4 worldPlinth = localPlinth;
			
			Tonearm.SetMatrix(worldTonearm);
			Adjuster.SetMatrix(worldAdjuster);
			Record.SetMatrix(worldRecord);
			Platter.SetMatrix(worldPlatter);
			Plinth.SetMatrix(worldPlinth);
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				i_motionCounter = 1000;
				i_degTonearm = 0.0f;
				i_degAdjuster = 0.0f;
				i_degPlatter = 0.0f;
				i_degRot = 0.0f;
				i_PLAY = false;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.S) && i_motionCounter > 240)
			{
				i_PLAY = !i_PLAY;
				i_motionCounter = 0;
				i_degRot = i_PLAY ? 0.0f : 3.0f;
			}
			
			i_motionCounter++;
			
			if (i_motionCounter <= 60)
			{
				float t = i_motionCounter / 60.0f;
				i_degTonearm = (i_PLAY) ? 8.0f * t : 2.0f + 6.0f * t;
			}
			else if (i_motionCounter <= 180)
			{
				float t = (i_motionCounter - 60) / 120.0f;
				i_degAdjuster = (i_PLAY) ? 20.0f * t : 20.0f - 20.0f * t;
			}
			else if (i_motionCounter <= 240)
			{
				float t = (i_motionCounter - 180) / 60.0f;
				i_degTonearm = (i_PLAY) ? 8.0f - 6.0f * t : 8.0f - 8.0f * t;
			}
			
			Matrix4x4 rotTonearm = TH3DMath.GetRotation4x4(i_degTonearm, Vector3.right);
			Matrix4x4 traTonearm = TH3DMath.GetTranslation4x4(c_attachPos_Tonearm);
			Matrix4x4 localTonearm = traTonearm * rotTonearm;
			
			Matrix4x4 rotAdjuster = TH3DMath.GetRotation4x4(i_degAdjuster, Vector3.up);
			Matrix4x4 traAdjuster = TH3DMath.GetTranslation4x4(c_attachPos_Adjuster);
			Matrix4x4 localAdjuster = traAdjuster * rotAdjuster;
			
			i_degPlatter += i_degRot;
			Matrix4x4 rotPlatter = TH3DMath.GetRotation4x4(i_degPlatter, Vector3.up);
			Matrix4x4 traPlatter = TH3DMath.GetTranslation4x4(c_attachPos_Platter);
			Matrix4x4 localPlatter = traPlatter * rotPlatter;
			if (i_PLAY && i_degRot < 2.9999f)
			{
				i_degRot += 0.02f;
				if (i_degRot > 3.0f) { i_degRot = 3.0f; }
			}
			else if (!i_PLAY && i_degRot > 0.0001f)
			{
				i_degRot -= 0.02f;
				if (i_degRot < 0.0f) { i_degRot = 0.0f; }
			}
			
			Matrix4x4 localRecord = TH3DMath.GetTranslation4x4(c_attachPos_Record);
			Matrix4x4 localPlinth = TH3DMath.GetTranslation4x4(32.0f, 16.0f, -8.0f);
			
			Matrix4x4 worldTonearm = localPlinth * localAdjuster * localTonearm;
			Matrix4x4 worldAdjuster = localPlinth * localAdjuster;
			Matrix4x4 worldRecord = localPlinth * localPlatter * localRecord;
			Matrix4x4 worldPlatter = localPlinth * localPlatter;
			Matrix4x4 worldPlinth = localPlinth;
			
			Tonearm.SetMatrix(worldTonearm);
			Adjuster.SetMatrix(worldAdjuster);
			Record.SetMatrix(worldRecord);
			Platter.SetMatrix(worldPlatter);
			Plinth.SetMatrix(worldPlinth);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec416 pr_instance = new Sec416(5);

		public static Sec416 PRGSI()
		{
			return pr_instance;
		}


		private Sec416(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Plinth = CreateObject_Obj3D("Plinth", true);
				Platter = CreateObject_Obj3D("Platter", true);
				Record = CreateObject_Obj3D("Record", true);
				Tonearm = CreateObject_Obj3D("Tonearm", true);
				Adjuster = CreateObject_Obj3D("Adjuster", true);
				Stand = CreateObject_Obj3D("Stand");
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			CrdSys.transform.localScale = new Vector3(8.0f, 1.0f, 8.0f);
			
			if (MatchCase(1, 2, 3, 4, 5))
			{
				int vis = MatchCase(2) ? 0 : 1;
				BSAuxFuncs.SetVisible(1, Plinth.gameObject);
				BSAuxFuncs.SetVisible(1, Platter.gameObject);
				BSAuxFuncs.SetVisible(1, Record.gameObject);
				BSAuxFuncs.SetVisible(vis, Tonearm.gameObject);
				BSAuxFuncs.SetVisible(vis, Adjuster.gameObject);
				BSAuxFuncs.SetVisible(1, Stand.gameObject);
				Plinth.SetMatrix(Matrix4x4.identity);
				Platter.SetMatrix(Matrix4x4.identity);
				Record.SetMatrix(Matrix4x4.identity);
				Tonearm.SetMatrix(Matrix4x4.identity);
				Adjuster.SetMatrix(Matrix4x4.identity);
				
				var posStand = new Vector3(32, 0, -8);
				Stand.SetLocalPosition(posStand);
				
				i_degAdjuster = 0.0f;
				i_degTonearm = 0.0f;
				i_degPlatter = 0.0f;
				i_PLAY = false;
				
				CamPos = new Vector3(35.91798f, 21.25527f, -11.91495f);
				TrgPos = new Vector3(35.41713f, 20.43271f, -11.24197f);
				uJ = new Vector3(-0.3801722f, 0.7132513f, 0.5888478f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(66, -30, 0);
				}
			}
		}

	}

}
