//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGAuxFuncs04;



namespace OQJ_Sec418
{
	public class Sec418 : BSSuperExe
	{
		// Objects
		
		THObject3D CSvL;	
		
		THObject3D[] UpperLeg = new THObject3D[3];	
		THObject3D[] LowerLeg = new THObject3D[3];
		THObject3D[] DoorA = new THObject3D[3];
		THObject3D[] DoorB = new THObject3D[3];	
		
		THObject3D Runway;	
		
		THObject3D BackPlane;	
		GameObject SubCamera;
		GameObject MainCamera;
		
		GameObject DLight;
		GameObject SubLight;
		
		Camera pr_COMP_SubCamera;
		MeshRenderer pr_MRE_BackPlane;


		// Instance Variables
		float i_degUpperLeg;
		float i_degLowerLeg;
		float i_degDoorA;
		float i_degDoorB;
		
		
		readonly Vector3 pr_AirplaneStartPosition = new Vector3(1503.0f, 0.60f, -480.0f);
		
		readonly Vector3 pr_difV0 = new Vector3(0.499023f, 3.286449f, -3.976807f);
		readonly Quaternion pr_Q0 = new Quaternion(0.3614976f, 0.0f, 0.0f, 0.932373f);
		
		readonly Vector3 pr_difV1 = new Vector3(-6.198303f, 2.384536f, -4.451813f);
		readonly Quaternion pr_Q1 = new Quaternion(0.1684094f, 0.4050006f, -0.09423006f, 0.8937189f);
		
		readonly Vector3 pr_difV2 = new Vector3(-6.20f, 2.38f, -4.45f);		
		readonly Quaternion pr_Q2 = Quaternion.Euler(22.17f, 48.37f, -1.981f);
		
		readonly Vector3 pr_difV3 = new Vector3(2.59f, 1.81f, -5.43f);
		readonly Quaternion pr_Q3 = Quaternion.Euler(20.18f, 335.70f, 332.58f);
		
		readonly Vector3 pr_difV5 = new Vector3(-7.33f, 2.74f, -1.68f);
		readonly Quaternion pr_Q5 = Quaternion.Euler(22.08f, 98.66f, 2.12f);
		
		readonly Vector3 pr_difV6 = new Vector3(-0.90f, 3.38f, -5.37f);
		readonly Quaternion pr_Q6 = Quaternion.Euler(32.01f, 9.43f, 3.56f);
		
		readonly Vector3 pr_difV7 = new Vector3(-3.56f, 2.16f, -2.69f);
		readonly Quaternion pr_Q7 = Quaternion.Euler(25.99f, 44.57f, 2.76f);
		
		
		readonly int c_ASCENT = 1;	
		readonly int c_DESCENT = 2;	
		
		readonly Vector3[] c_attachPos_UpperLeg =
		{
			new Vector3(-1.036088f, 0.152180f, 0.0f),
			new Vector3(0.328351f,  0.150820f,  0.118694f),
			new Vector3(0.328351f,  0.150820f, -0.118694f)
		};
		
		readonly Vector3 c_attachPos_LowerLeg = new Vector3(-0.155461f, 0.0f, 0.0f);
		
		readonly Vector3[] c_attachPos_DoorA =
		{
			new Vector3(-1.2530000f, 0.0018442f, -0.0761302f),
			new Vector3(0.1114389f, 0.0004838f,  0.0425581f),
			new Vector3(0.1114389f, 0.0004838f, -0.1948225f)
		};
		
		readonly Vector3[] c_attachPos_DoorB =
		{
			new Vector3(-0.9329223f, 0.0018442f, -0.0761302f),
			new Vector3(0.4315166f, 0.0004838f, 0.0425596f),
			new Vector3(0.4315166f, 0.0004838f, -0.1948291f)
		};
		
		Vector3 pr_initDifVtr;
		Quaternion pr_initRot;
		
		Vector3 pr_initDist_IJK;





		void Code1()
		{
			if (!i_INITIALIZED)
			{
				// 0:開始地点    1:離陸前    2:着陸前
				SetStartPosition(0);

				i_INITIALIZED = true;
			}

			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}

			if (!i_MOVE) { return; }

			Matrix4x4 localAirplane = GetAirplaneMatrix();

			int state = GetAirplaneState();

			if (state == c_ASCENT)
			{

			}
			else if (state == c_DESCENT)
			{

			}


		}




///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		Matrix4x4 GetAirplaneMatrix()
		{
			return RGAuxFuncs04.GetAirplaneMatrix();
		}
		
		
		int GetAirplaneState()
		{
			int state = -1;
			
			if (1322 <= i_frameCount && i_frameCount < 2122)
			{
				state = c_ASCENT;
			}
			
			
			if (4727 <= i_frameCount && i_frameCount < 5527)
			{
				state = c_DESCENT;
			}
			
			return state;
		}
		
		
		void SetStartPosition(int nr)
		{
			pr_MRE_BackPlane.enabled = false;
			pr_COMP_SubCamera.enabled = false;
			
			if (nr == 0)
			{
				i_frameCount = 0;
				RGAuxFuncs04.Standby(UpperLeg, LowerLeg, DoorA, DoorB, pr_AirplaneStartPosition);
			}
			else if (nr == 1)
			{
				i_frameCount = 1099;
				RGAuxFuncs04.Standby(UpperLeg, LowerLeg, DoorA, DoorB, pr_AirplaneStartPosition, 1);
				
				MainCamera.transform.localPosition = new Vector3(659.6971f, 7.502478f, -484.4121f);
				MainCamera.transform.localRotation = new Quaternion(0.186547f, 0.373337f, -0.086863f, 0.904585f);
			}
			else if (nr == 2)
			{
				i_frameCount = 4499;
				RGAuxFuncs04.Standby(UpperLeg, LowerLeg, DoorA, DoorB, pr_AirplaneStartPosition, 2);
				
				MainCamera.transform.localPosition = new Vector3(144.7504f, 174.5947f, -485.37f);
				MainCamera.transform.localRotation = new Quaternion(0.277110f, 0.070440f, 0.007104f, 0.958227f);
			}
			
		}



		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			if (i_MOVE)
			{
				i_frameCount++;
				RGAuxFuncs04.AirplaneMotion(i_frameCount);
			}
		}


		void PostProcess()
		{
			if (i_MOVE)
			{
				Vector3 APos = RGAuxFuncs04.GetAirplanePosition();
				
				if (i_frameCount <= 1350)
				{
					float t = (float)i_frameCount / 1350.0f;
					float dians90 = (t * 90) * Mathf.Deg2Rad;
					float ts = Mathf.Pow(Mathf.Cos(dians90), 2.0f);
					float t2 = 1.0f - ts;
					Vector3 dif = (1.0f - t2) * pr_difV0 + t2 * pr_difV1;
					MainCamera.transform.localPosition = APos + dif;
					
					Quaternion q = Quaternion.Slerp(pr_Q0, pr_Q1, t2);
					MainCamera.transform.localRotation = q;
				}
				else if (i_frameCount <= 1960)
				{
					float t = (float)(i_frameCount - 1350.0f) / (1960.0f - 1350.0f);
					float dians90 = (t * 90) * Mathf.Deg2Rad;
					float t2 = 1.0f - Mathf.Cos(dians90);
					Vector3 dif = (1.0f - t2) * pr_difV2 + t2 * pr_difV3;
					MainCamera.transform.localPosition = APos + dif;
					
					Quaternion q = Quaternion.Slerp(pr_Q2, pr_Q3, t2);
					MainCamera.transform.localRotation = q;
				}
				else if (2438 <= i_frameCount && i_frameCount <= 2960)
				{
					float t = (float)(i_frameCount - 2438.0f) / (2960.0f - 2438.0f);
					float dians90 = (t * 90) * Mathf.Deg2Rad;
					float ts = Mathf.Pow(Mathf.Cos(dians90), 2.0f);
					float t2 = 1.0f - ts;
					Vector3 dif = (1.0f - t2) * pr_difV3 + t2 * pr_difV5;
					MainCamera.transform.localPosition = APos + dif;
					
					Quaternion q = Quaternion.Slerp(pr_Q3, pr_Q5, t2);
					MainCamera.transform.localRotation = q;
				}
				else if (3920 <= i_frameCount && i_frameCount <= 4206)
				{
					float t = (float)(i_frameCount - 3920.0f) / (4206.0f - 3920.0f);
					float dians90 = (t * 90) * Mathf.Deg2Rad;
					float ts = Mathf.Pow(Mathf.Cos(dians90), 2.0f);
					float t2 = 1.0f - ts;
					Vector3 dif = (1.0f - t2) * pr_difV5 + t2 * pr_difV6;
					MainCamera.transform.localPosition = APos + dif;
					
					Quaternion q = Quaternion.Slerp(pr_Q5, pr_Q6, t2);
					MainCamera.transform.localRotation = q;
				}
				else if (4501 <= i_frameCount && i_frameCount <= 5700)	
				{
					float t = (float)(i_frameCount - 4501.0f) / (5700.0f - 4501.0f);
					float dians90 = (t * 90) * Mathf.Deg2Rad;
					float t2 = Mathf.Sin(dians90); 
					Vector3 dif = (1.0f - t2) * pr_difV6 + t2 * pr_difV7;
					MainCamera.transform.localPosition = APos + dif;
					
					Quaternion q = Quaternion.Slerp(pr_Q6, pr_Q7, t2);
					MainCamera.transform.localRotation = q;
				}
				else
				{
					MainCamera.transform.localPosition += RGAuxFuncs04.GetAirplaneMotionVector();
				}
				
				
				if(i_frameCount == 1100)
				{
					pr_MRE_BackPlane.enabled = true;
					pr_COMP_SubCamera.enabled = true;
				}
				else if(i_frameCount == 2122)
				{
					pr_MRE_BackPlane.enabled = false;
					pr_COMP_SubCamera.enabled = false;
				}
				else if(i_frameCount == 4500)
				{
					pr_MRE_BackPlane.enabled = true;
					pr_COMP_SubCamera.enabled = true;
				}
				else if(i_frameCount == 5580)
				{
					pr_MRE_BackPlane.enabled = false;
					pr_COMP_SubCamera.enabled = false;
				}
				
				
				if(pr_COMP_SubCamera.enabled)
				{	
					
					Quaternion ARot = RGAuxFuncs04.GetAirplaneRotation();
					SubCamera.transform.localPosition = APos + ARot * pr_initDifVtr;
					SubCamera.transform.localRotation = (ARot * pr_initRot).normalized;
					
					
					Matrix4x4 wM = MainCamera.transform.localToWorldMatrix;
					Vector3 uI = wM.GetColumn(0).normalized;
					Vector3 uJ = wM.GetColumn(1).normalized;
					Vector3 uK = wM.GetColumn(2).normalized;
					BackPlane.transform.localPosition = MainCamera.transform.localPosition + 
					pr_initDist_IJK.x * uI + pr_initDist_IJK.y * uJ + pr_initDist_IJK.z * uK;
					
					BackPlane.transform.localRotation = MainCamera.transform.localRotation;
				}
				
				BSData.CameraController3D.UpdateCameraData();
			}
		}


		static Sec418 pr_instance = new Sec418(1);

		public static Sec418 PRGSI()
		{
			return pr_instance;
		}


		private Sec418(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				
				CSvL = CreateObject_Obj3D("CSvL");
				Runway = CreateObject_Obj3D("Runway2", true);
				
				UpperLeg[0] = CreateObject_Obj3D("UpperLeg", true);
				LowerLeg[0] = CreateObject_Obj3D("LowerLeg", true);
				DoorA[0] = CreateObject_Obj3D("DoorA");
				DoorB[0] = CreateObject_Obj3D("DoorB");
				for (int i = 1; i < 3; i++)
				{
					UpperLeg[i] = CreateObject_InsObj3D(UpperLeg[0].gameObject, true);
					LowerLeg[i] = CreateObject_InsObj3D(LowerLeg[0].gameObject, true);
					DoorA[i] = CreateObject_InsObj3D(DoorA[0].gameObject);
					DoorB[i] = CreateObject_InsObj3D(DoorB[0].gameObject);
				}
				
				RGAuxFuncs04.SetAirplane(CreateObject_Obj3D("Airplane", true));
				
				
				BackPlane = CreateObject_Obj3D("UnitRect");
				Shader unlit_color = Shader.Find("Unlit/Color");
				if (unlit_color == null) { unlit_color = Shader.Find("Standard"); };
				Material objMtrl = new Material(unlit_color);
				objMtrl.SetColor("_Color", new Color(0.0f, 0.447f, 0.545f));
				pr_MRE_BackPlane = BackPlane.gameObject.GetComponent<MeshRenderer>();
				pr_MRE_BackPlane.material = objMtrl;
				
				
				MainCamera = GameObject.Find("Main Camera");
				if(MainCamera == null)
				{
					MainCamera = GameObject.Find("MainCamera");
				}
				MainCamera.GetComponent<Camera>().nearClipPlane = 0.5f;
				MainCamera.GetComponent<Camera>().farClipPlane = 2500.0f;
				
				
				GameObject go = new GameObject();
				go.name = "Camera_2";
				pr_COMP_SubCamera = go.AddComponent<Camera>();
				pr_COMP_SubCamera.nearClipPlane = 0.01f;
				pr_COMP_SubCamera.farClipPlane = 50.0f;
				pr_COMP_SubCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_COMP_SubCamera.backgroundColor = BSData.BackgroundColor;
				pr_COMP_SubCamera.orthographic = false;  
				pr_COMP_SubCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_COMP_SubCamera.enabled = false;
				SubCamera = go;  
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();
				
				
				SubLight = new GameObject();
				Light LC = SubLight.AddComponent<Light>();
				LC.color = new Color(0.8301f , 0.7955f, 0.7009f);
				LC.type = LightType.Directional;
				LC.enabled = true;
				SubLight.transform.rotation = Quaternion.Euler(-50, 0, 0);

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			
			
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, CSvL.gameObject);
				CSvL.SetMatrix(Matrix4x4.identity);
				
				BSAuxFuncs.SetVisible(1, Runway.gameObject);
				Runway.SetLocalPosition(0.0f, 0.355f, -480.0f);
				
				for (int i = 0; i < 3; i++)
				{
					BSAuxFuncs.SetVisible(1, UpperLeg[i].gameObject);
					BSAuxFuncs.SetVisible(1, LowerLeg[i].gameObject);
					BSAuxFuncs.SetVisible(1, DoorA[i].gameObject);
					BSAuxFuncs.SetVisible(1, DoorB[i].gameObject);
					UpperLeg[i].SetMatrix(Matrix4x4.identity);
					LowerLeg[i].SetMatrix(Matrix4x4.identity);
					DoorA[i].SetMatrix(Matrix4x4.identity);
					DoorB[i].SetMatrix(Matrix4x4.identity);
				}
				
				
				
				CamPos = new Vector3(1503.499f, 3.886449f, -483.9768f);
				TrgPos = new Vector3(1503.499f, 1.864146f, -481.7609f);
				uJ = new Vector3(0f, 0.7386382f, 0.6741021f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				
				
				if (SubCamera != null)
				{
					pr_COMP_SubCamera.enabled = false;
					
					CamPos = new Vector3(1503.561f, 0.2629415f, -480.8343f);
					TrgPos = new Vector3(1502.44f, 0.9852138f, -479.233f);
					uJ = new Vector3(0.1985241f, 0.9380078f, -0.2841293f);
					
					Vector3 dir = (TrgPos - CamPos).normalized;
					Quaternion rot = TH3DMath.CalcRotation_VW1toVW2(Vector3.forward, Vector3.up, dir, uJ);
					
					SubCamera.transform.localPosition = CamPos;
					SubCamera.transform.localRotation = rot;
					
					pr_initDifVtr = CamPos - pr_AirplaneStartPosition;
					pr_initRot = rot;
					
					
					Rect rectBounds = new Rect(0.66f, 0.66f, 0.32f, 0.32f);
					pr_COMP_SubCamera.depth = 2;
					pr_COMP_SubCamera.rect = rectBounds;
					
					float[] rbds2 = { rectBounds.x+rectBounds.width/2.0f,
					rectBounds.y+rectBounds.height/2.0f, rectBounds.width * 1.04f, rectBounds.height * 1.057f   };
					
					float[] iplaneData = BSAuxFuncs.CalcImagePlaneData(); 
					
					
					Vector3[] initCamData = { Vector3.zero, Vector3.right, Vector3.up, Vector3.forward };
					object[] trs = BSAuxFuncs.CalcUIRectTransformN_LowerLeft(rbds2, initCamData, iplaneData);
					pr_initDist_IJK = (Vector3)trs[0];  
					
					trs = BSAuxFuncs.CalcUIRectTransformN_LowerLeft(rbds2, iplaneData);
					Vector3 pos = (Vector3)trs[0];
					rot = (Quaternion)trs[1];
					Vector3 scale = (Vector3)trs[2];
					BackPlane.transform.localPosition = pos;
					BackPlane.transform.localRotation = rot;
					BackPlane.transform.localScale = scale;
					
					
					int LAYER_UI = 5;
					BackPlane.gameObject.layer = LAYER_UI;              
					int layerMask = 1 << LAYER_UI;
					pr_COMP_SubCamera.cullingMask = ~layerMask;  
					
				}	
				
			}
		}

	}

}
