//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGAuxFuncs04;
using OQJ_KeyframeData;



namespace OQJ_Sec419
{
	public class Sec419 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Body;	
		THObject3D Turret;	
		THObject3D Barrel;	
		
		THObject3D[] TrackBitR;	
		THObject3D[] TrackBitL;	
		THObject3D[] WheelR;	
		THObject3D[] WheelL;	
		
		GameObject MainCamera;
		GameObject DLight;
		
		Material Mtrl_opqBody;
		Material Mtrl_trpBody;


		// Instance Variables
		KeyframeData[] i_keyframes;
		float i_totalFrames;	
		float i_speedBody;	
		
		float i_degBody = 0.0f;	
		float i_degTurret = 0.0f;	
		float i_degBarrel = 0.0f;	
		
		float i_mileage = 0.0f;
		
		readonly float c_WheelLength = 0.42f * 2.0f * Mathf.PI;	
		readonly float c_BitW = 0.234108f;
		readonly float c_SingleBitFrames = 0.234108f * 100.0f;
		readonly float c_TrackLength = 0.234108f * 58;		
		readonly float c_TiRectH = 0.06f;
		readonly float c_TrackRad = 0.588471f;		
		
		readonly Vector3 c_attachPos_Turret = new Vector3(0.0f, 1.30125f, 0.0f);
		
		readonly Vector3 c_attachPos_Barrel = new Vector3(1.45353f, 0.20131f, 0.0f);	
		
		readonly Vector3 c_TrackCenterPosR = new Vector3(0.0f, 0.61847f, -1.5f);
		readonly Vector3 c_TrackCenterPosL = new Vector3(0.0f, 0.61847f,  1.5f);
		
		readonly Vector3[] c_attachPos_WheelR =
		{
		new Vector3(2.589446f, 0.587908f, -1.502279f),
		new Vector3(1.710000f, 0.482200f, -1.502279f),
		new Vector3(0.855000f, 0.482200f, -1.502279f),
		new Vector3(0.000000f, 0.482200f, -1.502279f),
		new Vector3(-0.855000f, 0.482200f, -1.502279f),
		new Vector3(-1.710000f, 0.482200f, -1.502279f),
		new Vector3(-2.589446f, 0.587908f, -1.502279f)
		};
		
		readonly Vector3[] c_attachPos_WheelL =
		{
		new Vector3(2.589446f, 0.587908f,  1.502279f),
		new Vector3(1.710000f, 0.482200f,  1.502279f),
		new Vector3(0.855000f, 0.482200f,  1.502279f),
		new Vector3(0.000000f, 0.482200f,  1.502279f),
		new Vector3(-0.855000f, 0.482200f,  1.502279f),
		new Vector3(-1.710000f, 0.482200f,  1.502279f),
		new Vector3(-2.589446f, 0.587908f,  1.502279f)
		};
		
		readonly int c_LINE = 21;
		readonly int c_CIRCLE = 22;
		readonly int c_SPIRAL = 23;
		readonly int c_END = 24;
		
		Vector3 pr_preTankPosition;	




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				i_MOVE = false;
				i_degBody = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody -= 0.25f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody += 0.25f;
			}
			
			Matrix4x4 localBarrel = TH3DMath.GetTranslation4x4(c_attachPos_Barrel);
			Matrix4x4 localTurret = TH3DMath.GetTranslation4x4(c_attachPos_Turret);
			
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 forwardDir = rotBody * Vector3.right;
			float speed = i_MOVE ? 0.01f : 0f;
			Vector3 pos = Body.GetWorldPosition() + speed * forwardDir;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(pos);
			Matrix4x4 localBody = traBody * rotBody;
			
			Matrix4x4 worldBarrel = localBody * localTurret * localBarrel;
			Matrix4x4 worldTurret = localBody * localTurret;
			Matrix4x4 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Turret.SetMatrix(worldTurret);
			Body.SetMatrix(worldBody);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				i_MOVE = false;
				i_degBody = 0.0f;
				i_degTurret = 0.0f;
				i_degBarrel = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody -= 0.25f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody += 0.25f;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degTurret -= 1.0f;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degTurret += 1.0f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				i_degBarrel -= 0.2f;
				if (i_degBarrel < 0.0f) { i_degBarrel = 0.0f; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_degBarrel += 0.2f;
				if (i_degBarrel > 20.0f) { i_degBarrel = 20.0f; }
			}
			
			
			Matrix4x4 traBarrel = TH3DMath.GetTranslation4x4(c_attachPos_Barrel);
			Matrix4x4 rotBarrel = TH3DMath.GetRotation4x4(i_degBarrel, Vector3.forward);
			Matrix4x4 localBarrel = traBarrel * rotBarrel;
			
			Matrix4x4 traTurret = TH3DMath.GetTranslation4x4(c_attachPos_Turret);
			Matrix4x4 rotTurret = TH3DMath.GetRotation4x4(i_degTurret, Vector3.up);
			Matrix4x4 localTurret = traTurret * rotTurret;
			
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 forwardDir = rotBody * Vector3.right;
			float speed = i_MOVE ? 0.01f : 0f;
			Vector3 pos = Body.GetWorldPosition() + speed * forwardDir;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(pos);
			Matrix4x4 localBody = traBody * rotBody;
			
			Matrix4x4 worldBarrel = localBody * localTurret * localBarrel;
			Matrix4x4 worldTurret = localBody * localTurret;
			Matrix4x4 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Turret.SetMatrix(worldTurret);
			Body.SetMatrix(worldBody);
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				i_mileage = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			
			i_mileage += 0.02f;
			
			Matrix4x4 localBody = Matrix4x4.identity;
			
			Matrix4x4[] mtrcs = GetTrackBitMatrices(i_mileage);
			Matrix4x4 moveRight = TH3DMath.GetTranslation4x4(c_TrackCenterPosR);
			Matrix4x4 moveLeft = TH3DMath.GetTranslation4x4(c_TrackCenterPosL);
			for (int i = 0; i < TrackBitR.Length; i++)
			{
				Matrix4x4 localTrackBitR = moveRight * mtrcs[i];
				Matrix4x4 worldTrackBitR = localBody * localTrackBitR;
				TrackBitR[i].SetMatrix(worldTrackBitR);
				
				Matrix4x4 localTrackBitL = moveLeft * mtrcs[i];
				Matrix4x4 worldTrackBitL = localBody * localTrackBitL;
				TrackBitL[i].SetMatrix(worldTrackBitL);
			}
			
			Matrix4x4 worldBody = localBody;
			Body.SetMatrix(worldBody);
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_mileage = 0.0f;
				
				i_INITIALIZED = true;
			}
			
			
			i_mileage += 0.01f;
			
			Matrix4x4 localBody = Matrix4x4.identity;
			
			// Wheels
			float degWheel = (i_mileage / c_WheelLength) * 360.0f;
			
			Matrix4x4 rotWheel = TH3DMath.GetRotation4x4(-degWheel, Vector3.forward);
			for (int i = 0; i < WheelR.Length; i++)
			{
				Matrix4x4 traR = TH3DMath.GetTranslation4x4(c_attachPos_WheelR[i]);
				Matrix4x4 localWheelR = traR * rotWheel;
				Matrix4x4 worldWheelR = localBody * localWheelR;
				WheelR[i].SetMatrix(worldWheelR);
				
				Matrix4x4 traL = TH3DMath.GetTranslation4x4(c_attachPos_WheelL[i]);
				Matrix4x4 localWheelL = traL * rotWheel;
				Matrix4x4 worldWheelL = localBody * localWheelL;
				WheelL[i].SetMatrix(worldWheelL);
			}
			
			
			// TrackBits
			Matrix4x4[] mtrcs = GetTrackBitMatrices(i_mileage);
			Matrix4x4 moveRight = TH3DMath.GetTranslation4x4(c_TrackCenterPosR);
			Matrix4x4 moveLeft = TH3DMath.GetTranslation4x4(c_TrackCenterPosL);
			for (int i = 0; i < TrackBitR.Length; i++)
			{
				Matrix4x4 localTrackBitR = moveRight * mtrcs[i];
				Matrix4x4 worldTrackBitR = localBody * localTrackBitR;
				TrackBitR[i].SetMatrix(worldTrackBitR);
				
				Matrix4x4 localTrackBitL = moveLeft * mtrcs[i];
				Matrix4x4 worldTrackBitL = localBody * localTrackBitL;
				TrackBitL[i].SetMatrix(worldTrackBitL);
			}
			
			Matrix4x4 worldBody = localBody;
			Body.SetMatrix(worldBody);
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				i_MOVE = false;
				i_mileage = 0.0f;
				i_degBody = 0.0f;
				i_degTurret = 0.0f;
				i_degBarrel = 0.0f;
				i_speedBody = 0.01f;        
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			if (Input.GetKey(KeyCode.H))
			{
				i_degBody -= 0.25f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody += 0.25f;
			}
			
			if (Input.GetKey(KeyCode.D))
			{
				i_degTurret -= 1.0f;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degTurret += 1.0f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				i_degBarrel -= 0.2f;
				if (i_degBarrel < 0.0f) { i_degBarrel = 0.0f; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_degBarrel += 0.2f;
				if (i_degBarrel > 20.0f) { i_degBarrel = 20.0f; }
			}
			
			if (Input.GetKey(KeyCode.N))
			{
				i_speedBody -= 0.0001f;
				if (i_speedBody < 0.01f) { i_speedBody = 0.01f; }
			}
			else if (Input.GetKey(KeyCode.M))
			{
				i_speedBody += 0.0001f;
				if (i_speedBody > 0.06f) { i_speedBody = 0.06f; }
			}
			
			
			// Barrel
			Matrix4x4 traBarrel = TH3DMath.GetTranslation4x4(c_attachPos_Barrel);
			Matrix4x4 rotBarrel = TH3DMath.GetRotation4x4(i_degBarrel, Vector3.forward);
			Matrix4x4 localBarrel = traBarrel * rotBarrel;
			
			// Turret
			Matrix4x4 traTurret = TH3DMath.GetTranslation4x4(c_attachPos_Turret);
			Matrix4x4 rotTurret = TH3DMath.GetRotation4x4(i_degTurret, Vector3.up);
			Matrix4x4 localTurret = traTurret * rotTurret;
			
			// Body
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 forwardDir = rotBody * Vector3.right;
			float speed = i_MOVE ? i_speedBody : 0f;      
			Vector3 pos = Body.GetWorldPosition() + speed * forwardDir;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(pos);
			Matrix4x4 localBody = traBody * rotBody;
			
			Matrix4x4 worldBarrel = localBody * localTurret * localBarrel;
			Matrix4x4 worldTurret = localBody * localTurret;
			Matrix4x4 worldBody = localBody;
			
			Barrel.SetMatrix(worldBarrel);
			Turret.SetMatrix(worldTurret);
			Body.SetMatrix(worldBody);
			
			i_mileage += speed;
			
			// Wheels
			float degWheel = (i_mileage / c_WheelLength) * 360.0f;
			Matrix4x4 rotWheel = TH3DMath.GetRotation4x4(-degWheel, Vector3.forward);
			for (int i = 0; i < WheelR.Length; i++)
			{
				Matrix4x4 traR = TH3DMath.GetTranslation4x4(c_attachPos_WheelR[i]);
				Matrix4x4 localWheelR = traR * rotWheel;
				Matrix4x4 worldWheelR = localBody * localWheelR;
				WheelR[i].SetMatrix(worldWheelR);
				
				Matrix4x4 traL = TH3DMath.GetTranslation4x4(c_attachPos_WheelL[i]);
				Matrix4x4 localWheelL = traL * rotWheel;
				Matrix4x4 worldWheelL = localBody * localWheelL;
				WheelL[i].SetMatrix(worldWheelL);
			}
			
			// TrackBits
			Matrix4x4[] mtrcs = GetTrackBitMatrices(i_mileage);
			Matrix4x4 moveRight = TH3DMath.GetTranslation4x4(c_TrackCenterPosR);
			Matrix4x4 moveLeft = TH3DMath.GetTranslation4x4(c_TrackCenterPosL);
			for (int i = 0; i < TrackBitR.Length; i++)
			{
				Matrix4x4 localTrackBitR = moveRight * mtrcs[i];
				Matrix4x4 worldTrackBitR = localBody * localTrackBitR;
				TrackBitR[i].SetMatrix(worldTrackBitR);
				
				Matrix4x4 localTrackBitL = moveLeft * mtrcs[i];
				Matrix4x4 worldTrackBitL = localBody * localTrackBitL;
				TrackBitL[i].SetMatrix(worldTrackBitL);
			}
		}




		Matrix4x4[] GetTrackBitMatrices(float mileage)
		{
			Matrix4x4[] mtrcs = new Matrix4x4[TrackBitR.Length];
			
			float frameNr = mileage * 100.0f;  
			
			for (int i = 0; i < TrackBitR.Length; i++)
			{
				float wfr = (frameNr + i * c_SingleBitFrames) % i_totalFrames;
				mtrcs[i] = CalcTransform(wfr, i_keyframes);
			}
			
			return mtrcs;
		}
		
		
		
		Matrix4x4 CalcTransform(float fr, KeyframeData[] kfrs)
		{
			return RGAuxFuncs04.CalcTransform(fr, kfrs);
		}
		
		
		
		


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(5))
			{
				Vector3 wp = Body.GetWorldPosition();
				Vector3 moveVtr = wp - pr_preTankPosition;
				MainCamera.transform.localPosition += moveVtr;
				
				BSData.CameraController3D.UpdateCameraData();
				
				pr_preTankPosition = wp;
			}
		}


		static Sec419 pr_instance = new Sec419(5);

		public static Sec419 PRGSI()
		{
			return pr_instance;
		}


		private Sec419(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Body = CreateObject_Obj3D("TankBody");
				Turret = CreateObject_Obj3D("Turret_2");
				Barrel = CreateObject_Obj3D("Barrel");
				
				// TrackBit
				TrackBitR = new THObject3D[58];
				TrackBitR[0] = CreateObject_Obj3D("TrackBit");
				TrackBitL = new THObject3D[58];
				TrackBitL[0] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
				for (int i = 1; i < TrackBitR.Length; i++)
				{
					TrackBitR[i] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
					TrackBitL[i] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
				}
				
				// Wheel
				WheelR = new THObject3D[7];
				WheelR[0] = CreateObject_Obj3D("TrackWheel");
				WheelL = new THObject3D[7];
				WheelL[0] = CreateObject_InsObj3D(WheelR[0].gameObject);
				for (int i = 1; i < WheelR.Length; i++)
				{
					WheelR[i] = CreateObject_InsObj3D(WheelR[0].gameObject);
					WheelL[i] = CreateObject_InsObj3D(WheelR[0].gameObject);
				}
				
				// Body Materials
				MeshRenderer mre = Body.gameObject.GetComponent<MeshRenderer>();
				Mtrl_opqBody = mre.material;
				Mtrl_trpBody = BSAuxFuncs.GetSemiTransparentMaterial(0.190f, 0.226f, 0.127f, 0.50f);
				
				// Main Camera
				MainCamera = BSAuxFuncs.GetMainCamera();
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();
				
				// Init Keyframes
				i_keyframes = RGAuxFuncs04.InitKeyframes_19();
				i_totalFrames = i_keyframes[i_keyframes.Length - 1].frameNumber;

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			
			MeshRenderer mre = Body.gameObject.GetComponent<MeshRenderer>();
			mre.material = MatchCase(3, 4) ? Mtrl_trpBody : Mtrl_opqBody;
			
			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Turret.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Turret.SetMatrix(Matrix4x4.identity);
				Barrel.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(6.396873f, 5.987775f, -8.102569f);
				TrgPos = new Vector3(1.841599f, 0.5622811f, 0.4879524f);
				uJ = new Vector3(-0.2227026f, 0.8732418f, 0.4334193f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
				}
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				
				for (int i = 0; i < TrackBitR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, TrackBitR[i].gameObject);
					TrackBitR[i].SetMatrix(Matrix4x4.identity);
					
					BSAuxFuncs.SetVisible(1, TrackBitL[i].gameObject);
					TrackBitL[i].SetMatrix(Matrix4x4.identity);
				}
				
				CamPos = new Vector3(3.746045f, 3.652374f, -5.073771f);
				TrgPos = new Vector3(-0.1534616f, -1.012148f, -0.09436762f);
				uJ = new Vector3(-0.3482892f, 0.8045945f, 0.4809598f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(80, -30, 0);
				}
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				
				for (int i = 0; i < TrackBitR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, TrackBitR[i].gameObject);
					TrackBitR[i].SetMatrix(Matrix4x4.identity);
					
					BSAuxFuncs.SetVisible(1, TrackBitL[i].gameObject);
					TrackBitL[i].SetMatrix(Matrix4x4.identity);
				}
				
				for (int i = 0; i < WheelR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, WheelR[i].gameObject);
					WheelR[i].SetMatrix(Matrix4x4.identity);
					
					BSAuxFuncs.SetVisible(1, WheelL[i].gameObject);
					WheelL[i].SetMatrix(Matrix4x4.identity);
				}
				
				CamPos = new Vector3(4.093256f, 2.503828f, 4.993123f);  
				TrgPos = new Vector3(-0.1049493f, -0.7846948f, -0.1989437f);
				uJ = new Vector3(-0.3047826f, 0.8965504f, -0.3214109f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(70, -180, 0);  
				}
			}
			else if (MatchCase(5))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Turret.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Turret.SetMatrix(Matrix4x4.identity);
				Barrel.SetMatrix(Matrix4x4.identity);
				
				for (int i = 0; i < TrackBitR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, TrackBitR[i].gameObject);
					TrackBitR[i].SetMatrix(Matrix4x4.identity);
					
					BSAuxFuncs.SetVisible(1, TrackBitL[i].gameObject);
					TrackBitL[i].SetMatrix(Matrix4x4.identity);
				}
				
				for (int i = 0; i < WheelR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, WheelR[i].gameObject);
					WheelR[i].SetMatrix(Matrix4x4.identity);
					
					BSAuxFuncs.SetVisible(1, WheelL[i].gameObject);
					WheelL[i].SetMatrix(Matrix4x4.identity);
				}
				
				pr_preTankPosition = Vector3.zero;
				
				CamPos = new Vector3(5.758025f, 4.142655f, -6.153786f);
				TrgPos = new Vector3(2.216308f, 1.6052f, -1.918961f);
				uJ = new Vector3(-0.2462879f, 0.9082599f, 0.3382398f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
				}
			}
		}

	}

}
