//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGAuxFuncs04;
using OQJ_KeyframeData;
using OQJ_THAlphaObject3D;
using OQJ_THShell3D;



namespace OQJ_Sec420
{
	public class Sec420 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;

		THAlphaObject3D Navy;

		THObject3D Body;
		THObject3D Turret;
		THAlphaObject3D Barrel;

		THObject3D[] TrackBitR;
		THObject3D[] TrackBitL;
		THObject3D[] WheelR;
		THObject3D[] WheelL;

		THShell3D[] Shell;
		THShell3D[] FShell;
		THShell3D[] TShell;

		GameObject MainCamera;
		GameObject DLight;


		// Instance Variables
		KeyframeData[] i_keyframes;
		float i_totalFrames;

		float i_speedBody;
		float i_degBody = 0.0f;
		float i_degTurret = 0.0f;
		float i_degBarrel = 0.0f;
		float i_mileage = 0.0f;
		readonly float c_WheelLength = 0.42f * 2.0f * Mathf.PI;
		readonly float c_SingleBitFrames = 0.234108f * 100.0f;

		Quaternion pr_staRot;
		Quaternion pr_endRot;
		Quaternion pr_endRot2;
		Quaternion pr_orieXZ;
		Quaternion pr_orieY;

		int pr_autoRotCounter;
		int pr_autoState;
		int pr_cntMax;
		Vector3 pr_preTankPosition;

		float pr_adg;

		bool pr_Y_UP;
		bool pr_Y_UP_2;

		readonly Vector3 c_attachPos_Turret = new Vector3(0.0f, 1.30125f, 0.0f);

		readonly Vector3 c_attachPos_Barrel = new Vector3(1.45353f, 0.20131f, 0.0f);

		readonly Vector3 c_TrackCenterPosR = new Vector3(0.0f, 0.61847f, -1.5f);
		readonly Vector3 c_TrackCenterPosL = new Vector3(0.0f, 0.61847f, 1.5f);

		readonly Vector3[] c_attachPos_WheelR =
		{
			new Vector3(2.589446f, 0.587908f, -1.502279f),
			new Vector3(1.710000f, 0.482200f, -1.502279f),
			new Vector3(0.855000f, 0.482200f, -1.502279f),
			new Vector3(0.000000f, 0.482200f, -1.502279f),
			new Vector3(-0.855000f, 0.482200f, -1.502279f),
			new Vector3(-1.710000f, 0.482200f, -1.502279f),
			new Vector3(-2.589446f, 0.587908f, -1.502279f)
		};

		readonly Vector3[] c_attachPos_WheelL =
		{
			new Vector3(2.589446f, 0.587908f,  1.502279f),
			new Vector3(1.710000f, 0.482200f,  1.502279f),
			new Vector3(0.855000f, 0.482200f,  1.502279f),
			new Vector3(0.000000f, 0.482200f,  1.502279f),
			new Vector3(-0.855000f, 0.482200f,  1.502279f),
			new Vector3(-1.710000f, 0.482200f,  1.502279f),
			new Vector3(-2.589446f, 0.587908f,  1.502279f)
		};




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Navy.lastShootFrame = 0;
				Navy.shootInterval = 8;
				Navy.initShootDirection = Vector3.forward;
				Navy.shellSpeed = 1.0f;
				Navy.initShootPositions = new Vector4[]
				{
					new Vector4( 0.358f, -0.13965f ,0.2371f, 1.0f),
					new Vector4(-0.358f, -0.13965f ,0.2371f, 1.0f)
				};

				i_INITIALIZED = true;
			}

			NavyMotion();


			i_frameCount++;

			Matrix4x4 M = Matrix4x4.identity;
			Matrix4x4 R = Matrix4x4.identity;
			int numShooter = 0;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Navy.lastShootFrame + Navy.shootInterval)
				{
					Navy.lastShootFrame = i_frameCount;
					numShooter = 2;

					M = Navy.GetMatrix();
					R = M;
					R.SetColumn(3, new Vector4(0, 0, 0, 1));
				}
			}


			Vector3 posNavy = Navy.GetWorldPosition();
			foreach (var shell in Shell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.position + shell.speed * shell.direction;
					if ((posNavy - newShellPos).magnitude > 80.0f)
					{
						shell.active = false;
						shell.position = new Vector3(-1000, 0, 0);
					}
					else
					{
						Matrix4x4 mtx = TH3DMath.GetTranslation4x4(newShellPos) * shell.rotMatrix;
						shell.SetMatrix(mtx);
					}

				}
				else
				{
					if (numShooter > 0)
					{
						Vector3 startPos = M * Navy.initShootPositions[2 - numShooter];
						Vector3 shootDir = M * Navy.initShootDirection;

						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Navy.shellSpeed;
						shell.rotMatrix = R;

						Matrix4x4 mtx = TH3DMath.GetTranslation4x4(startPos) * shell.rotMatrix;
						shell.SetMatrix(mtx);

						numShooter--;
					}
				}
			}
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Barrel.lastShootFrame = 0;
				Barrel.shootInterval = 30;
				Barrel.initShootDirection = Vector3.right;
				Barrel.shellSpeed = 0.8f;
				Barrel.initShootPosition = new Vector3(4.8f, 0.0f, 0.0f);

				i_INITIALIZED = true;
			}

			TankMotion();


			i_frameCount++;

			Matrix4x4 M = Matrix4x4.identity;
			Matrix4x4 R = Matrix4x4.identity;
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Barrel.lastShootFrame + Barrel.shootInterval)
				{
					Barrel.lastShootFrame = i_frameCount;
					bShoot = true;

					M = Barrel.GetMatrix();
					R = M;
					R.SetColumn(3, new Vector4(0, 0, 0, 1));
				}
			}


			Vector3 posBarrel = Barrel.GetWorldPosition();
			foreach (var shell in Shell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.position + shell.speed * shell.direction;
					if ((posBarrel - newShellPos).magnitude > 80.0f)
					{
						shell.active = false;
						shell.position = new Vector3(-1000, 0, 0);
					}
					else
					{
						Matrix4x4 mtx = TH3DMath.GetTranslation4x4(newShellPos) * shell.rotMatrix;
						shell.SetMatrix(mtx);
					}

				}
				else
				{
					if (bShoot)
					{
						Vector3 startPos = M * Barrel.initShootPosition;
						Vector3 shootDir = M * Barrel.initShootDirection;

						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Barrel.shellSpeed;
						shell.rotMatrix = R;

						Matrix4x4 mtx = TH3DMath.GetTranslation4x4(startPos) * shell.rotMatrix;
						shell.SetMatrix(mtx);

						bShoot = false;
					}
				}
			}
		}




		void NavyMotion()
		{
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}


			bool keyH = Input.GetKey(KeyCode.H);
			bool keyL = Input.GetKey(KeyCode.L);

			if (pr_autoState >= 0)
			{
				if (Input.GetKey(KeyCode.J))
				{
					Quaternion rot = (Navy.GetWorldRotation() * Quaternion.AngleAxis(1.0f, Vector3.right)).normalized;
					Navy.SetWorldRotation(rot);
				}
				else if (Input.GetKey(KeyCode.K))
				{
					Quaternion rot = (Navy.GetWorldRotation() * Quaternion.AngleAxis(-1.0f, Vector3.right)).normalized;
					Navy.SetWorldRotation(rot);
				}

				if (Input.GetKeyUp(KeyCode.H) || Input.GetKeyUp(KeyCode.L))
				{
					pr_autoState = -1;
					pr_autoRotCounter = 0;
					pr_cntMax = 30;

					float deg = CalcElevationAngle(false);


					pr_staRot = pr_orieXZ;

					if (!pr_Y_UP && pr_Y_UP_2)
					{
						Quaternion qq = Quaternion.AngleAxis(180, Vector3.up);
						pr_staRot = qq * pr_orieXZ;
						pr_orieY = qq * pr_orieY;
						pr_Y_UP = false;
					}

					pr_endRot = Quaternion.Euler(deg, 0.0f, 0.0f);
				}
				else if (keyH || keyL)
				{
					if (pr_autoState == 0)
					{
						pr_autoState = 1;
						pr_autoRotCounter = 0;
						pr_cntMax = 25;

						float deg = CalcElevationAngle();
						pr_staRot = Quaternion.AngleAxis(deg, Vector3.right);

						if (keyH)
						{
							if (pr_Y_UP)
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, 60);
								pr_endRot2 = Quaternion.Euler(0, 0, 45);
								pr_adg = -1.0f;
							}
							else
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, -60);
								pr_endRot2 = Quaternion.Euler(0, 180, 45);
								pr_adg = -1.0f;
							}
						}
						else if (keyL)
						{
							if (pr_Y_UP)
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, -60);
								pr_endRot2 = Quaternion.Euler(0, 0, -45);
								pr_adg = 1.0f;
							}
							else
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, 60);
								pr_endRot2 = Quaternion.Euler(0, 180, -45);
								pr_adg = 1.0f;
							}
						}
					}
					else if (pr_autoState == 1)
					{
						if (pr_autoRotCounter == pr_cntMax)
						{
							pr_autoState = 2;
							pr_autoRotCounter = 0;
							pr_cntMax = 150;


							pr_staRot = pr_orieXZ;
							pr_endRot = pr_endRot2;
						}
					}
					else if (pr_autoState >= 2)
					{
						if (pr_autoState == 2 && pr_autoRotCounter == pr_cntMax)
						{
							pr_autoState = 3;
						}


						pr_orieY = (Quaternion.AngleAxis(pr_adg, Vector3.up) * pr_orieY).normalized;
					}
				}
			}
			else
			{
				if (pr_autoRotCounter == pr_cntMax)
				{
					pr_autoState = 0;
					pr_orieXZ = Quaternion.identity;
				}
			}

			if (IsAutoRotation())
			{
				pr_autoRotCounter++;

				if (pr_autoRotCounter <= pr_cntMax)
				{
					float t = pr_autoRotCounter / (float)pr_cntMax;
					pr_orieXZ = Quaternion.Slerp(pr_staRot, pr_endRot, t);
				}
			}

			if (pr_autoState != 0)
			{
				Quaternion rot = pr_orieY * pr_orieXZ;
				Navy.SetWorldRotation(rot);
			}



			Quaternion rotNavy = Navy.GetWorldRotation();
			Vector3 forwardDir = rotNavy * Vector3.forward;
			Vector3 curPos = Navy.GetWorldPosition();
			Vector3 newPos = (i_MOVE) ? curPos + 0.2f * forwardDir : curPos;
			Navy.SetWorldPosition(newPos);
		}



		float CalcElevationAngle(bool updt = true)
		{
			Matrix4x4 mtx = Navy.GetMatrix();
			Vector3 lx = mtx.GetColumn(0).normalized;
			Vector3 ly = mtx.GetColumn(1).normalized;
			Vector3 lz = mtx.GetColumn(2).normalized;

			bool y_up = (Vector3.Dot(ly, Vector3.up) >= 0.0f);
			Vector3 prjZ = new Vector3(lz.x, 0.0f, lz.z).normalized;
			if (!y_up)
			{
				prjZ = -prjZ;
			}

			if (updt) { pr_Y_UP = y_up; }
			pr_Y_UP_2 = y_up;

			float deg = TH3DMath.CalcDegree_V1toV2(prjZ, lz, lx);

			return deg;
		}



		bool IsAutoRotation()
		{
			if (pr_autoState == -1 || pr_autoState == 1 || pr_autoState == 2)
			{
				return true;
			}

			return false;
		}



		void TankMotion()
		{
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}

			if (Input.GetKey(KeyCode.H))
			{
				i_degBody -= 0.25f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				i_degBody += 0.25f;
			}

			if (Input.GetKey(KeyCode.D))
			{
				i_degTurret -= 1.0f;
			}
			else if (Input.GetKey(KeyCode.F))
			{
				i_degTurret += 1.0f;
			}

			if (Input.GetKey(KeyCode.J))
			{
				i_degBarrel -= 0.2f;
				if (i_degBarrel < 0.0f) { i_degBarrel = 0.0f; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_degBarrel += 0.2f;
				if (i_degBarrel > 20.0f) { i_degBarrel = 20.0f; }
			}

			if (Input.GetKey(KeyCode.N))
			{
				i_speedBody -= 0.0001f;
				if (i_speedBody < 0.01f) { i_speedBody = 0.01f; }
			}
			else if (Input.GetKey(KeyCode.M))
			{
				i_speedBody += 0.0001f;
				if (i_speedBody > 0.06f) { i_speedBody = 0.06f; }
			}


			// Barrel
			Matrix4x4 traBarrel = TH3DMath.GetTranslation4x4(c_attachPos_Barrel);
			Matrix4x4 rotBarrel = TH3DMath.GetRotation4x4(i_degBarrel, Vector3.forward);
			Matrix4x4 localBarrel = traBarrel * rotBarrel;

			// Turret
			Matrix4x4 traTurret = TH3DMath.GetTranslation4x4(c_attachPos_Turret);
			Matrix4x4 rotTurret = TH3DMath.GetRotation4x4(i_degTurret, Vector3.up);
			Matrix4x4 localTurret = traTurret * rotTurret;

			// Body
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 forwardDir = rotBody * Vector3.right;
			float speed = i_MOVE ? i_speedBody : 0f;
			Vector3 pos = Body.GetLocalPosition() + speed * forwardDir;
			Matrix4x4 traBody = TH3DMath.GetTranslation4x4(pos);
			Matrix4x4 localBody = traBody * rotBody;

			Matrix4x4 worldBarrel = localBody * localTurret * localBarrel;
			Matrix4x4 worldTurret = localBody * localTurret;
			Matrix4x4 worldBody = localBody;

			Barrel.SetMatrix(worldBarrel);
			Turret.SetMatrix(worldTurret);
			Body.SetMatrix(worldBody);

			i_mileage += speed;

			// Wheels
			float degWheel = (i_mileage / c_WheelLength) * 360.0f;
			Matrix4x4 rotWheel = TH3DMath.GetRotation4x4(-degWheel, Vector3.forward);
			for (int i = 0; i < WheelR.Length; i++)
			{
				Matrix4x4 traR = TH3DMath.GetTranslation4x4(c_attachPos_WheelR[i]);
				Matrix4x4 localWheelR = traR * rotWheel;
				Matrix4x4 worldWheelR = localBody * localWheelR;
				WheelR[i].SetMatrix(worldWheelR);

				Matrix4x4 traL = TH3DMath.GetTranslation4x4(c_attachPos_WheelL[i]);
				Matrix4x4 localWheelL = traL * rotWheel;
				Matrix4x4 worldWheelL = localBody * localWheelL;
				WheelL[i].SetMatrix(worldWheelL);
			}

			// TrackBits
			Matrix4x4[] mtrcs = GetTrackBitMatrices(i_mileage);
			Matrix4x4 moveRight = TH3DMath.GetTranslation4x4(c_TrackCenterPosR);
			Matrix4x4 moveLeft = TH3DMath.GetTranslation4x4(c_TrackCenterPosL);
			for (int i = 0; i < TrackBitR.Length; i++)
			{
				Matrix4x4 localTrackBitR = moveRight * mtrcs[i];
				Matrix4x4 worldTrackBitR = localBody * localTrackBitR;
				TrackBitR[i].SetMatrix(worldTrackBitR);

				Matrix4x4 localTrackBitL = moveLeft * mtrcs[i];
				Matrix4x4 worldTrackBitL = localBody * localTrackBitL;
				TrackBitL[i].SetMatrix(worldTrackBitL);
			}
		}



		Matrix4x4[] GetTrackBitMatrices(float mileage)
		{
			Matrix4x4[] mtrcs = new Matrix4x4[TrackBitR.Length];

			float frameNr = mileage * 100.0f;

			for (int i = 0; i < TrackBitR.Length; i++)
			{
				float wfr = (frameNr + i * c_SingleBitFrames) % i_totalFrames;
				mtrcs[i] = RGAuxFuncs04.CalcTransform(wfr, i_keyframes);
			}

			return mtrcs;
		}



		///////////////////////////////////////////////////////////////////////////
		//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1: Code1(); break;

				case 2: Code2(); break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1))
			{
				var wpos = Navy.GetWorldPosition();

				var distC = new Vector3(4.82114f, 4.708963f, -5.310163f);
				var distT = new Vector3(-1.102283f, -2.86507f, 4.208788f);
				var uJ = new Vector3(-0.3925574f, 0.8230028f, 0.4105668f);

				var CamPos = distC + wpos;
				var TrgPos = distT + wpos;

				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2))
			{
				Vector3 wp = Body.GetWorldPosition();
				Vector3 moveVtr = wp - pr_preTankPosition;
				MainCamera.transform.position += moveVtr;

				BSData.CameraController3D.UpdateCameraData();

				pr_preTankPosition = wp;
			}
		}


		static Sec420 pr_instance = new Sec420(2);

		public static Sec420 PRGSI()
		{
			return pr_instance;
		}


		private Sec420(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if (!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");

				// ++ Navy ++
				Navy = CreateObject_Alpha3D("FNavy");
				var src_FShell = CreateObject_Shell3D("FShell");
				FShell = new THShell3D[60];
				for (int i = 0; i < FShell.Length; i++)
				{
					FShell[i] = CreateObject_InsShell3D(src_FShell.gameObject);
				}

				// ++ Tank ++
				Body = CreateObject_Obj3D("TankBody");
				Turret = CreateObject_Obj3D("Turret_2");
				Barrel = CreateObject_Alpha3D("Barrel");

				// TrackBit
				TrackBitR = new THObject3D[58];
				TrackBitR[0] = CreateObject_Obj3D("TrackBit");
				TrackBitL = new THObject3D[58];
				TrackBitL[0] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
				for (int i = 1; i < TrackBitR.Length; i++)
				{
					TrackBitR[i] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
					TrackBitL[i] = CreateObject_InsObj3D(TrackBitR[0].gameObject);
				}

				// Wheel
				WheelR = new THObject3D[7];
				WheelR[0] = CreateObject_Obj3D("TrackWheel");
				WheelL = new THObject3D[7];
				WheelL[0] = CreateObject_InsObj3D(WheelR[0].gameObject);
				for (int i = 1; i < WheelR.Length; i++)
				{
					WheelR[i] = CreateObject_InsObj3D(WheelR[0].gameObject);
					WheelL[i] = CreateObject_InsObj3D(WheelR[0].gameObject);
				}

				var src_TShell = CreateObject_Shell3D("TShell");
				TShell = new THShell3D[30];
				for (int i = 0; i < TShell.Length; i++)
				{
					TShell[i] = CreateObject_InsShell3D(src_TShell.gameObject);
				}


				// Init Keyframes
				i_keyframes = RGAuxFuncs04.InitKeyframes_19();
				i_totalFrames = i_keyframes[i_keyframes.Length - 1].frameNumber;


				// Main Camera
				MainCamera = BSAuxFuncs.GetMainCamera();

				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;

			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();

			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Navy.gameObject);
				Navy.SetIdentity();

				float yy = 24;
				Navy.SetWorldPosition(0, yy, 0);

				Shell = FShell;
				for (int i = 0; i < Shell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Shell[i].gameObject);
					Shell[i].SetIdentity();
					Shell[i].position = new Vector3(-1000, 0, 0);
				}

				pr_autoState = 0;
				pr_orieXZ = Quaternion.identity;
				pr_orieY = Quaternion.identity;
				pr_Y_UP = false;

				CamPos = new Vector3(15.08402f, 12.56928f + yy, -15.28707f);
				TrgPos = new Vector3(3.866516f, 4.686559f + yy, -2.338552f);
				uJ = new Vector3(-0.3025928f, 0.9077742f, 0.2904888f);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Turret.gameObject);
				BSAuxFuncs.SetVisible(1, Barrel.gameObject);
				Body.SetMatrix(Matrix4x4.identity);
				Turret.SetMatrix(Matrix4x4.identity);
				Barrel.SetMatrix(Matrix4x4.identity);

				for (int i = 0; i < TrackBitR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, TrackBitR[i].gameObject);
					TrackBitR[i].SetMatrix(Matrix4x4.identity);

					BSAuxFuncs.SetVisible(1, TrackBitL[i].gameObject);
					TrackBitL[i].SetMatrix(Matrix4x4.identity);
				}

				for (int i = 0; i < WheelR.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, WheelR[i].gameObject);
					WheelR[i].SetMatrix(Matrix4x4.identity);

					BSAuxFuncs.SetVisible(1, WheelL[i].gameObject);
					WheelL[i].SetMatrix(Matrix4x4.identity);
				}


				Shell = TShell;
				for (int i = 0; i < Shell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Shell[i].gameObject);
					Shell[i].SetIdentity();
					Shell[i].position = new Vector3(-1000, 0, 0);
				}

				i_MOVE = false;
				i_mileage = 0.0f;
				i_degBody = 0.0f;
				i_degTurret = 0.0f;
				i_degBarrel = 0.0f;
				i_speedBody = 0.01f;
				pr_preTankPosition = Vector3.zero;

				CamPos = new Vector3(5.546137f, 4.945885f, -6.328987f);
				TrgPos = new Vector3(2.00442f, 2.308429f, -2.094159f);
				uJ = new Vector3(-0.2884468f, 0.9022065f, 0.3206584f);

				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 30, 0);
				}
			}


			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}

	}

}
