//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject3D;
using OQJ_THShell3D;
using static OQJ_THCollisionTest.THCollisionTest;



namespace OQJ_Sec422
{
	public class Sec422 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THShell3D Shell;
		THShell3D[] UShell;
		THShell3D[] EShell;
		THAlphaObject3D[] Stand;
		
		THAlphaObject3D Sphere;
		THAlphaObject3D Box;
		THAlphaObject3D BoxMG;
		THAlphaObject3D Box01;
		THAlphaObject3D Cylinder;
		
		THAlphaObject3D Star;
		THAlphaObject3D Torus;
		THAlphaObject3D Crystal;
		
		THShell3D MiniShell;
		THAlphaObject3D Cannon;
		THObject3D Ray;
		THObject3D VisRay;
		
		THObject3D BackPlane;	
		GameObject SubCamera;
		
		GameObject RefLight;
		Light pr_Comp_RefLight;	
		
		GameObject DLight;


		// Instance Variables
		THAlphaObject3D[] i_targetObjects;
		
		Vector3 i_reboundVel;
		
		float i_radShell;			
		readonly Vector4 c_initShootPosition = new Vector4(0.0f, 0.0f, 1.15f, 1.0f);
		
		float pr_degCannon;
		
		float pr_shmA;
		float pr_shmB;
		float pr_degA;
		float pr_degB;
		
		Mesh pr_Mesh_VisRay;
		Camera pr_Comp_SubCamera;
		MeshRenderer pr_MRE_BackPlane;
		
		readonly int c_DISK = 10;
		readonly int c_RECT = 11;
		readonly int c_NORMAL = 100;
		readonly int c_HIT = 101;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Sphere.SetWorldPosition(12, 4, 0);
				Box.SetWorldPosition(12, 4, 6);
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
			Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L))
			{
				MoveSphere();
			}
			
			if (Input.GetKey(KeyCode.R))
			{
				MoveBox();
			}
			
			Vector2 C = Sphere.positionXZ;
			Vector2 T = Box.positionXZ;
			Vector2 lx = Box.localAxisXZ[0];
			Vector2 lz = Box.localAxisXZ[1];
			var bColl = CollisionTest_Disk_Rect(C, Sphere.radius, T, Box.width, Box.height, lx, lz);
			ChangeObjectColor(bColl);
		}


		void Code2()
		{
			if (Input.GetKeyDown(KeyCode.A))
			{
				Shell.position = Cannon.GetMatrix() * c_initShootPosition;
				Shell.direction = Vector3.forward;
				Shell.rebound = false;
			}
			
			if (Shell.rebound)
			{
				i_reboundVel.y -= 0.002f;
				Shell.position += i_reboundVel;
				return;
			}
			
			Shell.position += 0.075f * Shell.direction;
			
			Vector2 C = new Vector2(Shell.position.x, Shell.position.z);
			float   r = i_radShell;
			Vector2 T = Stand[0].positionXZ;
			float   w = Stand[0].width;
			float   h = Stand[0].height;
			bool bColl = CollisionTest_Disk_Rect(C, r, T, w, h, Vector2.right, Vector2.up);
			
			if (bColl)
			{
				i_reboundVel = -0.016f * Shell.direction;
				Shell.rebound = true;
			}
		}


		void Code3()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				ChangeDirection();
			}
			
			
			if (Input.GetKeyDown(KeyCode.A))
			{
				Matrix4x4 M = Cannon.GetMatrix();
				Shell.position  = M * c_initShootPosition;
				Shell.direction = M * Vector3.forward;
				Shell.rebound = false;
			}
			
			if (Shell.rebound)
			{
				i_reboundVel.y -= 0.002f;
				Shell.position += i_reboundVel;
				return;
			}
			
			Shell.position += 0.15f * Shell.direction;  
			
			Vector2 C = new Vector2(Shell.position.x, Shell.position.z);
			float   r = i_radShell;
			float   w = Stand[0].width;
			float   h = Stand[0].height;
			
			for (int i = 0; i < Stand.Length; i++)
			{
				Vector2 T = Stand[i].positionXZ;
				bool bColl = CollisionTest_Disk_Rect(C, r, T, w, h, Vector2.right, Vector2.up);
				
				if (bColl)
				{
					i_reboundVel = -0.025f * Shell.direction;
					Shell.rebound = true;
					break;
				}
			}
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = new THAlphaObject3D[] { Cylinder, Box, Sphere };
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				ChangeDirection();
			}
			
			ObjectMotion();
			
			if (Input.GetKeyDown(KeyCode.A))
			{
				Matrix4x4 M = Cannon.GetMatrix();
				Shell.position  = M * c_initShootPosition;
				Shell.direction = M * Vector3.forward;
				Shell.rebound = false;
			}
			
			if (Shell.rebound)
			{
				i_reboundVel.y -= 0.002f;
				Shell.position += i_reboundVel;
				return;
			}
			
			Shell.position += 0.18f * Shell.direction;
			
			Vector2 C = new Vector2(Shell.position.x, Shell.position.z);
			float   r = i_radShell;
			
			foreach (var target in i_targetObjects)
			{
				Vector2 T = target.positionXZ;
				
				bool bColl = false;
				if (target.type == c_DISK)
				{
					bColl = CollisionTest_Disk_Disk(C, r, T, target.radius);
				}
				else if (target.type == c_RECT)
				{
					Vector2 lx = target.localAxisXZ[0];
					Vector2 lz = target.localAxisXZ[1];
					
					bColl = CollisionTest_Disk_Rect(C, r, T, target.width, target.height, lx, lz);
				}
				
				if (bColl)
				{
					i_reboundVel = -0.025f * Shell.direction;
					Shell.rebound = true;
					break;		
				}
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				Cannon.lastShootFrame = 0;
				Cannon.shootInterval = 40;
				Cannon.initShootDirection = Vector3.forward;
				Cannon.initShootPosition = new Vector3(0.0f, 0.0f, 1.15f);
				Cannon.shellSpeed = 0.2f;
				
				i_targetObjects = new THAlphaObject3D[] { Cylinder, Box, Sphere };
				i_frameCount = 0;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				ChangeDirection();
			}
			
			ObjectMotion();
			
			
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Cannon.lastShootFrame + Cannon.shootInterval)
				{
					Cannon.lastShootFrame = i_frameCount;
					bShoot = true;
				}
			}
			
			
			Vector3 posCannon = Cannon.GetWorldPosition();
			foreach (var shell in UShell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.position + shell.speed * shell.direction;
					if ((newShellPos - posCannon).magnitude >= 60.0f)
					{
						shell.active = false;
						newShellPos = new Vector3(-1000, 0, 0);
					}
					
					shell.position = newShellPos;
				}
				else
				{
					if (bShoot)
					{
						Matrix4x4 M = Cannon.GetMatrix();
						
						Vector3 startPos = M * Cannon.initShootPosition;
						Vector3 shootDir = M * Cannon.initShootDirection;
						
						shell.active = true;
						shell.direction = shootDir.normalized;
						shell.speed = Cannon.shellSpeed;
						
						shell.position = startPos;
						
						bShoot = false;
					}
				}
			}
			
			
			
			
			foreach (var shell in UShell)
			{
				if (!shell.active) { continue; }
				
				Vector2 posShell = new Vector2(shell.position.x, shell.position.z);
				
				foreach (var target in i_targetObjects)
				{
					Vector2 posTarget = target.positionXZ;
					
					bool bColl = false;
					if (target.type == c_DISK)
					{
						bColl = CollisionTest_Disk_Disk(posShell, i_radShell, posTarget, target.radius);
					}
					else if (target.type == c_RECT)
					{
						Vector2 lx = target.localAxisXZ[0];
						Vector2 lz = target.localAxisXZ[1];
						bColl = CollisionTest_Disk_Rect(posShell, i_radShell, posTarget, target.width, target.height, lx, lz);
					}
					
					if (bColl)
					{
						Hit(target);
						
						shell.SetWorldPosition(-1000, 0, 0);
						shell.active = false;
					}
				} 
			} 
			
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				Cannon.lastShootFrame = 0;
				Cannon.shootInterval = 40;
				Cannon.initShootDirection = Vector3.forward;
				Cannon.initShootPosition = new Vector3(0.0f, 0.0f, 1.15f);
				Cannon.shellSpeed = 0.2f;
				
				i_targetObjects = new THAlphaObject3D[] { Star, Torus, Crystal };
				i_frameCount = 0;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				ChangeDirection();
			}
			
			ObjectMotion();
			
			UShellMotion();
			
			CollisionTest_ShellPhase(UShell, i_targetObjects);
		}




		void UShellMotion()		
		{
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Cannon.lastShootFrame + Cannon.shootInterval)
				{
					Cannon.lastShootFrame = i_frameCount;
					bShoot = true;
				}
			}
			
			
			Vector3 posCannon = Cannon.GetWorldPosition();
			foreach (var shell in UShell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.position + shell.speed * shell.direction;
					if ((newShellPos - posCannon).magnitude >= 60.0f)
					{
						shell.active = false;
						newShellPos = new Vector3(-1000, 0, 0);
					}
					
					shell.position = newShellPos;
				}
				else
				{
					if (bShoot)
					{
						Matrix4x4 M = Cannon.GetMatrix();
						
						Vector3 startPos = M * Cannon.initShootPosition;
						Vector3 shootDir = M * Cannon.initShootDirection;
						
						shell.active = true;
						shell.direction = shootDir.normalized;
						shell.speed = Cannon.shellSpeed;
						
						shell.position = startPos;
						
						bShoot = false;
					}
				}
			}
		}
		
		
		
		
		void CollisionTest_ShellPhase(THShell3D[] shell_array, THAlphaObject3D[] target_array)
		{
			foreach (var shell in shell_array)
			{
				if (!shell.active) { continue; }
				
				Vector2 posShell = new Vector2(shell.position.x, shell.position.z);
				
				foreach (var target in target_array)
				{
					Vector2 posTarget = target.positionXZ;
					
					bool bColl = false;
					if (target.type == c_DISK)
					{
						bColl = CollisionTest_Disk_Disk(posShell, i_radShell, posTarget, target.radius);
					}
					else if (target.type == c_RECT)
					{
						Vector2 lx = target.localAxisXZ[0];
						Vector2 lz = target.localAxisXZ[1];
						bColl = CollisionTest_Disk_Rect(posShell, i_radShell, posTarget, target.width, target.height, lx, lz);
					}
					
					if (bColl)
					{
						Hit(target);
						
						shell.SetWorldPosition(-1000, 0, 0);
						shell.active = false;
					}
				} 
			} 
		}
		
		
		
		void ObjectMotion()
		{
			if (MatchCase(4, 5))
			{
				// Cylinder
				Quaternion rot = Cylinder.GetWorldRotation();
				rot = Quaternion.AngleAxis(0.15f, Vector3.up) * rot;
				Cylinder.SetWorldRotation(rot.normalized);
				
				// Box
				pr_shmB += 0.45f;
				float A = 60.0f;
				float deg = A * Mathf.Sin(pr_shmB * Mathf.Deg2Rad);
				Box.SetWorldRotation(deg, Vector3.up);
				
				// Sphere
				pr_shmA += 0.3f;
				A = 1.1f;
				float scl = 0.5f * A * (-Mathf.Cos(pr_shmA * Mathf.Deg2Rad) + 1.0f) + 0.5f;
				Sphere.SetLocalScale(scl);
			}
			else if (MatchCase(6))
			{
				// Star
				pr_shmA += 0.5f;
				float t = 0.5f * (-Mathf.Cos(pr_shmA * Mathf.Deg2Rad)) + 0.5f;  
				Vector3 P1 = new Vector3(5, 1, 5);
				Vector3 P2 = new Vector3(6, 1, 7);
				Vector3 sp = (1.0f - t) * P1 + t * P2;
				
				float min = -70f;
				float max = 20f;
				float A = max - min;
				float deg = 0.5f * A * (-Mathf.Cos(pr_shmA * Mathf.Deg2Rad) + 1.0f) + min;
				Star.SetWorldPosition(sp);
				Star.SetWorldRotation(deg, Vector3.up);
				
				// Torus
				pr_degA += 0.55f;
				Vector3 tp = Quaternion.AngleAxis(pr_degA, Vector3.up) * Vector3.right;
				tp += new Vector3(12f, 2.5f, 5.5f);
				Torus.SetWorldPosition(tp);
				
				// Crystal
				pr_degB += 0.3f;
				Crystal.SetWorldRotation(pr_degB, Vector3.up);
				
			}
		}
		
		
		
		void MoveSphere()
		{
			Vector3 pos = Sphere.GetWorldPosition();
			
			if (Input.GetKey(KeyCode.H))
			{
				pos.x -= 0.05f;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				pos.x += 0.05f;
			}
			
			if (Input.GetKey(KeyCode.J))
			{
				pos.z -= 0.05f;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				pos.z += 0.05f;
			}
			
			Sphere.SetWorldPosition(pos);
		}
		
		
		
		void MoveBox()
		{
			pr_degA  += THUtil.IsShiftDown() ? -0.5f : 0.5f;
			Matrix4x4 M = TH3DMath.GetTranslation4x4(12, 4, 6) * 
			TH3DMath.GetRotation4x4(pr_degA, Vector3.up);
			Box.SetMatrix(M);
		}
		
		
		
		void ChangeObjectColor(bool bColl)	
		{
			if (!MatchCase(1)) { return; }
			
			if (bColl)
			{
				Sphere.state = c_HIT;
				Box.state = c_HIT;
			}
			else 
			{
				Sphere.state = c_NORMAL;
				Box.state = c_NORMAL;
			}
			
		}
		
		
		
		void ChangeDirection()
		{
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				pr_degCannon += Input.GetKey(KeyCode.H) ? -0.25f : 0.25f;
				Cannon.SetWorldRotation(pr_degCannon, Vector3.up);
				
				Ray.SetWorldRotation(pr_degCannon, Vector3.up);
				Ray.SetWorldPosition(Cannon.GetMatrix() * c_initShootPosition);
			}
		}	
		
		
		
		void Hit(THAlphaObject3D obj)
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = 12;
				obj.state = c_HIT;
			}
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			if (MatchCase(1, 2))
			{
			}
		}


		void PostProcess()
		{
			// Ref Light
			if (MatchCase(3, 4, 5, 6))
			{
				Vector3 wp = Cannon.GetMatrix() * new Vector4(0.0f, 0.0f, 0.9f, 1.0f);
				Ray.SetWorldPosition(wp);
				
				Quaternion q = Cannon.GetWorldRotation();
				Ray.SetWorldRotation(q);	
				
				Vector3 dir = q * Vector3.forward;
				
				Vector2 R0 = new Vector2(wp.x, wp.z);
				Vector2 v = new Vector2(dir.x, dir.z).normalized;
				float t = -1.0f;
				foreach (var target in i_targetObjects)
				{
					if (target.type == c_DISK)
					{
						Vector2 C = target.positionXZ;
						
						Vector3 P = CollisionTest_Ray_Disk_Coplanar(R0, v, C, target.radius);
						if(P.z >= 0.0f)
						{
							t = P.z;
							break;
						}	
					}
					else 
					{
						Vector2 T = target.positionXZ;
						Vector2[] ax = target.localAxisXZ;
						
						Vector3 P = CollisionTest_Ray_Rect_Coplanar(R0, v, T, ax[0], ax[1], target.width, target.height);
						if(P.z >= 0.0f)
						{
							t = P.z;
							break;
						}	
					}
				}
				
				if (t >= 0.0f)
				{
					Vector3 pp = wp + (t - 0.05f) * dir;
					RefLight.transform.position = pp;	
					Ray.SetLocalScale(1, 1, t - 0.05f);
					
					Vector3 vp = VisRay.GetWorldPosition();
					pr_Mesh_VisRay.SetVertices(new List<Vector3>() { wp - vp, pp - vp });
				}
				else
				{
					RefLight.transform.position = new Vector3(-1000, 0.0f, 0.0f);
					Ray.SetLocalScale(1, 1, 60.0f);
					
					Vector3 pp = wp + 60.0f * dir;
					Vector3 vp = VisRay.GetWorldPosition();
					pr_Mesh_VisRay.SetVertices(new List<Vector3>() { wp - vp, pp - vp });
				}
			}
			
			// Hit Color
			if (MatchCase(5, 6))
			{
				foreach (var o in i_targetObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							o.state = c_NORMAL;
						}
					}
				}
			}
		}


		static Sec422 pr_instance = new Sec422(6);

		public static Sec422 PRGSI()
		{
			return pr_instance;
		}


		private Sec422(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				MiniShell = CreateObject_Shell3D("MiniShell");
				Sphere = CreateObject_Alpha3D("PointB");
				BoxMG = CreateObject_Alpha3D("BoxMG"); 
				Box01 = CreateObject_Alpha3D("UnitCube01");
				Cylinder = CreateObject_Alpha3D("XZCylinder01");
				Star = CreateObject_Alpha3D("CStar");
				Torus = CreateObject_Alpha3D("CTorus");
				Crystal = CreateObject_Alpha3D("CCrystal");
				
				Stand = new THAlphaObject3D[5];
				for (int i = 1; i <= 5; i++)
				{
					Stand[i - 1] = CreateObject_Alpha3D("NrSt_" + i);
				}
				
				var src_Shell = CreateObject_Shell3D("MlShell_A");
				UShell = new THShell3D[30];
				for(int i=0; i<UShell.Length; i++)
				{
					UShell[i] = CreateObject_InsShell3D(src_Shell.gameObject);
				}
				
				EShell = new THShell3D[40];
				Material rMt = BSAuxFuncs.GetStandardMaterial(0.60f, 0.12f, 0.12f, 0.1f);
				for(int i=0; i<EShell.Length; i++)
				{
					EShell[i] = CreateObject_InsShell3D(src_Shell.gameObject);
					EShell[i].gameObject.GetComponent<MeshRenderer>().material = rMt;
					EShell[i].gameObject.name = "EShell_" + i;
				}
				
				Cannon = CreateObject_Alpha3D("MiniCannon");
				Ray = CreateObject_Obj3D("AuxRay");
				VisRay = CreateObject_InsObj3D(Ray.gameObject);
				
				Mesh mesh = Ray.gameObject.GetComponent<MeshFilter>().mesh;
				mesh.SetVertices(new List<Vector3>() { Vector3.zero, Vector3.forward });
				
				pr_Mesh_VisRay = VisRay.gameObject.GetComponent<MeshFilter>().mesh;
				
				
				Material mtrlNml = BSAuxFuncs.GetStandardMaterial(0.35f, 0.68f, 0.60f);
				Material mtrlHit = BSAuxFuncs.GetStandardMaterial(0.75f, 0.24f, 0.18f, 0.1f);
				BoxMG.gameObject.GetComponent<MeshRenderer>().material = mtrlNml;
				Box01.gameObject.GetComponent<MeshRenderer>().material = mtrlNml;
				BoxMG.SetData(c_NORMAL, new Material[] { mtrlNml });
				BoxMG.SetData(c_HIT, new Material[] { mtrlHit });
				Box01.SetData(c_NORMAL, new Material[] { mtrlNml });
				Box01.SetData(c_HIT, new Material[] { mtrlHit });
				
				mtrlNml = BSAuxFuncs.GetStandardMaterial(0.755f, 0.543f, 0.367f, 0.0f);  
				mtrlHit = BSAuxFuncs.GetStandardMaterial(0.80f, 0.34f, 0.34f, 0.0f);
				Sphere.SetData(c_NORMAL, new Material[] { mtrlNml });
				Sphere.SetData(c_HIT, new Material[] { mtrlHit });
				
				Cylinder.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetStandardMaterial(0.3f, 0.4f, 0.7f);
				Cylinder.SetData(c_NORMAL, new Material[] { BSAuxFuncs.GetStandardMaterial(0.3f, 0.4f, 0.7f) });
				Cylinder.SetData(c_HIT, new Material[] { mtrlHit });
				
				mtrlNml = Star.gameObject.GetComponent<MeshRenderer>().material;
				Star.SetData(c_NORMAL, new Material[] { mtrlNml });
				Star.SetData(c_HIT, new Material[] { mtrlHit });
				
				mtrlNml = Torus.gameObject.GetComponent<MeshRenderer>().material;
				Torus.SetData(c_NORMAL, new Material[] { mtrlNml });
				Torus.SetData(c_HIT, new Material[] { mtrlHit });
				
				mtrlNml = Crystal.gameObject.GetComponent<MeshRenderer>().material;
				Crystal.SetData(c_NORMAL, new Material[] { mtrlNml });
				Crystal.SetData(c_HIT, new Material[] { mtrlHit });
				
				
				
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();
				
				// Sub Camera
				GameObject go = new GameObject();
				go.name = "Camera_2";
				pr_Comp_SubCamera = go.AddComponent<Camera>();
				pr_Comp_SubCamera.nearClipPlane = 0.01f;
				pr_Comp_SubCamera.farClipPlane = 50.0f;
				pr_Comp_SubCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_Comp_SubCamera.backgroundColor = BSData.BackgroundColor;
				pr_Comp_SubCamera.orthographic = true;  
				pr_Comp_SubCamera.orthographicSize = 4.0f;
				pr_Comp_SubCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_Comp_SubCamera.enabled = false;
				SubCamera = go;  
				
				// BackPlane (init XY 1x1)
				BackPlane = CreateObject_Obj3D("UnitRect");
				Material objMtrl = BSAuxFuncs.GetUnlitMaterial(0.0f, 0.447f, 0.545f);
				if(objMtrl == null) { BSAuxFuncs.GetStandardMaterial(0.0f, 0.447f, 0.545f); }
				pr_MRE_BackPlane = BackPlane.gameObject.GetComponent<MeshRenderer>();
				pr_MRE_BackPlane.material = objMtrl;
				
				// PointLight
				RefLight = new GameObject("AuxPointLight");
				pr_Comp_RefLight = RefLight.AddComponent<Light>();
				pr_Comp_RefLight.type = LightType.Point;
				pr_Comp_RefLight.color = new Color(0.6f, 0.6f, 0.0f);
				pr_Comp_RefLight.range = 1.0f;
				pr_Comp_RefLight.intensity = 1.0f;		

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			pr_Comp_SubCamera.enabled = true;
			pr_Comp_RefLight.enabled = true;
			
			{
				THAlphaObject3D[] objArr = { Cylinder, BoxMG, Box01, Sphere, Star, Torus, Crystal };
				foreach(var o in objArr)
				{
					o.collCount = 0;
					o.state = c_NORMAL;
				}
				
				pr_degCannon = 0.0f;
			}
			
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				BSAuxFuncs.SetVisible(1, BoxMG.gameObject);
				Sphere.SetIdentity();
				BoxMG.SetIdentity();
				
				Box = BoxMG;
				
				Sphere.type = c_DISK;
				Sphere.radius = 1.0f;
				
				Box.type = c_RECT;
				Box.width = 6.0f;
				Box.height = 4.0f;
				Box.initRectVerts = new Vector3[]
				{
					new Vector3(3.0f, 0.0f,  2.0f),
					new Vector3(3.0f, 0.0f,  -2.0f),
					new Vector3(-3.0f, 0.0f, -2.0f),
					new Vector3(-3.0f, 0.0f, 2.0f)
				};
				
				pr_degA = 0.0f;	
				
				CamPos = new Vector3(13.57501f, 11.12496f, -8.550022f);
				TrgPos = new Vector3(13.57501f, 4.325006f, 0f);
				uJ = new Vector3(0f, 0.7826544f, 0.6224565f);
				
				pr_Comp_RefLight.enabled = false;
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -40, 0);
				}
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, MiniShell.gameObject);
				MiniShell.SetIdentity();
				Shell = MiniShell;
				
				BSAuxFuncs.SetVisible(1, Stand[0].gameObject);
				Stand[0].SetIdentity();
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				
				Cannon.SetWorldPosition(13.0f, 1.5f, -5.9f);
				Cannon.SetLocalScale(0.6f);
				
				Stand[0].SetWorldPosition(13, 0, 0);		
				Stand[0].width = 0.5177f;  
				Stand[0].height = 0.1f;
				
				Shell.SetWorldPosition(-1000, 0, 0);
				
				i_radShell = 0.09f;
				
				CamPos = new Vector3(11.44768f, 2.25562f, -7.070923f);
				TrgPos = new Vector3(15.82598f, -0.3505313f, 3.167265f);
				uJ = new Vector3(-0.0271975f, 0.9658951f, 0.2575014f);
				
				pr_Comp_RefLight.enabled = false;
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, 40, 0);
				}
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, MiniShell.gameObject);
				MiniShell.SetIdentity();
				Shell = MiniShell;
				
				BSAuxFuncs.SetVisible(1, Ray.gameObject);
				Ray.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				BSAuxFuncs.SetVisible(1, VisRay.gameObject);
				VisRay.SetIdentity();
				
				Vector3[] prr =
				{
					new Vector3(13, 0, 0), new Vector3(11, 0, 2), new Vector3(15, 0, 2),
					new Vector3(9, 0, 4), new Vector3(17, 0, 4)
				};
				
				for (int i = 0; i < 5; i++)
				{
					BSAuxFuncs.SetVisible(1, Stand[i].gameObject);
					Stand[i].SetIdentity();
					
					Stand[i].SetWorldPosition(prr[i]);
					Stand[i].width = 0.5177f;  
					Stand[i].height = 0.1f;
				}
				
				Cannon.SetWorldPosition(13.0f, 1.5f, -5.9f);
				Cannon.SetLocalScale(0.6f);
				
				Shell.SetWorldPosition(-1000, 0, 0);
				
				i_radShell = 0.09f;

				i_targetObjects = Stand;
				
				Ray.SetWorldPosition(Cannon.GetWorldPosition() + 0.9f * Vector3.forward);
				Ray.SetLocalScale(1, 1, 50);
				
				pr_Comp_RefLight.range = 0.2f;
				pr_Comp_RefLight.intensity = 3.0f;
				
				CamPos = new Vector3(11.44768f, 2.25562f, -7.070923f);
				TrgPos = new Vector3(15.82598f, -0.3505313f, 3.167265f);
				uJ = new Vector3(-0.0271975f, 0.9658951f, 0.2575014f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, 40, 0);
				}
			}
			else if (MatchCase(4, 5))
			{
				BSAuxFuncs.SetVisible(1, Ray.gameObject);
				Ray.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				Sphere.SetIdentity();
				BSAuxFuncs.SetVisible(1, Box01.gameObject);
				Box01.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cylinder.gameObject);
				Cylinder.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, VisRay.gameObject);
				VisRay.SetIdentity();
				
				Box = Box01;
				
				i_radShell = 0.18f;
				if (MatchCase(4))
				{
					BSAuxFuncs.SetVisible(1, UShell[0].gameObject);
					Shell = UShell[0];
					Shell.SetIdentity();
					Shell.SetWorldPosition(-1000, 0, 0);
				}
				else  
				{
					for (int i = 0; i < UShell.Length; i++)
					{
						BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
						UShell[i].SetIdentity();
						UShell[i].SetWorldPosition(-1000, 0, 0);
					}
				}
				
				Sphere.type = c_DISK;
				Sphere.SetWorldPosition(18.0f, 1.5f, 6.0f);
				
				Box.type = c_RECT;
				Box.SetWorldPosition(12.0f, 1.5f, 5.8f);
				Box.SetWorldRotation(0.0f, -40.0f, 0.0f);
				Box.SetLocalScale(3.0f, 2.5f, 1.25f);
				Box.initRectVerts = new Vector3[]	
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				
				Cylinder.type = c_RECT;
				Cylinder.SetWorldPosition(6.0f, 1.5f, 5.0f);
				Cylinder.SetWorldRotation(0.0f, 50.0f, 0.0f);
				Cylinder.SetLocalScale(1.4f, 1.4f, 4.0f);
				Cylinder.initRectVerts = new Vector3[]
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				
				Cannon.SetWorldPosition(12.0f, 1.5f, -5.0f);
				
				Ray.SetWorldPosition(Cannon.GetWorldPosition() + 0.9f * Vector3.forward);
				Ray.SetLocalScale(1, 1, 50);
				
				pr_Comp_RefLight.range = 1.0f;
				pr_Comp_RefLight.intensity = 1.0f;
				
				CamPos = new Vector3(10.75209f, 4.910862f, -8.329793f);
				TrgPos = new Vector3(13.17706f, 0.4521329f, 3.276584f);
				uJ = new Vector3(-0.01186869f, 0.932589f, 0.360745f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, 40, 0);
				}
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Ray.gameObject);
				Ray.SetIdentity();
				BSAuxFuncs.SetVisible(1, Cannon.gameObject);
				Cannon.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, VisRay.gameObject);
				VisRay.SetIdentity();
				
				i_radShell = 0.18f;
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetIdentity();
					UShell[i].SetWorldPosition(-1000, 0, 0);
				}
				
				BSAuxFuncs.SetVisible(1, Star.gameObject);
				Star.SetIdentity();
				BSAuxFuncs.SetVisible(1, Torus.gameObject);
				Torus.SetIdentity();
				BSAuxFuncs.SetVisible(1, Crystal.gameObject);
				Crystal.SetIdentity();
				
				Star.type = c_RECT;
				Star.SetWorldPosition(5.0f, 1.0f, 5.0f);
				Star.SetWorldRotation(0.0f, -60.0f, 0.0f);
				Star.initRectVerts = new Vector3[]
				{
					new Vector3( 1.5f, 1.5f,   0.4f),
					new Vector3( 1.5f, 1.5f,  -0.4f),
					new Vector3(-1.5f, 1.5f,  -0.4f),
					new Vector3(-1.5f, 1.5f,   0.4f)
				};
				Star.width = 3.0f;
				Star.height = 0.64f;
				
				Torus.type = c_DISK;
				Torus.SetWorldPosition(12.0f, 2.5f, 6.0f);
				Torus.radius = 1.68f;
				
				Crystal.type = c_RECT;
				Crystal.SetWorldPosition(18.6f, 2.5f, 5.5f);
				Crystal.SetWorldRotation(0.0f, 50.0f, 0.0f);
				Crystal.initRectVerts = new Vector3[]
				{
					new Vector3( 1.0f, 0.0f,  1.0f),
					new Vector3( 1.0f, 0.0f, -1.0f),
					new Vector3(-1.0f, 0.0f, -1.0f),
					new Vector3(-1.0f, 0.0f,  1.0f)
				};
				Crystal.width = 2.0f;
				Crystal.height = 2.0f;
				
				Cannon.SetWorldPosition(12.0f, 2.5f, -5.0f);
				
				Ray.SetWorldPosition(Cannon.GetWorldPosition() + 0.9f * Vector3.forward);
				Ray.SetLocalScale(1, 1, 50);
				
				pr_Comp_RefLight.range = 1.0f;
				pr_Comp_RefLight.intensity = 1.0f;		
				
				CamPos = new Vector3(12f, 6.648917f, -9.466519f);
				TrgPos = new Vector3(12f, 3.050638f, 3.159437f);
				uJ = new Vector3(0f, 0.9617076f, 0.2740777f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, 30, 0);
				}
			}		
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
			
			
			// SubCamera 
			{
				BSAuxFuncs.SetVisible(1, BackPlane.gameObject);
				BackPlane.SetIdentity();
				
				Vector3 subCP = new Vector3(0, 10, 0);
				if (MatchCase(1))
				{
					subCP = new Vector3(12.0f, 15.0f, 6.0f);
					pr_Comp_SubCamera.orthographicSize = 6.0f;
				}
				else if (MatchCase(2))
				{
					subCP = new Vector3(13.0f, 3.6f, 0.0f);        
					pr_Comp_SubCamera.orthographicSize = 1.5f;
				}
				else if (MatchCase(3))
				{
					subCP = new Vector3(13.0f, 8.0f, 1.8f);
					pr_Comp_SubCamera.orthographicSize = 2.5f;
				}
				else if (MatchCase(4, 5))
				{
					subCP = new Vector3(12.0f, 8.0f, 5.0f);     
					pr_Comp_SubCamera.orthographicSize = 5.0f;
				}
				else if (MatchCase(6))
				{
					subCP = new Vector3(12.0f, 8.0f, 5.0f);     
					pr_Comp_SubCamera.orthographicSize = 4.5f;
				}
				
				var vp = subCP;
				vp.y = 1.5f;
				VisRay.SetWorldPosition(vp);
				
				SubCamera.transform.position = subCP;
				SubCamera.transform.rotation = Quaternion.AngleAxis(90, Vector3.right);
				
				Rect rectBounds = new Rect(0.66f, 0.66f, 0.32f, 0.32f);
				pr_Comp_SubCamera.depth = 2;
				pr_Comp_SubCamera.rect = rectBounds;
				
				// BackPlane
				float[] rbds2 = { rectBounds.x+rectBounds.width/2.0f,
				rectBounds.y+rectBounds.height/2.0f, rectBounds.width * 1.04f, rectBounds.height * 1.057f   };
				
				float[] iplaneData = BSAuxFuncs.CalcImagePlaneData(); 
				
				object[] trs = BSAuxFuncs.CalcUIRectTransformN_LowerLeft(rbds2, iplaneData);
				Vector3 pos = (Vector3)trs[0];
				Quaternion rot = (Quaternion)trs[1];
				Vector3 scale = (Vector3)trs[2];
				BackPlane.transform.position = pos;
				BackPlane.transform.rotation = rot;
				BackPlane.transform.localScale = scale;
			}
		}

	}

}
