//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject3D;
using static OQJ_THCollisionTest.THCollisionTest;



namespace OQJ_Sec423
{
	public class Sec423 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THAlphaObject3D Tank;
		THAlphaObject3D Cylinder;
		THAlphaObject3D Box;
		THAlphaObject3D Sphere;
		THAlphaObject3D[] Coin;
		THAlphaObject3D[] Letter;
		THAlphaObject3D Chaser;
		THAlphaObject3D Cube;
		
		THObject3D CS_2D;
		THObject2D Square;
		THObject2D Rect;
		
		THObject3D Floor;
		
		THObject3D BackPlane;	
		GameObject SubCamera;
		
		GameObject MainCamera;
		GameObject DLight;


		// Instance Variables
		THAlphaObject3D[] i_targetObjects;
		
		float i_degTank;
		int i_reboundCount;
		Vector2 i_reboundDirection;
		
		float i_degSquare;
		float i_cosA;
		
		THAlphaObject3D[] pr_collisionObjects;
		
		Vector3 pr_difCamVtr;
		int pr_autoSettingCounter;
		bool pr_AUTO_SETTING;	
		Vector3 pr_utrCamZ;
		
		float pr_shmS;
		float pr_shmE;
		float pr_degObj;
		float pr_acc;
		float pr_velChaser;
		
		
		readonly int c_DISK = 10;
		readonly int c_RECT = 11;
		
		readonly int c_NORMAL = 100;
		readonly int c_HIT = 101;
		readonly int c_TEMP = 102;
		
		Camera pr_COMP_SubCamera;
		MeshRenderer pr_MRE_BackPlane;




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = new THAlphaObject3D[] { Cylinder, Box, Sphere };
				
				i_INITIALIZED = true;
			}
			
			UserMotion();
			
			ObjectMotion();
			
			
			THAlphaObject3D collObj = null;
			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				if (bColl)
				{
					collObj = target;
					break;	
				}
			}
			
			Collision(collObj);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = Coin;  
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.R))
			{
				ResetCoins();
			}
			
			UserMotion();
			
			ObjectMotion();
			
			
			THAlphaObject3D collObj = null;
			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				
				if (bColl)
				{
					collObj = target;
					break;	
				}
			}
			
			Collision(collObj);
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				Vector2 u = new Vector2(2.0f + 0.5f, 0.8f + 0.5f).normalized;  
				i_cosA = u.x;
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKey(KeyCode.R))
			{
				RectMotion();
			}
			
			
			THMatrix3x3 M, T, R;
			if (i_reboundCount > 0)
			{
				Vector2 wp = Square.GetPosition();
				wp += 0.03f * i_reboundDirection;
				
				R = TH2DMath.GetRotation3x3(i_degSquare);
				T = TH2DMath.GetTranslation3x3(wp);
				M = T * R;
				Square.SetMatrix(M);
				
				i_reboundCount--;
				return;
			}
			
			
			bool MOVE = false;
			if (Input.GetKey(KeyCode.S))
			{
				MOVE = true;
			}
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				i_degSquare += Input.GetKey(KeyCode.L) ? -2.5f : 2.5f;
			}
			
			R = TH2DMath.GetRotation3x3(i_degSquare);
			Vector2 forwardDir = R * Vector2.up;
			
			float mv = MOVE ? 0.05f : 0.0f;
			mv = THUtil.IsShiftDown() ? -mv : mv;
			Vector2 S = Square.GetPosition() + mv * forwardDir;
			T = TH2DMath.GetTranslation3x3(S);
			
			M = T * R;
			Square.SetMatrix(M);
			
			
			Vector2[] vertsA = GetVerts(Square);
			Vector2[] vertsB = GetVerts(Rect);
			bool bColl = CollisionTest_Rect_Rect(vertsA, vertsB);
			
			if (bColl)
			{
				i_reboundCount = 4;
				
				THMatrix3x3 mtx = Rect.GetMatrix();
				Vector2 E = Rect.GetPosition();
				Vector2 lx = mtx.GetColumn(0).normalized;
				Vector2 ly = mtx.GetColumn(1).normalized;
				
				Vector2 u = (S - E).normalized;
				float cosB = Vector2.Dot(u, lx);
				
				if (i_cosA <= cosB && cosB <= 1.0f)
				{
					i_reboundDirection = lx;
				}
				else if (-1.0f <= cosB && cosB <= -i_cosA)
				{
					i_reboundDirection = -lx;
				}
				else
				{
					if (Vector3.Dot(u, ly) >= 0.0f)
					{
						i_reboundDirection = ly;
					}
					else
					{
						i_reboundDirection = -ly;
					}
				}
			}
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = new THAlphaObject3D[] { Cylinder, Box, Sphere };
				
				i_INITIALIZED = true;
			}
			
			UserMotion();
			
			ObjectMotion();
			
			
			
			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				
				if (bColl)
				{
					Tank.reboundCount = 1;
					Tank.reboundSpeed = 0.084f;
					
					Vector3 wp = Tank.GetWorldPosition();
					Tank.reboundDirection = CalcReboundDirection(wp, target);
					
					break;
				}
			}
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = Letter;  
				
				i_INITIALIZED = true;
			}
			
			UserMotion();
			
			
			
			
			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				
				if (bColl)
				{
					Tank.reboundCount = 1;
					Tank.reboundSpeed = 0.08f;
					
					Vector3 wp = Tank.GetWorldPosition();
					Tank.reboundDirection = CalcReboundDirection(wp, target);
					
					break;
				}
			}
		}


		void Code6()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = new THAlphaObject3D[] { Chaser };
				
				i_INITIALIZED = true;
			}
			
			UserMotion();
			
			ObjectMotion();
			
			
			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				
				if (bColl)
				{
					float speed = GetChaserSpeed();
					Tank.reboundCount = (speed < 0.14f) ? 12 : (int)(speed * 125);
					Tank.reboundSpeed = (speed < 0.14f) ? 0.084f : speed * 3.5f; 
					
					Vector3 wp = Tank.GetWorldPosition();
					Tank.reboundDirection = CalcReboundDirection(wp, target);
					
					Hit(Tank);
					break;
				}
			}
		}


		void Code7()
		{
			if (!i_INITIALIZED)
			{
				i_targetObjects = new THAlphaObject3D[] { Cube };

				i_INITIALIZED = true;
			}

			UserMotion();

			if (Cube.reboundCount > 0)
			{
				Vector3 posCube = Cube.GetWorldPosition() + Cube.reboundSpeed * Cube.reboundDirection;
				Cube.SetWorldPosition(posCube);

				Cube.reboundCount--;
			}


			foreach (var target in i_targetObjects)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
													Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}


				if (bColl)
				{
					target.reboundCount = 1;
					target.reboundSpeed = 0.03f;

					Vector3 posTank = Tank.GetWorldPosition();
					target.reboundDirection = -CalcReboundDirection(posTank, target);

					Vector3 forwardDir = Tank.GetMatrix().GetColumn(2);
					posTank -= 0.055f * forwardDir;
					Tank.SetWorldPosition(posTank);

					break;
				}
			}
		}




		Vector3 CalcReboundDirection(Vector3 posTank, THAlphaObject3D target)
		{
			Vector3 posTrg = target.GetWorldPosition();
			Vector3 vtr = posTank - posTrg;
			vtr.y = 0.0f;
			Vector3 u = vtr.normalized;
			
			if (target.type == c_DISK)
			{
				return u;
			}
			// c_RECT
			{
				Matrix4x4 M = target.GetMatrix();
				Vector3 lx = M.GetColumn(0).normalized;
				Vector3 lz = M.GetColumn(2).normalized;
				
				float cosB = Vector3.Dot(u, lx);
				
				if (target.cosA <= cosB && cosB <= 1.0f)
				{
					return lx;
				}
				else if (-1.0f <= cosB && cosB <= -target.cosA)
				{
					return -lx;
				}
				
				if (Vector3.Dot(u, lz) > 0.0f)
				{
					return lz;
				}
				
				return -lz;
			}
		}
		
		
		void UserMotion()
		{
			if (Tank.reboundCount > 0)
			{
				Vector3 wp = Tank.GetWorldPosition() + Tank.reboundSpeed * Tank.reboundDirection;  
				Tank.SetWorldPosition(wp);
				
				Tank.reboundCount--;
				return;
			}
			
			bool MOVE = false;	
			if (Input.GetKey(KeyCode.S))
			{
				MOVE = true;
			}
			
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				i_degTank += Input.GetKey(KeyCode.H) ? -3f : 3f;
			}
			
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degTank, Vector3.up);
			Vector3 forwardDir = R * Vector3.forward;
			
			float mv = MOVE ? 0.08f : 0.0f;
			if (MatchCase(6) && MOVE)
			{
				if (Input.GetKey(KeyCode.A) && (pr_acc < 0.22f))
				{
					pr_acc += THUtil.IsShiftDown() ? -0.0002f : 0.0002f;
				}
				
				mv += pr_acc;	
			}
			
			mv = THUtil.IsShiftDown() ? -mv : mv;
			Vector3 pos = Tank.GetWorldPosition() + mv * forwardDir;
			Matrix4x4 T = TH3DMath.GetTranslation4x4(pos);
			
			Matrix4x4 M = T * R;
			Tank.SetMatrix(M);
		}
		
		
		void ObjectMotion()
		{
			if (MatchCase(1, 4))
			{
				// Cylinder
				Quaternion rot = Cylinder.GetWorldRotation();
				rot = Quaternion.AngleAxis(0.15f, Vector3.up) * rot;
				Cylinder.SetWorldRotation(rot.normalized);
				
				// Box
				pr_shmE += 0.45f;
				float A = 60.0f;
				float deg = A * Mathf.Sin(pr_shmE * Mathf.Deg2Rad);
				Box.SetWorldRotation(deg, Vector3.up);
				
				// Sphere
				pr_shmS += 0.3f;
				A = 1.1f;
				float scl = 0.5f * A * (-Mathf.Cos(pr_shmS * Mathf.Deg2Rad) + 1.0f) + 0.5f;
				Sphere.SetLocalScale(scl);
			}
			else if (MatchCase(2))
			{
				pr_degObj += 0.75f;
				Quaternion rot = Quaternion.AngleAxis(pr_degObj, Vector3.up);
				
				for (int i = 0; i < Coin.Length; i++)
				{
					Coin[i].SetWorldRotation(rot);
				}
			}
			else if (MatchCase(6))
			{
				Vector3 posTank = Tank.GetWorldPosition();
				Vector3 posSph = Chaser.GetWorldPosition();
				Vector3 w = posTank - posSph;
				w.y = 0.0f;
				Vector3 dir = w.normalized;
				Vector3 axis = Vector3.Cross(Vector3.up, dir).normalized;
				
				float dist = w.magnitude;
				if (dist >= 3.0f)
				{
					float acc = 0.0001f;
					if(pr_velChaser >= 0.2f)
					{
						acc = 0.00005f;
					}
					
					pr_velChaser += acc;
				}
				
				float r = 1.8f;
				float deg = pr_velChaser / (2.0f * Mathf.PI * r) * 360.0f;
				Quaternion q = Quaternion.AngleAxis(deg, axis);
				
				posSph += pr_velChaser * dir;
				Chaser.SetWorldPosition(posSph);
				
				Quaternion rot = q * Chaser.GetWorldRotation();
				Chaser.SetWorldRotation(rot);
			}
		}
		
		
		void RectMotion()
		{
			if (Input.GetKey(KeyCode.R))
			{
				pr_degObj += THUtil.IsShiftDown() ? -0.5f : 0.5f;
				
				Rect.SetMatrix(	TH2DMath.GetTranslation3x3(8, 5) * TH2DMath.GetRotation3x3(pr_degObj) *
				TH2DMath.GetScale3x3(4.0f, 1.6f));
			}
		}
		
		
		void ResetCoins()
		{
			Vector3 wvtr = new Vector3(0.0f, 1.6f, 10.0f);	
			Vector3 C0 = new Vector3(13, 0, -1);	
			for (int i = 0; i < Coin.Length; i++)
			{
				float deg = -60 + 30 * i;
				Vector3 cp = C0 + Quaternion.AngleAxis(deg, Vector3.up) * wvtr;
				Coin[i].SetWorldPosition(cp);
			}
		}
		
		
		void Collision(THAlphaObject3D coll_obj)
		{
			if (MatchCase(1))
			{
				if(coll_obj == null)
				{
					Tank.state = c_NORMAL;
					foreach (var trg in i_targetObjects)
					{
						if (trg.state == c_HIT) { trg.state = c_NORMAL; }
					}
				}
				else
				{
					Tank.state = c_HIT;
					coll_obj.state = c_HIT;
				}
			}
			else if (MatchCase(2))
			{
				if(coll_obj == null || coll_obj.GetWorldPosition().x < -999) 
				{ return; }
				
				Tank.collCount = 16;
				Tank.state = c_TEMP;
				
				coll_obj.SetWorldPosition(-1000, 0, 0);
			}
		}
		
		
		void Hit(THAlphaObject3D obj)		
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = (pr_velChaser < 0.14f) ? 12 : (int)(pr_velChaser * 125);
				obj.state = c_HIT;
				
				pr_acc = 0.0f;
				pr_velChaser = 0.025f;
			}
		}
		
		
		Vector2[] GetVerts(THObject2D obj)	 
		{
			Vector3[] initVerts =
			{
				new Vector3( 0.5f,  0.5f, 1.0f),
				new Vector3( 0.5f, -0.5f, 1.0f),
				new Vector3(-0.5f, -0.5f, 1.0f),
				new Vector3(-0.5f,  0.5f, 1.0f)
			};
			
			THMatrix3x3 M = obj.GetMatrix();
			Vector2[] verts =
			{
				M * initVerts[0],
				M * initVerts[1],
				M * initVerts[2],
				M * initVerts[3]
			};
			
			return verts;
		}
		
		
		float GetChaserSpeed()
		{
			return pr_velChaser;
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			if (MatchCase(6))
			{
				if (Input.GetKeyDown(KeyCode.V) && !pr_AUTO_SETTING)
				{
					pr_AUTO_SETTING = true;
					pr_autoSettingCounter = 0;
				}
				
				
				if (pr_AUTO_SETTING)
				{
					int max = 90;
					float t = 1.0f / (float)(max - pr_autoSettingCounter);
					t = Mathf.Pow(t, 0.9f);
					
					pr_autoSettingCounter++;
					
					Vector3 posTank = Tank.GetWorldPosition();
					
					Vector3 wvtr = Chaser.GetWorldPosition() - posTank;
					wvtr.y = 0.0f;
					Vector3 u1 = wvtr.normalized;
					
					float deg;
					if (pr_autoSettingCounter > 80)
					{
						deg = TH3DMath.CalcDegree_V1toV2(Vector3.forward, pr_utrCamZ, Vector3.up);
					}
					else
					{
						deg = TH3DMath.CalcDegree_V1toV2(Vector3.forward, u1, Vector3.up);
						pr_utrCamZ = u1;
					}
					
					Quaternion staQ = MainCamera.transform.rotation;
					Quaternion endQ = Quaternion.Euler(30, deg, 0);
					Quaternion rot = Quaternion.Slerp(staQ, endQ, t);
					
					Vector3 pos = rot * new Vector3(0.0f, 0.0f, -11.5f);
					pos.y += 0.6f;
					pr_difCamVtr = pos;
					
					pos += posTank;
					
					MainCamera.transform.position = pos;
					MainCamera.transform.rotation = rot;
					
					if (pr_autoSettingCounter == max)
					{
						pr_AUTO_SETTING = false;
					}
				}
			}
		}


		void PostProcess()
		{
			if (MatchCase(2, 6))	
			{
				foreach (var o in pr_collisionObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							o.state = c_NORMAL;
						}
					}
				}
			}
			else if (MatchCase(3))
			{
				Vector3 wp = Square.transform.position;
				wp.z -= 0.02f;
				Square.transform.position = wp;
			}
			
			
			if (MatchCase(6))	
			{
				Vector3 posTank = Tank.GetWorldPosition();
				Vector3 CP = posTank + pr_difCamVtr;
				MainCamera.transform.position = CP;
				
				BSData.CameraController3D.UpdateCameraData();
			}
		}


		static Sec423 pr_instance = new Sec423(7);

		public static Sec423 PRGSI()
		{
			return pr_instance;
		}


		private Sec423(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Tank = CreateObject_Alpha3D("ZTank", true);
				Sphere = CreateObject_Alpha3D("PointB", true);
				Box = CreateObject_Alpha3D("UnitCube01", true);
				Cylinder = CreateObject_Alpha3D("XZCylinder01", true);
				Cube = CreateObject_Alpha3D("DBlCube", true);
				Floor = CreateObject_Obj3D("FloorCh", true);
				
				Chaser = CreateObject_Alpha3D("Chaser");
				Square = CreateObject_Obj2D("Square01");
				Rect = CreateObject_Obj2D("UnitRect");
				CS_2D = CreateObject_Obj3D("TmpCS_2D");
				
				var src_Coin = CreateObject_Alpha3D("CCoin", true);
				Coin = new THAlphaObject3D[5];
				for (int i = 0; i < Coin.Length; i++)
				{
					Coin[i] = CreateObject_InsAlpha3D(src_Coin.gameObject, true);
				}
				
				Letter = new THAlphaObject3D[5];
				for (int i = 0; i < Letter.Length; i++)
				{
					Letter[i] = CreateObject_Alpha3D("LCM_" + i);
				}
				
				
				Material mtrlNml = BSAuxFuncs.GetStandardMaterial(0.755f, 0.543f, 0.367f, 0.0f);  
				Material mtrlHit = BSAuxFuncs.GetStandardMaterial(0.80f, 0.34f, 0.34f, 0.1f);
				Sphere.gameObject.GetComponent<MeshRenderer>().material = mtrlNml;
				Sphere.SetData(c_NORMAL, new Material[] { mtrlNml });
				Sphere.SetData(c_HIT, new Material[] { mtrlHit });
				
				mtrlNml = BSAuxFuncs.GetStandardMaterial(0.35f, 0.68f, 0.60f);
				Box.gameObject.GetComponent<MeshRenderer>().material = mtrlNml;
				Box.SetData(c_NORMAL, new Material[] { mtrlNml });
				Box.SetData(c_HIT, new Material[] { mtrlHit });
				
				Cylinder.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetStandardMaterial(0.3f, 0.4f, 0.7f);
				Cylinder.SetData(c_NORMAL, new Material[] { BSAuxFuncs.GetStandardMaterial(0.3f, 0.4f, 0.7f) });
				Cylinder.SetData(c_HIT, new Material[] { mtrlHit });
				
				
				var nMts = Tank.gameObject.GetComponent<MeshRenderer>().materials;
				var hMts = new Material[]
				{
					BSAuxFuncs.GetStandardMaterial(0.78f, 0.23f, 0.18f),
					BSAuxFuncs.GetStandardMaterial(0.86f, 0.38f, 0.43f)
				};
				var tMts = new Material[]
				{
					BSAuxFuncs.GetStandardMaterial(0.81f, 0.79f, 0.65f),
					BSAuxFuncs.GetStandardMaterial(0.85f, 0.85f, 0.48f)
				};
				
				Tank.SetData(c_NORMAL, nMts);
				Tank.SetData(c_HIT, hMts);
				Tank.SetData(c_TEMP, tMts);	
				
				Rect.gameObject.GetComponent<MeshRenderer>().material =
				BSAuxFuncs.GetUnlitMaterial(0.625f, 0.878f, 0.703f);	
				
				// Main Camera
				MainCamera = BSAuxFuncs.GetMainCamera();
				
				// Sub Camera
				GameObject go = new GameObject();
				go.name = "Camera_2";
				pr_COMP_SubCamera = go.AddComponent<Camera>();
				pr_COMP_SubCamera.nearClipPlane = 0.01f;
				pr_COMP_SubCamera.farClipPlane = 50.0f;
				pr_COMP_SubCamera.clearFlags = CameraClearFlags.SolidColor;
				pr_COMP_SubCamera.backgroundColor = BSData.BackgroundColor;
				pr_COMP_SubCamera.orthographic = true;  
				pr_COMP_SubCamera.orthographicSize = 4.0f;
				pr_COMP_SubCamera.fieldOfView = BSData.c_DefaultFOV;
				pr_COMP_SubCamera.enabled = false;
				SubCamera = go;  
				
				// BackPlane (init XY 1x1)
				BackPlane = CreateObject_InsObj3D(Rect.gameObject);
				Material objMtrl = BSAuxFuncs.GetUnlitMaterial(0.0f, 0.447f, 0.545f);
				if(objMtrl == null) { BSAuxFuncs.GetStandardMaterial(0.0f, 0.447f, 0.545f); }
				pr_MRE_BackPlane = BackPlane.gameObject.GetComponent<MeshRenderer>();
				pr_MRE_BackPlane.material = objMtrl;
				
				// Directional Light
				DLight = BSAuxFuncs.GetDirectionalLight();		

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			pr_COMP_SubCamera.enabled = false;
			
			{
				BSAuxFuncs.SetVisible(1, Tank.gameObject);
				Tank.SetIdentity();
				
				Tank.type = c_RECT;
				Tank.initRectVerts = new Vector3[]
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				
				i_degTank = 0;
				
				THAlphaObject3D[] objArr = { Tank, Cylinder, Box, Sphere };
				foreach(var o in objArr)
				{
					o.collCount = 0;
					o.state = c_NORMAL;
				}
			}
			
			if (DLight != null)
			{
				DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
			}
			
			if (MatchCase(1, 4))
			{
				CrdSys.SetWorldPosition(0.0f, -0.05f, 0.0f);
				
				BSAuxFuncs.SetVisible(1, Cylinder.gameObject);
				BSAuxFuncs.SetVisible(1, Box.gameObject);
				BSAuxFuncs.SetVisible(1, Sphere.gameObject);
				Cylinder.SetIdentity();
				Box.SetIdentity();
				Sphere.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, Floor.gameObject);
				Floor.SetIdentity();
				Floor.SetWorldPosition(12.0f, 0.0f, 12.0f);
				
				Floor.SetWorldPosition(15.0f, 0.0f, 16.0f);     
				Floor.SetLocalScale(2, 1, 2);
				
				i_degTank = 45;
				Tank.SetWorldPosition(8.6f, 0.0f, 0.0f);
				Tank.SetWorldRotation(i_degTank, Vector3.up);
				
				Sphere.type = c_DISK;
				Sphere.SetWorldPosition(21.5f, 1.5f, 5.0f);
				
				Box.type = c_RECT;
				Box.SetWorldPosition(14.0f, 1.5f, 5.8f);
				Box.SetWorldRotation(0.0f, -40.0f, 0.0f);
				Box.SetLocalScale(5.0f, 3.0f, 1.4f);
				Box.initRectVerts = new Vector3[]
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				Vector2 cv = new Vector2(2.5f + 0.5f, 0.7f + 0.5f).normalized;
				Box.cosA = cv.x;
				
				Cylinder.type = c_RECT;
				Cylinder.SetWorldPosition(6.0f, 1.5f, 5.0f);
				Cylinder.SetWorldRotation(0.0f, 50.0f, 0.0f);
				Cylinder.SetLocalScale(2.0f, 2.0f, 6.0f);
				Cylinder.initRectVerts = new Vector3[]
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				cv = new Vector2(1.0f + 0.5f, 3.0f + 0.5f).normalized;
				Cylinder.cosA = cv.x;
				
				CamPos = new Vector3(13.80001f, 6.675005f, -7.425028f);
				TrgPos = new Vector3(13.80001f, 3.325002f, 0f);
				uJ = new Vector3(0f, 0.9115195f, 0.4112568f);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, 0, 0);
				}
			}
			else if (MatchCase(2))
			{
				CrdSys.SetWorldPosition(0.0f, -0.05f, 0.0f);
				
				BSAuxFuncs.SetVisible(1, Floor.gameObject);
				Floor.SetIdentity();
				Floor.SetWorldPosition(12.0f, 0.0f, 12.0f);
				
				i_degTank = 45;
				Tank.SetWorldPosition(8.6f, 0.0f, 0.0f);
				Tank.SetWorldRotation(i_degTank, Vector3.up);
				
				// Coin
				float wid = 1.8f, hgt = 0.32f;
				float w2 = wid * 0.5f, h2 = hgt * 0.5f;
				Vector3[] initVrts =
				{
					new Vector3( w2, 0.0f,  h2),
					new Vector3( w2, 0.0f, -h2),
					new Vector3(-w2, 0.0f, -h2),
					new Vector3(-w2, 0.0f,  h2)
				};
				
				for (int i = 0; i < Coin.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Coin[i].gameObject);
					Coin[i].SetIdentity();
					
					Coin[i].type = c_RECT;
					Coin[i].initRectVerts = initVrts;
					Coin[i].width = wid;
					Coin[i].height = hgt;
				}
				
				ResetCoins();	
				
				pr_collisionObjects = new THAlphaObject3D[]{ Tank };
				pr_degObj = 60;	
				
				CamPos = new Vector3(13f, 8.899996f, -6.675025f);
				TrgPos = new Vector3(13f, 4.325006f, 0f);
				uJ = new Vector3(0f, 0.8248539f, 0.565346f);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(0, CrdSys.gameObject);
				BSAuxFuncs.SetVisible(0, Tank.gameObject);
				
				BSAuxFuncs.SetVisible(1, CS_2D.gameObject);
				CS_2D.SetIdentity();
				BSAuxFuncs.SetVisible(1, Square.gameObject);
				Square.SetIdentity();
				BSAuxFuncs.SetVisible(1, Rect.gameObject);
				Rect.SetIdentity();
				
				Square.SetPosition(4, 4);
				
				Rect.SetPosition(8, 5);
				Rect.SetScale(4.0f, 1.6f);
				
				pr_degObj = 0;	
				
				CamPos = new Vector3(8f, 5.0f, -12.79996f);
				TrgPos = new Vector3(8f, 5.0f, 0f);
				uJ = new Vector3(0f, 1f, 0f);
			}
			else if (MatchCase(5))
			{
				Tank.SetWorldPosition(20.5f, 0.0f, -4.0f);
				
				float s = 1.25f;  
				Vector3[] urvs =   
				{
					new Vector3(1.125797f*s, 0.0f, 0.361412f), new Vector3(0.177092f*s, 0.0f, 0.361412f),
					new Vector3(1.142061f*s, 0.0f, 0.361412f), new Vector3(1.111341f*s, 0.0f, 0.361412f),
					new Vector3(1.205308f*s, 0.0f, 0.361412f)
				};
				
				for (int i = 0; i < Letter.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Letter[i].gameObject);
					Letter[i].SetIdentity();
					Letter[i].SetWorldPosition(9.0f + i * 4.7f, 0.0f, 0.0f);
					
					Letter[i].type = c_RECT;
					
					Vector3 v = urvs[i];
					Letter[i].initRectVerts = new Vector3[]
					{
						new Vector3( v.x, 0.0f,  v.z),
						new Vector3( v.x, 0.0f, -v.z),
						new Vector3(-v.x, 0.0f, -v.z),
						new Vector3(-v.x, 0.0f,  v.z)
					};
					
					Vector2 cv = new Vector2(v.x + 0.5f, v.z + 0.5f).normalized;
					Letter[i].cosA = cv.x;
				}	
				
				CamPos = new Vector3(24.83004f, 4.826806f, -12.09338f);
				TrgPos = new Vector3(20.26129f, 1.130534f, 1.546135f);
				uJ = new Vector3(0.005683256f, 0.9646886f, 0.2633317f);
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Chaser.gameObject);
				Chaser.SetIdentity();
				
				Vector3 posTank = new Vector3(8.6f, 0.0f, 0.0f);
				Tank.SetWorldPosition(posTank);
				
				Chaser.type = c_DISK;
				Chaser.radius = 1.8f;
				Chaser.SetWorldPosition(12.0f, 1.8f, 18.8f);
				
				pr_collisionObjects = new THAlphaObject3D[]{ Tank };
				pr_velChaser = 0.035f;
				
				CamPos = new Vector3(8.6f, 6.35f, -9.959292f);
				TrgPos = new Vector3(8.6f, 2.021785f, -2.462605f);
				uJ = new Vector3(0f, 0.8660254f, 0.5000001f);
				
				pr_difCamVtr = CamPos - posTank;
			}
			else if (MatchCase(7))
			{
				BSAuxFuncs.SetVisible(1, Cube.gameObject);
				Cube.SetIdentity();
				
				Tank.SetWorldPosition(8.6f, 0.0f, 0.0f);
				
				Cube.type = c_RECT;
				Cube.SetWorldPosition(12.0f, 1.5f, 5.8f);
				Cube.initRectVerts = new Vector3[]
				{
					new Vector3(1.5f, 0.0f,  1.5f),
					new Vector3(1.5f, 0.0f,  -1.5f),
					new Vector3(-1.5f, 0.0f, -1.5f),
					new Vector3(-1.5f, 0.0f, 1.5f)
				};
				Vector2 cv = new Vector2(1.5f + 0.5f, 1.5f + 0.5f).normalized;
				Cube.cosA = cv.x;
				
				CamPos = new Vector3(13.80001f, 6.675005f, -7.425028f);
				TrgPos = new Vector3(13.80001f, 3.325002f, 0f);
				uJ = new Vector3(0f, 0.9115195f, 0.4112568f);
			}
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);		
			
			
			// Sub Camera 
			if (MatchCase(1, 2, 4, 5, 7))
			{
				BSAuxFuncs.SetVisible(1, BackPlane.gameObject);
				BackPlane.SetIdentity();
				
				pr_COMP_SubCamera.enabled = true;
				
				Vector3 subCP = new Vector3(0, 10, 0);
				if (MatchCase(1, 4))
				{
					subCP = new Vector3(13.0f, 15.0f, 6.0f);
					pr_COMP_SubCamera.orthographicSize = 6.0f;
				}
				else if (MatchCase(2))
				{
					subCP = new Vector3(13.0f, 8.0f, 6.4f); 
					pr_COMP_SubCamera.orthographicSize = 5.5f;
				}
				else if (MatchCase(5))
				{
					subCP = new Vector3(18.5f, 8.0f, 0.0f);     
					pr_COMP_SubCamera.orthographicSize = 6.6f;
				}
				else if (MatchCase(7))
				{
					subCP = new Vector3(12.0f, 8.0f, 5.5f);     
					pr_COMP_SubCamera.orthographicSize = 6f;
				}
				
				SubCamera.transform.position = subCP;
				SubCamera.transform.rotation = Quaternion.AngleAxis(90, Vector3.right);
				
				Rect rectBounds = new Rect(0.66f, 0.66f, 0.32f, 0.32f);
				pr_COMP_SubCamera.depth = 2;
				pr_COMP_SubCamera.rect = rectBounds;
				
				// BackPlane
				float[] rbds2 = { rectBounds.x+rectBounds.width/2.0f,
				rectBounds.y+rectBounds.height/2.0f, rectBounds.width * 1.04f, rectBounds.height * 1.057f   };
				
				float[] iplaneData = BSAuxFuncs.CalcImagePlaneData(); 
				object[] trs = BSAuxFuncs.CalcUIRectTransformN_LowerLeft(rbds2, iplaneData);
				Vector3 posBP = (Vector3)trs[0];
				Quaternion rot = (Quaternion)trs[1];
				Vector3 scale = (Vector3)trs[2];
				BackPlane.transform.position = posBP;
				BackPlane.transform.rotation = rot;
				BackPlane.transform.localScale = scale;
				
				// Layer setting --
				int LAYER_UI = 5;
				BackPlane.gameObject.layer = LAYER_UI;              
				int layerMask = 1 << LAYER_UI;
				pr_COMP_SubCamera.cullingMask = ~layerMask;  
			}
		}

	}

}
