//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_RGAuxFuncs04;
using OQJ_THAlphaObject3D;
using OQJ_KeyframeData;
using OQJ_THShell3D;
using static OQJ_THCollisionTest.THCollisionTest;



namespace OQJ_Sec424
{
	public class Sec424 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THShell3D[] UShell;
		THShell3D[] EShell;
		
		THAlphaObject3D Tank;
		THAlphaObject3D[] Disk;
		THAlphaObject3D[] SnArray;		
		
		GameObject MainCamera;
		GameObject DLight;


		// Instance Variables
		THAlphaObject3D[] i_targetObjects;
		THAlphaObject3D[] i_shootObjects;
		THAlphaObject3D[] pr_collisionObjects;
		
		float i_degTank;
		float i_velTank;
		
		readonly float c_radShell  = 0.09f;
		readonly float c_radShell2 = 0.27f;
		
		int pr_autoSettingCounter;
		bool pr_AUTO_SETTING;	
		Vector3 pr_utrCamZ;
		Vector3 pr_difCamVtr;
		
		System.Random pr_random = new System.Random();
		
		Motion24[] pr_motionsA = new Motion24[8];		
		Motion24[] pr_motionsB = new Motion24[8];		
		
		float pr_SinglePartFrames = 1.16f / 0.1f;  
		
		float pr_shmS;
		float pr_vel;
		float pr_TopFrameNr;
		float pr_totalFrames;
		
		Motion24 pr_curMotion;
		Motion24 pr_preMotion;
		
		Material[] pr_SnMtrls;
		
		readonly int c_DISK = 10;
		readonly int c_RECT = 11;
		readonly int c_NORMAL = 100;
		readonly int c_HIT = 101;
		
		
		class Motion24
		{
		public int startDN;
		public int endDN;
		
		public KeyframeData[] keyframes;
		
		}




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Tank.lastShootFrame = 0;
				Tank.shootInterval = 30;
				Tank.initShootDirection = Vector3.forward;
				Tank.initShootPosition = new Vector3(0.0f, 1.5f, 1.0f);
				Tank.shellSpeed = 0.35f;
				i_velTank = 0.08f;
				
				for (int i = 0; i < Disk.Length; i++)
				{
					Disk[i].lastShootFrame = 0;
					Disk[i].shootInterval = 24;
					Disk[i].initShootDirection = Vector3.forward;
					Disk[i].initShootPosition = new Vector3(0.0f, 0.0f, 3.0f);
					Disk[i].shellSpeed = 0.15f;
				}
				
				i_targetObjects = Disk;
				i_shootObjects = Disk;
				
				i_frameCount = 0;
				
				i_INITIALIZED = true;
			}
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
			
			EnemyMotion();
			EShellMotion();
			
			CollisionTest_ShellPhase(UShell, i_targetObjects);
			CollisionTest_ShellPhase(EShell, new THAlphaObject3D[] { Tank });
			
			CollisionTest_ObjectPhase(i_targetObjects);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Tank.lastShootFrame = 0;
				Tank.shootInterval = 30;
				Tank.initShootDirection = Vector3.forward;
				Tank.initShootPosition = new Vector3(0.0f, 1.5f, 1.0f);
				Tank.shellSpeed = 0.35f;
				i_velTank = 0.14f;
				
				SnArray[0].lastShootFrame = 0;
				SnArray[0].shootInterval = 120;
				SnArray[0].initShootDirection = Vector3.forward;
				SnArray[0].initShootPosition = new Vector3(0.0f, 0.0f, 1.0f);
				SnArray[0].shellSpeed = 0.4f;
				
				i_targetObjects = SnArray;
				i_shootObjects = new THAlphaObject3D[] { SnArray[0] };
				
				i_frameCount = 0;
				
				i_INITIALIZED = true;
			}
			
			
			i_frameCount++;
			
			UserMotion();
			UShellMotion();
			
			EnemyMotion();
			EShellMotion();
			
			CollisionTest_ShellPhase(UShell, i_targetObjects);
			CollisionTest_ShellPhase(EShell, new THAlphaObject3D[] { Tank });
			
			CollisionTest_ObjectPhase(i_targetObjects);
		}




		void UShellMotion()		
		{
			bool bShoot = false;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Tank.lastShootFrame + Tank.shootInterval)
				{
					Tank.lastShootFrame = i_frameCount;
					bShoot = true;
				}
			}
			
			
			Vector3 posTank = Tank.GetWorldPosition();
			foreach (var shell in UShell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.position + shell.speed * shell.direction;
					if ((newShellPos - posTank).magnitude >= 60.0f)
					{
						shell.active = false;
						newShellPos = new Vector3(-1000, 0, 0);
					}
					
					shell.position = newShellPos;
				}
				else
				{
					if (bShoot)
					{
						Matrix4x4 M = Tank.GetMatrix();
						
						Vector3 startPos = M * Tank.initShootPosition;
						Vector3 shootDir = M * Tank.initShootDirection;
						
						shell.active = true;
						shell.direction = shootDir.normalized;
						shell.speed = Tank.shellSpeed;
						
						shell.position = startPos;
						
						bShoot = false;
					}
				}
			}
		}
		
		
		void EShellMotion()		
		{
			var curShooters = new List<THAlphaObject3D>();
			
			foreach (var obj in i_shootObjects)
			{
				if (i_frameCount >= obj.lastShootFrame + obj.shootInterval)
				{
					obj.lastShootFrame = i_frameCount;
					curShooters.Add(obj);
				}
			}
			
			
			Vector3 posTank = Tank.GetWorldPosition();
			foreach (var shell in EShell)
			{
				if (shell.active)
				{
					if (MatchCase(1))
					{
						Vector3 newShellPos = shell.GetWorldPosition() + shell.speed * shell.direction;
						if ((newShellPos - posTank).magnitude >= 60.0f)
						{
							shell.active = false;
							shell.SetWorldPosition(-1000, 0, 0);
						}
						
						shell.SetWorldPosition(newShellPos);
					}
					else
					{
						shell.count++;
						
						Vector3 dir;
						if (shell.count <= 30)
						{
							
						}
						else
						{
							Vector3 w = posTank - shell.position;
							if (w.magnitude >= 8.0f && shell.count < 10000)
							{
								w.y = 0.0f;
								dir = w.normalized;
								
								int cnt = shell.count - 30;
								int max = 60;
								if (cnt < max)
								{
									float t = 1.0f / (max - cnt);
									float deg = TH3DMath.CalcDegree_V1toV2(shell.direction, dir, Vector3.up);
									dir = Quaternion.AngleAxis(deg * t, Vector3.up) * shell.direction;
								}
								
								shell.direction = dir;
							}
							else
							{
								shell.count = 10000;
							}
						}
						
						Vector3 newShellPos = shell.GetWorldPosition() + shell.speed * shell.direction;
						bool bO = false;
						for(int i= 0; i<SnArray.Length; i++)
						{
							Vector3 b = newShellPos - SnArray[i].GetWorldPosition();	
							float l = b.magnitude;
							if(l <= 1.0f)
							{
								bO = true;
								break;
							}
						}
						
						if ((newShellPos - posTank).magnitude >= 120.0f || bO)
						{
							shell.active = false;
							shell.SetWorldPosition(-1000, 0, 0);
						}
						else
						{
							shell.SetWorldPosition(newShellPos);
						}
					}
				}
				else
				{
					if (curShooters.Count > 0)
					{
						var obj = curShooters[0];	
						Matrix4x4 M = obj.GetMatrix();
						
						Vector3 startPos = M * obj.initShootPosition;
						Vector3 shootDir = M * obj.initShootDirection;
						
						shell.active = true;
						shell.direction = shootDir.normalized;
						shell.speed = obj.shellSpeed;
						
						shell.SetWorldPosition(startPos);
						
						curShooters.RemoveAt(0);
					}
				}
			}
		}
		
		
		
		void CollisionTest_ShellPhase(THShell3D[] shell_array, THAlphaObject3D[] target_array)
		{
			float radShell = c_radShell;
			if (MatchCase(2))
			{
				if(target_array.Length == 1) { radShell = c_radShell2; }
			}
			
			
			foreach (var shell in shell_array)
			{
				if (!shell.active) { continue; }
				
				Vector3 wp = shell.GetWorldPosition();
				Vector2 posShell = new Vector2(wp.x, wp.z);
				
				foreach (var target in target_array)
				{
					Vector2 posTarget = target.positionXZ;
					
					bool bColl = false;
					if (target.type == c_DISK)
					{
						bColl = CollisionTest_Disk_Disk(posShell, radShell, posTarget, target.radius);
					}
					else if (target.type == c_RECT)
					{
						Vector2 lx = target.localAxisXZ[0];
						Vector2 lz = target.localAxisXZ[1];
						bColl = CollisionTest_Disk_Rect(posShell, radShell, posTarget, target.width, target.height, lx, lz);
					}
					
					if (bColl)
					{
						Hit(target);
						
						shell.SetWorldPosition(-1000, 0, 0);
						shell.active = false;
					}
				} 
			} 
		}
		
		
		
		void CollisionTest_ObjectPhase(THAlphaObject3D[] target_arrahy)
		{
			foreach(var target in target_arrahy)
			{
				bool bColl = false;
				if (target.type == c_DISK)
				{
					Vector2 lx = Tank.localAxisXZ[0];
					Vector2 lz = Tank.localAxisXZ[1];
					bColl = CollisionTest_Disk_Rect(target.positionXZ, target.radius, 
					Tank.positionXZ, Tank.width, Tank.height, lx, lz);
				}
				else if (target.type == c_RECT)
				{
					bColl = CollisionTest_Rect_Rect(Tank.rectVertsXZ, target.rectVertsXZ);
				}
				
				if (bColl)
				{
					Tank.reboundCount = 8;
					Tank.reboundSpeed = 0.25f;
					
					Vector3 posTank = Tank.GetWorldPosition();
					Tank.reboundDirection = CalcReboundDirection(posTank, target);
					
					Hit(Tank);
					break;
				}
			}
		}
		
		
		Vector3 CalcReboundDirection(Vector3 posTank, THAlphaObject3D target)
		{
			Vector3 posTrg = target.GetWorldPosition();
			Vector3 vtr = posTank - posTrg;
			vtr.y = 0.0f;
			Vector3 utr = vtr.normalized;
			
			if (target.type == c_RECT)
			{
				Matrix4x4 M = target.GetMatrix();
				Vector3 lx = M.GetColumn(0).normalized;
				Vector3 lz = M.GetColumn(2).normalized;
				
				float dt = Vector3.Dot(utr, lx);
				
				if (target.cosA <= dt && dt <= 1.0f)
				{
					return lx;
				}
				else if (-1.0f <= dt && dt <= -target.cosA)
				{
					return -lx;
				}
				
				if (Vector3.Dot(utr, lz) > 0.0f)
				{
					return lz;
				}
				
				return -lz;
			}
			// c_DISK
			{
				return utr;
			}
		}
		
		
		void UserMotion()
		{
			if (Tank.reboundCount > 0)
			{
				Vector3 wp = Tank.GetWorldPosition() + Tank.reboundSpeed * Tank.reboundDirection;  
				Tank.SetWorldPosition(wp);
				
				Tank.reboundCount--;
				return;
			}
			
			bool MOVE = false;	
			if (Input.GetKey(KeyCode.S))
			{
				MOVE = true;
			}
			
			
			if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.L))
			{
				i_degTank += Input.GetKey(KeyCode.H) ? -3f : 3f;
			}
			
			Matrix4x4 R = TH3DMath.GetRotation4x4(i_degTank, Vector3.up);
			Vector3 forwardDir = R * Vector3.forward;
			
			float mv = MOVE ? i_velTank : 0.0f;  
			mv = THUtil.IsShiftDown() ? -mv : mv;
			Vector3 pos = Tank.GetWorldPosition() + mv * forwardDir;
			Matrix4x4 T = TH3DMath.GetTranslation4x4(pos);
			
			Matrix4x4 M = T * R;
			Tank.SetMatrix(M);
		}
		
		
		
		void EnemyMotion()
		{
			if (MatchCase(1))
			{
				Matrix4x4 T = TH3DMath.GetTranslation4x4(24.0f, 1.5f, 15.0f);
				pr_shmS += 0.8f;
				float A = 60.0f;
				float deg = 0.5f * A * (-Mathf.Cos(pr_shmS * Mathf.Deg2Rad) + 1.0f) + 170.0f;  
				Matrix4x4 R = TH3DMath.GetRotation4x4(deg, Vector3.up);
				Disk[0].SetMatrix(T * R);
				
				
				T = TH3DMath.GetTranslation4x4(4.0f, 1.5f, 9.0f);
				A = 45.0f;
				deg = 0.5f * A * (-Mathf.Cos(pr_shmS * Mathf.Deg2Rad) + 1.0f) + 80.0f;  
				R = TH3DMath.GetRotation4x4(deg, Vector3.up);
				Disk[1].SetMatrix(T * R);
			}
			else if (MatchCase(2))
			{
				Matrix4x4[] mtrcs = GetSnakeMatrices(pr_vel);
				for (int i = 0; i < SnArray.Length; i++)
				{
					SnArray[i].SetMatrix(mtrcs[i]);
				}
			}
			
		}
		
		
		Matrix4x4[] GetSnakeMatrices(float vel)
		{
			Matrix4x4[] mtrcs = new Matrix4x4[SnArray.Length];
			
			float baseSpeed = 0.1f;
			pr_TopFrameNr += vel / baseSpeed;
			if(pr_TopFrameNr > pr_totalFrames)
			{
				pr_TopFrameNr -= pr_totalFrames;
				
				bool A = pr_random.Next(0, 1 + 1) == 0 ? true : false;      
				
				int dn = pr_curMotion.endDN;
				pr_preMotion = pr_curMotion;
				
				pr_curMotion = A ? pr_motionsA[dn] : pr_motionsB[dn];
			}
			
			for (int i = 0; i < SnArray.Length; i++)
			{
				float wfr = pr_TopFrameNr - i * pr_SinglePartFrames;
				Motion24 mtn;
				if(wfr >= 0.0f)
				{
					mtn = pr_curMotion;
				}
				else
				{
					mtn = pr_preMotion;
					wfr += pr_totalFrames;
				}
				
				mtrcs[i] = CalcTransform(wfr, mtn.keyframes);
			}
			
			return mtrcs;
		}
		
		
		void Hit(THAlphaObject3D obj)
		{
			if (obj.collCount <= 0)
			{
				obj.collCount = 12;
				
				if (obj == Tank || MatchCase(1))
				{
					obj.state = c_HIT;
				}
				else
				{
					pr_SnMtrls[0].SetColor("_Color", new Color(0.52f, 0.15f, 0.15f));
					pr_SnMtrls[1].SetColor("_Color", new Color(0.82f, 0.34f, 0.28f));
					pr_SnMtrls[2].SetColor("_Color", new Color(0.0f, 0.0f, 0.0f));
				}
			}
		}
		
		
		Matrix4x4 CalcTransform(float fr, KeyframeData[] kfrs)
		{
			return RGAuxFuncs04.CalcTransform(fr, kfrs);
		}


///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
			if (MatchCase(2))
			{
				if (Input.GetKeyDown(KeyCode.V) && !pr_AUTO_SETTING)
				{
					pr_AUTO_SETTING = true;
					pr_autoSettingCounter = 0;
				}
				
				
				if (pr_AUTO_SETTING)
				{
					int max = 90;
					float t = 1.0f / (float)(max - pr_autoSettingCounter);
					t = Mathf.Pow(t, 0.9f);
					
					pr_autoSettingCounter++;
					
					Vector3 posTank = Tank.GetWorldPosition();
					
					Vector3 wvtr = SnArray[12].GetWorldPosition() - posTank;
					wvtr.y = 0.0f;
					Vector3 u1 = wvtr.normalized;
					
					float deg;
					if (pr_autoSettingCounter > 80)
					{
						deg = TH3DMath.CalcDegree_V1toV2(Vector3.forward, pr_utrCamZ, Vector3.up);
					}
					else
					{
						deg = TH3DMath.CalcDegree_V1toV2(Vector3.forward, u1, Vector3.up);
						pr_utrCamZ = u1;
					}
					
					Quaternion staQ = MainCamera.transform.rotation;
					Quaternion endQ = Quaternion.Euler(30, deg, 0);
					Quaternion rot = Quaternion.Slerp(staQ, endQ, t);
					
					Vector3 pos = rot * new Vector3(0.0f, 0.0f, -11.5f);
					pos.y += 0.6f;
					pr_difCamVtr = pos;
					
					pos += posTank;
					
					MainCamera.transform.position = pos;
					MainCamera.transform.rotation = rot;
					
					if (pr_autoSettingCounter == max)
					{
						pr_AUTO_SETTING = false;
					}
				}
			}
		}


		void PostProcess()
		{
			// Hit Color
			if (MatchCase(1, 2))
			{
				foreach (var o in pr_collisionObjects)
				{
					if (o.collCount > 0)
					{
						o.collCount--;
						
						if (o.collCount == 0)
						{
							if (o == Tank || MatchCase(1))
							{
								o.state = c_NORMAL;
							}
							else
							{
								pr_SnMtrls[0].SetColor("_Color", new Color(0.0754f, 0.0754f, 0.0754f));
								pr_SnMtrls[1].SetColor("_Color", new Color(0.205f, 0.110f, 0.755f));
								pr_SnMtrls[2].SetColor("_Color", new Color(0.0f, 0.0f, 0.0f));
							}
						}
					}
				}
			}
			
			// Camera
			if (MatchCase(2))
			{
				Vector3 posTank = Tank.GetWorldPosition();
				Vector3 CP = posTank + pr_difCamVtr;
				MainCamera.transform.position = CP;
				
				BSData.CameraController3D.UpdateCameraData();
			}
		}


		static Sec424 pr_instance = new Sec424(2);

		public static Sec424 PRGSI()
		{
			return pr_instance;
		}


		private Sec424(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Tank = CreateObject_Alpha3D("ZTank");
				var src_Disk = CreateObject_Alpha3D("DBlDisk");
				var src_Sn = CreateObject_Alpha3D("SnPart");		
				
				Disk = new THAlphaObject3D[2];
				for (int i = 0; i < Disk.Length; i++)
				{
					Disk[i] = CreateObject_InsAlpha3D(src_Disk.gameObject);
				}
				
				pr_SnMtrls = new Material[]
				{
					BSAuxFuncs.GetStandardMaterial(0.0754f, 0.0754f, 0.0754f, 0.0f),
					BSAuxFuncs.GetStandardMaterial(0.205f, 0.110f, 0.755f, 0.0f),
					BSAuxFuncs.GetStandardMaterial(0.0f, 0.0f, 0.0f, 0.0f)
				};
				SnArray = new THAlphaObject3D[24];
				for(int i=0; i<SnArray.Length; i++)
				{
					SnArray[i] = CreateObject_InsAlpha3D(src_Sn.gameObject);
					SnArray[i].gameObject.GetComponent<MeshRenderer>().materials = pr_SnMtrls;
				}
				
				
				var src_Shell = CreateObject_Shell3D("MlShell_A");
				UShell = new THShell3D[30];
				for(int i=0; i<UShell.Length; i++)
				{
					UShell[i] = CreateObject_InsShell3D(src_Shell.gameObject);
				}
				
				EShell = new THShell3D[60];
				Material rMt = BSAuxFuncs.GetStandardMaterial(0.60f, 0.12f, 0.12f, 0.1f);
				for(int i=0; i<EShell.Length; i++)
				{
					EShell[i] = CreateObject_InsShell3D(src_Shell.gameObject);
					EShell[i].gameObject.GetComponent<MeshRenderer>().material = rMt;
					EShell[i].gameObject.name = "EShell_" + i;
				}
				
				
				var nMts = Tank.gameObject.GetComponent<MeshRenderer>().materials;
				var hMts = new Material[]
				{
					BSAuxFuncs.GetStandardMaterial(0.78f, 0.23f, 0.18f),
					BSAuxFuncs.GetStandardMaterial(0.86f, 0.38f, 0.43f)
				};
				Tank.SetData(c_NORMAL, nMts);
				Tank.SetData(c_HIT, hMts);
				
				
				hMts = new Material[]
				{
					BSAuxFuncs.GetStandardMaterial(0.78f, 0.23f, 0.18f),
					BSAuxFuncs.GetStandardMaterial(0.86f, 0.38f, 0.43f),
					BSAuxFuncs.GetStandardMaterial(0.80f, 0.12f, 0.30f),
					BSAuxFuncs.GetUnlitMaterial(0.0f, 0.0f, 0.0f)
				};
				for (int i = 0; i < Disk.Length; i++)
				{
					nMts = Disk[i].gameObject.GetComponent<MeshRenderer>().materials;
					Disk[i].SetData(c_NORMAL, nMts);
					Disk[i].SetData(c_HIT, hMts);
				}
				
				
				
				MainCamera = BSAuxFuncs.GetMainCamera();
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos = new Vector3(0, 0, -10);
			Vector3 TrgPos = Vector3.zero;
			Vector3 uJ = Vector3.up;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			
			{
				BSAuxFuncs.SetVisible(1, Tank.gameObject);
				Tank.SetIdentity();
				
				Tank.type = c_RECT;
				Tank.initRectVerts = new Vector3[]
				{
					new Vector3(0.5f, 0.0f,  0.5f),
					new Vector3(0.5f, 0.0f,  -0.5f),
					new Vector3(-0.5f, 0.0f, -0.5f),
					new Vector3(-0.5f, 0.0f, 0.5f)
				};
				
				THAlphaObject3D[] objArr = { Tank, Disk[0], Disk[1] };
				foreach (var o in objArr)
				{
					o.collCount = 0;
					o.state = c_NORMAL;
				}
				
				i_degTank = 0;	
				
				for (int i = 0; i < UShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, UShell[i].gameObject);
					UShell[i].SetIdentity();
					UShell[i].SetWorldPosition(-1000, 0, 0);
				}
				
				for (int i = 0; i < EShell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, EShell[i].gameObject);
					EShell[i].SetIdentity();
					EShell[i].SetWorldPosition(-1000, 0, 0);
				}
			}
			
			
			if (MatchCase(1))
			{
				Tank.SetWorldPosition(12, 0, 0);
				
				for (int i = 0; i < Disk.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Disk[i].gameObject);
					Disk[i].SetIdentity();
					Disk[i].type = c_DISK;
					Disk[i].radius = 2.88f;
				}
				
				pr_collisionObjects = new THAlphaObject3D[] { Disk[0], Disk[1], Tank };
				
				CamPos = new Vector3(14.5f, 8.7f, -10f);
				TrgPos = new Vector3(14.5f, 2.9f, 0f);
				uJ = new Vector3(0f, 0.8650313f, 0.5017179f).normalized;
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(30, 40, 0);
				}
			}
			else if (MatchCase(2))
			{
				Vector3 posTank = new Vector3(5.0f, 0.0f, -23.0f);
				Tank.SetWorldPosition(posTank);
				
				for (int i = 0; i < EShell.Length; i++)
				{
					EShell[i].SetLocalScale(3.0f);
				}
				
				// SnArray
				float wid = 1.2f;
				float hgt = 1.2f;
				float w2 = wid * 0.5f  ,  h2 = hgt * 0.5f;
				Vector3[] rvts =
				{
					new Vector3( w2, 0.0f,  h2),
					new Vector3( w2, 0.0f, -h2),
					new Vector3(-w2, 0.0f, -h2),
					new Vector3(-w2, 0.0f,  h2)
				};
				
				
				for (int i = 0; i < SnArray.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, SnArray[i].gameObject);
					SnArray[i].SetIdentity();
					SnArray[i].type = c_RECT;
					SnArray[i].initRectVerts = rvts;
					SnArray[i].width  = wid;
					SnArray[i].height = hgt;
					
					SnArray[i].collCount = 0;
				}
				
				pr_SnMtrls[0].SetColor("_Color", new Color(0.0754f, 0.0754f, 0.0754f));
				pr_SnMtrls[1].SetColor("_Color", new Color(0.205f, 0.110f, 0.755f));
				pr_SnMtrls[2].SetColor("_Color", new Color(0.0f, 0.0f, 0.0f));
				
				
				for (int i = 0; i < 8; i++)
				{
					pr_motionsA[i] = new Motion24();
					pr_motionsA[i].keyframes = RGAuxFuncs04.InitKeyframes_24(i);
					pr_motionsA[i].startDN = i;
					pr_motionsA[i].endDN = (i + 3) % 8;
					
					pr_motionsB[i] = new Motion24();
					pr_motionsB[i].keyframes = RGAuxFuncs04.InitKeyframes_24(i, true);
					pr_motionsB[i].startDN = i;
					pr_motionsB[i].endDN = (i + 5) % 8;
				}
				
				pr_totalFrames = pr_motionsA[0].keyframes[pr_motionsA[0].keyframes.Length - 1].frameNumber;
				
				pr_collisionObjects = new THAlphaObject3D[SnArray.Length + 1];
				pr_collisionObjects[0] = Tank;
				for (int i = 1; i < pr_collisionObjects.Length; i++)
				{
					pr_collisionObjects[i] = SnArray[i - 1];
				}
				
				
				pr_vel = 0.20f;
				pr_TopFrameNr = 320;
				pr_curMotion = pr_motionsA[0];
				pr_preMotion = pr_motionsA[0];
				pr_AUTO_SETTING = false;
				pr_autoSettingCounter = 0;
				
				CamPos = new Vector3(5.0f, 6.35f, -32.959292f);
				TrgPos = new Vector3(5.0f, 2.021785f, -25.462605f);
				uJ = new Vector3(0f, 0.8660254f, 0.5000001f);
				
				pr_difCamVtr = CamPos - posTank;
				
				if (DLight != null)		
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -30, 0);
				}
			}
			
			SetCameraPosition_3D(CamPos, TrgPos, uJ);
		}

	}

}
