﻿//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using UnityEngine;
using System.Collections.Generic;



namespace OQJ_THAlphaObject3D
{
	public class THAlphaObject3D : THObject3D
	{

		public THAlphaObject3D(GameObject go, bool shadow=false) : base(go, shadow) { }


		public int type;		// DISK 10  ;;  RECT 11

		int m_state;
		public int state
		{
			get { return m_state; }

			set
			{
				m_state = value;

				if(m_state == NORMAL)
				{
					gameObject.GetComponent<MeshRenderer>().materials = m_normalMtrls;		
				}	
				else if(m_state == HIT)
				{
					gameObject.GetComponent<MeshRenderer>().materials = m_hitMtrls;		
				}
				else if(m_state == TEMP)
				{
					gameObject.GetComponent<MeshRenderer>().materials = (m_tempMtrls != null) ? m_tempMtrls : m_normalMtrls;
				}
			}

		}

		static readonly int NORMAL = 100;
		static readonly int HIT  = 101;
		static readonly int TEMP  = 102;


		Material[] m_normalMtrls;
		Material[] m_hitMtrls;
		Material[] m_tempMtrls;


		float m_radius = 1.0f; 
		public float radius
		{
			get
			{
				if(THUtil.IsEqual(transform.localScale.x, 1.0f))
				{
					return m_radius;
				}
				else
				{
					return transform.localScale.x;
				}
			}

			set
			{
				m_radius = value;
			}
		}


		// // width : ローカル x軸方向の長さ
		float m_width = 1.0f;	
		public float width
		{
			get
			{
				if(THUtil.IsEqual(transform.localScale.x, 1.0f))
				{
					return m_width;
				}
				else
				{
					return transform.localScale.x;		
				}
			}

			set
			{
				m_width = value;
			}
		}


		// // height : ローカル z軸方向の長さ
		float m_height = 1.0f;	
		public float height
		{
			get
			{
				if(THUtil.IsEqual(transform.localScale.z, 1.0f))
				{
					return m_height;
				}
				else
				{
					return transform.localScale.z;
				}
			}

			set
			{
				m_height = value;
			}
		}


		public Vector2 positionXZ
		{
			get
			{
				return new Vector2(transform.position.x, transform.position.z);
			}
		}



		public Vector2[] localAxisXZ
		{
			get
			{
				Matrix4x4 M = transform.localToWorldMatrix;
				Vector3 c0 = M.GetColumn(0);
				Vector3 c2 = M.GetColumn(2);

				return new Vector2[] { new Vector2(c0.x, c0.z).normalized, new Vector2(c2.x, c2.z).normalized};
			}
		}


		public float cosA;




		Vector4[] m_initRectVerts;
		public Vector3[] initRectVerts
		{
			set
			{
				if(value.Length != 4) { m_initRectVerts = null; }

				m_initRectVerts = new Vector4[4];
				for (int i = 0; i < 4; i++)
				{
					m_initRectVerts[i] = TH3DMath.ToVector4(value[i]);
				}
			}
		}



		public Vector2[] rectVertsXZ
		{
			get
			{
				Vector2[] verts = new Vector2[4];

				Matrix4x4 M = transform.localToWorldMatrix;
				for (int i = 0; i < 4; i++)
				{
					Vector3 v3 = M * m_initRectVerts[i];
					verts[i] = new Vector2(v3.x, v3.z);	
				}

				return verts;
			}
		}


		public int reboundCount;

		public float reboundSpeed;

		public Vector3 reboundDirection;


		public int collCount;



		// // Shooting Data 
		public int lastShootFrame;

		public int shootInterval;

		Vector3 m_initShootDir;
		public Vector4 initShootDirection
		{
			get
			{
				return m_initShootDir;		// w=0
			}

			set
			{
				m_initShootDir = value;
			}
		}


		Vector4 m_initShootPos;
		public Vector4 initShootPosition
		{
			get
			{
				return m_initShootPos;	
			}

			set
			{
				m_initShootPos = value;
				m_initShootPos.w = 1.0f;
			}
		}


		Vector4[] m_initShootPositions;
		public Vector4[] initShootPositions
		{
			get
			{
				return m_initShootPositions;
			}
			set
			{
				m_initShootPositions = new Vector4[value.Length];
				for (int i = 0; i < value.Length; i++)
				{
					m_initShootPositions[i] = new Vector4(value[i].x, value[i].y, value[i].z, 1.0f);
				}
			}	
		}


		public float shellSpeed;



		// // ---- ----

		public void SetData(int id, Material[] mtrls)
		{
			if (id == NORMAL)
			{
				m_normalMtrls = mtrls;
			}	
			else if (id == HIT)
			{
				m_hitMtrls = mtrls;
			}	
			else if (id == TEMP)
			{
				m_tempMtrls = mtrls;
			}	
		}
	}

}
