//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec501
{
	public class Sec501 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Stick1;	
		THObject3D Stick2;	
		THObject3D Stick3;	
		
		GameObject DLight;


		// Instance Variables
		bool i_switch_Stick1 = true;
		bool i_switch_Stick2 = true;
		bool i_switch_Stick3 = true;
		
		int i_degStick1 = 0;
		int i_degStick2 = 0;
		int i_degStick3 = 0;
		readonly Vector3 c_attachPos_Stick2 = new Vector3(6.0f, 0.3f, 0.0f);
		readonly Vector3 c_attachPos_Stick3 = new Vector3(4.0f, 0.3f, 0.0f);




		void Code1()
		{
			Matrix4x4 localStick3 = TH3DMath.GetTranslation4x4(c_attachPos_Stick3);
			Matrix4x4 localStick2 = TH3DMath.GetTranslation4x4(c_attachPos_Stick2);
			Matrix4x4 localStick1 = Matrix4x4.identity;
			
			Matrix4x4 worldStick3 = localStick1 * localStick2 * localStick3;
			Matrix4x4 worldStick2 = localStick1 * localStick2;
			Matrix4x4 worldStick1 = localStick1;
			
			Stick3.SetMatrix(worldStick3);
			Stick2.SetMatrix(worldStick2);
			Stick1.SetMatrix(worldStick1);
		}


		void Code2()
		{
			i_degStick3 += 16;
			Matrix4x4 rotStick3 = TH3DMath.GetRotation4x4(i_degStick3, Vector3.up);
			Matrix4x4 traStick3 = TH3DMath.GetTranslation4x4(c_attachPos_Stick3);
			Matrix4x4 localStick3 = traStick3 * rotStick3;
			
			i_degStick2 += 4;
			Matrix4x4 rotStick2 = TH3DMath.GetRotation4x4(i_degStick2, Vector3.up);
			Matrix4x4 traStick2 = TH3DMath.GetTranslation4x4(c_attachPos_Stick2);
			Matrix4x4 localStick2 = traStick2 * rotStick2;
			
			i_degStick1 += 1;
			Matrix4x4 localStick1 = TH3DMath.GetRotation4x4(i_degStick1, Vector3.up);
			
			Matrix4x4 worldStick3 = localStick1 * localStick2 * localStick3;
			Matrix4x4 worldStick2 = localStick1 * localStick2;
			Matrix4x4 worldStick1 = localStick1;
			
			Stick3.SetMatrix(worldStick3);
			Stick2.SetMatrix(worldStick2);
			Stick1.SetMatrix(worldStick1);
		}


		void Code3()
		{
			if (Input.GetKeyDown(KeyCode.J))
			{
				i_switch_Stick1 = !i_switch_Stick1;	
			}
			else if (Input.GetKeyDown(KeyCode.K))
			{
				i_switch_Stick2 = !i_switch_Stick2;	
			}
			else if (Input.GetKeyDown(KeyCode.L))
			{
				i_switch_Stick3 = !i_switch_Stick3;	
			}
			
			if (i_switch_Stick3) { i_degStick3 += 16; } 
			Matrix4x4 rotStick3 = TH3DMath.GetRotation4x4(i_degStick3, Vector3.up);
			Matrix4x4 traStick3 = TH3DMath.GetTranslation4x4(c_attachPos_Stick3);
			Matrix4x4 localStick3 = traStick3 * rotStick3; 
			
			if (i_switch_Stick2) { i_degStick2 += 4; } 
			Matrix4x4 rotStick2 = TH3DMath.GetRotation4x4(i_degStick2, Vector3.up);
			Matrix4x4 traStick2 = TH3DMath.GetTranslation4x4(c_attachPos_Stick2);
			Matrix4x4 localStick2 = traStick2 * rotStick2; 
			
			if (i_switch_Stick1) { i_degStick1 += 1; } 
			Matrix4x4 localStick1 = TH3DMath.GetRotation4x4(i_degStick1, Vector3.up);
			
			Matrix4x4 worldStick3 = localStick1 * localStick2 * localStick3;
			Matrix4x4 worldStick2 = localStick1 * localStick2;
			Matrix4x4 worldStick1 = localStick1;
			
			Stick3.SetMatrix(worldStick3);
			Stick2.SetMatrix(worldStick2);
			Stick1.SetMatrix(worldStick1);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec501 pr_instance = new Sec501(3);

		public static Sec501 PRGSI()
		{
			return pr_instance;
		}


		private Sec501(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Stick1 = CreateObject_Obj3D("Stick1");
				Stick2 = CreateObject_Obj3D("Stick2");
				Stick3 = CreateObject_Obj3D("Stick3");
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(40, -30, 0);
				}

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Stick1.gameObject);
				BSAuxFuncs.SetVisible(1, Stick2.gameObject);
				BSAuxFuncs.SetVisible(1, Stick3.gameObject);
				Stick1.SetMatrix(Matrix4x4.identity);
				Stick2.SetMatrix(Matrix4x4.identity);
				Stick3.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(8.872377f, 9.230135f, -11.9978f);
				TrgPos = new Vector3(5.337572f, -3.0667f, 2.413074f);
				uJ = new Vector3(-0.01180232f, 0.7620729f, 0.6473837f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2, 3))
			{
				BSAuxFuncs.SetVisible(1, Stick1.gameObject);
				BSAuxFuncs.SetVisible(1, Stick2.gameObject);
				BSAuxFuncs.SetVisible(1, Stick3.gameObject);
				Stick1.SetMatrix(Matrix4x4.identity);
				Stick2.SetMatrix(Matrix4x4.identity);
				Stick3.SetMatrix(Matrix4x4.identity);
				
				i_degStick1 = 0;
				i_degStick2 = 0;
				i_degStick3 = 0;
				i_switch_Stick1 = true;
				i_switch_Stick2 = true;
				i_switch_Stick3 = true;
				
				CamPos = new Vector3(5.981528f, 12.31484f, -17.51619f);
				TrgPos = new Vector3(1.238464f, -4.445543f, 1.820937f);
				uJ = new Vector3(-0.02605342f, 0.7585568f, 0.6510858f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
