//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;




namespace OQJ_Sec503
{
	public class Sec503 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		THObject3D Ball;
		THObject3D Ring;
		THObject3D Grabber;
		
		GameObject DLight;


		// Instance Variables
		readonly Vector3 c_attachPos_Ring = new Vector3(-11.0f, 0.0f, 0.0f);




		void Code1()
		{
			if (!i_INITIALIZED)
			{
				Ball.transform.SetParent(Ring.transform);
				
				i_INITIALIZED = true;
			}
			
			// Ball
			Ball.transform.localScale = new Vector3(1.5f, 1.5f, 1.5f);
			Ball.transform.localRotation = Quaternion.AngleAxis(-50.0f, Vector3.up);
			Ball.transform.localPosition = TH3DMath.GetRotation4x4(70.0f, Vector3.forward) *
			new Vector4(2, 0, 0, 1);
			
			// Ring
			Ring.transform.localRotation = Quaternion.AngleAxis(45.0f, Vector3.up) *
			Quaternion.AngleAxis(-60.0f, Vector3.right);
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				Ball.transform.SetParent(Ring.transform);
				Ring.transform.SetParent(Grabber.transform);
				
				i_INITIALIZED = true;
			}
			
			// Ball
			Ball.transform.localScale = new Vector3(1.5f, 1.5f, 1.5f);
			Ball.transform.localRotation = Quaternion.AngleAxis(-135.0f, Vector3.up);
			Ball.transform.localPosition = TH3DMath.GetRotation4x4(150.0f, Vector3.forward) *
			new Vector4(2, 0, 0, 1);
			
			// Ring
			Ring.transform.localRotation = Quaternion.AngleAxis(40.0f, Vector3.up);
			Ring.transform.localPosition = c_attachPos_Ring;
			
			// Grabber
			Grabber.transform.localRotation = Quaternion.AngleAxis(50.0f, Vector3.up) *
			Quaternion.AngleAxis(15.0f, Vector3.right);
			Grabber.transform.localPosition = new Vector3(0, 8, 0);
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
		}


		static Sec503 pr_instance = new Sec503(2);

		public static Sec503 PRGSI()
		{
			return pr_instance;
		}


		private Sec503(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Ball = CreateObject_Obj3D("WhBall");
				Ring = CreateObject_Obj3D("Ring");
				Grabber = CreateObject_Obj3D("Grabber");
				
				
				DLight = BSAuxFuncs.GetDirectionalLight();

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				Ball.transform.SetParent(null);		
				Ring.transform.SetParent(null);
				
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				Ball.SetMatrix(Matrix4x4.identity);
				Ring.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(7.591183f, 3.794933f, 2.005502f);
				TrgPos = new Vector3(3.229023f, 1.337933f, 1.079663f);
				uJ = new Vector3(-0.4643821f, 0.8752996f, -0.134907f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -50, 0);
				}
				
				pr_LOOP_CODE = false;
			}
			else if (MatchCase(2))
			{
				Ball.transform.SetParent(null);		
				Ring.transform.SetParent(null);
				
				BSAuxFuncs.SetVisible(1, Ball.gameObject);
				BSAuxFuncs.SetVisible(1, Ring.gameObject);
				BSAuxFuncs.SetVisible(1, Grabber.gameObject);
				Ball.SetMatrix(Matrix4x4.identity);
				Ring.SetMatrix(Matrix4x4.identity);
				Grabber.SetMatrix(Matrix4x4.identity);
				
				CamPos = new Vector3(10.24197f, 12.30226f, 17.14771f);
				TrgPos = new Vector3(-2.730373f, 6.777371f, 1.137726f);
				uJ = new Vector3(-0.143856f, 0.9655911f, -0.2166546f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
				if (DLight != null)
				{
					DLight.transform.localRotation = Quaternion.Euler(50, -90, 0);
				}
				
				pr_LOOP_CODE = false;
			}
		}

	}

}
