//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THAlphaObject3D;
using OQJ_THShell3D;



namespace OQJ_Sec504
{
	public class Sec504 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		// Code1
		THObject3D Sphere;
		THObject3D Sphere_1;
		THObject3D PointB;
		THObject3D AuxCircle;
		THObject3D AuxLine;
		
		// Code2
		THObject3D Body;
		THObject3D Propeller;
		
		// Code3
		THObject3D Disk;
		THObject3D Arm;
		THObject3D Sphere_3;
		
		// Code4, 5
		THObject3D Pole;
		THObject3D[] ArmArray = new THObject3D[3];
		THObject3D[] BoxArray = new THObject3D[3];
		THObject3D[] PropellerArray = new THObject3D[3];
		THObject3D YLine;
		
		// Code6
		THAlphaObject3D Navy;
		THShell3D[] Shell;
		
		
		// Code7
		THObject3D Dice;
		THObject3D DPath;
		
		// Code8
		THObject3D[] Face = new THObject3D[6];	
		THObject3D[] CubeSide = new THObject3D[3];	
		THObject3D[] CubeSide2 = new THObject3D[2];	
		THObject3D CubeBottom;	


		// Instance Variables
		// Code1
		float i_degRev = 0.0f;
		float i_degRot = 0.0f;
		Vector3 c_centerPos = new Vector3(5, 3, -5);
		
		// Code2
		int i_degPropeller = 0;
		int i_degBody = 0;
		Vector3 c_attachPos_Propeller = new Vector3(0.0f, 0.0f, 2.5f);
		
		// Code3
		float i_shmSphere = 0;
		float i_degArm = 0;
		float i_shmDisk = 0;
		readonly Vector3 c_attachPos_Sphere = new Vector3(5, 4, 0);
		readonly Vector3 c_attachPos_Arm = new Vector3(2, 0, 0);
		
		// Code4, 5
		int i_shmArm = 0;
		int i_degPole = 0;
		int i_degBox0 = 0;
		bool i_switch_Pole = true;
		bool i_switch_Arm = true;
		
		readonly Vector3 c_attachPos_Box = new Vector3(4.75641f, 10.0f - 0.5f, 0.0f);
		
		
		// Code6
		Quaternion pr_staRot;
		Quaternion pr_endRot;
		Quaternion pr_endRot2;
		Quaternion pr_orieXZ;
		Quaternion pr_orieY;
		
		int pr_autoRotCounter;
		int pr_autoState;
		int pr_cntMax;
		
		float pr_adg;
		
		bool pr_Y_UP;
		bool pr_Y_UP_2;
		
		
		// Code7
		int i_motionCounter;
		int i_curSquare;
		List<Vector3[]> i_edgeList;
		
		readonly Vector3 A = new Vector3(4.5f, 0.5f, 2.5f);	
		
		readonly Vector3[] V =
		{
		new Vector3(4, 0, 3), new Vector3(5, 0, 3), new Vector3(6, 0, 3), new Vector3(4, 0, 4),  
		new Vector3(5, 0, 4), new Vector3(6, 0, 4), new Vector3(7, 0, 4), new Vector3(4, 0, 5),  
		new Vector3(5, 0, 5), new Vector3(6, 0, 5), new Vector3(7, 0, 5), new Vector3(8, 0, 5),  
		new Vector3(5, 0, 6), new Vector3(6, 0, 6), new Vector3(7, 0, 6)						
		};
		
		// Code8
		float i_deg1 = 0.0f;
		float i_deg2 = 0.0f;
		float i_deg3 = 0.0f;
		float i_deg4 = 0.0f;
		float i_deg5 = 0.0f;




		void Code1()
		{
			i_degRot += 4.0f;
			Quaternion rotY = Quaternion.AngleAxis(i_degRot, Vector3.up);
			Quaternion rotZ = Quaternion.AngleAxis(-20.0f, Vector3.forward);
			Quaternion rot = rotZ * rotY;
			
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Vector3 pos = np + c_centerPos;
			
			Sphere.transform.localPosition = pos;
			Sphere.transform.localRotation = rot;
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				// 親子関係の設定
				Propeller.transform.SetParent(Body.transform);
				
				i_INITIALIZED = true;
			}
			
			// Propeller
			i_degPropeller += 15;
			Quaternion rotProp = Quaternion.AngleAxis(i_degPropeller, Vector3.forward);
			Propeller.transform.localPosition = c_attachPos_Propeller;
			Propeller.transform.localRotation = rotProp;
			
			// Body
			i_degBody -= 2;
			Quaternion rotBody = Quaternion.AngleAxis(i_degBody, Vector3.up);
			Vector3 posBody = rotBody * new Vector3(8.0f, 10.0f, 0.0f);
			Body.transform.localPosition = posBody;
			Body.transform.localRotation = rotBody;
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				// 親子関係の設定
				Sphere.transform.SetParent(Arm.transform);
				Arm.transform.SetParent(Disk.transform);
				
				i_INITIALIZED = true;
			}
			
			i_shmSphere += 1.0f;
			float scale = 0.5f * Mathf.Sin(i_shmSphere * Mathf.Deg2Rad) + 1.5f; // 1 -- 2
			Sphere.transform.localScale = new Vector3(scale, scale, scale);
			Sphere.transform.localPosition = c_attachPos_Sphere;
			
			
			i_degArm--;
			Arm.transform.localRotation = Quaternion.AngleAxis(i_degArm, Vector3.up);
			Arm.transform.localPosition = c_attachPos_Arm;
			
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12
			Disk.transform.localRotation = Quaternion.AngleAxis(-15.0f, Vector3.right);
			Disk.transform.localPosition = new Vector3(0.0f, posY, 0.0f);
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				// 親子関係の設定
				ArmArray[0].transform.SetParent(Pole.transform);
				BoxArray[0].transform.SetParent(ArmArray[0].transform);
				PropellerArray[0].transform.SetParent(BoxArray[0].transform);
				
				// 実行中に変更することのないデータの設定
				PropellerArray[0].transform.localPosition = c_attachPos_Propeller;
				BoxArray[0].transform.localPosition = c_attachPos_Box;
				
				i_INITIALIZED = true;
			}
			
			
			// Propeller
			i_degPropeller -= 6;
			PropellerArray[0].transform.localRotation = Quaternion.AngleAxis(i_degPropeller, Vector3.forward);
			
			// Arm
			i_shmArm += 1;
			float my = 3.0f * Mathf.Sin(i_shmArm * Mathf.Deg2Rad);    // -3.0 -- 3.0
			ArmArray[0].transform.localPosition = new Vector3(0.0f, my, 0.0f);
			
			// Pole
			i_degPole += 1;
			Pole.transform.localRotation = Quaternion.AngleAxis(i_degPole, Vector3.up);
		}


		void Code5()
		{
			if (!i_INITIALIZED)
			{
				// 親子関係の設定
				ArmArray[0].transform.SetParent(Pole.transform);
				ArmArray[1].transform.SetParent(Pole.transform);
				ArmArray[2].transform.SetParent(Pole.transform);
				
				BoxArray[0].transform.SetParent(ArmArray[0].transform);
				BoxArray[1].transform.SetParent(ArmArray[1].transform);
				BoxArray[2].transform.SetParent(ArmArray[2].transform);
				
				PropellerArray[0].transform.SetParent(BoxArray[0].transform);
				PropellerArray[1].transform.SetParent(BoxArray[1].transform);
				PropellerArray[2].transform.SetParent(BoxArray[2].transform);
				
				// 実行中に更新することのないデータの設定
				for (int i = 0; i < 3; i++)
				{
					PropellerArray[i].transform.localPosition = c_attachPos_Propeller;
					BoxArray[i].transform.localPosition = c_attachPos_Box;
					ArmArray[i].transform.localRotation = Quaternion.AngleAxis(i * 120, Vector3.up);
				}
				
				i_INITIALIZED = true;
			}
			
			if (Input.GetKeyDown(KeyCode.J))
			{
				i_switch_Pole = !i_switch_Pole;
			}
			else if (Input.GetKeyDown(KeyCode.K))
			{
				i_switch_Arm = !i_switch_Arm;
			}
			else if (Input.GetKey(KeyCode.L))  // 長押しに対応 
			{
				// Box0 (Box1、Box2は実行中のデータ更新はなし)
				i_degBox0 = THUtil.IsShiftDown() ? i_degBox0 - 2 : i_degBox0 + 2;
				BoxArray[0].transform.localRotation = Quaternion.AngleAxis(i_degBox0, Vector3.up);
			}
			
			// Propeller
			i_degPropeller -= 6;
			for (int i = 0; i < 3; i++)
			{
				PropellerArray[i].transform.localRotation = Quaternion.AngleAxis(i_degPropeller, Vector3.forward);
			}
			
			// Arm
			if (i_switch_Arm)
			{
				i_shmArm++;
				for (int i = 0; i < 3; i++)
				{
					float my = 3.0f * Mathf.Sin((i_shmArm + i * 60) * Mathf.Deg2Rad);    // -3.0 -- 3.0
					ArmArray[i].transform.localPosition = new Vector3(0.0f, my, 0.0f);
				}
			}
			
			// Pole
			if (i_switch_Pole)
			{
				i_degPole++;
				Pole.transform.localRotation = Quaternion.AngleAxis(i_degPole, Vector3.up);
			}
		}



		void Code6()
		{
			if (!i_INITIALIZED)
			{
				Navy.lastShootFrame = 0;
				Navy.shootInterval = 8;
				Navy.initShootDirection = Vector3.forward;
				Navy.shellSpeed = 1.0f;
				Navy.initShootPositions = new Vector4[]
				{
					new Vector4( 0.358f, -0.13965f ,0.2371f, 1.0f),  
					new Vector4(-0.358f, -0.13965f ,0.2371f, 1.0f)
				};
				
				i_INITIALIZED = true;
			}
			
			NavyMotion();
			
			
			i_frameCount++;
			
			Matrix4x4 M = Matrix4x4.identity;
			Quaternion rotNavy = Quaternion.identity;
			int numShooter = 0;
			if (Input.GetKey(KeyCode.A))
			{
				if (i_frameCount >= Navy.lastShootFrame + Navy.shootInterval)
				{
					Navy.lastShootFrame = i_frameCount;
					numShooter = 2;
					
					M = Navy.transform.localToWorldMatrix;
					rotNavy = Navy.transform.rotation;
				}
			}
			
			
			Vector3 posNavy = Navy.GetWorldPosition();
			foreach (var shell in Shell)
			{
				if (shell.active)
				{
					Vector3 newShellPos = shell.transform.position + shell.speed * shell.direction;
					if ((posNavy - newShellPos).magnitude > 80.0f)
					{
						shell.active = false;
						shell.transform.position = new Vector3(-1000, 0, 0);
					}
					else
					{
						shell.transform.position = newShellPos;
					}
					
				}
				else
				{
					if (numShooter > 0)
					{
						Vector3 startPos = M * Navy.initShootPositions[2 - numShooter];
						Vector3 shootDir = M * Navy.initShootDirection;
						
						shell.active = true;
						shell.direction = shootDir;
						shell.speed = Navy.shellSpeed;
						
						shell.transform.position = startPos;
						shell.transform.rotation = rotNavy;
						
						numShooter--;
					}
				}
			}
		}




		void Code7()
		{
			if (!i_INITIALIZED)
			{
				Dice.transform.localPosition = A;
				Dice.transform.localRotation = Quaternion.identity;

				i_INITIALIZED = true;
			}


			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = true;
			}

			if (!i_MOVE) { return; }


		}



		void Code8()
		{
			if (!i_INITIALIZED)
			{
				
				i_INITIALIZED = true;
			}
			

		}





///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////




		void NavyMotion()
		{
			if (Input.GetKeyDown(KeyCode.S))
			{
				i_MOVE = !i_MOVE;
			}
			
			
			bool keyH = Input.GetKey(KeyCode.H);
			bool keyL = Input.GetKey(KeyCode.L);
			
			if (pr_autoState >= 0)
			{
				if (Input.GetKey(KeyCode.J))
				{
					Quaternion rot = (Navy.GetWorldRotation() * Quaternion.AngleAxis(1.0f, Vector3.right)).normalized;
					Navy.SetWorldRotation(rot);
				}
				else if (Input.GetKey(KeyCode.K))
				{
					Quaternion rot = (Navy.GetWorldRotation() * Quaternion.AngleAxis(-1.0f, Vector3.right)).normalized;
					Navy.SetWorldRotation(rot);
				}
				
				if (Input.GetKeyUp(KeyCode.H) || Input.GetKeyUp(KeyCode.L))
				{
					pr_autoState = -1;
					pr_autoRotCounter = 0;
					pr_cntMax = 30; 
					
					float deg = CalcElevationAngle(false);
					
					
					pr_staRot = pr_orieXZ;  
					
					if (!pr_Y_UP && pr_Y_UP_2)
					{
						Quaternion qq = Quaternion.AngleAxis(180, Vector3.up);
						pr_staRot = qq * pr_orieXZ;
						pr_orieY  = qq * pr_orieY;
						pr_Y_UP = false;
					}
					
					pr_endRot = Quaternion.Euler(deg, 0.0f, 0.0f);
				}
				else if (keyH || keyL)
				{
					if (pr_autoState == 0)
					{
						pr_autoState = 1;
						pr_autoRotCounter = 0;
						pr_cntMax = 25; 
						
						float deg = CalcElevationAngle();
						pr_staRot = Quaternion.AngleAxis(deg, Vector3.right);
						
						if (keyH)
						{
							if (pr_Y_UP)
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, 60);
								pr_endRot2 = Quaternion.Euler(0, 0, 45);
								pr_adg = -1.0f;
							}
							else
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, -60);
								pr_endRot2 = Quaternion.Euler(0, 180, 45);
								pr_adg = -1.0f;
							}
						}
						else if (keyL)
						{
							if (pr_Y_UP)
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, -60);
								pr_endRot2 = Quaternion.Euler(0, 0, -45);
								pr_adg = 1.0f;
							}
							else
							{
								pr_endRot = pr_staRot * Quaternion.Euler(0, 0, 60);
								pr_endRot2 = Quaternion.Euler(0, 180, -45);
								pr_adg = 1.0f;
							}
						}
					}
					else if (pr_autoState == 1)
					{
						if (pr_autoRotCounter == pr_cntMax)
						{
							pr_autoState = 2;
							pr_autoRotCounter = 0;
							pr_cntMax = 150;
							
							
							pr_staRot = pr_orieXZ;  
							pr_endRot = pr_endRot2;
						}
					}
					else if (pr_autoState >= 2)
					{
						if (pr_autoState == 2 && pr_autoRotCounter == pr_cntMax)
						{
							pr_autoState = 3;
						}
						
						
						pr_orieY = (Quaternion.AngleAxis(pr_adg, Vector3.up) * pr_orieY).normalized;
					}
				}
			}
			else    
			{
				if (pr_autoRotCounter == pr_cntMax)
				{
					pr_autoState = 0;
					pr_orieXZ = Quaternion.identity;
				}
			}
			
			if (IsAutoRotation())
			{
				pr_autoRotCounter++;
				
				if (pr_autoRotCounter <= pr_cntMax)
				{
					float t = pr_autoRotCounter / (float)pr_cntMax;
					pr_orieXZ = Quaternion.Slerp(pr_staRot, pr_endRot, t);
				}
			}
			
			if (pr_autoState != 0)
			{
				Quaternion rot = pr_orieY * pr_orieXZ;
				Navy.SetWorldRotation(rot);
			}
			
			
			
			Quaternion rotNavy = Navy.GetWorldRotation();
			Vector3 forwardDir = rotNavy * Vector3.forward;
			Vector3 curPos = Navy.GetWorldPosition();
			Vector3 newPos = (i_MOVE) ? curPos + 0.2f * forwardDir : curPos;  
			Navy.SetWorldPosition(newPos);
		}
		
		
		
		float CalcElevationAngle(bool updt=true)
		{
			Matrix4x4 mtx = Navy.GetMatrix();
			Vector3 lx = mtx.GetColumn(0).normalized;
			Vector3 ly = mtx.GetColumn(1).normalized;
			Vector3 lz = mtx.GetColumn(2).normalized;
			
			bool y_up = (Vector3.Dot(ly, Vector3.up) >= 0.0f);
			Vector3 prjZ = new Vector3(lz.x, 0.0f, lz.z).normalized;
			if (!y_up)
			{
				prjZ = -prjZ;
			}
			
			if (updt) { pr_Y_UP = y_up; }
			pr_Y_UP_2 = y_up;
			
			float deg = TH3DMath.CalcDegree_V1toV2(prjZ, lz, lx);
			
			return deg;
		}
		
		
		
		bool IsAutoRotation()
		{
			if(pr_autoState == -1 || pr_autoState == 1 || pr_autoState == 2)
			{
				return true;
			}
			
			return false;
		}




		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

				case 5 :	Code5();  break;

				case 6 :	Code6();  break;

				case 7 :	Code7();  break;

				case 8 :	Code8();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1))
			{
				AuxLine.transform.localScale = new Vector3(1.0f, 2.0f, 1.0f);
				AuxLine.transform.localRotation = Quaternion.AngleAxis(-20.0f, Vector3.forward);
				AuxLine.transform.localPosition = Sphere.GetLocalPosition();
			}
			else if (MatchCase(5))
			{
				Matrix4x4 lBox0 = TH3DMath.GetTranslation4x4(c_attachPos_Box) *
				TH3DMath.GetRotation4x4(i_degBox0, Vector3.up);
				
				float my = 3.0f * Mathf.Sin(i_shmArm * Mathf.Deg2Rad);
				Matrix4x4 lArm0 = TH3DMath.GetTranslation4x4(0.0f, my, 0.0f);
				
				Matrix4x4 lPole = TH3DMath.GetRotation4x4(i_degPole, Vector3.up);
				
				Matrix4x4 M = lPole * lArm0 * lBox0;
				YLine.SetMatrix(M);
			}
			else if (MatchCase(6))
			{
				var wpos = Navy.GetWorldPosition();
				
				var distC = new Vector3(4.82114f, 4.708963f, -5.310163f);
				var distT = new Vector3(-1.102283f, -2.86507f, 4.208788f);
				var uJ = new Vector3(-0.3925574f, 0.8230028f, 0.4105668f);
				
				var CamPos = distC + wpos;
				var TrgPos = distT + wpos;
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}


		static Sec504 pr_instance = new Sec504(8);

		public static Sec504 PRGSI()
		{
			return pr_instance;
		}


		private Sec504(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				Sphere_1 = CreateObject_Obj3D("Sphere_half");
				PointB = CreateObject_Obj3D("PointB");
				Body = CreateObject_Obj3D("PlaneBody");
				Propeller = CreateObject_Obj3D("Propeller");
				Disk = CreateObject_Obj3D("GapDisk");
				Arm = CreateObject_Obj3D("DBrwnArm");
				Sphere_3 = CreateObject_Obj3D("R2Sphere");
				Pole = CreateObject_Obj3D("MBlPole");
				ArmArray[0] = CreateObject_Obj3D("MBlArm");
				BoxArray[0] = CreateObject_Obj3D("SkyBox");
				PropellerArray[0] = CreateObject_Obj3D("BrProp");
				YLine = CreateObject_Obj3D("YLine");
				
				Navy = CreateObject_Alpha3D("FNavy");
				var src_FShell = CreateObject_Shell3D("FShell");
				Shell = new THShell3D[60];
				for (int i = 0; i < Shell.Length; i++)
				{
					Shell[i] = CreateObject_InsShell3D(src_FShell.gameObject);
				}
				
				Dice = CreateObject_Obj3D("Dice");
				DPath = CreateObject_Obj3D("DPath");
				CubeBottom = CreateObject_Obj3D("CubeBottom");
				CubeSide[0] = CreateObject_Obj3D("CubeSide");
				CubeSide2[0] = CreateObject_Obj3D("CubeSide2");
				
				// ArmArray
				for (int i = 1; i < ArmArray.Length; i++)
				{
					ArmArray[i] = CreateObject_InsObj3D(ArmArray[0].gameObject);
				}
				
				// BoxArray
				for (int i = 1; i < BoxArray.Length; i++)
				{
					BoxArray[i] = CreateObject_InsObj3D(BoxArray[0].gameObject);
				}
				
				// PropellerArray
				for (int i = 1; i < PropellerArray.Length; i++)
				{
					PropellerArray[i] = CreateObject_InsObj3D(PropellerArray[0].gameObject);
				}
				
				// CubeSide
				for (int i = 1; i < CubeSide.Length; i++)
				{
					CubeSide[i] = CreateObject_InsObj3D(CubeSide[0].gameObject);
				}
				
				// CubeSide2
				for (int i = 1; i < CubeSide2.Length; i++)
				{
					CubeSide2[i] = CreateObject_InsObj3D(CubeSide2[0].gameObject);
				}
				
				
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				AuxLine = CreateObject_Obj3D("AuxLine");
				
				
				Mesh mesh = AuxLine.gameObject.GetComponent<MeshFilter>().mesh;
				List<Vector3> verts = new List<Vector3>();
				verts.Add(new Vector3(0.0f, -0.5f, 0.0f));
				verts.Add(new Vector3(0.0f, 0.5f, 0.0f));
				mesh.SetVertices(verts);
				
				
				MeshRenderer mre = PointB.gameObject.GetComponent<MeshRenderer>();
				Material wmtrl = mre.material;
				wmtrl.SetColor("_Color", new Color(0.6792f, 0.6665f, 0.4773f, 1.0f));

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Sphere_1.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				BSAuxFuncs.SetVisible(1, PointB.gameObject);
				Sphere = Sphere_1;
				
				Sphere.SetIdentity();
				AuxCircle.SetIdentity();
				AuxLine.SetIdentity();
				PointB.SetIdentity();
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(5, 3, -5);
				
				PointB.transform.localScale = new Vector3(0.2f, 0.2f, 0.2f);
				PointB.transform.localPosition = new Vector3(5, 3, -5);
				
				i_degRev = 0.0f;
				i_degRot = 0.0f;
				
				CamPos = new Vector3(8.704226f, 6.571349f, -14.06188f);
				TrgPos = new Vector3(3.191605f, 0.5071556f, -0.02890308f);
				uJ = new Vector3(-0.1914269f, 0.9261311f, 0.3250184f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Body.transform.SetParent(null);
				
				Body.SetIdentity();
				Propeller.SetIdentity();
				AuxCircle.SetIdentity();
				
				AuxCircle.transform.localScale = new Vector3(8, 8, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(0, 10, 0);
				
				c_attachPos_Propeller = new Vector3(0.0f, 0.0f, 2.5f);
				i_degPropeller = 0;
				i_degBody = 0;
				
				CamPos = new Vector3(12.62591f, 25.24535f, -19.98914f);
				TrgPos = new Vector3(0.6450775f, 7.977334f, -0.9672379f);
				uJ = new Vector3(-0.3318465f, 0.7930158f, 0.5108854f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Sphere_3.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Sphere = Sphere_3;
				
				Arm.transform.SetParent(null);
				Sphere.transform.SetParent(null);
				
				Sphere.SetIdentity();
				Arm.SetIdentity();
				Disk.SetIdentity();
				
				i_shmDisk = 0;
				i_degArm = 0;
				i_shmSphere = 270;
				
				CamPos = new Vector3(12.2925f, 12.65544f, -24.1966f);
				TrgPos = new Vector3(-0.90039f, 9.064087f, 1.753844f);
				uJ = new Vector3(-0.04559816f, 0.9924152f, 0.1141616f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, PropellerArray[0].gameObject);
				BSAuxFuncs.SetVisible(1, BoxArray[0].gameObject);
				BSAuxFuncs.SetVisible(1, ArmArray[0].gameObject);
				BSAuxFuncs.SetVisible(1, Pole.gameObject);
				
				ArmArray[0].transform.SetParent(null);
				BoxArray[0].transform.SetParent(null);
				PropellerArray[0].transform.SetParent(null);
				
				PropellerArray[0].SetIdentity();
				BoxArray[0].SetIdentity();
				ArmArray[0].SetIdentity();
				Pole.SetIdentity();
				
				c_attachPos_Propeller = new Vector3(0, 0, -1);
				i_degPropeller = 0;
				i_shmArm = 0;
				i_degPole = 0;
				
				CamPos = new Vector3(12.0253f, 20.00275f, -13.07123f);
				TrgPos = new Vector3(-0.3658276f, 7.516417f, 0.8944302f);
				uJ = new Vector3(-0.3822502f, 0.8311056f, 0.403916f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(5))
			{
				for (int i = 0; i < 3; i++)
				{
					BSAuxFuncs.SetVisible(1, PropellerArray[i].gameObject);
					BSAuxFuncs.SetVisible(1, BoxArray[i].gameObject);
					BSAuxFuncs.SetVisible(1, ArmArray[i].gameObject);
					
					ArmArray[i].transform.SetParent(null);
					BoxArray[i].transform.SetParent(null);
					PropellerArray[i].transform.SetParent(null);
					
					PropellerArray[i].SetIdentity();
					BoxArray[i].SetIdentity();
					ArmArray[i].SetIdentity();
				}
				
				BSAuxFuncs.SetVisible(1, Pole.gameObject);
				Pole.SetIdentity();
				
				BSAuxFuncs.SetVisible(1, YLine.gameObject);
				YLine.SetIdentity();
				
				c_attachPos_Propeller = new Vector3(0, 0, -1);
				i_degPropeller = 0;
				i_shmArm = 0;
				i_degPole = 0;
				i_switch_Pole = false;
				i_switch_Arm = true;
				i_degBox0 = 0;
				
				CamPos = new Vector3(12.0253f, 20.00275f, -13.07123f);
				TrgPos = new Vector3(-0.3658276f, 7.516417f, 0.8944302f);
				uJ = new Vector3(-0.3822502f, 0.8311056f, 0.403916f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(6))
			{
				BSAuxFuncs.SetVisible(1, Navy.gameObject);
				Navy.SetIdentity();
				
				float yy = 24;
				Navy.SetWorldPosition(0, yy, 0);
				
				for (int i = 0; i < Shell.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, Shell[i].gameObject);
					Shell[i].SetIdentity();
					Shell[i].position = new Vector3(-1000, 0, 0);
				}
				
				i_frameCount = 0;
				pr_autoState = 0;
				pr_orieXZ = Quaternion.identity;
				pr_orieY = Quaternion.identity;
				pr_Y_UP = false;
				
				CamPos = new Vector3(15.08402f, 12.56928f + yy, -15.28707f);     
				TrgPos = new Vector3(3.866516f, 4.686559f + yy, -2.338552f);
				uJ = new Vector3(-0.3025928f, 0.9077742f, 0.2904888f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(7))
			{
				BSAuxFuncs.SetVisible(1, Dice.gameObject);
				BSAuxFuncs.SetVisible(1, DPath.gameObject);
				Dice.SetMatrix(Matrix4x4.identity);
				DPath.SetMatrix(Matrix4x4.identity);
				DPath.SetLocalPosition(4.0f, 0.02f, 2.0f);
				
				i_edgeList = new List<Vector3[]>()
				{
					new Vector3[]{ V[0],  V[1]},
					new Vector3[]{ V[3],  V[4]},
					new Vector3[]{ V[7],  V[8]},
					new Vector3[]{ V[12], V[8]},
					new Vector3[]{ V[13], V[9]},
					new Vector3[]{ V[14], V[10]},
					new Vector3[]{ V[11], V[10]},
					new Vector3[]{ V[6],  V[10]},
					new Vector3[]{ V[5],  V[9]},
					new Vector3[]{ V[5],  V[4]},
					new Vector3[]{ V[2],  V[1]}
				};
				
				CamPos = new Vector3(4.886537f, 5.291785f, -3.634611f);
				TrgPos = new Vector3(4.886537f, 0.8763647f, 1.994584f);
				uJ = new Vector3(0f, 0.7868286f, 0.6171716f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(8))
			{
				BSAuxFuncs.SetVisible(1, CubeBottom.gameObject);
				BSAuxFuncs.SetVisible(1, CubeSide[0].gameObject);
				BSAuxFuncs.SetVisible(1, CubeSide[1].gameObject);
				BSAuxFuncs.SetVisible(1, CubeSide[2].gameObject);
				BSAuxFuncs.SetVisible(1, CubeSide2[0].gameObject);
				BSAuxFuncs.SetVisible(1, CubeSide2[1].gameObject);
				
				CubeSide[0].transform.SetParent(null);
				CubeSide[1].transform.SetParent(null);
				CubeSide[2].transform.SetParent(null);
				CubeSide2[0].transform.SetParent(null);
				CubeSide2[1].transform.SetParent(null);
				
				CubeBottom.SetIdentity();
				CubeSide[0].SetIdentity();
				CubeSide[1].SetIdentity();
				CubeSide[2].SetIdentity();
				CubeSide2[0].SetIdentity();
				CubeSide2[1].SetIdentity();
				
				Face[0] = CubeBottom;
				Face[1] = CubeSide[0];
				Face[2] = CubeSide[1];
				Face[3] = CubeSide[2];
				Face[4] = CubeSide2[0];
				Face[5] = CubeSide2[1]; 
				
				CrdSys.SetLocalPosition(0.0f, -0.005f, 0.0f);
				
				CamPos = new Vector3(2.251693f, 2.613619f, -3.940248f);
				TrgPos = new Vector3(0.9813164f, 0.8267338f, -1.397259f);
				uJ = new Vector3(-0.1836902f, 0.8450986f, 0.5020621f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
