//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using OQJ_THBetaObject3D;



namespace OQJ_Sec508
{
	public class Sec508 : BSSuperExe
	{
		// Objects
		THObject3D CrdSys;
		
		// Code1
		THBetaObject3D Sphere;
		THBetaObject3D Sphere_1;
		THBetaObject3D PointB;
		THObject3D AuxCircle;
		THObject3D AuxLine;
		
		// Code2
		THBetaObject3D Body;
		THBetaObject3D Propeller;
		
		// Code3
		THBetaObject3D Disk;
		THBetaObject3D Arm;
		THBetaObject3D Sphere_3;
		
		// Code4
		THBetaObject3D[] Face = new THBetaObject3D[6];	
		THBetaObject3D[] CubeSide = new THBetaObject3D[5];	
		THBetaObject3D CubeBottom;	
		


		// Instance Variables
		// Code1
		float i_degRev = 0.0f;
		float i_degRot = 0.0f;
		Vector3 c_centerPos = new Vector3(5, 3, -5);
		
		// Code2
		int i_degPropeller = 0;
		int i_degBody = 0;
		Vector3 c_attachPos_Propeller = new Vector3(0.0f, 0.0f, 2.5f);
		
		// Code3
		float i_shmSphere = 0;
		float i_degArm = 0;
		float i_shmDisk = 0;
		readonly Vector3 c_attachPos_Sphere = new Vector3(5, 4, 0);
		readonly Vector3 c_attachPos_Arm = new Vector3(2, 0, 0);
		
		// Code4
		float i_degPrimarySide = 0.0f;
		float i_degSide = 0.0f;
		float i_degTop = 0.0f;
		int i_motionCounter;




		void Code1()
		{
			i_degRot += 4.0f;
			Matrix4x4 rotY = TH3DMath.GetRotation4x4(i_degRot, Vector3.up);
			Matrix4x4 rotZ = TH3DMath.GetRotation4x4(-20.0f, Vector3.forward);
			Matrix4x4 rot = rotZ * rotY;
			
			i_degRev += 1.0f;
			Matrix4x4 rev = TH3DMath.GetRotation4x4(i_degRev, Vector3.up);
			Vector3 np = rev * new Vector4(3.0f, 0.0f, 0.0f, 1.0f);
			Vector3 pos = np + c_centerPos;
			
			Sphere.thTransform.thLocalPosition = pos;
			Sphere.thTransform.thLocalRotation = rot;
			
			
			Sphere.thTransform.UpdateChain();
		}


		void Code2()
		{
			if (!i_INITIALIZED)
			{
				
				Propeller.thTransform.SetParent(Body.thTransform);
				
				i_INITIALIZED = true;
			}
			
			// Propeller
			i_degPropeller += 15;
			Propeller.thTransform.thLocalPosition = c_attachPos_Propeller;
			Propeller.thTransform.thLocalRotation = TH3DMath.GetRotation4x4(i_degPropeller, Vector3.forward);
			
			// Body
			i_degBody -= 2;
			Matrix4x4 rotBody = TH3DMath.GetRotation4x4(i_degBody, Vector3.up);
			Vector3 posBody = rotBody * new Vector4(8.0f, 10.0f, 0.0f, 1.0f);
			Body.thTransform.thLocalPosition = posBody;
			Body.thTransform.thLocalRotation = rotBody;
			
			
			Body.thTransform.UpdateChain();
		}


		void Code3()
		{
			if (!i_INITIALIZED)
			{
				// 親子関係の設定
				Sphere.thTransform.SetParent(Arm.thTransform);
				Arm.thTransform.SetParent(Disk.thTransform);
				
				i_INITIALIZED = true;
			}
			
			i_shmSphere += 1.0f;
			float scale = 0.5f * Mathf.Sin(i_shmSphere * Mathf.Deg2Rad) + 1.5f; // 1 -- 2
			Sphere.thTransform.thLocalScale = new Vector3(scale, scale, scale);
			Sphere.thTransform.thLocalPosition = c_attachPos_Sphere;
			
			i_degArm--;
			Arm.thTransform.thLocalRotation = TH3DMath.GetRotation4x4(i_degArm, Vector3.up);
			Arm.thTransform.thLocalPosition = c_attachPos_Arm;
			
			i_shmDisk += 1.0f;
			float posY = 4.0f * Mathf.Sin(i_shmDisk * Mathf.Deg2Rad) + 8.0f; // 4 -- 12
			Disk.thTransform.thLocalRotation = TH3DMath.GetRotation4x4(-15.0f, Vector3.right);
			Disk.thTransform.thLocalPosition = new Vector3(0.0f, posY, 0.0f);
			
			
			Disk.thTransform.UpdateChain();
		}


		void Code4()
		{
			if (!i_INITIALIZED)
			{
				i_degPrimarySide = 0.0f;
				i_degSide = 0.0f;
				i_degTop = 0.0f;
				i_motionCounter = 0;
				
				// 親子関係の設定
				Face[1].thTransform.SetParent(Face[0].thTransform);
				Face[2].thTransform.SetParent(Face[1].thTransform);
				Face[3].thTransform.SetParent(Face[2].thTransform);
				Face[4].thTransform.SetParent(Face[3].thTransform);
				Face[5].thTransform.SetParent(Face[4].thTransform);
				
				i_INITIALIZED = true;
			}
			
			
			if (Input.GetKey(KeyCode.J))
			{
				i_motionCounter++;
				if (i_motionCounter > 240) { i_motionCounter = 240; }
			}
			else if (Input.GetKey(KeyCode.K))
			{
				i_motionCounter--;
				if (i_motionCounter < 0) { i_motionCounter = 0; }
			}
			
			if (0 <= i_motionCounter && i_motionCounter <= 60)
			{
				float t = i_motionCounter / 60.0f;
				i_degPrimarySide = -90.0f * t;
			}
			if (60 <= i_motionCounter && i_motionCounter <= 180)
			{
				float t = (i_motionCounter - 60) / 120.0f;
				i_degSide = 90.0f * t;
			}
			if (180 <= i_motionCounter && i_motionCounter <= 240)
			{
				float t = (i_motionCounter - 180) / 60.0f;
				i_degTop = -90.0f * t;
			}
			
			
			Face[5].thTransform.thLocalPosition = new Vector3(0.0f, 0.0f, 1.0f);
			Face[5].thTransform.thLocalRotation = TH3DMath.GetRotation4x4(i_degTop, Vector3.right);
			
			for (int i = 2; i <= 4; i++)
			{
				Face[i].thTransform.thLocalPosition = new Vector3(1.0f, 0.0f, 0.0f);
				Face[i].thTransform.thLocalRotation = TH3DMath.GetRotation4x4(i_degSide, Vector3.forward);
			}
			
			Face[1].thTransform.thLocalRotation = TH3DMath.GetRotation4x4(i_degPrimarySide, Vector3.right);
			
			Face[0].thTransform.UpdateChain();
		}






///////////////////////////////////////////////////////////////////////////
//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

				case 3 :	Code3();  break;

				case 4 :	Code4();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1))
			{
				AuxLine.transform.localScale = new Vector3(1.0f, 2.0f, 1.0f);
				AuxLine.transform.localRotation = Quaternion.AngleAxis(-20.0f, Vector3.forward);
				AuxLine.transform.localPosition = Sphere.GetLocalPosition();
			}
		}


		static Sec508 pr_instance = new Sec508(4);

		public static Sec508 PRGSI()
		{
			return pr_instance;
		}


		private Sec508(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj3D("CoordinateSystem3D");
				
				Sphere_1 = CreateObject_Beta3D("Sphere_half");
				PointB = CreateObject_Beta3D("PointB");
				Body = CreateObject_Beta3D("PlaneBody");
				Propeller = CreateObject_Beta3D("Propeller");
				Disk = CreateObject_Beta3D("GapDisk");
				Arm = CreateObject_Beta3D("DBrwnArm");
				Sphere_3 = CreateObject_Beta3D("R2Sphere");
				CubeBottom = CreateObject_Beta3D("CubeBottom");
				
				AuxCircle = CreateObject_Obj3D("AuxCircle");
				AuxLine = CreateObject_Obj3D("AuxLine");
				
				CubeSide[0] = CreateObject_Beta3D("CubeSide");
				for (int i = 0; i < CubeSide.Length; i++)
				{
					CubeSide[i] = CreateObject_InsBeta3D(CubeSide[0].gameObject);
				}
				
				
				
				Mesh mesh = AuxLine.gameObject.GetComponent<MeshFilter>().mesh;
				List<Vector3> verts = new List<Vector3>();
				verts.Add(new Vector3(0.0f, -0.5f, 0.0f));
				verts.Add(new Vector3(0.0f, 0.5f, 0.0f));
				mesh.SetVertices(verts);
				
				
				MeshRenderer mre = PointB.gameObject.GetComponent<MeshRenderer>();
				Material wmtrl = mre.material;
				wmtrl.SetColor("_Color", new Color(0.6792f, 0.6665f, 0.4773f, 1.0f));

				pr_LOAD_OBJECTS = true;
			}


			Preset();
		}


		protected override void Preset()
		{
			Vector3 CamPos;
			Vector3 TrgPos;
			Vector3 uJ;
			
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetMatrix(Matrix4x4.identity);
			
			if (MatchCase(1))
			{
				BSAuxFuncs.SetVisible(1, Sphere_1.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				BSAuxFuncs.SetVisible(1, AuxLine.gameObject);
				BSAuxFuncs.SetVisible(1, PointB.gameObject);
				Sphere = Sphere_1;
				
				Sphere.thTransform.Clear();
				
				Sphere.SetIdentity();
				AuxCircle.SetIdentity();
				AuxLine.SetIdentity();
				PointB.SetIdentity();
				
				AuxCircle.transform.localScale = new Vector3(3, 3, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(5, 3, -5);
				
				PointB.transform.localScale = new Vector3(0.2f, 0.2f, 0.2f);
				PointB.transform.localPosition = new Vector3(5, 3, -5);
				
				i_degRev = 0.0f;
				i_degRot = 0.0f;
				
				CamPos = new Vector3(8.704226f, 6.571349f, -14.06188f);
				TrgPos = new Vector3(3.191605f, 0.5071556f, -0.02890308f);
				uJ = new Vector3(-0.1914269f, 0.9261311f, 0.3250184f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(2))
			{
				BSAuxFuncs.SetVisible(1, Body.gameObject);
				BSAuxFuncs.SetVisible(1, Propeller.gameObject);
				BSAuxFuncs.SetVisible(1, AuxCircle.gameObject);
				
				Body.thTransform.Clear();
				Propeller.thTransform.Clear();
				
				Body.SetIdentity();
				Propeller.SetIdentity();
				AuxCircle.SetIdentity();
				
				AuxCircle.transform.localScale = new Vector3(8, 8, 1);
				AuxCircle.transform.localRotation = Quaternion.AngleAxis(90.0f, Vector3.right);
				AuxCircle.transform.localPosition = new Vector3(0, 10, 0);
				
				c_attachPos_Propeller = new Vector3(0.0f, 0.0f, 2.5f);
				i_degPropeller = 0;
				i_degBody = 0;
				
				CamPos = new Vector3(12.62591f, 25.24535f, -19.98914f);
				TrgPos = new Vector3(0.6450775f, 7.977334f, -0.9672379f);
				uJ = new Vector3(-0.3318465f, 0.7930158f, 0.5108854f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
				
			}
			else if (MatchCase(3))
			{
				BSAuxFuncs.SetVisible(1, Sphere_3.gameObject);
				BSAuxFuncs.SetVisible(1, Arm.gameObject);
				BSAuxFuncs.SetVisible(1, Disk.gameObject);
				Sphere = Sphere_3;
				
				Disk.thTransform.Clear();
				Arm.thTransform.Clear();
				Sphere.thTransform.Clear();
				
				Sphere.SetIdentity();
				Arm.SetIdentity();
				Disk.SetIdentity();
				
				i_shmDisk = 0;
				i_degArm = 0;
				i_shmSphere = 270;
				
				CamPos = new Vector3(12.2925f, 12.65544f, -24.1966f);
				TrgPos = new Vector3(-0.90039f, 9.064087f, 1.753844f);
				uJ = new Vector3(-0.04559816f, 0.9924152f, 0.1141616f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
			else if (MatchCase(4))
			{
				BSAuxFuncs.SetVisible(1, CubeBottom.gameObject);
				CubeBottom.thTransform.Clear();
				CubeBottom.SetIdentity();
				
				for (int i = 0; i < CubeSide.Length; i++)
				{
					BSAuxFuncs.SetVisible(1, CubeSide[i].gameObject);
					CubeSide[i].thTransform.Clear();
					CubeSide[i].SetIdentity();
				}
				
				Face[0] = CubeBottom;
				Face[1] = CubeSide[0];
				Face[2] = CubeSide[1];
				Face[3] = CubeSide[2];
				Face[4] = CubeSide[3];
				Face[5] = CubeSide[4];  
				
				CrdSys.SetLocalPosition(0.0f, -0.005f, 0.0f);
				
				CamPos = new Vector3(4.030374f, 3.147966f, -3.108043f);
				TrgPos = new Vector3(2.186804f, 0.5344113f, -0.2183516f);
				uJ = new Vector3(-0.3673171f, 0.7942415f, 0.4840027f);
				
				SetCameraPosition_3D(CamPos, TrgPos, uJ);
			}
		}

	}

}
