//-------------------------------------------
// Copyright (c) 2020-2025, Redpoll's 60.
// https://www.eproph527tw.com/
//-------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using ClEllipse = OQJ_MainPB.ClEllipse;



namespace OQJ_SecB01
{
	public class SecB01 : BSSuperExe
	{
		// Objects
		THObject2D CrdSys;

		ClEllipse White;
		ClEllipse Blue;

		ClEllipse pr_Target;

		GameObject MainCamera;	


		// Instance Variables

		bool pr_isHitColor;



		void Code1()
		{
			if (!i_INITIALIZED)
			{
				White.scaleX = 2.34f;
				White.scaleY = 1.23f;
				White.degree = 25.0f;
				White.position = new Vector2(-3.55f, 1.25f);

				Blue.scaleX = 1.25f;
				Blue.scaleY = 2.21f;
				Blue.degree = 26.5f;
				Blue.position = new Vector2(2.65f, 0.2f);

				i_INITIALIZED = true;
			}

			ObjectMotion();

			Vector2 E1 = White.position;
			Vector2 lx1 = White.localX;
			Vector2 ly1 = White.localY;
			float sx1 = White.scaleX;
			float sy1 = White.scaleY;

			Vector2 E2 = Blue.position;
			Vector2 lx2 = Blue.localX;
			Vector2 ly2 = Blue.localY;
			float sx2 = Blue.scaleX;
			float sy2 = Blue.scaleY;

			bool bColl = CollisionTest_Ellipse_Ellipse(E1, lx1, ly1, sx1, sy1, E2, lx2, ly2, sx2, sy2);
			Hit(bColl);

		}



		void Code2()
		{
			if (!i_INITIALIZED)
			{
				White.scaleX = 0.9f;
				White.scaleY = 1.7f;
				White.position = new Vector2(-2.4f, 2.3f);

				i_INITIALIZED = true;
			}

			ObjectMotion();

			Vector2 E = White.position;
			float sx = White.scaleX;
			float sy = White.scaleY;

			E.x = -Mathf.Abs(E.x);
			E.y = Mathf.Abs(E.y);

			float a = sx;
			float b = sy;

			if (sx > sy)
			{
				a = sy;
				b = sx;
				float w = E.x;
				E.x = -E.y;  
				E.y = -w;
			}

			bool bColl = CollisionTest_AAEllipse_UnitCircle(E, a, b);
			Hit(bColl);
		}




		bool CollisionTest_Ellipse_Ellipse(	Vector2 E1, Vector2 lx1, Vector2 ly1, float sx1, float sy1,
											Vector2 E2, Vector2 lx2, Vector2 ly2, float sx2, float sy2)
		{
			Vector3[] m1c =	new Vector3[]	// cols, M1
			{
				new Vector3(sx1 * lx1.x, sx1 * lx1.y, 0),
				new Vector3(sy1 * ly1.x, sy1 * ly1.y, 0),
				new Vector3(E1.x, E1.y, 1)
			};

			Vector3[] im2r = CalcInverseRows(E2, lx2, ly2, sx2, sy2);     // rows, invM2

			//// A = (col1, col2, col3) = invM2 * M1
			Vector3 col1 = new Vector3(Vector3.Dot(im2r[0], m1c[0]), Vector3.Dot(im2r[1], m1c[0]), 0);
			Vector3 col2 = new Vector3(Vector3.Dot(im2r[0], m1c[1]), Vector3.Dot(im2r[1], m1c[1]), 0);
			Vector3 col3 = new Vector3(Vector3.Dot(im2r[0], m1c[2]), Vector3.Dot(im2r[1], m1c[2]), 1);

			float a = col1.x;
			float c = col1.y;
			float b = col2.x;
			float d = col2.y;
			float e = a * a + b * b;
			float f = a * c + b * d;
			float g = c * c + d * d;

			float ja = e + g;
			float jb = Mathf.Sqrt((e - g) * (e - g) + 4.0f * f * f);
			float G1 = (ja + jb) * 0.5f;
			float G2 = (ja - jb) * 0.5f;

			Vector2 u1 = new Vector2(1.0f, (G1 - e) / f).normalized;   
			Vector2 u2 = new Vector2(-u1.y, u1.x);                    

			float r_a = Mathf.Sqrt(G1);
			float r_b = Mathf.Sqrt(G2);

			Vector2 wp = col3;
			float x0 = Vector2.Dot(-wp, u1); 
			float y0 = Vector2.Dot(-wp, u2);

			Vector2 E = new Vector2(-Mathf.Abs(-x0), Mathf.Abs(-y0)); 

			if (r_a > r_b)
			{
				float w = r_a;
				r_a = r_b;
				r_b = w;
				w = E.x;
				E.x = -E.y;   
				E.y = -w;
			}

			return CollisionTest_AAEllipse_UnitCircle(E, r_a, r_b);
		}



		Vector3[] CalcInverseRows(Vector2 pos, Vector2 lx, Vector2 ly, float sx, float sy)
		{
			float p = pos.x;
			float q = pos.y;
			float a = lx.x;
			float b = ly.x;
			float c = lx.y;
			float d = ly.y;

			float inv_s = 1.0f / sx;
			float inv_t = 1.0f / sy;

			Vector3[] rows =
			{
				new Vector3(a * inv_s, c * inv_s, -(a * p + c * q) * inv_s),
				new Vector3(b * inv_t, d * inv_t, -(b * p + d * q) * inv_t),
				new Vector3(0, 0, 1)
			};

			return rows;
		}



		bool CollisionTest_AAEllipse_UnitCircle(Vector2 E, float a, float b)
		{
			if (E.x + a < -1.0f || E.y - b > 1.0f) { return false; }

			float k = a / b;
			float inv_a = 1.0f / a;
			float tanTh, x;

			if (E.x < -1.0f)
			{
				float d = -1.0f - E.x;
				float cosBt = d * inv_a;  // d / a;
				float sinBt = -Mathf.Sqrt(1 - cosBt * cosBt);
				Vector2 P = new Vector2(E.x + d, E.y + b * sinBt);	// d == a*cosBt

				if (P.y <= 0.0f) { return true; }
				else if (P.y >= 1.0f) { return false; }

				float sinAl = P.y;
				float cosAl = -(Mathf.Sqrt(1.0f - sinAl * sinAl));

				float mEll = -(b * cosBt) / (a * sinBt);
				float mCir = -cosAl / sinAl;
				if(mEll >= mCir) { return false; }

				float A = sinAl / cosAl;	// tanAl
				float B = sinBt / cosBt;	// tanBt

				float s = k * A - B;
				float t = (k + 1) * (A * B + 1);
				float u = k * B - A;
				float D = t * t - 4 * s * u;
				float sqD = Mathf.Sqrt(D);

				x = (-t + sqD) / (2 * s);
				tanTh = (B + x) / (1.0f - B * x);	// tan(Beta + x)
			}
			else
			{
				Vector2 P = new Vector2(E.x, E.y - b);

				if(P.sqrMagnitude <= 1.0f || P.y <= 0.0f) { return true; }

				float sinAl = P.y;
				float cosAl = -(Mathf.Sqrt(1.0f - sinAl * sinAl));

				float A = sinAl / cosAl;	// tanAl
				// float s = 1.0f;
				float t = -(A + k * A);
				float u = -k;
				float D = t * t - 4 * u;
				float sqD = Mathf.Sqrt(D);

				x = (-t + sqD) * 0.5f;
				tanTh = -1.0f / x;
			}

			float cosTh = Mathf.Sqrt(1.0f / (1.0f + tanTh * tanTh));
			float sinTh = tanTh * cosTh;
			Vector2 Q = new Vector2(E.x + a * cosTh, E.y + b * sinTh);

			if(Q.sqrMagnitude <= 1.0f) { return true; }


			// [CASE 2]
			//
			//tanPh = (A - x) / (1.0f + A * x);
			float tanPh = k * tanTh;	// tan(A-x) == k*tan(B+x)
			float cosPh = -Mathf.Sqrt(1.0f / (1.0f + tanPh * tanPh));
			float sinPh = tanPh * cosPh;
			Vector2 Qd = new Vector2(cosPh, sinPh);		// Q'

			Vector2 L = Qd - E;		// new Vector2(Qd.x - E.x, Qd.y - E.y);
			L.y *= k;

			float a2 = a * a;
			if (L.sqrMagnitude <= a2) { return true; }


			// [CASE 3]
			const float EPS = 0.0002f;
			if(Q.x < Qd.x)
			{
				Vector2 dirQ = new Vector2(-a * sinTh, b * cosTh).normalized;

				float t = Vector2.Dot(Q, dirQ);
				float u = Q.sqrMagnitude - 1.0f;
				float D = t * t - u;

				if (D < 0.0f || (-t - Mathf.Sqrt(D)) < 0.0f)
				{ return false;	}

				Vector2 Rd = Q + (-t - Mathf.Sqrt(D)) * dirQ;	// R'
				float m = -Rd.x / Rd.y;     // = -1 / (k*tanR)
				float tanGm = -1.0f / (k * m);
				float cosGm = Mathf.Sqrt(1.0f / (1.0f + tanGm * tanGm));
				float sinGm = tanGm * cosGm;

				Vector2 R = new Vector2(E.x + a * cosGm, E.y + b * sinGm);

				if (R.magnitude <= 1.0f + EPS) { return true; }
			}	

			// [CASE 4 (and CASE 3)]
			{
				Vector2 v1 = -new Vector2(-Qd.y, Qd.x);

				L.y = -b * Mathf.Sqrt(1.0f - (L.x * L.x) / a2);
				Vector2 v2 = new Vector2(-L.y * k, L.x * b * inv_a).normalized; 
				Vector2 v3 = (v1 + v2).normalized;

				Vector2 Sd = new Vector2(-v3.y, v3.x);
				Vector2 S = Sd - E;
				S.y *= k;

				return (S.magnitude * inv_a) <= (1.0f + EPS);  
			}
		}



		void ObjectMotion()
		{
			if (!(Input.GetKeyDown(KeyCode.A) ||
				Input.GetKey(KeyCode.S) || Input.GetKey(KeyCode.T) || Input.GetKey(KeyCode.R) ||
				Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.J) ||
				Input.GetKey(KeyCode.K) || Input.GetKey(KeyCode.L)))
			{ return; }


			if (MatchCase(1) && THUtil.IsShiftDown() && Input.GetKeyDown(KeyCode.A))
			{
				pr_Target = (pr_Target == Blue) ? White : Blue;
			}


			float s = 0.01f;
			if (Input.GetKey(KeyCode.S))
			{
				pr_Target.scaleX += THUtil.IsShiftDown() ? -s : s;
				if (pr_Target.scaleX < s) { pr_Target.scaleX = s; }
			}
			else if (Input.GetKey(KeyCode.T))
			{
				pr_Target.scaleY += THUtil.IsShiftDown() ? -s : s;
				if (pr_Target.scaleY < s) { pr_Target.scaleY = s; }
			}


			if (MatchCase(1) && Input.GetKey(KeyCode.R))
			{
				float deg = pr_Target.degree;
				deg += THUtil.IsShiftDown() ? -0.25f : 0.25f;

				if (deg >= 360.0f) { deg -= 360.0f; }
				else if (deg < 0.0f) { deg += 360.0f; }

				pr_Target.degree = deg;
			}


			Vector2 pos = pr_Target.position;
			float mv = 0.025f;
			if (Input.GetKey(KeyCode.H))
			{
				pos.x -= mv;
			}
			else if (Input.GetKey(KeyCode.L))
			{
				pos.x += mv;
			}

			if (Input.GetKey(KeyCode.J))
			{
				pos.y -= mv;
			}
			else if (Input.GetKey(KeyCode.K))
			{
				pos.y += mv;
			}

			pr_Target.position = pos;
		}



		void Hit(bool b)
		{
			var mre = White.gameObject.GetComponent<MeshRenderer>();
			if (b)
			{
				mre.material.SetColor("_Color", new Color(0.7f, 0.1f, 0.1f, 0.55f));
			}
			else 
			{
				mre.material.SetColor("_Color", new Color(0.8f, 0.8f, 0.8f, 0.55f));
			}
		}



		///////////////////////////////////////////////////////////////////////////
		//////////////////////////////// Backstage ////////////////////////////////


		public override void Update()
		{
			PreProcess();

			switch (THSettings.CODE_NR)
			{
				case 1 :	Code1();  break;

				case 2 :	Code2();  break;

			}

			PostProcess();
		}


		void PreProcess()
		{
		}


		void PostProcess()
		{
			if (MatchCase(1, 2))
			{
				Vector3 pos = White.transform.localPosition;
				pos.z = -0.001f;
				White.transform.localPosition = pos;
			}
		}


		static SecB01 pr_instance = new SecB01(2);

		public static SecB01 PRGSI()
		{
			return pr_instance;
		}


		private SecB01(int num_code) : base(num_code) { }


		public override void Initialize()
		{
			if(!pr_LOAD_OBJECTS)
			{
				CrdSys = CreateObject_Obj2D("CoordinateSystem2D");
				White = (ClEllipse)CreateObject_Others2D("SHR_W");
				Blue  = (ClEllipse)CreateObject_Others2D("SHR_B");
				
				var trpMtrl = BSAuxFuncs.GetSemiTransparentMaterial(0.8f, 0.8f, 0.8f, 0.55f);
				var mre = White.gameObject.GetComponent<MeshRenderer>();
				mre.material = trpMtrl;

				BSAuxFuncs.SetUnlitMaterial(Blue, 0.0f, 0.0f, 0.8f);

				BSData.c_SHOW_COORDINATE_SCALE = false;

				MainCamera = BSAuxFuncs.GetMainCamera();
				MainCamera.GetComponent<Camera>().nearClipPlane = 0.03f;
				MainCamera.GetComponent<Camera>().farClipPlane  = 20.0f;

				pr_LOAD_OBJECTS = true;
			}

			SetOutputImageSize(4.8f, 400);
			SetCameraPosition_2D(0, 0);
			
			SetFPS(60);

			Preset();
		}


		protected override void Preset()
		{
			BSAuxFuncs.SetVisible(1, CrdSys.gameObject);
			CrdSys.SetIdentity();
			

			if (MatchCase(1, 2))
			{
				BSAuxFuncs.SetVisible(1, White.gameObject);
				White.SetIdentity();

				BSAuxFuncs.SetVisible(1, Blue.gameObject);
				Blue.SetIdentity();

				pr_Target = White;
			}
		}
	}

}

